@@ -640,9 +640,9 @@ namespace python
 
     template <int dim, int spacedim>
     std::pair<int, int>
-    find_active_cell_around_point(PointWrapper &          p,
-                                  MappingQGenericWrapper &mapping_wrapper,
-                                  void *                  triangulation)
+    find_active_cell_around_point(PointWrapper &   p,
+                                  MappingQWrapper &mapping_wrapper,
+                                  void *           triangulation)
     {
       Triangulation<dim, spacedim> *tria =
         static_cast<Triangulation<dim, spacedim> *>(triangulation);
@@ -651,8 +651,8 @@ namespace python
 
       if (mapping_wrapper.get_mapping() != nullptr)
         {
-          const MappingQGeneric<dim, spacedim> *mapping =
-            static_cast<const MappingQGeneric<dim, spacedim> *>(
+          const MappingQ<dim, spacedim> *mapping =
+            static_cast<const MappingQ<dim, spacedim> *>(
               mapping_wrapper.get_mapping());
 
           auto cell_pair =
@@ -671,9 +671,9 @@ namespace python
     template <int dim, int spacedim>
     boost::python::list
     compute_aspect_ratio_of_cells(
-      const MappingQGenericWrapper &mapping_wrapper,
-      const QuadratureWrapper &     quadrature_wrapper,
-      const TriangulationWrapper &  triangulation_wrapper)
+      const MappingQWrapper &     mapping_wrapper,
+      const QuadratureWrapper &   quadrature_wrapper,
+      const TriangulationWrapper &triangulation_wrapper)
     {
       const Triangulation<dim, spacedim> *tria =
         static_cast<const Triangulation<dim, spacedim> *>(
@@ -682,8 +682,8 @@ namespace python
       const Quadrature<dim> *quad = static_cast<const Quadrature<dim> *>(
         quadrature_wrapper.get_quadrature());
 
-      const MappingQGeneric<dim, spacedim> *mapping =
-        static_cast<const MappingQGeneric<dim, spacedim> *>(
+      const MappingQ<dim, spacedim> *mapping =
+        static_cast<const MappingQ<dim, spacedim> *>(
           mapping_wrapper.get_mapping());
 
       auto aspect_ratios =
@@ -1655,9 +1655,8 @@ namespace python
 
 
   CellAccessorWrapper
-  TriangulationWrapper::find_active_cell_around_point(
-    PointWrapper &         p,
-    MappingQGenericWrapper mapping)
+  TriangulationWrapper::find_active_cell_around_point(PointWrapper &  p,
+                                                      MappingQWrapper mapping)
   {
     std::pair<int, int> level_index_pair;
     if ((dim == 2) && (spacedim == 2))
@@ -1699,8 +1698,8 @@ namespace python
 
   boost::python::list
   TriangulationWrapper::compute_aspect_ratio_of_cells(
-    const MappingQGenericWrapper &mapping,
-    const QuadratureWrapper &     quadrature)
+    const MappingQWrapper &  mapping,
+    const QuadratureWrapper &quadrature)
   {
     if ((dim == 2) && (spacedim == 2))
       return internal::compute_aspect_ratio_of_cells<2, 2>(mapping,
