@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2020 - 2021 by the deal.II authors
+ * Copyright (C) 2020 - 2022 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -144,7 +144,7 @@ namespace Step19
     void output_results() const;
 
     Triangulation<dim>        triangulation;
-    MappingQGeneric<dim>      mapping;
+    MappingQ<dim>             mapping;
     FE_Q<dim>                 fe;
     DoFHandler<dim>           dof_handler;
     AffineConstraints<double> constraints;
@@ -539,7 +539,7 @@ namespace Step19
 
   // Let us now turn to the functions that deal with particles. The first one
   // is about the creation of particles. As mentioned in the introduction,
-  // we want to create a particle at points of the cathode if the the electric
+  // we want to create a particle at points of the cathode if the electric
   // field $\mathbf E=\nabla V$ exceeds a certain threshold, i.e., if
   // $|\mathbf E| \ge E_\text{threshold}$, and if furthermore the electric field
   // points into the domain (i.e., if $\mathbf E \cdot \mathbf n < 0$). As is
@@ -707,7 +707,7 @@ namespace Step19
                 const Tensor<1, dim> new_velocity =
                   old_velocity + acceleration * dt;
 
-                particle->set_properties(make_array_view(new_velocity));
+                particle->set_properties(new_velocity);
 
                 // With the new velocity, we can then also update the location
                 // of the particle and tell the particle about it.
@@ -903,12 +903,14 @@ namespace Step19
 
 
   // With this, the `output_results()` function becomes relatively
-  // straightforward: We use the DataOut class as we have in almost every one of
-  // the previous tutorial programs to output the solution (the "electric
-  // potential") and we use the postprocessor defined above to also output its
-  // gradient (the "electric field"). This all is then written into a file in
-  // VTU format after also associating the current time and time step number
-  // with this file.
+  // straightforward: We use the DataOut class as we have in almost
+  // every one of the previous tutorial programs to output the
+  // solution (the "electric potential") and we use the postprocessor
+  // defined above to also output its gradient (the "electric
+  // field"). This all is then written into a file in VTU format after
+  // also associating the current time and time step number with this
+  // file, and providing physical units for the two fields to be
+  // output.
   template <int dim>
   void CathodeRaySimulator<dim>::output_results() const
   {
@@ -920,8 +922,13 @@ namespace Step19
       data_out.add_data_vector(solution, electric_field);
       data_out.build_patches();
 
-      data_out.set_flags(
-        DataOutBase::VtkFlags(time.get_current_time(), time.get_step_number()));
+      DataOutBase::VtkFlags output_flags;
+      output_flags.time  = time.get_current_time();
+      output_flags.cycle = time.get_step_number();
+      output_flags.physical_units["electric_potential"] = "V";
+      output_flags.physical_units["electric_field"]     = "V/m";
+
+      data_out.set_flags(output_flags);
 
       std::ofstream output("solution-" +
                            Utilities::int_to_string(time.get_step_number(), 4) +
@@ -936,15 +943,19 @@ namespace Step19
     // properties -- namely, as a single vector indicating the velocity, rather
     // than as `dim` scalar properties. The rest is then the same as above:
     {
-      Particles::DataOut<dim, dim> particle_out;
+      Particles::DataOut<dim> particle_out;
       particle_out.build_patches(
         particle_handler,
         std::vector<std::string>(dim, "velocity"),
         std::vector<DataComponentInterpretation::DataComponentInterpretation>(
           dim, DataComponentInterpretation::component_is_part_of_vector));
 
-      particle_out.set_flags(
-        DataOutBase::VtkFlags(time.get_current_time(), time.get_step_number()));
+      DataOutBase::VtkFlags output_flags;
+      output_flags.time                       = time.get_current_time();
+      output_flags.cycle                      = time.get_step_number();
+      output_flags.physical_units["velocity"] = "m/s";
+
+      particle_out.set_flags(output_flags);
 
       std::ofstream output("particles-" +
                            Utilities::int_to_string(time.get_step_number(), 4) +
