@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2006 - 2020 by the deal.II authors
+ * Copyright (C) 2006 - 2022 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -614,7 +614,7 @@ namespace Step25
 
         std::cout << std::endl
                   << "Time step #" << timestep_number << "; "
-                  << "advancing to t = " << time << "." << std::endl;
+                  << "advancing to t = " << time << '.' << std::endl;
 
         // At the beginning of each time step we must solve the nonlinear
         // equation in the split formulation via Newton's method ---
