@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2012 - 2021 by the deal.II authors
+ * Copyright (C) 2012 - 2022 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -469,7 +469,7 @@ namespace Step42
       hy = 1.0 / (ny - 1);
 
       if (Utilities::MPI::this_mpi_process(MPI_COMM_WORLD) == 0)
-        std::cout << "Read obstacle from file <" << name << ">" << std::endl
+        std::cout << "Read obstacle from file <" << name << '>' << std::endl
                   << "Resolution of the scanned obstacle picture: " << nx
                   << " x " << ny << std::endl;
     }
@@ -612,7 +612,7 @@ namespace Step42
     void compute_dirichlet_constraints();
     void update_solution_and_constraints();
     void
-         assemble_mass_matrix_diagonal(TrilinosWrappers::SparseMatrix &mass_matrix);
+    assemble_mass_matrix_diagonal(TrilinosWrappers::SparseMatrix &mass_matrix);
     void assemble_newton_system(
       const TrilinosWrappers::MPI::Vector &linearization_point);
     void compute_nonlinear_residual(
@@ -973,9 +973,8 @@ namespace Step42
       dof_handler.distribute_dofs(fe);
 
       locally_owned_dofs = dof_handler.locally_owned_dofs();
-      locally_relevant_dofs.clear();
-      DoFTools::extract_locally_relevant_dofs(dof_handler,
-                                              locally_relevant_dofs);
+      locally_relevant_dofs =
+        DoFTools::extract_locally_relevant_dofs(dof_handler);
     }
 
     /* setup hanging nodes and Dirichlet constraints */
@@ -1716,7 +1715,7 @@ namespace Step42
                  !transfer_solution)
           constitutive_law.set_sigma_0(correct_sigma);
 
-        pcout << " " << std::endl;
+        pcout << ' ' << std::endl;
         pcout << "   Newton iteration " << newton_step << std::endl;
         pcout << "      Updating active set..." << std::endl;
 
@@ -2154,7 +2153,7 @@ namespace Step42
         Utilities::System::MemoryStats stats;
         Utilities::System::get_memory_stats(stats);
         pcout << "Peak virtual memory used, resident in kB: " << stats.VmSize
-              << " " << stats.VmRSS << std::endl;
+              << ' ' << stats.VmRSS << std::endl;
 
         if (base_mesh == "box")
           output_contact_force();
