@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2011 - 2020 by the deal.II authors
+ * Copyright (C) 2011 - 2021 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -38,19 +38,15 @@
 
 #include <deal.II/grid/tria.h>
 #include <deal.II/grid/grid_generator.h>
-#include <deal.II/grid/tria_accessor.h>
-#include <deal.II/grid/tria_iterator.h>
 #include <deal.II/grid/grid_refinement.h>
 
 #include <deal.II/dofs/dof_tools.h>
-#include <deal.II/dofs/dof_accessor.h>
 
 #include <deal.II/fe/fe_q.h>
 #include <deal.II/fe/fe_nothing.h>
 #include <deal.II/fe/fe_system.h>
 #include <deal.II/fe/fe_values.h>
 
-#include <deal.II/hp/dof_handler.h>
 #include <deal.II/hp/fe_collection.h>
 #include <deal.II/hp/fe_values.h>
 
@@ -70,7 +66,7 @@ namespace Step46
 
   // This is the main class. It is, if you want, a combination of step-8 and
   // step-22 in that it has member variables that either address the global
-  // problem (the Triangulation and hp::DoFHandler objects, as well as the
+  // problem (the Triangulation and DoFHandler objects, as well as the
   // hp::FECollection and various linear algebra objects) or that pertain to
   // either the elasticity or Stokes sub-problems. The general structure of
   // the class, however, is like that of most of the other programs
@@ -104,10 +100,10 @@ namespace Step46
     };
 
     static bool cell_is_in_fluid_domain(
-      const typename hp::DoFHandler<dim>::cell_iterator &cell);
+      const typename DoFHandler<dim>::cell_iterator &cell);
 
     static bool cell_is_in_solid_domain(
-      const typename hp::DoFHandler<dim>::cell_iterator &cell);
+      const typename DoFHandler<dim>::cell_iterator &cell);
 
 
     void make_grid();
@@ -132,7 +128,7 @@ namespace Step46
     FESystem<dim>         stokes_fe;
     FESystem<dim>         elasticity_fe;
     hp::FECollection<dim> fe_collection;
-    hp::DoFHandler<dim>   dof_handler;
+    DoFHandler<dim>       dof_handler;
 
     AffineConstraints<double> constraints;
 
@@ -249,7 +245,7 @@ namespace Step46
 
   template <int dim>
   bool FluidStructureProblem<dim>::cell_is_in_fluid_domain(
-    const typename hp::DoFHandler<dim>::cell_iterator &cell)
+    const typename DoFHandler<dim>::cell_iterator &cell)
   {
     return (cell->material_id() == fluid_domain_id);
   }
@@ -257,7 +253,7 @@ namespace Step46
 
   template <int dim>
   bool FluidStructureProblem<dim>::cell_is_in_solid_domain(
-    const typename hp::DoFHandler<dim>::cell_iterator &cell)
+    const typename DoFHandler<dim>::cell_iterator &cell)
   {
     return (cell->material_id() == solid_domain_id);
   }
@@ -364,39 +360,41 @@ namespace Step46
     // introduction:
     {
       std::vector<types::global_dof_index> local_face_dof_indices(
-        stokes_fe.dofs_per_face);
+        stokes_fe.n_dofs_per_face());
       for (const auto &cell : dof_handler.active_cell_iterators())
         if (cell_is_in_fluid_domain(cell))
-          for (unsigned int face_no : GeometryInfo<dim>::face_indices())
-            {
-              bool face_is_on_interface = false;
+          for (const auto face_no : cell->face_indices())
+            if (cell->face(face_no)->at_boundary() == false)
+              {
+                bool face_is_on_interface = false;
 
-              if ((cell->neighbor(face_no)->has_children() == false) &&
-                  (cell_is_in_solid_domain(cell->neighbor(face_no))))
-                face_is_on_interface = true;
-              else if (cell->neighbor(face_no)->has_children() == true)
-                {
-                  for (unsigned int sf = 0;
-                       sf < cell->face(face_no)->n_children();
-                       ++sf)
-                    if (cell_is_in_solid_domain(
-                          cell->neighbor_child_on_subface(face_no, sf)))
-                      {
-                        face_is_on_interface = true;
-                        break;
-                      }
-                }
+                if ((cell->neighbor(face_no)->has_children() == false) &&
+                    (cell_is_in_solid_domain(cell->neighbor(face_no))))
+                  face_is_on_interface = true;
+                else if (cell->neighbor(face_no)->has_children() == true)
+                  {
+                    for (unsigned int sf = 0;
+                         sf < cell->face(face_no)->n_children();
+                         ++sf)
+                      if (cell_is_in_solid_domain(
+                            cell->neighbor_child_on_subface(face_no, sf)))
+                        {
+                          face_is_on_interface = true;
+                          break;
+                        }
+                  }
 
-              if (face_is_on_interface)
-                {
-                  cell->face(face_no)->get_dof_indices(local_face_dof_indices,
-                                                       0);
-                  for (unsigned int i = 0; i < local_face_dof_indices.size();
-                       ++i)
-                    if (stokes_fe.face_system_to_component_index(i).first < dim)
-                      constraints.add_line(local_face_dof_indices[i]);
-                }
-            }
+                if (face_is_on_interface)
+                  {
+                    cell->face(face_no)->get_dof_indices(local_face_dof_indices,
+                                                         0);
+                    for (unsigned int i = 0; i < local_face_dof_indices.size();
+                         ++i)
+                      if (stokes_fe.face_system_to_component_index(i).first <
+                          dim)
+                        constraints.add_line(local_face_dof_indices[i]);
+                  }
+              }
     }
 
     // At the end of all this, we can declare to the constraints object that
@@ -498,8 +496,9 @@ namespace Step46
 
     // ...to objects that are needed to describe the local contributions to
     // the global linear system...
-    const unsigned int stokes_dofs_per_cell     = stokes_fe.dofs_per_cell;
-    const unsigned int elasticity_dofs_per_cell = elasticity_fe.dofs_per_cell;
+    const unsigned int stokes_dofs_per_cell = stokes_fe.n_dofs_per_cell();
+    const unsigned int elasticity_dofs_per_cell =
+      elasticity_fe.n_dofs_per_cell();
 
     FullMatrix<double> local_matrix;
     FullMatrix<double> local_interface_matrix(elasticity_dofs_per_cell,
@@ -538,9 +537,9 @@ namespace Step46
 
         const FEValues<dim> &fe_values = hp_fe_values.get_present_fe_values();
 
-        local_matrix.reinit(cell->get_fe().dofs_per_cell,
-                            cell->get_fe().dofs_per_cell);
-        local_rhs.reinit(cell->get_fe().dofs_per_cell);
+        local_matrix.reinit(cell->get_fe().n_dofs_per_cell(),
+                            cell->get_fe().n_dofs_per_cell());
+        local_rhs.reinit(cell->get_fe().n_dofs_per_cell());
 
         // With all of this done, we continue to assemble the cell terms for
         // cells that are part of the Stokes and elastic regions. While we
@@ -558,7 +557,7 @@ namespace Step46
         // documentation module for the elasticity equations:
         if (cell_is_in_fluid_domain(cell))
           {
-            const unsigned int dofs_per_cell = cell->get_fe().dofs_per_cell;
+            const unsigned int dofs_per_cell = cell->get_fe().n_dofs_per_cell();
             Assert(dofs_per_cell == stokes_dofs_per_cell, ExcInternalError());
 
             for (unsigned int q = 0; q < fe_values.n_quadrature_points; ++q)
@@ -584,7 +583,7 @@ namespace Step46
           }
         else
           {
-            const unsigned int dofs_per_cell = cell->get_fe().dofs_per_cell;
+            const unsigned int dofs_per_cell = cell->get_fe().n_dofs_per_cell();
             Assert(dofs_per_cell == elasticity_dofs_per_cell,
                    ExcInternalError());
 
@@ -622,7 +621,7 @@ namespace Step46
         // along since the elimination of nonzero boundary values requires the
         // modification of local and consequently also global right hand side
         // values:
-        local_dof_indices.resize(cell->get_fe().dofs_per_cell);
+        local_dof_indices.resize(cell->get_fe().n_dofs_per_cell());
         cell->get_dof_indices(local_dof_indices);
         constraints.distribute_local_to_global(local_matrix,
                                                local_rhs,
@@ -640,119 +639,122 @@ namespace Step46
         // boundary and the potential neighbor behind it is part of the fluid
         // domain. Let's start with these conditions:
         if (cell_is_in_solid_domain(cell))
-          for (unsigned int f : GeometryInfo<dim>::face_indices())
-            {
-              // At this point we know that the current cell is a candidate
-              // for integration and that a neighbor behind face
-              // <code>f</code> exists. There are now three possibilities:
-              //
-              // - The neighbor is at the same refinement level and has no
-              //   children.
-              // - The neighbor has children.
-              // - The neighbor is coarser.
-              //
-              // In all three cases, we are only interested in it if it is
-              // part of the fluid subdomain. So let us start with the first
-              // and simplest case: if the neighbor is at the same level,
-              // has no children, and is a fluid cell, then the two cells
-              // share a boundary that is part of the interface along which
-              // we want to integrate interface terms. All we have to do is
-              // initialize two FEFaceValues object with the current face
-              // and the face of the neighboring cell (note how we find out
-              // which face of the neighboring cell borders on the current
-              // cell) and pass things off to the function that evaluates
-              // the interface terms (the third through fifth arguments to
-              // this function provide it with scratch arrays). The result
-              // is then again copied into the global matrix, using a
-              // function that knows that the DoF indices of rows and
-              // columns of the local matrix result from different cells:
-              if ((cell->neighbor(f)->level() == cell->level()) &&
-                  (cell->neighbor(f)->has_children() == false) &&
-                  cell_is_in_fluid_domain(cell->neighbor(f)))
-                {
-                  elasticity_fe_face_values.reinit(cell, f);
-                  stokes_fe_face_values.reinit(cell->neighbor(f),
-                                               cell->neighbor_of_neighbor(f));
+          for (const auto f : cell->face_indices())
+            if (cell->face(f)->at_boundary() == false)
+              {
+                // At this point we know that the current cell is a candidate
+                // for integration and that a neighbor behind face
+                // <code>f</code> exists. There are now three possibilities:
+                //
+                // - The neighbor is at the same refinement level and has no
+                //   children.
+                // - The neighbor has children.
+                // - The neighbor is coarser.
+                //
+                // In all three cases, we are only interested in it if it is
+                // part of the fluid subdomain. So let us start with the first
+                // and simplest case: if the neighbor is at the same level,
+                // has no children, and is a fluid cell, then the two cells
+                // share a boundary that is part of the interface along which
+                // we want to integrate interface terms. All we have to do is
+                // initialize two FEFaceValues object with the current face
+                // and the face of the neighboring cell (note how we find out
+                // which face of the neighboring cell borders on the current
+                // cell) and pass things off to the function that evaluates
+                // the interface terms (the third through fifth arguments to
+                // this function provide it with scratch arrays). The result
+                // is then again copied into the global matrix, using a
+                // function that knows that the DoF indices of rows and
+                // columns of the local matrix result from different cells:
+                if ((cell->neighbor(f)->level() == cell->level()) &&
+                    (cell->neighbor(f)->has_children() == false) &&
+                    cell_is_in_fluid_domain(cell->neighbor(f)))
+                  {
+                    elasticity_fe_face_values.reinit(cell, f);
+                    stokes_fe_face_values.reinit(cell->neighbor(f),
+                                                 cell->neighbor_of_neighbor(f));
 
-                  assemble_interface_term(elasticity_fe_face_values,
-                                          stokes_fe_face_values,
-                                          elasticity_phi,
-                                          stokes_symgrad_phi_u,
-                                          stokes_phi_p,
-                                          local_interface_matrix);
+                    assemble_interface_term(elasticity_fe_face_values,
+                                            stokes_fe_face_values,
+                                            elasticity_phi,
+                                            stokes_symgrad_phi_u,
+                                            stokes_phi_p,
+                                            local_interface_matrix);
 
-                  cell->neighbor(f)->get_dof_indices(neighbor_dof_indices);
-                  constraints.distribute_local_to_global(local_interface_matrix,
-                                                         local_dof_indices,
-                                                         neighbor_dof_indices,
-                                                         system_matrix);
-                }
+                    cell->neighbor(f)->get_dof_indices(neighbor_dof_indices);
+                    constraints.distribute_local_to_global(
+                      local_interface_matrix,
+                      local_dof_indices,
+                      neighbor_dof_indices,
+                      system_matrix);
+                  }
 
-              // The second case is if the neighbor has further children. In
-              // that case, we have to loop over all the children of the
-              // neighbor to see if they are part of the fluid subdomain. If
-              // they are, then we integrate over the common interface,
-              // which is a face for the neighbor and a subface of the
-              // current cell, requiring us to use an FEFaceValues for the
-              // neighbor and an FESubfaceValues for the current cell:
-              else if ((cell->neighbor(f)->level() == cell->level()) &&
-                       (cell->neighbor(f)->has_children() == true))
-                {
-                  for (unsigned int subface = 0;
-                       subface < cell->face(f)->n_children();
-                       ++subface)
-                    if (cell_is_in_fluid_domain(
-                          cell->neighbor_child_on_subface(f, subface)))
-                      {
-                        elasticity_fe_subface_values.reinit(cell, f, subface);
-                        stokes_fe_face_values.reinit(
-                          cell->neighbor_child_on_subface(f, subface),
-                          cell->neighbor_of_neighbor(f));
+                // The second case is if the neighbor has further children. In
+                // that case, we have to loop over all the children of the
+                // neighbor to see if they are part of the fluid subdomain. If
+                // they are, then we integrate over the common interface,
+                // which is a face for the neighbor and a subface of the
+                // current cell, requiring us to use an FEFaceValues for the
+                // neighbor and an FESubfaceValues for the current cell:
+                else if ((cell->neighbor(f)->level() == cell->level()) &&
+                         (cell->neighbor(f)->has_children() == true))
+                  {
+                    for (unsigned int subface = 0;
+                         subface < cell->face(f)->n_children();
+                         ++subface)
+                      if (cell_is_in_fluid_domain(
+                            cell->neighbor_child_on_subface(f, subface)))
+                        {
+                          elasticity_fe_subface_values.reinit(cell, f, subface);
+                          stokes_fe_face_values.reinit(
+                            cell->neighbor_child_on_subface(f, subface),
+                            cell->neighbor_of_neighbor(f));
 
-                        assemble_interface_term(elasticity_fe_subface_values,
-                                                stokes_fe_face_values,
-                                                elasticity_phi,
-                                                stokes_symgrad_phi_u,
-                                                stokes_phi_p,
-                                                local_interface_matrix);
+                          assemble_interface_term(elasticity_fe_subface_values,
+                                                  stokes_fe_face_values,
+                                                  elasticity_phi,
+                                                  stokes_symgrad_phi_u,
+                                                  stokes_phi_p,
+                                                  local_interface_matrix);
 
-                        cell->neighbor_child_on_subface(f, subface)
-                          ->get_dof_indices(neighbor_dof_indices);
-                        constraints.distribute_local_to_global(
-                          local_interface_matrix,
-                          local_dof_indices,
-                          neighbor_dof_indices,
-                          system_matrix);
-                      }
-                }
+                          cell->neighbor_child_on_subface(f, subface)
+                            ->get_dof_indices(neighbor_dof_indices);
+                          constraints.distribute_local_to_global(
+                            local_interface_matrix,
+                            local_dof_indices,
+                            neighbor_dof_indices,
+                            system_matrix);
+                        }
+                  }
 
-              // The last option is that the neighbor is coarser. In that
-              // case we have to use an FESubfaceValues object for the
-              // neighbor and a FEFaceValues for the current cell; the rest
-              // is the same as before:
-              else if (cell->neighbor_is_coarser(f) &&
-                       cell_is_in_fluid_domain(cell->neighbor(f)))
-                {
-                  elasticity_fe_face_values.reinit(cell, f);
-                  stokes_fe_subface_values.reinit(
-                    cell->neighbor(f),
-                    cell->neighbor_of_coarser_neighbor(f).first,
-                    cell->neighbor_of_coarser_neighbor(f).second);
+                // The last option is that the neighbor is coarser. In that
+                // case we have to use an FESubfaceValues object for the
+                // neighbor and a FEFaceValues for the current cell; the rest
+                // is the same as before:
+                else if (cell->neighbor_is_coarser(f) &&
+                         cell_is_in_fluid_domain(cell->neighbor(f)))
+                  {
+                    elasticity_fe_face_values.reinit(cell, f);
+                    stokes_fe_subface_values.reinit(
+                      cell->neighbor(f),
+                      cell->neighbor_of_coarser_neighbor(f).first,
+                      cell->neighbor_of_coarser_neighbor(f).second);
 
-                  assemble_interface_term(elasticity_fe_face_values,
-                                          stokes_fe_subface_values,
-                                          elasticity_phi,
-                                          stokes_symgrad_phi_u,
-                                          stokes_phi_p,
-                                          local_interface_matrix);
+                    assemble_interface_term(elasticity_fe_face_values,
+                                            stokes_fe_subface_values,
+                                            elasticity_phi,
+                                            stokes_symgrad_phi_u,
+                                            stokes_phi_p,
+                                            local_interface_matrix);
 
-                  cell->neighbor(f)->get_dof_indices(neighbor_dof_indices);
-                  constraints.distribute_local_to_global(local_interface_matrix,
-                                                         local_dof_indices,
-                                                         neighbor_dof_indices,
-                                                         system_matrix);
-                }
-            }
+                    cell->neighbor(f)->get_dof_indices(neighbor_dof_indices);
+                    constraints.distribute_local_to_global(
+                      local_interface_matrix,
+                      local_dof_indices,
+                      neighbor_dof_indices,
+                      system_matrix);
+                  }
+              }
       }
   }
 
@@ -839,8 +841,7 @@ namespace Step46
   // Generating graphical output is rather trivial here: all we have to do is
   // identify which components of the solution vector belong to scalars and/or
   // vectors (see, for example, step-22 for a previous example), and then pass
-  // it all on to the DataOut class (with the second template argument equal
-  // to hp::DoFHandler instead of the usual default DoFHandler):
+  // it all on to the DataOut class:
   template <int dim>
   void FluidStructureProblem<dim>::output_results(
     const unsigned int refinement_cycle) const
@@ -859,12 +860,12 @@ namespace Step46
       data_component_interpretation.push_back(
         DataComponentInterpretation::component_is_part_of_vector);
 
-    DataOut<dim, hp::DoFHandler<dim>> data_out;
+    DataOut<dim> data_out;
     data_out.attach_dof_handler(dof_handler);
 
     data_out.add_data_vector(solution,
                              solution_names,
-                             DataOut<dim, hp::DoFHandler<dim>>::type_dof_data,
+                             DataOut<dim>::type_dof_data,
                              data_component_interpretation);
     data_out.build_patches();
 
@@ -949,7 +950,7 @@ namespace Step46
     // encountered when assembling interface terms in
     // <code>assemble_system</code>.
     for (const auto &cell : dof_handler.active_cell_iterators())
-      for (unsigned int f : GeometryInfo<dim>::face_indices())
+      for (const auto f : cell->face_indices())
         if (cell_is_in_solid_domain(cell))
           {
             if ((cell->at_boundary(f) == false) &&
