@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2014 - 2019 by the deal.II authors
+ * Copyright (C) 2014 - 2020 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -46,14 +46,9 @@
 #include <boost/iostreams/filter/gzip.hpp>
 #include <boost/iostreams/device/file.hpp>
 
-// The last include file is required because we will be using a feature that is
-// not part of the C++11 standard. As some of the C++14 features are very
-// useful, we provide their implementation in an internal namespace, if the
-// compiler does not support them:
-#include <deal.II/base/std_cxx14/memory.h>
-
-#include <iostream>
 #include <fstream>
+#include <iostream>
+#include <memory>
 
 
 // The final part of the top matter is to open a namespace into which to put
@@ -226,18 +221,16 @@ namespace Step53
   }
 
 
-  // This function is required by the interface of the Manifold base
-  // class, and allows you to clone the AfricaGeometry class. This is
-  // where we use a feature that is only available in C++14, namely the
-  // make_unique function, that simplifies the creation of
-  // std::unique_ptr objects. Notice that, while the function returns an
-  // std::unique_ptr<Manifold<3,3> >, we internally create a
-  // unique_ptr<AfricaGeometry>. C++11 knows how to handle these cases,
-  // and is able to transform a unique pointer to a derived class to a
-  // unique pointer to its base class automatically:
+  // The next function is required by the interface of the
+  // Manifold base class, and allows cloning the AfricaGeometry
+  // class. Notice that, while the function returns a
+  // `std::unique_ptr<Manifold<3,3>>`, we internally create a
+  // `unique_ptr<AfricaGeometry>`. In other words, the library
+  // requires a pointer-to-base-class, which we provide by creating a
+  // pointer-to-derived-class.
   std::unique_ptr<Manifold<3, 3>> AfricaGeometry::clone() const
   {
-    return std_cxx14::make_unique<AfricaGeometry>();
+    return std::make_unique<AfricaGeometry>();
   }
 
 
