@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2018 - 2021 by the deal.II authors
+ * Copyright (C) 2018 - 2022 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -564,7 +564,7 @@ namespace step62
     // height="200" />
     // where the brown color represents material_a and the green color
     // represents material_b.
-    for (unsigned int idx = 0; idx < nb_mirror_pairs; idx++)
+    for (unsigned int idx = 0; idx < nb_mirror_pairs; ++idx)
       {
         const double layer_transition_center =
           material_a_wavelength / 2 +
@@ -586,7 +586,7 @@ namespace step62
     // [subpixel
     // smoothing](https://meep.readthedocs.io/en/latest/Subpixel_Smoothing/)
     // which improves the precision of the simulation.
-    for (unsigned int idx = 0; idx < nb_mirror_pairs; idx++)
+    for (unsigned int idx = 0; idx < nb_mirror_pairs; ++idx)
       {
         const double layer_transition_center =
           material_a_wavelength / 2 +
@@ -612,7 +612,7 @@ namespace step62
       }
 
     // the material_a layers
-    for (unsigned int idx = 0; idx < nb_mirror_pairs; idx++)
+    for (unsigned int idx = 0; idx < nb_mirror_pairs; ++idx)
       {
         const double layer_center =
           material_a_wavelength / 2 +
@@ -627,7 +627,7 @@ namespace step62
       }
 
     // the material_b layers
-    for (unsigned int idx = 0; idx < nb_mirror_pairs; idx++)
+    for (unsigned int idx = 0; idx < nb_mirror_pairs; ++idx)
       {
         const double layer_center =
           material_a_wavelength / 2 +
@@ -729,7 +729,7 @@ namespace step62
             (Utilities::MPI::this_mpi_process(mpi_communicator) == 0))
     , computing_timer(mpi_communicator,
                       pcout,
-                      TimerOutput::summary,
+                      TimerOutput::never,
                       TimerOutput::wall_times)
   {}
 
@@ -748,7 +748,8 @@ namespace step62
     dof_handler.distribute_dofs(fe);
 
     locally_owned_dofs = dof_handler.locally_owned_dofs();
-    DoFTools::extract_locally_relevant_dofs(dof_handler, locally_relevant_dofs);
+    locally_relevant_dofs =
+      DoFTools::extract_locally_relevant_dofs(dof_handler);
 
     locally_relevant_solution.reinit(locally_owned_dofs,
                                      locally_relevant_dofs,
