@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2021 by the deal.II authors
+ * Copyright (C) 2021 - 2022 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -30,7 +30,7 @@
 #include <deal.II/dofs/dof_tools.h>
 
 #include <deal.II/fe/fe_q.h>
-#include <deal.II/fe/mapping_q_generic.h>
+#include <deal.II/fe/mapping_q.h>
 
 #include <deal.II/grid/grid_generator.h>
 #include <deal.II/grid/grid_out.h>
@@ -403,13 +403,10 @@ namespace Step66
     // SphericalManifold for boundary cells a TransfiniteInterpolationManifold
     // object for the mapping of the inner cells, which takes care of the inner
     // cells. In this example we use an isoparametric finite element approach
-    // and thus use the MappingQGeneric class. Note, that we could also create
-    // an instance of the MappingQ class and set the
-    // <code>use_mapping_q_on_all_cells</code> flags in the contructor call to
-    // <code>true</code>. For further details on the connection of MappingQ and
-    // MappingQGeneric you may read the detailed description of these classes.
+    // and thus use the MappingQ class. For further details you may read the
+    // detailed description of this class.
     parallel::distributed::Triangulation<dim> triangulation;
-    const MappingQGeneric<dim>                mapping;
+    const MappingQ<dim>                       mapping;
 
 
     // As usual we then define the Lagrangian finite elements FE_Q and a
@@ -547,8 +544,8 @@ namespace Step66
     dof_handler.distribute_dofs(fe);
     dof_handler.distribute_mg_dofs();
 
-    IndexSet locally_relevant_dofs;
-    DoFTools::extract_locally_relevant_dofs(dof_handler, locally_relevant_dofs);
+    const IndexSet locally_relevant_dofs =
+      DoFTools::extract_locally_relevant_dofs(dof_handler);
 
     constraints.clear();
     constraints.reinit(locally_relevant_dofs);
@@ -585,21 +582,18 @@ namespace Step66
     mg_matrices.resize(0, nlevels - 1);
     mg_solution.resize(0, nlevels - 1);
 
-    std::set<types::boundary_id> dirichlet_boundary;
-    dirichlet_boundary.insert(0);
+    const std::set<types::boundary_id> dirichlet_boundary_ids = {0};
     mg_constrained_dofs.initialize(dof_handler);
     mg_constrained_dofs.make_zero_boundary_constraints(dof_handler,
-                                                       dirichlet_boundary);
+                                                       dirichlet_boundary_ids);
 
     mg_transfer.initialize_constraints(mg_constrained_dofs);
     mg_transfer.build(dof_handler);
 
     for (unsigned int level = 0; level < nlevels; ++level)
       {
-        IndexSet relevant_dofs;
-        DoFTools::extract_locally_relevant_level_dofs(dof_handler,
-                                                      level,
-                                                      relevant_dofs);
+        const IndexSet relevant_dofs =
+          DoFTools::extract_locally_relevant_level_dofs(dof_handler, level);
 
         AffineConstraints<double> level_constraints;
         level_constraints.reinit(relevant_dofs);
@@ -1130,7 +1124,7 @@ namespace Step66
 
 
         timer.stop();
-        pcout << "Time for setup+solve (CPU/Wall) " << timer.cpu_time() << "/"
+        pcout << "Time for setup+solve (CPU/Wall) " << timer.cpu_time() << '/'
               << timer.wall_time() << " s" << std::endl;
         pcout << std::endl;
 
