@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2020 - 2021 by the deal.II authors
+ * Copyright (C) 2020 - 2022 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -26,7 +26,6 @@
 #include <deal.II/base/parameter_acceptor.h>
 #include <deal.II/base/timer.h>
 
-#include <deal.II/distributed/cell_weights.h>
 #include <deal.II/distributed/solution_transfer.h>
 #include <deal.II/distributed/tria.h>
 
@@ -323,7 +322,7 @@ namespace Step68
   // return value of this function (representing "work for this cell") is
   // calculated based on the number of particles in the current cell.
   // The function is
-  // connected to the cell_weight() signal inside the triangulation, and will be
+  // connected to the `weight` signal inside the triangulation, and will be
   // called once per cell, whenever the triangulation repartitions the domain
   // between ranks (the connection is created inside the
   // generate_particles() function of this class).
@@ -334,47 +333,45 @@ namespace Step68
     const typename parallel::distributed::Triangulation<dim>::CellStatus status)
     const
   {
-    // We do not assign any weight to cells we do not own (i.e., artificial
-    // or ghost cells)
-    if (!cell->is_locally_owned())
-      return 0;
+    // First, we introduce a base weight that will be assigned to every cell.
+    const unsigned int base_weight = 1;
 
-    // This determines how important particle work is compared to cell
-    // work (by default every cell has a weight of 1000).
-    // We set the weight per particle much higher to indicate that
-    // the particle load is the only one that is important to distribute the
-    // cells in this example. The optimal value of this number depends on the
-    // application and can range from 0 (cheap particle operations,
-    // expensive cell operations) to much larger than 1000 (expensive
-    // particle operations, cheap cell operations, like presumed in this
-    // example).
-    const unsigned int particle_weight = 10000;
+    // The following variable then determines how important particle work is
+    // compared to cell work. We set the weight per particle much higher to
+    // indicate that the particle load is the only one that is important to
+    // distribute the cells in this example. The optimal value of this number
+    // depends on the application and can range from 0 (cheap particle
+    // operations, expensive cell operations) to much larger than the base
+    // weight of 1 (expensive particle operations, cheap cell operations, like
+    // presumed in this example).
+    const unsigned int particle_weight = 10;
 
     // This example does not use adaptive refinement, therefore every cell
     // should have the status `CELL_PERSIST`. However this function can also
     // be used to distribute load during refinement, therefore we consider
     // refined or coarsened cells as well.
-    if (status == parallel::distributed::Triangulation<dim>::CELL_PERSIST ||
-        status == parallel::distributed::Triangulation<dim>::CELL_REFINE)
+    unsigned int n_particles_in_cell = 0;
+    switch (status)
       {
-        const unsigned int n_particles_in_cell =
-          particle_handler.n_particles_in_cell(cell);
-        return n_particles_in_cell * particle_weight;
-      }
-    else if (status == parallel::distributed::Triangulation<dim>::CELL_COARSEN)
-      {
-        unsigned int n_particles_in_cell = 0;
+        case parallel::distributed::Triangulation<dim>::CELL_PERSIST:
+        case parallel::distributed::Triangulation<dim>::CELL_REFINE:
+          n_particles_in_cell = particle_handler.n_particles_in_cell(cell);
+          break;
 
-        for (unsigned int child_index = 0; child_index < cell->n_children();
-             ++child_index)
-          n_particles_in_cell +=
-            particle_handler.n_particles_in_cell(cell->child(child_index));
+        case parallel::distributed::Triangulation<dim>::CELL_INVALID:
+          break;
 
-        return n_particles_in_cell * particle_weight;
+        case parallel::distributed::Triangulation<dim>::CELL_COARSEN:
+          for (const auto &child : cell->child_iterators())
+            n_particles_in_cell += particle_handler.n_particles_in_cell(child);
+          break;
+
+        default:
+          Assert(false, ExcInternalError());
+          break;
       }
 
-    Assert(false, ExcInternalError());
-    return 0;
+    return base_weight + particle_weight * n_particles_in_cell;
   }
 
 
@@ -392,32 +389,22 @@ namespace Step68
     background_triangulation.refine_global(par.fluid_refinement);
 
     // In order to consider the particles when repartitioning the triangulation
-    // the algorithm needs to know three things:
+    // the algorithm needs to know how much weight to assign to each cell
+    // (how many particles are in there).
     //
-    // 1. How much weight to assign to each cell (how many particles are in
-    // there);
-    // 2. How to pack the particles before shipping data around;
-    // 3. How to unpack the particles after repartitioning.
-    //
-    // We attach the correct functions to the signals inside
-    // parallel::distributed::Triangulation. These signal will be called every
-    // time the repartition() function is called. These connections only need to
-    // be created once, so we might as well have set them up in the constructor
+    // We attach a weight function to the signal inside
+    // parallel::distributed::Triangulation. This signal will be called every
+    // time the repartition() function is called. This connection only needs to
+    // be created once, so we might as well have set it up in the constructor
     // of this class, but for the purpose of this example we want to group the
     // particle related instructions.
-    background_triangulation.signals.cell_weight.connect(
+    background_triangulation.signals.weight.connect(
       [&](
         const typename parallel::distributed::Triangulation<dim>::cell_iterator
           &cell,
         const typename parallel::distributed::Triangulation<dim>::CellStatus
           status) -> unsigned int { return this->cell_weight(cell, status); });
 
-    background_triangulation.signals.pre_distributed_repartition.connect(
-      [this]() { this->particle_handler.register_store_callback_function(); });
-
-    background_triangulation.signals.post_distributed_repartition.connect(
-      [&]() { this->particle_handler.register_load_callback_function(false); });
-
     // This initializes the background triangulation where the particles are
     // living and the number of properties of the particles.
     particle_handler.initialize(background_triangulation, mapping, 1 + dim);
@@ -484,8 +471,8 @@ namespace Step68
   {
     fluid_dh.distribute_dofs(fluid_fe);
     const IndexSet locally_owned_dofs = fluid_dh.locally_owned_dofs();
-    IndexSet       locally_relevant_dofs;
-    DoFTools::extract_locally_relevant_dofs(fluid_dh, locally_relevant_dofs);
+    const IndexSet locally_relevant_dofs =
+      DoFTools::extract_locally_relevant_dofs(fluid_dh);
 
     velocity_field.reinit(locally_owned_dofs,
                           locally_relevant_dofs,
@@ -566,8 +553,7 @@ namespace Step68
     auto particle = particle_handler.begin();
     while (particle != particle_handler.end())
       {
-        const auto cell =
-          particle->get_surrounding_cell(background_triangulation);
+        const auto cell = particle->get_surrounding_cell();
         const auto dh_cell =
           typename DoFHandler<dim>::cell_iterator(*cell, &fluid_dh);
 
@@ -645,7 +631,7 @@ namespace Step68
                                   "interpolated-particles" :
                                   "analytical-particles");
 
-    pcout << "Writing particle output file: " << file_name << "-" << it
+    pcout << "Writing particle output file: " << file_name << '-' << it
           << std::endl;
 
     particle_output.write_vtu_with_pvtu_record(
@@ -680,7 +666,7 @@ namespace Step68
     const std::string output_folder(par.output_directory);
     const std::string file_name("background");
 
-    pcout << "Writing background field file: " << file_name << "-" << it
+    pcout << "Writing background field file: " << file_name << '-' << it
           << std::endl;
 
     data_out.write_vtu_with_pvtu_record(
@@ -705,7 +691,10 @@ namespace Step68
 
     pcout << "Repartitioning triangulation after particle generation"
           << std::endl;
+
+    particle_handler.prepare_for_coarsening_and_refinement();
     background_triangulation.repartition();
+    particle_handler.unpack_after_coarsening_and_refinement();
 
     // We set the initial property of the particles by doing an
     // explicit Euler iteration with a time-step of 0 both in the case
@@ -731,7 +720,10 @@ namespace Step68
 
         if ((discrete_time.get_step_number() % par.repartition_frequency) == 0)
           {
+            particle_handler.prepare_for_coarsening_and_refinement();
             background_triangulation.repartition();
+            particle_handler.unpack_after_coarsening_and_refinement();
+
             if (interpolated_velocity)
               setup_background_dofs();
           }
