@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2021 by the deal.II authors
+ * Copyright (C) 2021 - 2022 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -51,6 +51,7 @@
 #include <deal.II/lac/solver_cg.h>
 #include <deal.II/lac/trilinos_precondition.h>
 #include <deal.II/lac/trilinos_sparse_matrix.h>
+#include <deal.II/lac/trilinos_sparsity_pattern.h>
 #include <deal.II/lac/vector.h>
 
 #include <deal.II/numerics/data_out.h>
@@ -190,7 +191,7 @@ namespace Step75
     double p_refine_fraction  = 0.9;
     double p_coarsen_fraction = 0.9;
 
-    double weighting_factor   = 1e6;
+    double weighting_factor   = 1.;
     double weighting_exponent = 1.;
   };
 
@@ -324,9 +325,8 @@ namespace Step75
     {
       AffineConstraints<number> constraints_without_dbc;
 
-      IndexSet locally_relevant_dofs;
-      DoFTools::extract_locally_relevant_dofs(dof_handler,
-                                              locally_relevant_dofs);
+      const IndexSet locally_relevant_dofs =
+        DoFTools::extract_locally_relevant_dofs(dof_handler);
       constraints_without_dbc.reinit(locally_relevant_dofs);
 
       DoFTools::make_hanging_node_constraints(dof_handler,
@@ -427,6 +427,7 @@ namespace Step75
   void LaplaceOperator<dim, number>::compute_inverse_diagonal(
     VectorType &diagonal) const
   {
+    this->matrix_free.initialize_dof_vector(diagonal);
     MatrixFreeTools::compute_diagonal(matrix_free,
                                       diagonal,
                                       &LaplaceOperator::do_cell_integral_local,
@@ -573,7 +574,7 @@ namespace Step75
     MGLevelObject<typename SmootherType::AdditionalData> smoother_data(
       min_level, max_level);
 
-    for (unsigned int level = min_level; level <= max_level; level++)
+    for (unsigned int level = min_level; level <= max_level; ++level)
       {
         smoother_data[level].preconditioner =
           std::make_shared<SmootherPreconditionerType>();
@@ -668,8 +669,7 @@ namespace Step75
           dof_handler.get_triangulation());
     else
       coarse_grid_triangulations.emplace_back(
-        const_cast<Triangulation<dim> *>(&(dof_handler.get_triangulation())),
-        [](auto &) {});
+        &(dof_handler.get_triangulation()), [](auto *) {});
 
     // Determine the total number of levels for the multigrid operation and
     // allocate sufficient memory for all levels.
@@ -700,9 +700,8 @@ namespace Step75
         fe_index_for_degree[degree] = i;
       }
 
-    unsigned int minlevel   = 0;
-    unsigned int minlevel_p = n_h_levels;
-    unsigned int maxlevel   = n_h_levels + n_p_levels - 1;
+    unsigned int minlevel = 0;
+    unsigned int maxlevel = n_h_levels + n_p_levels - 1;
 
     dof_handlers.resize(minlevel, maxlevel);
     operators.resize(minlevel, maxlevel);
@@ -778,9 +777,8 @@ namespace Step75
         const auto &dof_handler = dof_handlers[level];
         auto &      constraint  = constraints[level];
 
-        IndexSet locally_relevant_dofs;
-        DoFTools::extract_locally_relevant_dofs(dof_handler,
-                                                locally_relevant_dofs);
+        const IndexSet locally_relevant_dofs =
+          DoFTools::extract_locally_relevant_dofs(dof_handler);
         constraint.reinit(locally_relevant_dofs);
 
         DoFTools::make_hanging_node_constraints(dof_handler, constraint);
@@ -802,17 +800,11 @@ namespace Step75
 
     // Set up intergrid operators and collect transfer operators within a single
     // operator as needed by the Multigrid solver class.
-    for (unsigned int level = minlevel; level < minlevel_p; ++level)
-      transfers[level + 1].reinit_geometric_transfer(dof_handlers[level + 1],
-                                                     dof_handlers[level],
-                                                     constraints[level + 1],
-                                                     constraints[level]);
-
-    for (unsigned int level = minlevel_p; level < maxlevel; ++level)
-      transfers[level + 1].reinit_polynomial_transfer(dof_handlers[level + 1],
-                                                      dof_handlers[level],
-                                                      constraints[level + 1],
-                                                      constraints[level]);
+    for (unsigned int level = minlevel; level < maxlevel; ++level)
+      transfers[level + 1].reinit(dof_handlers[level + 1],
+                                  dof_handlers[level],
+                                  constraints[level + 1],
+                                  constraints[level]);
 
     MGTransferGlobalCoarsening<dim, VectorType> transfer(
       transfers, [&](const auto l, auto &vec) {
@@ -911,7 +903,7 @@ namespace Step75
             (Utilities::MPI::this_mpi_process(mpi_communicator) == 0))
     , computing_timer(mpi_communicator,
                       pcout,
-                      TimerOutput::summary,
+                      TimerOutput::never,
                       TimerOutput::wall_times)
   {
     Assert(prm.min_h_level <= prm.max_h_level,
@@ -987,16 +979,13 @@ namespace Step75
     // for hp-adaptation and right before repartitioning for load balancing is
     // about to happen. Functions can be registered that will attach weights in
     // the form that $a (n_\text{dofs})^b$ with a provided pair of parameters
-    // $(a,b)$. We register such a function in the following. Every cell will be
-    // charged with a constant weight at creation, which is a value of 1000 (see
-    // Triangulation::Signals::cell_weight).
+    // $(a,b)$. We register such a function in the following.
     //
     // For load balancing, efficient solvers like the one we use should scale
-    // linearly with the number of degrees of freedom owned. Further, to
-    // increase the impact of the weights we would like to attach, make sure
-    // that the individual weight will exceed this base weight by orders of
-    // magnitude. We set the parameters for cell weighting correspondingly: A
-    // large weighting factor of $10^6$ and an exponent of $1$.
+    // linearly with the number of degrees of freedom owned. We set the
+    // parameters for cell weighting correspondingly: A weighting factor of $1$
+    // and an exponent of $1$ (see the definitions of the `weighting_factor` and
+    // `weighting_exponent` above).
     cell_weights = std::make_unique<parallel::CellWeights<dim>>(
       dof_handler,
       parallel::CellWeights<dim>::ndofs_weighting(
@@ -1057,9 +1046,12 @@ namespace Step75
   // remove one cell in every cell from the negative direction, but remove one
   // from the positive x-direction.
   //
-  // In the end, we supply the number of initial refinements that corresponds to
-  // the supplied minimal grid refinement level. Further, we set the initial
-  // active FE indices accordingly.
+  // On the coarse grid, we set the initial active FE indices and distribute the
+  // degrees of freedom once. We do that in order to assign the hp::FECollection
+  // to the DoFHandler, so that all cells know how many DoFs they are going to
+  // have. This step is mandatory for the weighted load balancing algorithm,
+  // which will be called implicitly in
+  // parallel::distributed::Triangulation::refine_global().
   template <int dim>
   void LaplaceProblem<dim>::initialize_grid()
   {
@@ -1087,12 +1079,14 @@ namespace Step75
     GridGenerator::subdivided_hyper_L(
       triangulation, repetitions, bottom_left, top_right, cells_to_remove);
 
-    triangulation.refine_global(prm.min_h_level);
-
     const unsigned int min_fe_index = prm.min_p_degree - 1;
     for (const auto &cell : dof_handler.active_cell_iterators())
       if (cell->is_locally_owned())
         cell->set_active_fe_index(min_fe_index);
+
+    dof_handler.distribute_dofs(fe_collection);
+
+    triangulation.refine_global(prm.min_h_level);
   }
 
 
@@ -1112,7 +1106,8 @@ namespace Step75
     dof_handler.distribute_dofs(fe_collection);
 
     locally_owned_dofs = dof_handler.locally_owned_dofs();
-    DoFTools::extract_locally_relevant_dofs(dof_handler, locally_relevant_dofs);
+    locally_relevant_dofs =
+      DoFTools::extract_locally_relevant_dofs(dof_handler);
 
     locally_relevant_solution.reinit(locally_owned_dofs,
                                      locally_relevant_dofs,
@@ -1216,7 +1211,7 @@ namespace Step75
       pcout << "   Frequencies of poly. degrees:";
       for (unsigned int i = 0; i < fe_collection.size(); ++i)
         if (n_fe_indices[i] > 0)
-          pcout << ' ' << fe_collection[i].degree << ":" << n_fe_indices[i];
+          pcout << ' ' << fe_collection[i].degree << ':' << n_fe_indices[i];
       pcout << std::endl;
     }
   }
