@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2020 by the deal.II authors
+// Copyright (C) 1998 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -317,18 +317,6 @@ Function<dim, RangeNumberType>::memory_consumption() const
 
 
 
-//---------------------------------------------------------------------------
-
-namespace Functions
-{
-  template <int dim, typename RangeNumberType>
-  ZeroFunction<dim, RangeNumberType>::ZeroFunction(
-    const unsigned int n_components)
-    : ConstantFunction<dim, RangeNumberType>(RangeNumberType(), n_components)
-  {}
-
-} // namespace Functions
-
 //---------------------------------------------------------------------------
 
 namespace Functions
@@ -341,6 +329,8 @@ namespace Functions
     , function_value_vector(n_components, value)
   {}
 
+
+
   template <int dim, typename RangeNumberType>
   ConstantFunction<dim, RangeNumberType>::ConstantFunction(
     const std::vector<RangeNumberType> &values)
@@ -349,6 +339,7 @@ namespace Functions
   {}
 
 
+
   template <int dim, typename RangeNumberType>
   ConstantFunction<dim, RangeNumberType>::ConstantFunction(
     const Vector<RangeNumberType> &values)
@@ -361,6 +352,7 @@ namespace Functions
   }
 
 
+
   template <int dim, typename RangeNumberType>
   ConstantFunction<dim, RangeNumberType>::ConstantFunction(
     const RangeNumberType *begin_ptr,
@@ -465,6 +457,7 @@ namespace Functions
   }
 
 
+
   template <int dim, typename RangeNumberType>
   void
   ConstantFunction<dim, RangeNumberType>::vector_gradient(
@@ -479,6 +472,7 @@ namespace Functions
   }
 
 
+
   template <int dim, typename RangeNumberType>
   void
   ConstantFunction<dim, RangeNumberType>::gradient_list(
@@ -494,6 +488,7 @@ namespace Functions
   }
 
 
+
   template <int dim, typename RangeNumberType>
   void
   ConstantFunction<dim, RangeNumberType>::vector_gradient_list(
@@ -530,6 +525,67 @@ namespace Functions
   {
     return 0;
   }
+
+
+
+  template <int dim, typename RangeNumberType>
+  ZeroFunction<dim, RangeNumberType>::ZeroFunction(
+    const unsigned int n_components)
+    : ConstantFunction<dim, RangeNumberType>(RangeNumberType(), n_components)
+  {}
+
+
+
+  template <int dim, typename RangeNumberType>
+  IdentityFunction<dim, RangeNumberType>::IdentityFunction()
+    : Function<dim, RangeNumberType>(dim)
+  {}
+
+
+
+  template <int dim, typename RangeNumberType>
+  RangeNumberType
+  IdentityFunction<dim, RangeNumberType>::value(
+    const Point<dim> & p,
+    const unsigned int component) const
+  {
+    AssertIndexRange(component, this->n_components);
+    return p[component];
+  }
+
+
+
+  template <int dim, typename RangeNumberType>
+  Tensor<1, dim, RangeNumberType>
+  IdentityFunction<dim, RangeNumberType>::gradient(
+    const Point<dim> &,
+    const unsigned int component) const
+  {
+    AssertIndexRange(component, this->n_components);
+    Tensor<1, dim, RangeNumberType> result;
+    result[component] = RangeNumberType(1);
+    return result;
+  }
+
+
+
+  template <int dim, typename RangeNumberType>
+  SymmetricTensor<2, dim, RangeNumberType>
+  IdentityFunction<dim, RangeNumberType>::hessian(const Point<dim> &,
+                                                  const unsigned int) const
+  {
+    return SymmetricTensor<2, dim, RangeNumberType>();
+  }
+
+
+
+  template <int dim, typename RangeNumberType>
+  RangeNumberType
+  IdentityFunction<dim, RangeNumberType>::laplacian(const Point<dim> &,
+                                                    const unsigned int) const
+  {
+    return 0;
+  }
 } // namespace Functions
 
 //---------------------------------------------------------------------------
@@ -539,7 +595,7 @@ ComponentSelectFunction<dim, RangeNumberType>::ComponentSelectFunction(
   const unsigned int    selected,
   const RangeNumberType value,
   const unsigned int    n_components)
-  : ConstantFunction<dim, RangeNumberType>(value, n_components)
+  : Functions::ConstantFunction<dim, RangeNumberType>(value, n_components)
   , selected_components(std::make_pair(selected, selected + 1))
 {}
 
@@ -549,7 +605,7 @@ template <int dim, typename RangeNumberType>
 ComponentSelectFunction<dim, RangeNumberType>::ComponentSelectFunction(
   const unsigned int selected,
   const unsigned int n_components)
-  : ConstantFunction<dim, RangeNumberType>(1., n_components)
+  : Functions::ConstantFunction<dim, RangeNumberType>(1., n_components)
   , selected_components(std::make_pair(selected, selected + 1))
 {
   AssertIndexRange(selected, n_components);
@@ -561,7 +617,7 @@ template <int dim, typename RangeNumberType>
 ComponentSelectFunction<dim, RangeNumberType>::ComponentSelectFunction(
   const std::pair<unsigned int, unsigned int> &selected,
   const unsigned int                           n_components)
-  : ConstantFunction<dim, RangeNumberType>(1., n_components)
+  : Functions::ConstantFunction<dim, RangeNumberType>(1., n_components)
   , selected_components(selected)
 {
   Assert(selected_components.first < selected_components.second,
@@ -577,7 +633,7 @@ ComponentSelectFunction<dim, RangeNumberType>::ComponentSelectFunction(
 template <int dim, typename RangeNumberType>
 void
 ComponentSelectFunction<dim, RangeNumberType>::substitute_function_value_with(
-  const ConstantFunction<dim, RangeNumberType> &f)
+  const Functions::ConstantFunction<dim, RangeNumberType> &f)
 {
   Point<dim> p;
   for (unsigned int i = 0; i < this->function_value_vector.size(); ++i)
@@ -626,8 +682,9 @@ ComponentSelectFunction<dim, RangeNumberType>::memory_consumption() const
   // No new complex data structure is introduced here, just evaluate how much
   // more memory is used *inside* the class via sizeof() and add that value to
   // parent class's memory_consumption()
-  return (sizeof(*this) - sizeof(ConstantFunction<dim, RangeNumberType>) +
-          ConstantFunction<dim, RangeNumberType>::memory_consumption());
+  return (
+    sizeof(*this) - sizeof(Functions::ConstantFunction<dim, RangeNumberType>) +
+    Functions::ConstantFunction<dim, RangeNumberType>::memory_consumption());
 }
 
 //---------------------------------------------------------------------------
