@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2021 by the deal.II authors
+// Copyright (C) 2009 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -22,10 +22,7 @@
 #include <deal.II/base/thread_management.h>
 #include <deal.II/base/utilities.h>
 
-#include <boost/serialization/vector.hpp>
-
 #include <algorithm>
-#include <iterator>
 #include <vector>
 
 
@@ -324,7 +321,8 @@ public:
    * sets must have the same size (though of course they do not have to have
    * the same number of indices).
    */
-  IndexSet operator&(const IndexSet &is) const;
+  IndexSet
+  operator&(const IndexSet &is) const;
 
   /**
    * This command takes an interval <tt>[begin, end)</tt> and returns the
@@ -658,12 +656,14 @@ public:
     /**
      * Dereferencing operator, returns an IntervalAccessor.
      */
-    const IntervalAccessor &operator*() const;
+    const IntervalAccessor &
+    operator*() const;
 
     /**
      * Dereferencing operator, returns a pointer to an IntervalAccessor.
      */
-    const IntervalAccessor *operator->() const;
+    const IntervalAccessor *
+    operator->() const;
 
     /**
      * Comparison.
@@ -734,7 +734,8 @@ public:
      * Dereferencing operator. The returned value is the index of the element
      * inside the IndexSet.
      */
-    size_type operator*() const;
+    size_type
+    operator*() const;
 
     /**
      * Does this iterator point to an existing element?
@@ -1116,8 +1117,8 @@ IndexSet::IntervalAccessor::operator=(const IndexSet::IntervalAccessor &other)
 
 
 inline bool
-IndexSet::IntervalAccessor::
-operator==(const IndexSet::IntervalAccessor &other) const
+IndexSet::IntervalAccessor::operator==(
+  const IndexSet::IntervalAccessor &other) const
 {
   Assert(index_set == other.index_set,
          ExcMessage(
@@ -1128,8 +1129,8 @@ operator==(const IndexSet::IntervalAccessor &other) const
 
 
 inline bool
-IndexSet::IntervalAccessor::
-operator<(const IndexSet::IntervalAccessor &other) const
+IndexSet::IntervalAccessor::operator<(
+  const IndexSet::IntervalAccessor &other) const
 {
   Assert(index_set == other.index_set,
          ExcMessage(
@@ -1195,16 +1196,16 @@ IndexSet::IntervalIterator::operator++(int)
 
 
 
-inline const IndexSet::IntervalAccessor &IndexSet::IntervalIterator::
-                                         operator*() const
+inline const IndexSet::IntervalAccessor &
+IndexSet::IntervalIterator::operator*() const
 {
   return accessor;
 }
 
 
 
-inline const IndexSet::IntervalAccessor *IndexSet::IntervalIterator::
-                                         operator->() const
+inline const IndexSet::IntervalAccessor *
+IndexSet::IntervalIterator::operator->() const
 {
   return &accessor;
 }
@@ -1212,8 +1213,8 @@ inline const IndexSet::IntervalAccessor *IndexSet::IntervalIterator::
 
 
 inline bool
-IndexSet::IntervalIterator::
-operator==(const IndexSet::IntervalIterator &other) const
+IndexSet::IntervalIterator::operator==(
+  const IndexSet::IntervalIterator &other) const
 {
   return accessor == other.accessor;
 }
@@ -1221,8 +1222,8 @@ operator==(const IndexSet::IntervalIterator &other) const
 
 
 inline bool
-IndexSet::IntervalIterator::
-operator!=(const IndexSet::IntervalIterator &other) const
+IndexSet::IntervalIterator::operator!=(
+  const IndexSet::IntervalIterator &other) const
 {
   return !(*this == other);
 }
@@ -1230,8 +1231,8 @@ operator!=(const IndexSet::IntervalIterator &other) const
 
 
 inline bool
-IndexSet::IntervalIterator::
-operator<(const IndexSet::IntervalIterator &other) const
+IndexSet::IntervalIterator::operator<(
+  const IndexSet::IntervalIterator &other) const
 {
   return accessor < other.accessor;
 }
@@ -1239,8 +1240,8 @@ operator<(const IndexSet::IntervalIterator &other) const
 
 
 inline int
-IndexSet::IntervalIterator::
-operator-(const IndexSet::IntervalIterator &other) const
+IndexSet::IntervalIterator::operator-(
+  const IndexSet::IntervalIterator &other) const
 {
   Assert(accessor.index_set == other.accessor.index_set,
          ExcMessage(
@@ -1307,7 +1308,8 @@ IndexSet::ElementIterator::is_valid() const
 
 
 
-inline IndexSet::size_type IndexSet::ElementIterator::operator*() const
+inline IndexSet::size_type
+IndexSet::ElementIterator::operator*() const
 {
   Assert(
     is_valid(),
@@ -1319,8 +1321,8 @@ inline IndexSet::size_type IndexSet::ElementIterator::operator*() const
 
 
 inline bool
-IndexSet::ElementIterator::
-operator==(const IndexSet::ElementIterator &other) const
+IndexSet::ElementIterator::operator==(
+  const IndexSet::ElementIterator &other) const
 {
   Assert(index_set == other.index_set,
          ExcMessage(
@@ -1379,8 +1381,8 @@ IndexSet::ElementIterator::operator++(int)
 
 
 inline bool
-IndexSet::ElementIterator::
-operator!=(const IndexSet::ElementIterator &other) const
+IndexSet::ElementIterator::operator!=(
+  const IndexSet::ElementIterator &other) const
 {
   return !(*this == other);
 }
@@ -1388,8 +1390,8 @@ operator!=(const IndexSet::ElementIterator &other) const
 
 
 inline bool
-IndexSet::ElementIterator::
-operator<(const IndexSet::ElementIterator &other) const
+IndexSet::ElementIterator::operator<(
+  const IndexSet::ElementIterator &other) const
 {
   Assert(index_set == other.index_set,
          ExcMessage(
@@ -1401,8 +1403,8 @@ operator<(const IndexSet::ElementIterator &other) const
 
 
 inline std::ptrdiff_t
-IndexSet::ElementIterator::
-operator-(const IndexSet::ElementIterator &other) const
+IndexSet::ElementIterator::operator-(
+  const IndexSet::ElementIterator &other) const
 {
   Assert(index_set == other.index_set,
          ExcMessage(
@@ -1718,7 +1720,7 @@ IndexSet::add_indices(const ForwardIterator &begin, const ForwardIterator &end)
       size_type       end_index   = begin_index + 1;
       ForwardIterator q           = p;
       ++q;
-      while ((q != end) && (*q == end_index))
+      while ((q != end) && (static_cast<size_type>(*q) == end_index))
         {
           ++end_index;
           ++q;
@@ -1731,7 +1733,7 @@ IndexSet::add_indices(const ForwardIterator &begin, const ForwardIterator &end)
       // than the end index of the one just identified, then we will have at
       // least one pair of ranges that are not sorted, and consequently the
       // whole collection of ranges is not sorted.
-      if (p != end && *p < end_index)
+      if (p != end && static_cast<size_type>(*p) < end_index)
         ranges_are_sorted = false;
     }
 
@@ -2015,19 +2017,19 @@ inline void
 IndexSet::print(StreamType &out) const
 {
   compress();
-  out << "{";
+  out << '{';
   std::vector<Range>::const_iterator p;
   for (p = ranges.begin(); p != ranges.end(); ++p)
     {
       if (p->end - p->begin == 1)
         out << p->begin;
       else
-        out << "[" << p->begin << "," << p->end - 1 << "]";
+        out << '[' << p->begin << ',' << p->end - 1 << ']';
 
       if (p != --ranges.end())
         out << ", ";
     }
-  out << "}" << std::endl;
+  out << '}' << std::endl;
 }
 
 
