@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2018 - 2020 by the deal.II authors
+// Copyright (C) 2018 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -236,12 +236,14 @@ public:
   /**
    * Dereferencing operator.
    */
-  reference operator*() const;
+  reference
+  operator*() const;
 
   /**
    * Dereferencing operator.
    */
-  pointer operator->() const;
+  pointer
+  operator->() const;
 
   /**
    * Comparison operator. Returns <code>true</code> if both iterators point to
@@ -328,8 +330,8 @@ protected:
 
 template <class DerivedIterator, class AccessorType>
 inline DerivedIterator &
-LinearIndexIterator<DerivedIterator, AccessorType>::
-operator=(const DerivedIterator &it)
+LinearIndexIterator<DerivedIterator, AccessorType>::operator=(
+  const DerivedIterator &it)
 {
   accessor.container    = it.container;
   accessor.linear_index = it.linear_index;
@@ -380,8 +382,8 @@ LinearIndexIterator<DerivedIterator, AccessorType>::operator--(int)
 
 template <class DerivedIterator, class AccessorType>
 inline DerivedIterator
-LinearIndexIterator<DerivedIterator, AccessorType>::
-operator+(const difference_type n) const
+LinearIndexIterator<DerivedIterator, AccessorType>::operator+(
+  const difference_type n) const
 {
   DerivedIterator copy(this->accessor);
   copy += n;
@@ -392,8 +394,8 @@ operator+(const difference_type n) const
 
 template <class DerivedIterator, class AccessorType>
 inline DerivedIterator
-LinearIndexIterator<DerivedIterator, AccessorType>::
-operator-(const difference_type n) const
+LinearIndexIterator<DerivedIterator, AccessorType>::operator-(
+  const difference_type n) const
 {
   DerivedIterator copy(this->accessor);
   copy += -n;
@@ -404,8 +406,8 @@ operator-(const difference_type n) const
 
 template <class DerivedIterator, class AccessorType>
 inline DerivedIterator &
-LinearIndexIterator<DerivedIterator, AccessorType>::
-operator+=(const difference_type n)
+LinearIndexIterator<DerivedIterator, AccessorType>::operator+=(
+  const difference_type n)
 {
   accessor.linear_index += n;
   return static_cast<DerivedIterator &>(*this);
@@ -415,8 +417,8 @@ operator+=(const difference_type n)
 
 template <class DerivedIterator, class AccessorType>
 inline DerivedIterator &
-LinearIndexIterator<DerivedIterator, AccessorType>::
-operator-=(const difference_type n)
+LinearIndexIterator<DerivedIterator, AccessorType>::operator-=(
+  const difference_type n)
 {
   return operator+=(-n);
 }
@@ -426,8 +428,8 @@ operator-=(const difference_type n)
 template <class DerivedIterator, class AccessorType>
 inline
   typename LinearIndexIterator<DerivedIterator, AccessorType>::difference_type
-  LinearIndexIterator<DerivedIterator, AccessorType>::
-  operator-(const DerivedIterator &other) const
+  LinearIndexIterator<DerivedIterator, AccessorType>::operator-(
+    const DerivedIterator &other) const
 {
   Assert(this->accessor.container == other.accessor.container,
          ExcMessage(
@@ -439,7 +441,7 @@ inline
 
 template <class DerivedIterator, class AccessorType>
 inline typename LinearIndexIterator<DerivedIterator, AccessorType>::reference
-  LinearIndexIterator<DerivedIterator, AccessorType>::operator*() const
+LinearIndexIterator<DerivedIterator, AccessorType>::operator*() const
 {
   return accessor;
 }
@@ -448,7 +450,7 @@ inline typename LinearIndexIterator<DerivedIterator, AccessorType>::reference
 
 template <class DerivedIterator, class AccessorType>
 inline typename LinearIndexIterator<DerivedIterator, AccessorType>::pointer
-  LinearIndexIterator<DerivedIterator, AccessorType>::operator->() const
+LinearIndexIterator<DerivedIterator, AccessorType>::operator->() const
 {
   return &accessor;
 }
@@ -457,8 +459,8 @@ inline typename LinearIndexIterator<DerivedIterator, AccessorType>::pointer
 
 template <class DerivedIterator, class AccessorType>
 inline bool
-LinearIndexIterator<DerivedIterator, AccessorType>::
-operator<=(const DerivedIterator &other) const
+LinearIndexIterator<DerivedIterator, AccessorType>::operator<=(
+  const DerivedIterator &other) const
 {
   return (*this == other) || (*this < other);
 }
@@ -467,8 +469,8 @@ operator<=(const DerivedIterator &other) const
 
 template <class DerivedIterator, class AccessorType>
 inline bool
-LinearIndexIterator<DerivedIterator, AccessorType>::
-operator>=(const DerivedIterator &other) const
+LinearIndexIterator<DerivedIterator, AccessorType>::operator>=(
+  const DerivedIterator &other) const
 {
   return !(*this < other);
 }
@@ -477,8 +479,8 @@ operator>=(const DerivedIterator &other) const
 
 template <class DerivedIterator, class AccessorType>
 inline bool
-LinearIndexIterator<DerivedIterator, AccessorType>::
-operator<(const DerivedIterator &other) const
+LinearIndexIterator<DerivedIterator, AccessorType>::operator<(
+  const DerivedIterator &other) const
 {
   Assert(this->accessor.container == other.accessor.container,
          ExcMessage(
@@ -490,8 +492,8 @@ operator<(const DerivedIterator &other) const
 
 template <class DerivedIterator, class AccessorType>
 inline bool
-LinearIndexIterator<DerivedIterator, AccessorType>::
-operator>(const DerivedIterator &other) const
+LinearIndexIterator<DerivedIterator, AccessorType>::operator>(
+  const DerivedIterator &other) const
 {
   return other < static_cast<const DerivedIterator &>(*this);
 }
