@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2020 by the deal.II authors
+// Copyright (C) 2000 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -306,7 +306,7 @@ namespace MemoryConsumption
   inline std::size_t
   memory_consumption(const std::string &s)
   {
-    return sizeof(s) + s.length();
+    return sizeof(s) + s.size();
   }
 
 
