@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2020 by the deal.II authors
+// Copyright (C) 2003 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -73,7 +73,7 @@ public:
   template <class... Args>
   MGLevelObject(const unsigned int minlevel,
                 const unsigned int maxlevel,
-                Args &&... args);
+                Args &&...args);
 
   /**
    * Constructor. Same as above but without arguments to be forwarded to the
@@ -85,7 +85,8 @@ public:
   /**
    * Access object on level @p level.
    */
-  Object &operator[](const unsigned int level);
+  Object &
+  operator[](const unsigned int level);
 
   /**
    * Access object on level @p level.
@@ -93,7 +94,14 @@ public:
    * This function can be called on a @p const object, and
    * consequently returns a @p const reference.
    */
-  const Object &operator[](const unsigned int level) const;
+  const Object &
+  operator[](const unsigned int level) const;
+
+  /**
+   * Return object on level max.
+   */
+  const Object &
+  back() const;
 
   /**
    * Delete all previous contents of this object and reset its size according
@@ -112,7 +120,7 @@ public:
   void
   resize(const unsigned int new_minlevel,
          const unsigned int new_maxlevel,
-         Args &&... args);
+         Args &&...args);
 
   /**
    * Call <tt>operator = (s)</tt> on all objects stored by this object.
@@ -146,6 +154,12 @@ public:
   unsigned int
   max_level() const;
 
+  /**
+   * Number of levels, i.e., `max_level()-min_level()+1`.
+   */
+  unsigned int
+  n_levels() const;
+
   /**
    * Apply the action @p action to every object stored in here. The
    * parameter @p action is expected to be a function object that accepts
@@ -186,7 +200,7 @@ template <class Object>
 template <class... Args>
 MGLevelObject<Object>::MGLevelObject(const unsigned int min,
                                      const unsigned int max,
-                                     Args &&... args)
+                                     Args &&...args)
   : minlevel(0)
 {
   resize(min, max, std::forward<Args>(args)...);
@@ -203,7 +217,8 @@ MGLevelObject<Object>::MGLevelObject(const unsigned int min,
 
 
 template <class Object>
-Object &MGLevelObject<Object>::operator[](const unsigned int i)
+Object &
+MGLevelObject<Object>::operator[](const unsigned int i)
 {
   Assert((i >= minlevel) && (i < minlevel + objects.size()),
          ExcIndexRange(i, minlevel, minlevel + objects.size()));
@@ -212,7 +227,8 @@ Object &MGLevelObject<Object>::operator[](const unsigned int i)
 
 
 template <class Object>
-const Object &MGLevelObject<Object>::operator[](const unsigned int i) const
+const Object &
+MGLevelObject<Object>::operator[](const unsigned int i) const
 {
   Assert((i >= minlevel) && (i < minlevel + objects.size()),
          ExcIndexRange(i, minlevel, minlevel + objects.size()));
@@ -220,12 +236,20 @@ const Object &MGLevelObject<Object>::operator[](const unsigned int i) const
 }
 
 
+template <class Object>
+const Object &
+MGLevelObject<Object>::back() const
+{
+  return this->operator[](this->max_level());
+}
+
+
 template <class Object>
 template <class... Args>
 void
 MGLevelObject<Object>::resize(const unsigned int new_minlevel,
                               const unsigned int new_maxlevel,
-                              Args &&... args)
+                              Args &&...args)
 {
   Assert(new_minlevel <= new_maxlevel, ExcInternalError());
   // note that on clear(), the
@@ -276,6 +300,14 @@ MGLevelObject<Object>::max_level() const
   return minlevel + objects.size() - 1;
 }
 
+
+template <class Object>
+unsigned int
+MGLevelObject<Object>::n_levels() const
+{
+  return objects.size();
+}
+
 template <class Object>
 template <typename ActionFunctionObjectType>
 void
