@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2020 - 2021 by the deal.II authors
+// Copyright (C) 2020 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -104,12 +104,12 @@ namespace Utilities
 
       // post recv
       AssertIndexRange(recv_ranks.size(), recv_ptr.size());
-      for (types::global_dof_index i = 0; i < recv_ranks.size(); i++)
+      for (types::global_dof_index i = 0; i < recv_ranks.size(); ++i)
         {
-          const auto ierr =
+          const int ierr =
             MPI_Irecv(buffers.data() + recv_ptr[i],
                       recv_ptr[i + 1] - recv_ptr[i],
-                      Utilities::MPI::internal::mpi_type_id(buffers.data()),
+                      Utilities::MPI::mpi_type_id_for_type<Number>,
                       recv_ranks[i],
                       tag,
                       communicator,
@@ -119,7 +119,7 @@ namespace Utilities
 
       // post send
       AssertIndexRange(send_ranks.size(), send_ptr.size());
-      for (types::global_dof_index i = 0, k = 0; i < send_ranks.size(); i++)
+      for (types::global_dof_index i = 0, k = 0; i < send_ranks.size(); ++i)
         {
           // collect data to be send
           for (types::global_dof_index j = send_ptr[i]; j < send_ptr[i + 1];
@@ -135,10 +135,10 @@ namespace Utilities
                    (send_ptr[i] == buffers.size() &&
                     send_ptr[i + 1] == send_ptr[i]),
                  ExcMessage("The input buffer doesn't contain enough entries"));
-          const auto ierr =
+          const int ierr =
             MPI_Isend(buffers.data() + send_ptr[i],
                       send_ptr[i + 1] - send_ptr[i],
-                      Utilities::MPI::internal::mpi_type_id(buffers.data()),
+                      Utilities::MPI::mpi_type_id_for_type<Number>,
                       send_ranks[i],
                       tag,
                       communicator,
@@ -164,7 +164,7 @@ namespace Utilities
       Assert(false, ExcNeedsMPI());
 #else
       // receive all data packages and copy data from buffers
-      for (types::global_dof_index proc = 0; proc < recv_ranks.size(); proc++)
+      for (types::global_dof_index proc = 0; proc < recv_ranks.size(); ++proc)
         {
           int        i;
           MPI_Status status;
@@ -182,7 +182,7 @@ namespace Utilities
         }
 
       // wait that all data packages have been sent
-      const auto ierr =
+      const int ierr =
         MPI_Waitall(send_ranks.size(), requests.data(), MPI_STATUSES_IGNORE);
       AssertThrowMPI(ierr);
 #endif
