@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2020 by the deal.II authors
+// Copyright (C) 2006 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -142,7 +142,9 @@ DEAL_II_NAMESPACE_CLOSE
 // import the whole Sacado header at this point to get the math
 // functions imported into the standard namespace.
 #ifdef DEAL_II_TRILINOS_WITH_SACADO
+DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #  include <Sacado.hpp>
+DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 #endif
 
 namespace std
@@ -428,6 +430,11 @@ namespace numbers
      */
     using real_type = number;
 
+    /**
+     * For this data type, alias the corresponding double type.
+     */
+    using double_type = double;
+
     /**
      * Return the complex-conjugate of the given number. Since the general
      * template is selected if number is not a complex data type, this
@@ -490,6 +497,11 @@ namespace numbers
      */
     using real_type = number;
 
+    /**
+     * For this data type, alias the corresponding double type.
+     */
+    using double_type = std::complex<double>;
+
     /**
      * Return the complex-conjugate of the given number.
      */
@@ -561,7 +573,7 @@ namespace numbers
 
   template <typename number>
   constexpr DEAL_II_CUDA_HOST_DEV const number &
-                                        NumberTraits<number>::conjugate(const number &x)
+  NumberTraits<number>::conjugate(const number &x)
   {
     return x;
   }
@@ -690,7 +702,7 @@ namespace internal
   struct NumberType
   {
     static constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV const T &
-                                                                       value(const T &t)
+    value(const T &t)
     {
       return t;
     }
@@ -705,8 +717,8 @@ namespace internal
     // Type T is constructible from F.
     template <typename F>
     static constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV T
-                                                                 value(const F &f,
-                                                                       typename std::enable_if<
+    value(const F &f,
+          typename std::enable_if<
             !std::is_same<typename std::decay<T>::type,
                           typename std::decay<F>::type>::value &&
             std::is_constructible<T, F>::value>::type * = nullptr)
@@ -717,8 +729,8 @@ namespace internal
     // Type T is explicitly convertible (but not constructible) from F.
     template <typename F>
     static constexpr DEAL_II_ALWAYS_INLINE T
-                                           value(const F &f,
-                                                 typename std::enable_if<
+    value(const F &f,
+          typename std::enable_if<
             !std::is_same<typename std::decay<T>::type,
                           typename std::decay<F>::type>::value &&
             !std::is_constructible<T, F>::value &&
