@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2021 by the deal.II authors
+// Copyright (C) 1998 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -1609,7 +1609,7 @@ public:
    */
   DeclException1(ExcEntryAlreadyExists,
                  std::string,
-                 << "The following entry already exists: " << arg1 << ".");
+                 << "The following entry already exists: " << arg1 << '.');
   /**
    * Exception
    */
@@ -1699,16 +1699,6 @@ public:
                    "The provided file could not be parsed as a "
                    "ParameterHandler description.");
 
-  /**
-   * Exception for when an entry in an XML parameter file does not match the
-   * provided pattern. The arguments are, in order, the entry value, entry
-   * name, and a description of the pattern.
-   *
-   * @deprecated Use ExcValueDoesNotMatchPattern instead of ExcInvalidEntryForPatternXML.
-   */
-  using ExcInvalidEntryForPatternXML DEAL_II_DEPRECATED =
-    ExcValueDoesNotMatchPattern;
-
   /**
    * Exception for when the file given in an include statement cannot be
    * open. The arguments are, in order, the line number of the include
