@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2021 by the deal.II authors
+// Copyright (C) 2011 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -21,13 +21,10 @@
 #include <deal.II/base/array_view.h>
 #include <deal.II/base/communication_pattern_base.h>
 #include <deal.II/base/index_set.h>
-#include <deal.II/base/memory_consumption.h>
 #include <deal.II/base/memory_space.h>
 #include <deal.II/base/mpi.h>
 #include <deal.II/base/types.h>
-#include <deal.II/base/utilities.h>
 
-#include <deal.II/lac/vector.h>
 #include <deal.II/lac/vector_operation.h>
 
 #include <limits>
@@ -291,7 +288,7 @@ namespace Utilities
        * @deprecated Use the more clearly named function locally_owned_size()
        * instead.
        */
-      DEAL_II_DEPRECATED_EARLY
+      DEAL_II_DEPRECATED
       unsigned int
       local_size() const;
 
@@ -655,7 +652,7 @@ namespace Utilities
                      types::global_dof_index,
                      unsigned int,
                      << "Global index " << arg1
-                     << " neither owned nor ghost on proc " << arg2 << ".");
+                     << " neither owned nor ghost on proc " << arg2 << '.');
 
       /**
        * Exception
