@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2021 by the deal.II authors
+// Copyright (C) 1998 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -83,7 +83,12 @@ public:
 
 /**
  * The midpoint rule for numerical quadrature. This one-point formula is exact
- * for linear polynomials.
+ * for linear integrands.
+ *
+ * @note This class only works for cells that are tensor product (hypercube) cells,
+ *   that is, are either ReferenceCells::Line, ReferenceCells::Quadrilateral,
+ *   or ReferenceCells::Hexahedron. For other cell shapes, this class is not
+ *   appropriate. Use ReferenceCell::get_midpoint_quadrature() instead.
  */
 template <int dim>
 class QMidpoint : public Quadrature<dim>
@@ -588,6 +593,9 @@ public:
  *
  * No transformation is applied to the weights, and the weights referring to
  * points that live outside the reference simplex are simply discarded.
+ * Because this leads to (or *may* lead to) a sum of quadrature weights that
+ * do not equal the area of the simplex, the resulting quadrature formula
+ * is not useful for actually computing integrals.
  *
  * The main use of this quadrature formula is not to chop tensor product
  * quadratures. Ideally you should pass to this class a quadrature formula
@@ -800,13 +808,21 @@ public:
  * QGauss quadrature object, even though the present quadrature formula is not
  * a tensor product. The given value is translated for n_points_1D=1,2,3,4 to
  * following number of quadrature points for 2D and 3D:
- *   - 2D: 1, 3, 7, 15
- *   - 3D: 1, 4, 10, 35
+ * - 2D: 1, 4, 7, 15
+ * - 3D: 1, 6, 14, 35
  *
  * For 1D, the quadrature rule degenerates to a
  * `dealii::QGauss<1>(n_points_1D)`.
  *
- * @ingroup simplex
+ * @note The quadrature rules implemented by this class come from a variety of
+ * sources, but all of them have positive quadrature weights.
+ *
+ * @note Several of the schemes implemented by this class are not symmetric with
+ * respect to the vertices - i.e., the locations of the mapped quadrature points
+ * depends on the numbering of the cell vertices. If you need rules that are
+ * independent of the vertex numbering then use QWitherdenVincentSimplex.
+ *
+ * @relates simplex
  */
 template <int dim>
 class QGaussSimplex : public QSimplex<dim>
@@ -824,30 +840,58 @@ public:
  *
  * Like QGauss, users should specify a number `n_points_1D` as an indication
  * of what polynomial degree to be integrated exactly (e.g., for $n$ points,
- * the rule can integrate polynomials of degree $2 n - 1$ exactly). The given
- * value for n_points_1D = 1, 2, 3, 4, 5 results in the following number of
- * quadrature points in 2D and 3D:
- * - 2D: 1, 6, 7, 15, 19
- * - 3D: 1, 8, 14, 35, 59
+ * the rule can integrate polynomials of degree $2 n - 1$ exactly).
+ * Additionally, since these rules were derived for simplices, there are
+ * also even-ordered rules (i.e., they integrate polynomials of degree $2 n$)
+ * available which do not have analogous 1D rules.
+ *
+ * The given value for n_points_1D = 1, 2, 3, 4, 5, 6, 7 (where the last two are
+ * only implemented in 2D) results in the following number of quadrature points
+ * in 2D and 3D:
+ * - 2D: odd (default): 1, 6, 7, 15, 19, 28, 37
+ * - 2D: even: 3, 6, 12, 16, 25, 33, 42
+ * - 3D: odd (default): 1, 8, 14, 35, 59
+ * - 3D: even: 4, 14, 24, 46, 81
  *
  * For 1D, the quadrature rule degenerates to a
- * `dealii::QGauss<1>(n_points_1D)`.
+ * `dealii::QGauss<1>(n_points_1D)` and @p use_odd_order is ignored.
  *
  * These rules match the ones listed for Witherden-Vincent in the quadpy
  * @cite quadpy library and were first described in
  * @cite witherden2015identification.
  *
- * @ingroup simplex
+ * @note Some rules (2D 2 odd and 3D 2 even) do not yet exist and instead a
+ * higher-order rule is used in their place.
+ *
+ * @relates simplex
  */
 template <int dim>
 class QWitherdenVincentSimplex : public QSimplex<dim>
 {
 public:
   /**
-   * Constructor taking the number of quadrature points in 1D direction
-   * @p n_points_1D.
+   * Constructor taking the equivalent number of quadrature points in 1D
+   * @p n_points_1D and boolean indicating whether the rule should be order
+   * $2 n - 1$ or $2 n$: see the general documentation of this class for more
+   * information.
    */
-  explicit QWitherdenVincentSimplex(const unsigned int n_points_1D);
+  explicit QWitherdenVincentSimplex(const unsigned int n_points_1D,
+                                    const bool         use_odd_order = true);
+};
+
+/**
+ * Iterated quadrature for simplices. Since simplex cannot be described as
+ * tensor products the base quadrature has equal dimension.
+ *
+ * At the moment @p n_copies must be a power of 2 due to the complexity of
+ * subdividing a simplex.
+ */
+template <int dim>
+class QIteratedSimplex : public Quadrature<dim>
+{
+public:
+  QIteratedSimplex(const Quadrature<dim> &base_quadrature,
+                   const unsigned int     n_copies);
 };
 
 /**
