@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2021 by the deal.II authors
+// Copyright (C) 1998 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -23,7 +23,6 @@
 #include <deal.II/base/table_indices.h>
 #include <deal.II/base/template_constraints.h>
 #include <deal.II/base/tensor_accessors.h>
-#include <deal.II/base/utilities.h>
 
 #ifdef DEAL_II_WITH_ADOLC
 #  include <adolc/adouble.h> // Taped double
@@ -31,9 +30,6 @@
 
 #include <cmath>
 #include <ostream>
-#include <utility>
-#include <vector>
-
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -177,31 +173,55 @@ public:
    * Move constructor
    */
   constexpr DEAL_II_CUDA_HOST_DEV
-    Tensor(Tensor<0, dim, Number> &&other) noexcept;
+  Tensor(Tensor<0, dim, Number> &&other) noexcept;
 #endif
 
   /**
    * Return a pointer to the first element of the underlying storage.
+   *
+   * @deprecated This function suggests that the elements of a Tensor
+   *   object are stored as a contiguous array, but this is not in fact true
+   *   and one should not pretend that this so. As a consequence, this function
+   *   is deprecated.
    */
+  DEAL_II_DEPRECATED_EARLY
   Number *
   begin_raw();
 
   /**
    * Return a const pointer to the first element of the underlying storage.
+   *
+   * @deprecated This function suggests that the elements of a Tensor
+   *   object are stored as a contiguous array, but this is not in fact true
+   *   and one should not pretend that this so. As a consequence, this function
+   *   is deprecated.
    */
+  DEAL_II_DEPRECATED_EARLY
   const Number *
   begin_raw() const;
 
   /**
    * Return a pointer to the element past the end of the underlying storage.
+   *
+   * @deprecated This function suggests that the elements of a Tensor
+   *   object are stored as a contiguous array, but this is not in fact true
+   *   and one should not pretend that this so. As a consequence, this function
+   *   is deprecated.
    */
+  DEAL_II_DEPRECATED_EARLY
   Number *
   end_raw();
 
   /**
    * Return a const pointer to the element past the end of the underlying
    * storage.
+   *
+   * @deprecated This function suggests that the elements of a Tensor
+   *   object are stored as a contiguous array, but this is not in fact true
+   *   and one should not pretend that this so. As a consequence, this function
+   *   is deprecated.
    */
+  DEAL_II_DEPRECATED_EARLY
   const Number *
   end_raw() const;
 
@@ -214,7 +234,8 @@ public:
    *
    * @note This function can also be used in CUDA device code.
    */
-  constexpr DEAL_II_CUDA_HOST_DEV operator Number &();
+  constexpr DEAL_II_CUDA_HOST_DEV
+  operator Number &();
 
   /**
    * Return a reference to the encapsulated Number object. Since rank-0
@@ -235,7 +256,7 @@ public:
    */
   template <typename OtherNumber>
   constexpr DEAL_II_CUDA_HOST_DEV Tensor &
-                                  operator=(const Tensor<0, dim, OtherNumber> &rhs);
+  operator=(const Tensor<0, dim, OtherNumber> &rhs);
 
 #if defined(__INTEL_COMPILER) || defined(DEAL_II_DELETED_MOVE_CONSTRUCTOR_BUG)
   /**
@@ -255,7 +276,7 @@ public:
    * Move assignment operator
    */
   constexpr DEAL_II_CUDA_HOST_DEV Tensor<0, dim, Number> &
-                                  operator=(Tensor<0, dim, Number> &&other) noexcept;
+  operator=(Tensor<0, dim, Number> &&other) noexcept;
 #endif
 
   /**
@@ -289,7 +310,7 @@ public:
    */
   template <typename OtherNumber>
   constexpr DEAL_II_CUDA_HOST_DEV Tensor &
-                                  operator+=(const Tensor<0, dim, OtherNumber> &rhs);
+  operator+=(const Tensor<0, dim, OtherNumber> &rhs);
 
   /**
    * Subtract another scalar.
@@ -298,7 +319,7 @@ public:
    */
   template <typename OtherNumber>
   constexpr DEAL_II_CUDA_HOST_DEV Tensor &
-                                  operator-=(const Tensor<0, dim, OtherNumber> &rhs);
+  operator-=(const Tensor<0, dim, OtherNumber> &rhs);
 
   /**
    * Multiply the scalar with a <tt>factor</tt>.
@@ -324,7 +345,7 @@ public:
    * @note This function can also be used in CUDA device code.
    */
   constexpr DEAL_II_CUDA_HOST_DEV Tensor
-                                  operator-() const;
+  operator-() const;
 
   /**
    * Reset all values to zero.
@@ -356,7 +377,18 @@ public:
    * @note This function can also be used in CUDA device code.
    */
   constexpr DEAL_II_CUDA_HOST_DEV real_type
-                                  norm_square() const;
+  norm_square() const;
+
+  /**
+   * Fill a range with all tensor elements. Since this type of Tensor only has
+   * one entry this just copies the value of this tensor into <tt>*begin</tt>.
+   *
+   * The template type Number must be convertible to the type of
+   * <tt>*begin</tt>.
+   */
+  template <class Iterator>
+  void
+  unroll(const Iterator begin, const Iterator end) const;
 
   /**
    * Read or write the data of this object to or from a stream for the purpose
@@ -382,10 +414,9 @@ private:
   /**
    * Internal helper function for unroll.
    */
-  template <typename OtherNumber>
-  void
-  unroll_recursion(Vector<OtherNumber> &result,
-                   unsigned int &       start_index) const;
+  template <typename Iterator>
+  Iterator
+  unroll_recursion(const Iterator current, const Iterator end) const;
 
   // Allow an arbitrary Tensor to access the underlying values.
   template <int, int, typename>
@@ -474,7 +505,7 @@ public:
   static_assert(rank_ >= 1,
                 "Tensors must have a rank greater than or equal to one.");
   static_assert(dim >= 0,
-                "Tensors must have a dimension greater than or equal to one.");
+                "Tensors must have a dimension greater than or equal to zero.");
   /**
    * Provide a way to get the dimension of an object without explicit
    * knowledge of it's data type. Implementation is this way instead of
@@ -517,7 +548,7 @@ public:
    * @note This function can also be used in CUDA device code.
    */
   constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
-                                  Tensor();
+  Tensor();
 
   /**
    * A constructor where the data is copied from a C-style array.
@@ -586,7 +617,8 @@ public:
    *
    * @note This function can also be used in CUDA device code.
    */
-  constexpr DEAL_II_CUDA_HOST_DEV value_type &operator[](const unsigned int i);
+  constexpr DEAL_II_CUDA_HOST_DEV value_type &
+                                  operator[](const unsigned int i);
 
   /**
    * Read-only access operator.
@@ -599,12 +631,14 @@ public:
   /**
    * Read access using TableIndices <tt>indices</tt>
    */
-  constexpr const Number &operator[](const TableIndices<rank_> &indices) const;
+  constexpr const Number &
+  operator[](const TableIndices<rank_> &indices) const;
 
   /**
    * Read and write access using TableIndices <tt>indices</tt>
    */
-  constexpr Number &operator[](const TableIndices<rank_> &indices);
+  constexpr Number &
+  operator[](const TableIndices<rank_> &indices);
 
   /**
    * Return a pointer to the first element of the underlying storage.
@@ -639,7 +673,7 @@ public:
    */
   template <typename OtherNumber>
   constexpr DEAL_II_CUDA_HOST_DEV Tensor &
-                                  operator=(const Tensor<rank_, dim, OtherNumber> &rhs);
+  operator=(const Tensor<rank_, dim, OtherNumber> &rhs);
 
   /**
    * This operator assigns a scalar to a tensor. To avoid confusion with what
@@ -685,7 +719,7 @@ public:
    */
   template <typename OtherNumber>
   constexpr DEAL_II_CUDA_HOST_DEV Tensor &
-                                  operator+=(const Tensor<rank_, dim, OtherNumber> &);
+  operator+=(const Tensor<rank_, dim, OtherNumber> &);
 
   /**
    * Subtract another tensor.
@@ -694,7 +728,7 @@ public:
    */
   template <typename OtherNumber>
   constexpr DEAL_II_CUDA_HOST_DEV Tensor &
-                                  operator-=(const Tensor<rank_, dim, OtherNumber> &);
+  operator-=(const Tensor<rank_, dim, OtherNumber> &);
 
   /**
    * Scale the tensor by <tt>factor</tt>, i.e. multiply all components by
@@ -721,7 +755,7 @@ public:
    * @note This function can also be used in CUDA device code.
    */
   constexpr DEAL_II_CUDA_HOST_DEV Tensor
-                                  operator-() const;
+  operator-() const;
 
   /**
    * Reset all values to zero.
@@ -765,11 +799,28 @@ public:
    * This function unrolls all tensor entries into a single, linearly numbered
    * vector. As usual in C++, the rightmost index of the tensor marches
    * fastest.
+   *
+   * @deprecated Use the more general function that takes a pair of iterators
+   * instead.
    */
   template <typename OtherNumber>
-  void
+  DEAL_II_DEPRECATED_EARLY void
   unroll(Vector<OtherNumber> &result) const;
 
+  /**
+   * Fill a range with all tensor elements.
+   *
+   * This function unrolls all tensor entries into a single, linearly numbered
+   * sequence. The order of the elements is the one given by
+   * component_to_unrolled_index().
+   *
+   * The template type Number must be convertible to the type of
+   * <tt>*begin</tt>.
+   */
+  template <class Iterator>
+  void
+  unroll(const Iterator begin, const Iterator end) const;
+
   /**
    * Return an unrolled index in the range $[0,\text{dim}^{\text{rank}}-1]$
    * for the element of the tensor indexed by the argument to the function.
@@ -818,10 +869,9 @@ private:
   /**
    * Internal helper function for unroll.
    */
-  template <typename OtherNumber>
-  void
-  unroll_recursion(Vector<OtherNumber> &result,
-                   unsigned int &       start_index) const;
+  template <typename Iterator>
+  Iterator
+  unroll_recursion(const Iterator current, const Iterator end) const;
 
   /**
    * This constructor is for internal use. It provides a way
@@ -887,7 +937,7 @@ namespace internal
   struct NumberType<Tensor<rank, dim, T>>
   {
     static constexpr DEAL_II_ALWAYS_INLINE const Tensor<rank, dim, T> &
-                                                 value(const Tensor<rank, dim, T> &t)
+    value(const Tensor<rank, dim, T> &t)
     {
       return t;
     }
@@ -908,7 +958,7 @@ namespace internal
 
 template <int dim, typename Number>
 constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
-                                Tensor<0, dim, Number>::Tensor()
+Tensor<0, dim, Number>::Tensor()
   // Some auto-differentiable numbers need explicit
   // zero initialization such as adtl::adouble.
   : Tensor{0.0}
@@ -919,7 +969,7 @@ constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
 template <int dim, typename Number>
 template <typename OtherNumber>
 constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
-                                Tensor<0, dim, Number>::Tensor(const OtherNumber &initializer)
+Tensor<0, dim, Number>::Tensor(const OtherNumber &initializer)
   : value(internal::NumberType<Number>::value(initializer))
 {}
 
@@ -928,7 +978,7 @@ constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
 template <int dim, typename Number>
 template <typename OtherNumber>
 constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
-                                Tensor<0, dim, Number>::Tensor(const Tensor<0, dim, OtherNumber> &p)
+Tensor<0, dim, Number>::Tensor(const Tensor<0, dim, OtherNumber> &p)
   : Tensor{p.value}
 {}
 
@@ -936,7 +986,7 @@ constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
 #  ifdef DEAL_II_DELETED_MOVE_CONSTRUCTOR_BUG
 template <int dim, typename Number>
 constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
-                                Tensor<0, dim, Number>::Tensor(const Tensor<0, dim, Number> &other)
+Tensor<0, dim, Number>::Tensor(const Tensor<0, dim, Number> &other)
   : value{other.value}
 {}
 
@@ -944,7 +994,7 @@ constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
 
 template <int dim, typename Number>
 constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
-                                Tensor<0, dim, Number>::Tensor(Tensor<0, dim, Number> &&other) noexcept
+Tensor<0, dim, Number>::Tensor(Tensor<0, dim, Number> &&other) noexcept
   : value{std::move(other.value)}
 {}
 #  endif
@@ -987,8 +1037,8 @@ Tensor<0, dim, Number>::end_raw() const
 
 
 template <int dim, typename Number>
-constexpr inline DEAL_II_ALWAYS_INLINE
-  DEAL_II_CUDA_HOST_DEV Tensor<0, dim, Number>::operator Number &()
+constexpr inline DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
+Tensor<0, dim, Number>::operator Number &()
 {
   // We cannot use Assert inside a CUDA kernel
 #  ifndef __CUDA_ARCH__
@@ -1012,6 +1062,7 @@ constexpr inline DEAL_II_ALWAYS_INLINE
 }
 
 
+
 template <int dim, typename Number>
 template <typename OtherNumber>
 constexpr inline DEAL_II_ALWAYS_INLINE
@@ -1037,7 +1088,7 @@ constexpr inline DEAL_II_ALWAYS_INLINE
 #  ifdef DEAL_II_DELETED_MOVE_CONSTRUCTOR_BUG
 template <int dim, typename Number>
 constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV Tensor<0, dim, Number> &
-  Tensor<0, dim, Number>::operator=(Tensor<0, dim, Number> &&other) noexcept
+Tensor<0, dim, Number>::operator=(Tensor<0, dim, Number> &&other) noexcept
 {
   value = std::move(other.value);
   return *this;
@@ -1062,7 +1113,7 @@ template <typename OtherNumber>
 constexpr inline bool
 Tensor<0, dim, Number>::operator==(const Tensor<0, dim, OtherNumber> &p) const
 {
-#  if defined(DEAL_II_ADOLC_WITH_ADVANCED_BRANCHING)
+#  ifdef DEAL_II_ADOLC_WITH_ADVANCED_BRANCHING
   Assert(!(std::is_same<Number, adouble>::value ||
            std::is_same<OtherNumber, adouble>::value),
          ExcMessage(
@@ -1112,7 +1163,7 @@ namespace internal
   {
     template <typename Number, typename OtherNumber>
     constexpr inline DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV void
-                                           multiply_assign_scalar(Number &val, const OtherNumber &s)
+    multiply_assign_scalar(Number &val, const OtherNumber &s)
     {
       val *= s;
     }
@@ -1120,7 +1171,7 @@ namespace internal
 #  ifdef __CUDA_ARCH__
     template <typename Number, typename OtherNumber>
     constexpr inline DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV void
-                                           multiply_assign_scalar(std::complex<Number> &, const OtherNumber &)
+    multiply_assign_scalar(std::complex<Number> &, const OtherNumber &)
     {
       printf("This function is not implemented for std::complex<Number>!\n");
       assert(false);
@@ -1184,19 +1235,25 @@ constexpr DEAL_II_CUDA_HOST_DEV inline DEAL_II_ALWAYS_INLINE
 }
 
 
+
 template <int dim, typename Number>
-template <typename OtherNumber>
-inline void
-Tensor<0, dim, Number>::unroll_recursion(Vector<OtherNumber> &result,
-                                         unsigned int &       index) const
+template <typename Iterator>
+Iterator
+Tensor<0, dim, Number>::unroll_recursion(const Iterator current,
+                                         const Iterator end) const
 {
+  (void)end;
   Assert(dim != 0,
          ExcMessage("Cannot unroll an object of type Tensor<0,0,Number>"));
-  result[index] = value;
-  ++index;
+  Assert(std::distance(current, end) >= 1,
+         ExcMessage("The provided iterator range must contain at least one "
+                    "element."));
+  *current = value;
+  return std::next(current);
 }
 
 
+
 template <int dim, typename Number>
 constexpr inline void
 Tensor<0, dim, Number>::clear()
@@ -1207,6 +1264,18 @@ Tensor<0, dim, Number>::clear()
 }
 
 
+
+template <int dim, typename Number>
+template <class Iterator>
+inline void
+Tensor<0, dim, Number>::unroll(const Iterator begin, const Iterator end) const
+{
+  AssertDimension(std::distance(begin, end), n_independent_components);
+  unroll_recursion(begin, end);
+}
+
+
+
 template <int dim, typename Number>
 template <class Archive>
 inline void
@@ -1225,7 +1294,7 @@ constexpr unsigned int Tensor<0, dim, Number>::n_independent_components;
 template <int rank_, int dim, typename Number>
 template <typename ArrayLike, std::size_t... indices>
 constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
-                                Tensor<rank_, dim, Number>::Tensor(const ArrayLike &initializer,
+Tensor<rank_, dim, Number>::Tensor(const ArrayLike &initializer,
                                    std::index_sequence<indices...>)
   : values{Tensor<rank_ - 1, dim, Number>(initializer[indices])...}
 {
@@ -1237,7 +1306,7 @@ constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
 
 template <int rank_, int dim, typename Number>
 constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
-                                Tensor<rank_, dim, Number>::Tensor()
+Tensor<rank_, dim, Number>::Tensor()
   // We would like to use =default, but this causes compile errors with some
   // MSVC versions and internal compiler errors with -O1 in gcc 5.4.
   : values{}
@@ -1247,7 +1316,7 @@ constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
 
 template <int rank_, int dim, typename Number>
 constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
-                                Tensor<rank_, dim, Number>::Tensor(const array_type &initializer)
+Tensor<rank_, dim, Number>::Tensor(const array_type &initializer)
   : Tensor(initializer, std::make_index_sequence<dim>{})
 {}
 
@@ -1256,7 +1325,7 @@ constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
 template <int rank_, int dim, typename Number>
 template <typename ElementType, typename MemorySpace>
 constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
-                                Tensor<rank_, dim, Number>::Tensor(
+Tensor<rank_, dim, Number>::Tensor(
   const ArrayView<ElementType, MemorySpace> &initializer)
 {
   AssertDimension(initializer.size(), n_independent_components);
@@ -1270,7 +1339,7 @@ constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
 template <int rank_, int dim, typename Number>
 template <typename OtherNumber>
 constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
-                                Tensor<rank_, dim, Number>::Tensor(
+Tensor<rank_, dim, Number>::Tensor(
   const Tensor<rank_, dim, OtherNumber> &initializer)
   : Tensor(initializer, std::make_index_sequence<dim>{})
 {}
@@ -1290,7 +1359,7 @@ Tensor<rank_, dim, Number>::Tensor(
 template <int rank_, int dim, typename Number>
 template <typename OtherNumber>
 constexpr DEAL_II_ALWAYS_INLINE Tensor<rank_, dim, Number>::
-                                operator Tensor<1, dim, Tensor<rank_ - 1, dim, OtherNumber>>() const
+operator Tensor<1, dim, Tensor<rank_ - 1, dim, OtherNumber>>() const
 {
   return Tensor<1, dim, Tensor<rank_ - 1, dim, Number>>(values);
 }
@@ -1370,9 +1439,9 @@ namespace internal
 
 
 template <int rank_, int dim, typename Number>
-constexpr inline DEAL_II_ALWAYS_INLINE             DEAL_II_CUDA_HOST_DEV
-  typename Tensor<rank_, dim, Number>::value_type &Tensor<rank_, dim, Number>::
-                                                   operator[](const unsigned int i)
+constexpr inline DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
+  typename Tensor<rank_, dim, Number>::value_type &
+  Tensor<rank_, dim, Number>::operator[](const unsigned int i)
 {
   return dealii::internal::TensorSubscriptor::subscript(
     values, i, std::integral_constant<int, dim>());
@@ -1381,8 +1450,8 @@ constexpr inline DEAL_II_ALWAYS_INLINE             DEAL_II_CUDA_HOST_DEV
 
 template <int rank_, int dim, typename Number>
 constexpr DEAL_II_ALWAYS_INLINE
-    DEAL_II_CUDA_HOST_DEV const typename Tensor<rank_, dim, Number>::value_type &
-    Tensor<rank_, dim, Number>::operator[](const unsigned int i) const
+  DEAL_II_CUDA_HOST_DEV const typename Tensor<rank_, dim, Number>::value_type &
+  Tensor<rank_, dim, Number>::operator[](const unsigned int i) const
 {
 #  ifndef DEAL_II_COMPILER_CUDA_AWARE
   AssertIndexRange(i, dim);
@@ -1394,8 +1463,7 @@ constexpr DEAL_II_ALWAYS_INLINE
 
 template <int rank_, int dim, typename Number>
 constexpr inline DEAL_II_ALWAYS_INLINE const Number &
-                                             Tensor<rank_, dim, Number>::
-                                             operator[](const TableIndices<rank_> &indices) const
+Tensor<rank_, dim, Number>::operator[](const TableIndices<rank_> &indices) const
 {
 #  ifndef DEAL_II_COMPILER_CUDA_AWARE
   Assert(dim != 0,
@@ -1408,8 +1476,8 @@ constexpr inline DEAL_II_ALWAYS_INLINE const Number &
 
 
 template <int rank_, int dim, typename Number>
-constexpr inline DEAL_II_ALWAYS_INLINE Number &Tensor<rank_, dim, Number>::
-                                               operator[](const TableIndices<rank_> &indices)
+constexpr inline DEAL_II_ALWAYS_INLINE Number &
+Tensor<rank_, dim, Number>::operator[](const TableIndices<rank_> &indices)
 {
 #  ifndef DEAL_II_COMPILER_CUDA_AWARE
   Assert(dim != 0,
@@ -1500,8 +1568,8 @@ Tensor<rank_, dim, Number>::operator=(const Tensor<rank_, dim, Number> &other)
 
 template <int rank_, int dim, typename Number>
 constexpr DEAL_II_ALWAYS_INLINE Tensor<rank_, dim, Number> &
-                                Tensor<rank_, dim, Number>::
-                                operator=(Tensor<rank_, dim, Number> &&other) noexcept
+Tensor<rank_, dim, Number>::operator=(
+  Tensor<rank_, dim, Number> &&other) noexcept
 {
   for (unsigned int i = 0; i < dim; ++i)
     values[i] = other.values[i];
@@ -1513,8 +1581,8 @@ constexpr DEAL_II_ALWAYS_INLINE Tensor<rank_, dim, Number> &
 template <int rank_, int dim, typename Number>
 template <typename OtherNumber>
 constexpr inline bool
-Tensor<rank_, dim, Number>::
-operator==(const Tensor<rank_, dim, OtherNumber> &p) const
+Tensor<rank_, dim, Number>::operator==(
+  const Tensor<rank_, dim, OtherNumber> &p) const
 {
   for (unsigned int i = 0; i < dim; ++i)
     if (values[i] != p.values[i])
@@ -1540,8 +1608,8 @@ Tensor<1, 0, double>::operator==(const Tensor<1, 0, double> &) const
 template <int rank_, int dim, typename Number>
 template <typename OtherNumber>
 constexpr bool
-Tensor<rank_, dim, Number>::
-operator!=(const Tensor<rank_, dim, OtherNumber> &p) const
+Tensor<rank_, dim, Number>::operator!=(
+  const Tensor<rank_, dim, OtherNumber> &p) const
 {
   return !((*this) == p);
 }
@@ -1551,8 +1619,8 @@ template <int rank_, int dim, typename Number>
 template <typename OtherNumber>
 constexpr inline DEAL_II_ALWAYS_INLINE
   DEAL_II_CUDA_HOST_DEV Tensor<rank_, dim, Number> &
-                        Tensor<rank_, dim, Number>::
-                        operator+=(const Tensor<rank_, dim, OtherNumber> &p)
+  Tensor<rank_, dim, Number>::operator+=(
+    const Tensor<rank_, dim, OtherNumber> &p)
 {
   for (unsigned int i = 0; i < dim; ++i)
     values[i] += p.values[i];
@@ -1564,8 +1632,8 @@ template <int rank_, int dim, typename Number>
 template <typename OtherNumber>
 constexpr inline DEAL_II_ALWAYS_INLINE
   DEAL_II_CUDA_HOST_DEV Tensor<rank_, dim, Number> &
-                        Tensor<rank_, dim, Number>::
-                        operator-=(const Tensor<rank_, dim, OtherNumber> &p)
+  Tensor<rank_, dim, Number>::operator-=(
+    const Tensor<rank_, dim, OtherNumber> &p)
 {
   for (unsigned int i = 0; i < dim; ++i)
     values[i] -= p.values[i];
@@ -1677,27 +1745,39 @@ constexpr inline DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
 }
 
 
+
 template <int rank_, int dim, typename Number>
 template <typename OtherNumber>
 inline void
 Tensor<rank_, dim, Number>::unroll(Vector<OtherNumber> &result) const
 {
-  AssertDimension(result.size(),
-                  (Utilities::fixed_power<rank_, unsigned int>(dim)));
-
-  unsigned int index = 0;
-  unroll_recursion(result, index);
+  unroll(result.begin(), result.end());
 }
 
 
+
 template <int rank_, int dim, typename Number>
-template <typename OtherNumber>
+template <class Iterator>
 inline void
-Tensor<rank_, dim, Number>::unroll_recursion(Vector<OtherNumber> &result,
-                                             unsigned int &       index) const
+Tensor<rank_, dim, Number>::unroll(const Iterator begin,
+                                   const Iterator end) const
 {
+  AssertDimension(std::distance(begin, end), n_independent_components);
+  unroll_recursion(begin, end);
+}
+
+
+
+template <int rank_, int dim, typename Number>
+template <typename Iterator>
+Iterator
+Tensor<rank_, dim, Number>::unroll_recursion(const Iterator current,
+                                             const Iterator end) const
+{
+  auto next = current;
   for (unsigned int i = 0; i < dim; ++i)
-    values[i].unroll_recursion(result, index);
+    next = values[i].unroll_recursion(next, end);
+  return next;
 }
 
 
@@ -1945,8 +2025,8 @@ DEAL_II_CUDA_HOST_DEV constexpr DEAL_II_ALWAYS_INLINE
  */
 template <int dim, typename Number, typename OtherNumber>
 constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
-                                Tensor<0, dim, typename ProductType<Number, OtherNumber>::type>
-                                operator+(const Tensor<0, dim, Number> &     p,
+  Tensor<0, dim, typename ProductType<Number, OtherNumber>::type>
+  operator+(const Tensor<0, dim, Number> &     p,
             const Tensor<0, dim, OtherNumber> &q)
 {
   return static_cast<const Number &>(p) + static_cast<const OtherNumber &>(q);
@@ -1962,8 +2042,8 @@ constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
  */
 template <int dim, typename Number, typename OtherNumber>
 constexpr DEAL_II_ALWAYS_INLINE DEAL_II_CUDA_HOST_DEV
-                                Tensor<0, dim, typename ProductType<Number, OtherNumber>::type>
-                                operator-(const Tensor<0, dim, Number> &     p,
+  Tensor<0, dim, typename ProductType<Number, OtherNumber>::type>
+  operator-(const Tensor<0, dim, Number> &     p,
             const Tensor<0, dim, OtherNumber> &q)
 {
   return static_cast<const Number &>(p) - static_cast<const OtherNumber &>(q);
@@ -2198,7 +2278,8 @@ inline constexpr DEAL_II_ALWAYS_INLINE
 
 
 /**
- * The dot product (single contraction) for tensors: Return a tensor of rank
+ * The dot product (single contraction) for tensors. This function return a
+ * tensor of rank
  * $(\text{rank}_1 + \text{rank}_2 - 2)$ that is the contraction of the last
  * index of a tensor @p src1 of rank @p rank_1 with the first index of a
  * tensor @p src2 of rank @p rank_2:
@@ -2211,11 +2292,24 @@ inline constexpr DEAL_II_ALWAYS_INLINE
  *
  * @note For the Tensor class, the multiplication operator only performs a
  * contraction over a single pair of indices. This is in contrast to the
- * multiplication operator for SymmetricTensor, which does the double
- * contraction.
+ * multiplication operator for SymmetricTensor, for which the corresponding
+ * `operator*()` performs a double contraction.  The origin of the difference in
+ * how `operator*()` is implemented between Tensor and SymmetricTensor is that
+ * for the former, the product between two Tensor objects of same rank and
+ * dimension results in another Tensor object -- that it, `operator*()`
+ * corresponds to the multiplicative group action within the group of tensors.
+ * On the other hand, there is no corresponding multiplicative group action with
+ * the set of symmetric tensors because, in general, the product of two
+ * symmetric tensors is a *nonsymmetric* tensor. As a consequence, for a
+ * mathematician, it is clear that `operator*()` for symmetric tensors must have
+ * a different meaning: namely the *dot* or *scalar product* that maps two
+ * symmetric tensors of rank 2 to a scalar. This corresponds to the double-dot
+ * (colon) operator whose meaning is then extended to the product of any two
+ * even-ranked symmetric tensors.
  *
- * @note In case the contraction yields a tensor of rank 0 the scalar number
- * is returned as an unwrapped number type.
+ * @note In case the contraction yields a tensor of rank 0, that is, if
+ *   `rank_1==rank_2==1`, then a scalar number is returned as an unwrapped
+ *   number type.
  *
  * @relatesalso Tensor
  */
@@ -2466,8 +2560,10 @@ constexpr inline DEAL_II_ALWAYS_INLINE
  * @relatesalso Tensor
  */
 template <template <int, int, typename> class TensorT1,
-          template <int, int, typename> class TensorT2,
-          template <int, int, typename> class TensorT3,
+          template <int, int, typename>
+          class TensorT2,
+          template <int, int, typename>
+          class TensorT3,
           int rank_1,
           int rank_2,
           int dim,
@@ -2536,7 +2632,7 @@ constexpr inline DEAL_II_ALWAYS_INLINE
  */
 template <int dim, typename Number>
 constexpr inline DEAL_II_ALWAYS_INLINE Tensor<1, dim, Number>
-                                       cross_product_2d(const Tensor<1, dim, Number> &src)
+cross_product_2d(const Tensor<1, dim, Number> &src)
 {
   Assert(dim == 2, ExcInternalError());
 
@@ -2595,7 +2691,7 @@ constexpr inline DEAL_II_ALWAYS_INLINE
  */
 template <int dim, typename Number>
 constexpr inline DEAL_II_ALWAYS_INLINE Number
-                                       determinant(const Tensor<2, dim, Number> &t)
+determinant(const Tensor<2, dim, Number> &t)
 {
   // Compute the determinant using the Laplace expansion of the
   // determinant. We expand along the last row.
@@ -2623,7 +2719,7 @@ constexpr inline DEAL_II_ALWAYS_INLINE Number
  */
 template <typename Number>
 constexpr DEAL_II_ALWAYS_INLINE Number
-                                determinant(const Tensor<2, 1, Number> &t)
+determinant(const Tensor<2, 1, Number> &t)
 {
   return t[0][0];
 }
@@ -2635,7 +2731,7 @@ constexpr DEAL_II_ALWAYS_INLINE Number
  */
 template <typename Number>
 constexpr DEAL_II_ALWAYS_INLINE Number
-                                determinant(const Tensor<2, 2, Number> &t)
+determinant(const Tensor<2, 2, Number> &t)
 {
   // hard-coded for efficiency reasons
   return t[0][0] * t[1][1] - t[1][0] * t[0][1];
@@ -2648,7 +2744,7 @@ constexpr DEAL_II_ALWAYS_INLINE Number
  */
 template <typename Number>
 constexpr DEAL_II_ALWAYS_INLINE Number
-                                determinant(const Tensor<2, 3, Number> &t)
+determinant(const Tensor<2, 3, Number> &t)
 {
   // hard-coded for efficiency reasons
   const Number C0 = internal::NumberType<Number>::value(t[1][1] * t[2][2]) -
@@ -2669,7 +2765,7 @@ constexpr DEAL_II_ALWAYS_INLINE Number
  */
 template <int dim, typename Number>
 constexpr inline DEAL_II_ALWAYS_INLINE Number
-                                       trace(const Tensor<2, dim, Number> &d)
+trace(const Tensor<2, dim, Number> &d)
 {
   Number t = d[0][0];
   for (unsigned int i = 1; i < dim; ++i)
@@ -2775,7 +2871,7 @@ constexpr inline DEAL_II_ALWAYS_INLINE Tensor<2, 3, Number>
  */
 template <int dim, typename Number>
 constexpr inline DEAL_II_ALWAYS_INLINE Tensor<2, dim, Number>
-                                       transpose(const Tensor<2, dim, Number> &t)
+transpose(const Tensor<2, dim, Number> &t)
 {
   Tensor<2, dim, Number> tt;
   for (unsigned int i = 0; i < dim; ++i)
