@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2014 - 2020 by the deal.II authors
+// Copyright (C) 2014 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -261,7 +261,7 @@ namespace TimeStepping
      * the output is value of f at this point. @p id_minus_tau_J_inverse is a
      * function that computes $ inv(I-\tau J)$ where $ I $ is the identity
      * matrix, $ \tau $ is given, and $ J $ is the Jacobian $ \frac{\partial
-     * J}{\partial y} $. The input parameters are the time, $ \tau $, and a
+     * f}{\partial y} $. The input parameters are the time, $ \tau $, and a
      * vector. The output is the value of function at this point.
      * evolve_one_time_step returns the time at the end of the time step.
      */
@@ -333,10 +333,12 @@ namespace TimeStepping
      * parameters are the time t and the vector y and the output is value of f
      * at this point. @p id_minus_tau_J_inverse is a function that computes $
      * inv(I-\tau J)$ where $ I $ is the identity matrix, $ \tau $ is given,
-     * and $ J $ is the Jacobian $ \frac{\partial J}{\partial y} $. The input
+     * and $ J $ is the Jacobian $ \frac{\partial f}{\partial y} $. The input
      * parameter are the time, $ \tau $, and a vector. The output is the value
      * of function at this point. evolve_one_time_step returns the time at the
      * end of the time step.
+     *
+     * @note @p id_minus_tau_J_inverse is ignored since the method is explicit.
      */
     double
     evolve_one_time_step(
@@ -435,7 +437,7 @@ namespace TimeStepping
      * parameters are the time t and the vector y and the output is value of f
      * at this point. @p id_minus_tau_J_inverse is a function that computes $
      * inv(I-\tau J)$ where $ I $ is the identity matrix, $ \tau $ is given,
-     * and $ J $ is the Jacobian $ \frac{\partial J}{\partial y} $. The input
+     * and $ J $ is the Jacobian $ \frac{\partial f}{\partial y} $. The input
      * parameters are the time, $ \tau $, and a vector. The output is the value
      * of function at this point. evolve_one_time_step returns the time at the
      * end of the time step.
@@ -558,7 +560,7 @@ namespace TimeStepping
      * parameters are the time t and the vector y and the output is value of f
      * at this point. @p id_minus_tau_J_inverse is a function that computes $
      * (I-\tau J)^{-1}$ where $ I $ is the identity matrix, $ \tau $ is given,
-     * and $ J $ is the Jacobian $ \frac{\partial J}{\partial y} $. The input
+     * and $ J $ is the Jacobian $ \frac{\partial f}{\partial y} $. The input
      * parameters this function receives are the time, $ \tau $, and a vector.
      * The output is the value of function at this point. evolve_one_time_step
      * returns the time at the end of the time step.
@@ -642,13 +644,6 @@ namespace TimeStepping
       VectorType &      tendency,
       VectorType &      residual) const;
 
-    /**
-     * When using SDIRK, there is no need to compute the linear combination of
-     * the stages. Thus, when this flag is true, the linear combination is
-     * skipped.
-     */
-    bool skip_linear_combi;
-
     /**
      * Maximum number of iterations of the Newton solver.
      */
@@ -722,10 +717,12 @@ namespace TimeStepping
      * parameters are the time t and the vector y and the output is value of f
      * at this point. @p id_minus_tau_J_inverse is a function that computes $
      * inv(I-\tau J)$ where $ I $ is the identity matrix, $ \tau $ is given,
-     * and $ J $ is the Jacobian $ \frac{\partial J}{\partial y} $. The input
+     * and $ J $ is the Jacobian $ \frac{\partial f}{\partial y} $. The input
      * parameters are the time, $ \tau $, and a vector. The output is the
      * value of function at this point. evolve_one_time_step returns the time
      * at the end of the time step.
+     *
+     * @note @p id_minus_tau_J_inverse is ignored since the method is explicit.
      */
     double
     evolve_one_time_step(
@@ -770,6 +767,10 @@ namespace TimeStepping
      */
     struct Status : public TimeStepping<VectorType>::Status
     {
+      Status()
+        : method(invalid)
+      {}
+
       runge_kutta_method             method;
       embedded_runge_kutta_time_step exit_delta_t;
       unsigned int                   n_iterations;
@@ -833,7 +834,7 @@ namespace TimeStepping
      * If the flag is true, the last stage is the same as the first stage and
      * one evaluation of f can be saved.
      */
-    bool last_same_as_first;
+    bool last_same_as_first = false;
 
     /**
      * Butcher tableau coefficients.
@@ -849,7 +850,7 @@ namespace TimeStepping
      * If the last_same_as_first flag is set to true, the last stage is saved
      * and reused as the first stage of the next time step.
      */
-    VectorType *last_stage;
+    VectorType *last_stage = nullptr;
 
     /**
      * Status structure of the object.
