@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2018 - 2021 by the deal.II authors
+// Copyright (C) 2018 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -53,12 +53,11 @@ namespace parallel
    * The connected weighting function may be changed anytime using the
    * CellWeights::reinit() function. The following code snippet demonstrates how
    * to achieve each cell being weighted by its current number of degrees of
-   * freedom. We chose a factor of `1000` that corresponds to the initial weight
-   * each cell is assigned to upon creation.
+   * freedom.
    * @code
    * parallel::CellWeights<dim, spacedim> cell_weights(
    *   hp_dof_handler,
-   *   parallel::CellWeights<dim, spacedim>::ndofs_weighting({1000, 1}));
+   *   parallel::CellWeights<dim, spacedim>::ndofs_weighting({1, 1}));
    * @endcode
    *
    * On the other hand, you are also able to take care of handling the signal
@@ -66,16 +65,15 @@ namespace parallel
    * this case, an analogous code example looks as follows.
    * @code
    * boost::signals2::connection connection =
-   *   hp_dof_handler.get_triangulation().signals.cell_weight.connect(
+   *   hp_dof_handler.get_triangulation().signals.weight.connect(
    *     parallel::CellWeights<dim, spacedim>::make_weighting_callback(
    *       hp_dof_handler,
-   *       parallel::CellWeights<dim, spacedim>::ndofs_weighting(
-   *         {1000, 1}));
+   *       parallel::CellWeights<dim, spacedim>::ndofs_weighting({1, 1}));
    * @endcode
    *
    * The use of this class is demonstrated in step-75.
    *
-   * @note See Triangulation::Signals::cell_weight for more information on
+   * @note See Triangulation::Signals::weight for more information on
    * weighting and load balancing.
    *
    * @note Be aware that this class connects the weight function to the
@@ -85,6 +83,17 @@ namespace parallel
    * the weight_callback() function. Use CellWeights::reinit() to deregister the
    * weighting function on the old Triangulation and connect it to the new one.
    *
+   * @note A hp::FECollection needs to be attached to your DoFHandler object via
+   * DoFHandler::distribute_dofs() <em>before</em> the
+   * Triangulation::Signals::weight signal will be triggered. Otherwise,
+   * your DoFHandler does not know many degrees of freedom your cells have. In
+   * other words, you need to call DoFHandler::distribute_dofs() once before you
+   * call
+   * parallel::distributed::Triangulation::execute_coarsening_and_refinement(),
+   * parallel::distributed::Triangulation::refine_global(),
+   * parallel::distributed::Triangulation::repartition(), or
+   * GridTools::partition_triangulation() for the very first time.
+   *
    * @ingroup distributed
    */
   template <int dim, int spacedim = dim>
@@ -156,7 +165,7 @@ namespace parallel
      * Choose a constant weight @p factor on each cell.
      */
     static WeightingFunction
-    constant_weighting(const unsigned int factor = 1000);
+    constant_weighting(const unsigned int factor = 1);
 
     /**
      * The pair of floating point numbers $(a,b)$ provided via
@@ -186,105 +195,15 @@ namespace parallel
      * @}
      */
 
-    /**
-     * @name Deprecated functions
-     * @{
-     */
-
-    /**
-     * Constructor.
-     *
-     * @param[in] dof_handler The DoFHandler which will be used to
-     *    determine each cell's finite element.
-     */
-    DEAL_II_DEPRECATED
-    CellWeights(const dealii::DoFHandler<dim, spacedim> &dof_handler);
-
-    /**
-     * Choose a constant weight @p factor on each cell.
-     *
-     * @deprecated Use CellWeights::constant_weighting() instead.
-     */
-    DEAL_II_DEPRECATED void
-    register_constant_weighting(const unsigned int factor = 1000);
-
-    /**
-     * Choose a weight for each cell that is proportional to its number of
-     * degrees of freedom with a factor @p factor.
-     *
-     * @deprecated Use CellWeights::ndofs_weighting() instead.
-     */
-    DEAL_II_DEPRECATED void
-    register_ndofs_weighting(const unsigned int factor = 1000);
-
-    /**
-     * Choose a weight for each cell that is proportional to its number of
-     * degrees of freedom <i>squared</i> with a factor @p factor.
-     *
-     * @deprecated Use CellWeights::ndofs_weighting() instead.
-     */
-    DEAL_II_DEPRECATED void
-    register_ndofs_squared_weighting(const unsigned int factor = 1000);
-
-    /**
-     * Register a custom weight for each cell by providing a function as a
-     * parameter.
-     *
-     * @param[in] custom_function The custom weighting function returning
-     *    the weight of each cell as an unsigned integer. It is required
-     *    to have two arguments, namely the FiniteElement that will be
-     *    active on the particular cell, and the cell itself of type
-     *    DoFHandler::cell_iterator. We require both to make sure to
-     *    get the right active FiniteElement on each cell in case that we
-     *    coarsen the Triangulation.
-     */
-    DEAL_II_DEPRECATED void
-    register_custom_weighting(
-      const std::function<
-        unsigned int(const FiniteElement<dim, spacedim> &,
-                     const typename DoFHandler<dim, spacedim>::cell_iterator &)>
-        custom_function);
-
-    /**
-     * @}
-     */
-
   private:
     /**
-     * @name Deprecated members
-     * @{
-     */
-
-    /**
-     * Pointer to the degree of freedom handler.
-     */
-    DEAL_II_DEPRECATED
-    SmartPointer<const dealii::DoFHandler<dim, spacedim>, CellWeights>
-      dof_handler;
-
-    /**
-     * Pointer to the Triangulation associated with the degree of freedom
-     * handler.
-     *
-     * We store both to make sure to always work on the correct combination of
-     * both.
-     */
-    DEAL_II_DEPRECATED
-    SmartPointer<const parallel::TriangulationBase<dim, spacedim>, CellWeights>
-      triangulation;
-
-    /**
-     * @}
-     */
-
-    /**
-     * A connection to the corresponding cell_weight signal of the Triangulation
+     * A connection to the corresponding `weight` signal of the Triangulation
      * which is attached to the DoFHandler.
      */
     boost::signals2::connection connection;
 
     /**
-     * A callback function that will be connected to the cell_weight signal of
+     * A callback function that will be connected to the `weight` signal of
      * the @p triangulation, to which the @p dof_handler is attached. Ultimately
      * returns the weight for each cell, determined by the @p weighting_function
      * provided as a parameter. Returns zero if @p dof_handler has not been
