@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2019 - 2021 by the deal.II authors
+// Copyright (C) 2019 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -19,10 +19,9 @@
 
 #include <deal.II/base/config.h>
 
+#include <deal.II/distributed/repartitioning_policy_tools.h>
 #include <deal.II/distributed/tria_base.h>
 
-#include <deal.II/grid/grid_tools.h>
-
 #include <vector>
 
 #ifdef DEAL_II_WITH_MPI
@@ -196,6 +195,22 @@ namespace parallel
                                  const unsigned int)> &partitioner,
         const TriangulationDescription::Settings &     settings);
 
+      /**
+       * Register a partitioner, which is used within the method
+       * repartition().
+       */
+      void
+      set_partitioner(
+        const RepartitioningPolicyTools::Base<dim, spacedim> &partitioner,
+        const TriangulationDescription::Settings &            settings);
+
+      /**
+       * Execute repartitioning and use the partitioner attached by the
+       * method set_partitioner();
+       */
+      void
+      repartition();
+
       /**
        * Coarsen and refine the mesh according to refinement and coarsening
        * flags set.
@@ -214,14 +229,6 @@ namespace parallel
       virtual bool
       prepare_coarsening_and_refinement() override;
 
-      /**
-       * Return true if the triangulation has hanging nodes.
-       *
-       * @note Not implemented yet.
-       */
-      virtual bool
-      has_hanging_nodes() const override;
-
       /**
        * Return the local memory consumption in bytes.
        */
@@ -252,8 +259,16 @@ namespace parallel
        * in with notify_ready_to_unpack() after calling load().
        */
       virtual void
-      load(const std::string &filename,
-           const bool         autopartition = false) override;
+      load(const std::string &filename) override;
+
+      /**
+       * @copydoc load()
+       *
+       * @deprecated The autopartition parameter has been removed.
+       */
+      DEAL_II_DEPRECATED_EARLY
+      virtual void
+      load(const std::string &filename, const bool autopartition) override;
 
     private:
       virtual unsigned int
@@ -278,6 +293,9 @@ namespace parallel
       virtual void
       update_cell_relations() override;
 
+      virtual void
+      update_number_cache() override;
+
       /**
        * store the Settings.
        */
@@ -290,6 +308,12 @@ namespace parallel
                          const unsigned int)>
         partitioner;
 
+      /**
+       * Partitioner used during repartition().
+       */
+      SmartPointer<const RepartitioningPolicyTools::Base<dim, spacedim>>
+        partitioner_distributed;
+
       /**
        * Sorted list of pairs of coarse-cell ids and their indices.
        */
