@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2020 by the deal.II authors
+// Copyright (C) 2009 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -31,11 +31,6 @@ DEAL_II_NAMESPACE_OPEN
 #ifndef DOXYGEN
 template <int dim, int spacedim>
 class DoFHandler;
-namespace hp
-{
-  template <int dim, int spacedim>
-  class DoFHandler;
-}
 #endif
 
 
@@ -90,11 +85,10 @@ namespace hp
  * levels of <tt>mg_vector</tt> will have the block structure needed on that
  * level.
  *
- * @todo Extend the functions local() and renumber() to the concept to
- * hp::DoFHandler.
+ * @todo Extend the functions local() and renumber() to allow for
+ * hp-capablilites.
  *
  * @ingroup dofs
- * @author Guido Kanschat, 2009
  */
 class BlockInfo : public Subscriptor
 {
@@ -118,17 +112,6 @@ public:
              bool levels_only = false,
              bool active_only = false);
 
-  /**
-   * The same as above but for hp::DoFHandler.
-   *
-   * @note Not implemented.
-   */
-  template <int dim, int spacedim>
-  void
-  initialize(const hp::DoFHandler<dim, spacedim> &,
-             bool levels_only = false,
-             bool active_only = false);
-
   /**
    * @brief Initialize block structure on cells and compute renumbering
    * between cell dofs and block cell dofs.
@@ -137,15 +120,6 @@ public:
   void
   initialize_local(const DoFHandler<dim, spacedim> &);
 
-  /**
-   * The same as above but for hp::DoFHandler.
-   *
-   * @note Not implemented.
-   */
-  template <int dim, int spacedim>
-  void
-  initialize_local(const hp::DoFHandler<dim, spacedim> &);
-
   /**
    * Access the BlockIndices structure of the global system.
    */
@@ -204,7 +178,8 @@ public:
 
   /**
    * Read or write the data of this object to or from a stream for the purpose
-   * of serialization
+   * of serialization using the [BOOST serialization
+   * library](https://www.boost.org/doc/libs/1_74_0/libs/serialization/doc/index.html).
    */
   template <class Archive>
   void
