@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2021 by the deal.II authors
+// Copyright (C) 1998 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -506,12 +506,12 @@ public:
   /**
    * Make the dimension available in function templates.
    */
-  static const unsigned int dimension = dim;
+  static constexpr unsigned int dimension = dim;
 
   /**
    * Make the space dimension available in function templates.
    */
-  static const unsigned int space_dimension = spacedim;
+  static constexpr unsigned int space_dimension = spacedim;
 
   /**
    * The default index of the finite element to be used on a given cell.
@@ -1192,58 +1192,6 @@ public:
   const IndexSet &
   locally_owned_mg_dofs(const unsigned int level) const;
 
-  /**
-   * Return a vector that stores the locally owned DoFs of each processor.
-   *
-   * @deprecated As of deal.II version 9.2, we do not populate a vector with
-   * the index sets of all processors by default any more due to a possibly
-   * large memory footprint on many processors. As a consequence, this
-   * function needs to call `Utilities::MPI::all_gather(comm,
-   * locally_owned_dofs())` upon the first invocation, including global
-   * communication. Use `Utilities::MPI::all_gather(comm,
-   * dof_handler.locally_owned_dofs())` instead if using up to a few thousands
-   * of MPI ranks or some variant involving local communication with more
-   * processors.
-   */
-  DEAL_II_DEPRECATED const std::vector<IndexSet> &
-                           locally_owned_dofs_per_processor() const;
-
-  /**
-   * Return a vector that stores the number of degrees of freedom each
-   * processor that participates in this triangulation owns locally. The sum
-   * of all these numbers equals the number of degrees of freedom that exist
-   * globally, i.e. what n_dofs() returns.
-   *
-   * @deprecated As of deal.II version 9.2, we do not populate a vector with
-   * the numbers of dofs of all processors by default any more due to a
-   * possibly large memory footprint on many processors. As a consequence,
-   * this function needs to call `Utilities::MPI::all_gather(comm,
-   * n_locally_owned_dofs()` upon the first invocation, including global
-   * communication. Use `Utilities::MPI::all_gather(comm,
-   * dof_handler.n_locally_owned_dofs()` instead if using up to a few thousands
-   * of MPI ranks or some variant involving local communication with more
-   * processors.
-   */
-  DEAL_II_DEPRECATED const std::vector<types::global_dof_index> &
-                           n_locally_owned_dofs_per_processor() const;
-
-  /**
-   * Return a vector that stores the locally owned DoFs of each processor on
-   * the given level @p level.
-   *
-   * @deprecated As of deal.II version 9.2, we do not populate a vector with
-   * the index sets of all processors by default any more due to a possibly
-   * large memory footprint on many processors. As a consequence, this
-   * function needs to call `Utilities::MPI::all_gather(comm,
-   * locally_owned_dofs_mg())` upon the first invocation, including global
-   * communication. Use `Utilities::MPI::all_gather(comm,
-   * dof_handler.locally_owned_dofs_mg())` instead if using up to a few
-   * thousands of MPI ranks or some variant involving local communication with
-   * more processors.
-   */
-  DEAL_II_DEPRECATED const std::vector<IndexSet> &
-                           locally_owned_mg_dofs_per_processor(const unsigned int level) const;
-
   /**
    * Return a constant reference to the indexth finite element object that is
    * used by this object.
@@ -1443,20 +1391,10 @@ private:
      * Return the index of the <code>dof_number</code>th degree of freedom for
      * the given level stored for the current vertex.
      */
-    types::global_dof_index
-    get_index(const unsigned int level,
-              const unsigned int dof_number,
-              const unsigned int dofs_per_vertex) const;
-
-    /**
-     * Set the index of the <code>dof_number</code>th degree of freedom for
-     * the given level stored for the current vertex to <code>index</code>.
-     */
-    void
-    set_index(const unsigned int            level,
-              const unsigned int            dof_number,
-              const unsigned int            dofs_per_vertex,
-              const types::global_dof_index index);
+    types::global_dof_index &
+    access_index(const unsigned int level,
+                 const unsigned int dof_number,
+                 const unsigned int dofs_per_vertex);
 
   private:
     /**
@@ -1682,27 +1620,6 @@ private:
   void
   clear_mg_space();
 
-  /**
-   * Return dof index of specified object.
-   */
-  template <int structdim>
-  types::global_dof_index
-  get_dof_index(const unsigned int obj_level,
-                const unsigned int obj_index,
-                const unsigned int fe_index,
-                const unsigned int local_index) const;
-
-  /**
-   * Return dof index of specified object.
-   */
-  template <int structdim>
-  void
-  set_dof_index(const unsigned int            obj_level,
-                const unsigned int            obj_index,
-                const unsigned int            fe_index,
-                const unsigned int            local_index,
-                const types::global_dof_index global_index) const;
-
   /**
    * Set up DoFHandler policy.
    */
@@ -1964,66 +1881,6 @@ DoFHandler<dim, spacedim>::locally_owned_mg_dofs(const unsigned int level) const
 
 
 
-template <int dim, int spacedim>
-const std::vector<types::global_dof_index> &
-DoFHandler<dim, spacedim>::n_locally_owned_dofs_per_processor() const
-{
-  if (number_cache.n_locally_owned_dofs_per_processor.empty() &&
-      number_cache.n_global_dofs > 0)
-    {
-      const_cast<dealii::internal::DoFHandlerImplementation::NumberCache &>(
-        number_cache)
-        .n_locally_owned_dofs_per_processor =
-        number_cache.get_n_locally_owned_dofs_per_processor(get_communicator());
-    }
-  return number_cache.n_locally_owned_dofs_per_processor;
-}
-
-
-
-template <int dim, int spacedim>
-const std::vector<IndexSet> &
-DoFHandler<dim, spacedim>::locally_owned_dofs_per_processor() const
-{
-  if (number_cache.locally_owned_dofs_per_processor.empty() &&
-      number_cache.n_global_dofs > 0)
-    {
-      const_cast<dealii::internal::DoFHandlerImplementation::NumberCache &>(
-        number_cache)
-        .locally_owned_dofs_per_processor =
-        number_cache.get_locally_owned_dofs_per_processor(get_communicator());
-    }
-  return number_cache.locally_owned_dofs_per_processor;
-}
-
-
-
-template <int dim, int spacedim>
-const std::vector<IndexSet> &
-DoFHandler<dim, spacedim>::locally_owned_mg_dofs_per_processor(
-  const unsigned int level) const
-{
-  Assert(level < this->get_triangulation().n_global_levels(),
-         ExcMessage("The given level index exceeds the number of levels "
-                    "present in the triangulation"));
-  Assert(
-    mg_number_cache.size() == this->get_triangulation().n_global_levels(),
-    ExcMessage(
-      "The level dofs are not set up properly! Did you call distribute_mg_dofs()?"));
-  if (mg_number_cache[level].locally_owned_dofs_per_processor.empty() &&
-      mg_number_cache[level].n_global_dofs > 0)
-    {
-      const_cast<dealii::internal::DoFHandlerImplementation::NumberCache &>(
-        mg_number_cache[level])
-        .locally_owned_dofs_per_processor =
-        mg_number_cache[level].get_locally_owned_dofs_per_processor(
-          get_communicator());
-    }
-  return mg_number_cache[level].locally_owned_dofs_per_processor;
-}
-
-
-
 template <int dim, int spacedim>
 inline const FiniteElement<dim, spacedim> &
 DoFHandler<dim, spacedim>::get_fe(const unsigned int number) const
@@ -2130,14 +1987,14 @@ DoFHandler<dim, spacedim>::save(Archive &ar, const unsigned int) const
 {
   if (this->hp_capability_enabled)
     {
-      ar & this->object_dof_indices;
-      ar & this->object_dof_ptr;
+      ar &this->object_dof_indices;
+      ar &this->object_dof_ptr;
 
-      ar & this->cell_dof_cache_indices;
-      ar & this->cell_dof_cache_ptr;
+      ar &this->cell_dof_cache_indices;
+      ar &this->cell_dof_cache_ptr;
 
-      ar & this->hp_cell_active_fe_indices;
-      ar & this->hp_cell_future_fe_indices;
+      ar &this->hp_cell_active_fe_indices;
+      ar &this->hp_cell_future_fe_indices;
 
       ar &hp_object_fe_ptr;
       ar &hp_object_fe_indices;
@@ -2157,14 +2014,14 @@ DoFHandler<dim, spacedim>::save(Archive &ar, const unsigned int) const
     }
   else
     {
-      ar & this->block_info_object;
+      ar &this->block_info_object;
       ar &number_cache;
 
-      ar & this->object_dof_indices;
-      ar & this->object_dof_ptr;
+      ar &this->object_dof_indices;
+      ar &this->object_dof_ptr;
 
-      ar & this->cell_dof_cache_indices;
-      ar & this->cell_dof_cache_ptr;
+      ar &this->cell_dof_cache_indices;
+      ar &this->cell_dof_cache_ptr;
 
       // write out the number of triangulation cells and later check during
       // loading that this number is indeed correct; same with something that
@@ -2186,14 +2043,14 @@ DoFHandler<dim, spacedim>::load(Archive &ar, const unsigned int)
 {
   if (this->hp_capability_enabled)
     {
-      ar & this->object_dof_indices;
-      ar & this->object_dof_ptr;
+      ar &this->object_dof_indices;
+      ar &this->object_dof_ptr;
 
-      ar & this->cell_dof_cache_indices;
-      ar & this->cell_dof_cache_ptr;
+      ar &this->cell_dof_cache_indices;
+      ar &this->cell_dof_cache_ptr;
 
-      ar & this->hp_cell_active_fe_indices;
-      ar & this->hp_cell_future_fe_indices;
+      ar &this->hp_cell_active_fe_indices;
+      ar &this->hp_cell_future_fe_indices;
 
       ar &hp_object_fe_ptr;
       ar &hp_object_fe_indices;
@@ -2224,18 +2081,18 @@ DoFHandler<dim, spacedim>::load(Archive &ar, const unsigned int)
     }
   else
     {
-      ar & this->block_info_object;
+      ar &this->block_info_object;
       ar &number_cache;
 
       object_dof_indices.clear();
 
       object_dof_ptr.clear();
 
-      ar & this->object_dof_indices;
-      ar & this->object_dof_ptr;
+      ar &this->object_dof_indices;
+      ar &this->object_dof_ptr;
 
-      ar & this->cell_dof_cache_indices;
-      ar & this->cell_dof_cache_ptr;
+      ar &this->cell_dof_cache_indices;
+      ar &this->cell_dof_cache_ptr;
 
       // these are the checks that correspond to the last block in the save()
       // function
@@ -2268,11 +2125,11 @@ DoFHandler<dim, spacedim>::load(Archive &ar, const unsigned int)
 
 
 template <int dim, int spacedim>
-inline types::global_dof_index
-DoFHandler<dim, spacedim>::MGVertexDoFs::get_index(
+inline types::global_dof_index &
+DoFHandler<dim, spacedim>::MGVertexDoFs::access_index(
   const unsigned int level,
   const unsigned int dof_number,
-  const unsigned int dofs_per_vertex) const
+  const unsigned int dofs_per_vertex)
 {
   Assert((level >= coarsest_level) && (level <= finest_level),
          ExcInvalidLevel(level));
@@ -2281,21 +2138,6 @@ DoFHandler<dim, spacedim>::MGVertexDoFs::get_index(
 
 
 
-template <int dim, int spacedim>
-inline void
-DoFHandler<dim, spacedim>::MGVertexDoFs::set_index(
-  const unsigned int            level,
-  const unsigned int            dof_number,
-  const unsigned int            dofs_per_vertex,
-  const types::global_dof_index index)
-{
-  Assert((level >= coarsest_level) && (level <= finest_level),
-         ExcInvalidLevel(level));
-  indices[dofs_per_vertex * (level - coarsest_level) + dof_number] = index;
-}
-
-
-
 extern template class DoFHandler<1, 1>;
 extern template class DoFHandler<1, 2>;
 extern template class DoFHandler<1, 3>;
