@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2015 - 2020 by the deal.II authors
+// Copyright (C) 2015 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -85,7 +85,9 @@ public:
    * the 3D version of FE_NedelecSZ has 12 degrees of freedom for `order = 0`
    * and 54 for `degree = 1`. It is important to have enough quadrature points
    * in order to perform the quadrature with sufficient accuracy.
-   * For example [QGauss<dim>(order + 2)](@ref QGauss) can be used for the
+   * For example
+   * [QGauss<dim>(order + 2)](@ref QGauss)
+   * can be used for the
    * quadrature formula, where `order` is the order of FE_NedelecSZ.
    */
   FE_NedelecSZ(const unsigned int order);
@@ -255,6 +257,16 @@ protected:
      */
     mutable std::vector<std::vector<DerivativeForm<1, dim, dim>>> shape_grads;
 
+    /**
+     * Storage for shape function hessians on the reference element. We only
+     * pre-compute cell-based DoFs, as the edge- and face-based DoFs depend on
+     * the cell.
+     *
+     * Due to the cell-dependent DoFs, this variable is declared mutable.
+     */
+    mutable std::vector<std::vector<DerivativeForm<2, dim, dim>>>
+      shape_hessians;
+
     /**
      * Storage for all possible edge parameterization between vertices. These
      * are required in the computation of edge- and face-based DoFs, which are
