@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2015 - 2019 by the deal.II authors
+// Copyright (C) 2015 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -253,8 +253,6 @@ DEAL_II_NAMESPACE_OPEN
  * The original paper for the P1 nonconforming element  by Park and Sheen
  * is accessible at https://doi.org/10.1137/S0036142902404923 ,
  * see @cite park2003p .
- *
- * @author Jaeryun Yim, 2015, 2016.
  */
 class FE_P1NC : public FiniteElement<2, 2>
 {
@@ -294,7 +292,7 @@ private:
    * x + b y + c$ on given cell. For each local shape function, the array
    * consists of three coefficients is in order of a,b and c.
    */
-  static std::array<std::array<double, 3>, 4>
+  static ndarray<double, 4, 3>
   get_linear_shape_coefficients(const Triangulation<2, 2>::cell_iterator &cell);
 
   /**
@@ -313,11 +311,13 @@ private:
     dealii::internal::FEValuesImplementation::FiniteElementRelatedData<2, 2>
       &output_data) const override;
 
+  using FiniteElement<2, 2>::get_face_data;
+
   virtual std::unique_ptr<FiniteElement<2, 2>::InternalDataBase>
   get_face_data(
     const UpdateFlags update_flags,
     const Mapping<2, 2> &,
-    const Quadrature<1> &quadrature,
+    const hp::QCollection<1> &quadrature,
     dealii::internal::FEValuesImplementation::FiniteElementRelatedData<2, 2>
       &output_data) const override;
 
@@ -345,6 +345,8 @@ private:
     internal::FEValuesImplementation::FiniteElementRelatedData<2, 2>
       &output_data) const override;
 
+  using FiniteElement<2, 2>::fill_fe_face_values;
+
   /**
    * Compute the data on the face of the current cell.
    */
@@ -352,7 +354,7 @@ private:
   fill_fe_face_values(
     const Triangulation<2, 2>::cell_iterator &cell,
     const unsigned int                        face_no,
-    const Quadrature<1> &                     quadrature,
+    const hp::QCollection<1> &                quadrature,
     const Mapping<2, 2> &                     mapping,
     const Mapping<2, 2>::InternalDataBase &   mapping_internal,
     const dealii::internal::FEValuesImplementation::MappingRelatedData<2, 2>
