@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2020 by the deal.II authors
+// Copyright (C) 1998 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -325,7 +325,8 @@ operator|=(UpdateFlags &f1, const UpdateFlags f2)
  *
  * @ref UpdateFlags
  */
-inline UpdateFlags operator&(const UpdateFlags f1, const UpdateFlags f2)
+inline UpdateFlags
+operator&(const UpdateFlags f1, const UpdateFlags f2)
 {
   return static_cast<UpdateFlags>(static_cast<unsigned int>(f1) &
                                   static_cast<unsigned int>(f2));
