@@ -513,6 +513,11 @@ public:
      */
     mutable std::vector<double> volume_elements;
 
+    /**
+     * Projected quadrature weights.
+     */
+    mutable std::vector<double> quadrature_weights;
+
     /**
      * Auxiliary vectors for internal use.
      */
@@ -529,7 +534,7 @@ public:
     mutable std::vector<double> local_dof_values;
   };
 
-private:
+protected:
   // documentation can be found in Mapping::get_data()
   virtual std::unique_ptr<typename Mapping<dim, spacedim>::InternalDataBase>
   get_data(const UpdateFlags, const Quadrature<dim> &quadrature) const override;
@@ -583,6 +588,12 @@ private:
    * @}
    */
 
+  /**
+   * Reference cell over which the mapping is defined. This class does not yet
+   * support mixed meshes.
+   */
+  ReferenceCell reference_cell;
+
   /**
    * Specifies whether we access unknowns on the active dofs (with a single
    * Euler vector) or on the level dofs (via a vector of Euler vectors).
