@@ -141,9 +141,10 @@ public:
   preserves_vertex_locations() const override;
 
   // for documentation, see the Mapping base class
-  virtual BoundingBox<spacedim>
-  get_bounding_box(const typename Triangulation<dim, spacedim>::cell_iterator
-                     &cell) const override;
+  virtual std::vector<Point<spacedim>>
+  compute_mapping_support_points(
+    const typename Triangulation<dim, spacedim>::cell_iterator &cell)
+    const override;
 
   /**
    * Exception which is thrown when the mapping is being evaluated at
@@ -182,7 +183,6 @@ protected:
                MappingQEulerian<dim, VectorType, spacedim>>
     euler_dof_handler;
 
-
 private:
   /**
    * Multigrid level at which the mapping is to be used.
@@ -190,89 +190,37 @@ private:
   const unsigned int level;
 
   /**
-   * A class derived from MappingQGeneric that provides the generic mapping
-   * with support points on boundary objects so that the corresponding Q3
-   * mapping ends up being C1.
+   * Special quadrature rule used to define the support points in the
+   * reference configuration.
    */
-  class MappingQEulerianGeneric : public MappingQGeneric<dim, spacedim>
+  class SupportQuadrature : public Quadrature<dim>
   {
   public:
     /**
-     * Constructor.
+     * Constructor, with an argument defining the desired polynomial degree.
      */
-    MappingQEulerianGeneric(
-      const unsigned int                                 degree,
-      const MappingQEulerian<dim, VectorType, spacedim> &mapping_q_eulerian);
-
-    /**
-     * Return the mapped vertices of the cell. For the current class, this
-     * function does not use the support points from the geometry of the
-     * current cell but instead evaluates an externally given displacement
-     * field in addition to the geometry of the cell.
-     */
-    virtual boost::container::small_vector<Point<spacedim>,
-                                           GeometryInfo<dim>::vertices_per_cell>
-    get_vertices(const typename Triangulation<dim, spacedim>::cell_iterator
-                   &cell) const override;
-
-    /**
-     * Compute the positions of the support points in the current
-     * configuration. See the documentation of
-     * MappingQGeneric::compute_mapping_support_points() for more information.
-     */
-    virtual std::vector<Point<spacedim>>
-    compute_mapping_support_points(
-      const typename Triangulation<dim, spacedim>::cell_iterator &cell)
-      const override;
-
-    /**
-     * Always return @p false because MappingQEulerianGeneric does not in general
-     * preserve vertex locations (unless the translation vector happens to
-     * provide for zero displacements at vertex locations).
-     */
-    virtual bool
-    preserves_vertex_locations() const override;
-
-  private:
-    /**
-     * Reference to the surrounding object off of which we live.
-     */
-    const MappingQEulerian<dim, VectorType, spacedim> &mapping_q_eulerian;
-
-
-    /**
-     * Special quadrature rule used to define the support points in the
-     * reference configuration.
-     */
-    class SupportQuadrature : public Quadrature<dim>
-    {
-    public:
-      /**
-       * Constructor, with an argument defining the desired polynomial degree.
-       */
-      SupportQuadrature(const unsigned int map_degree);
-    };
-
-    /**
-     * A member variable holding the quadrature points in the right order.
-     */
-    const SupportQuadrature support_quadrature;
-
-    /**
-     * FEValues object used to query the given finite element field at the
-     * support points in the reference configuration.
-     *
-     * The variable is marked as mutable since we have to call
-     * FEValues::reinit from compute_mapping_support_points, a function that
-     * is 'const'.
-     */
-    mutable FEValues<dim, spacedim> fe_values;
-
-    /**
-     * A variable to guard access to the fe_values variable.
-     */
-    mutable Threads::Mutex fe_values_mutex;
+    SupportQuadrature(const unsigned int map_degree);
   };
+
+  /**
+   * A member variable holding the quadrature points in the right order.
+   */
+  const SupportQuadrature support_quadrature;
+
+  /**
+   * FEValues object used to query the given finite element field at the
+   * support points in the reference configuration.
+   *
+   * The variable is marked as mutable since we have to call
+   * FEValues::reinit from compute_mapping_support_points, a function that
+   * is 'const'.
+   */
+  mutable FEValues<dim, spacedim> fe_values;
+
+  /**
+   * A variable to guard access to the fe_values variable.
+   */
+  mutable Threads::Mutex fe_values_mutex;
 };
 
 /*@}*/
