@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2019 by the deal.II authors
+// Copyright (C) 2016 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -24,7 +24,6 @@
 #include <deal.II/base/derivative_form.h>
 #include <deal.II/base/point.h>
 #include <deal.II/base/quadrature_lib.h>
-#include <deal.II/base/std_cxx14/memory.h>
 #include <deal.II/base/subscriptor.h>
 
 #include <deal.II/grid/manifold.h>
@@ -32,6 +31,8 @@
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/grid/tria_iterator.h>
 
+#include <memory>
+
 DEAL_II_NAMESPACE_OPEN
 
 /**
@@ -63,7 +64,6 @@ DEAL_II_NAMESPACE_OPEN
  * @tparam dim2 The dimension of the second ChartManifold
  *
  * @ingroup manifold
- * @author Luca Heltai, Timo Heister, 2016
  */
 template <int dim,
           int spacedim         = dim,
@@ -161,7 +161,7 @@ std::unique_ptr<Manifold<dim, spacedim>>
 CompositionManifold<dim, spacedim, chartdim, intermediate_dim, dim1, dim2>::
   clone() const
 {
-  return std_cxx14::make_unique<
+  return std::make_unique<
     CompositionManifold<dim, spacedim, chartdim, intermediate_dim, dim1, dim2>>(
     *F, *G);
 }
