@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2021 by the deal.II authors
+// Copyright (C) 1999 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -162,19 +162,18 @@ struct CellData;
  * read.
  *
  * <li> <tt>UNV</tt> format: this format is generated by the Salome mesh
- * generator, see http://www.salome-platform.org/ . The sections of the format
- * that the GridIn::read_unv function supports are documented here:
+ * generator, see http://www.salome-platform.org/ . The sections of this format
+ * that the GridIn::read_unv function supports are (only) 2411, 2412 and 2467.
+ * A detailed description of all the sections of the UNV file can be found in
+ * the following links:
  * <ul>
- * <li> section 2411:
- * http://www.sdrl.uc.edu/sdrl/referenceinfo/universalfileformats/file-format-storehouse/universal-dataset-number-2411
- * <li> section 2412:
- * http://www.sdrl.uc.edu/sdrl/referenceinfo/universalfileformats/file-format-storehouse/universal-dataset-number-2412
- * <li> section 2467:
- * http://www.sdrl.uc.edu/sdrl/referenceinfo/universalfileformats/file-format-storehouse/universal-dataset-number-2467
- * <li> all sections of this format, even if they may not be supported in our
- * reader, can be found here:
- * http://www.sdrl.uc.edu/sdrl/referenceinfo/universalfileformats/file-format-storehouse
+ * <li> https://www.ceas3.uc.edu/sdrluff/
+ * <li> https://docs.plm.automation.siemens.com/tdoc/nx/12/nx_help#uid:xid1128419:index_advanced:xid1404601:xid1404604
  * </ul>
+ *
+ * A Wikipedia page dedicated to Universal File Format is available here:
+ * https://en.wikipedia.org/wiki/Universal_File_Format
+ *
  * Note that Salome, let's say in 2D, can only make a quad mesh on an object
  * that has exactly 4 edges (or 4 pieces of the boundary). That means, that if
  * you have a more complicated object and would like to mesh it with quads,
@@ -185,6 +184,12 @@ struct CellData;
  * associated with some of the internal faces of this compound mesh are
  * removed.
  *
+ * @note It is essential that __all__ the mesh groups in Salome have purely
+ * integer names for this function to work as expected. For reference, the
+ * user guide of [OpenFCST](http://www.openfcst.mece.ualberta.ca/index.html)
+ * (a project that uses deal.II) has a section on creating deal.II-compatible
+ * meshes on Salome.
+ *
  * <li> <tt>VTK</tt> format: VTK Unstructured Grid Legacy file reader
  * generator. The reader can handle only Unstructured Grid format of data at
  * present for 2D & 3D geometries. The documentation for the general legacy
@@ -406,7 +411,7 @@ public:
    * The companion GridOut::write_vtk function can be used to write VTK files
    * compatible with this method.
    *
-   * @ingroup simplex
+   * @relates simplex
    */
   void
   read_vtk(std::istream &in);
@@ -508,15 +513,10 @@ public:
   read_xda(std::istream &in);
 
   /**
-   * Read grid data from an msh file, either version 1 or version 2 of that
-   * file format. The %Gmsh formats are documented at
+   * Read grid data from an msh file. The %Gmsh formats are documented at
    * http://www.gmsh.info/.
    *
-   * @note The input function of deal.II does not distinguish between newline
-   * and other whitespace. Therefore, deal.II will be able to read files in a
-   * slightly more general format than %Gmsh.
-   *
-   * @ingroup simplex
+   * @relates simplex
    */
   void
   read_msh(std::istream &in);
@@ -570,16 +570,55 @@ public:
    *   value, i.e., zero)
    * - some elements have manifold id 1 and material id equal to 2
    *
-   * If the physical groups are not named, then the behaviour is the same as
+   * If the physical groups are not named, then the behavior is the same as
    * the other read_msh() function, i.e., the physical tag itself is interpreted
-   * as a boundary or material id.
+   * as a boundary or material id.  Physical surface numbers created in Gmsh,
+   * which can be seen in the .geo file, become material IDs.
    *
-   * @ingroup simplex
+   * @relates simplex
    */
   void
   read_msh(const std::string &filename);
 #endif
 
+  /**
+   * Read grid data from a `.mphtxt` file. `.mphtxt` is one of the file formats
+   * typically generated by COMSOL. The file format is described at
+   * http://victorsndvg.github.io/FEconv/formats/mphtxt.xhtml .
+   *
+   * The reader interprets the "geometric entity indicators" that COMSOL
+   * writes into these files as either boundary indicators (for edges and faces
+   * of cells) or as material ids (for cells). See the glossary for a
+   * description of
+   * @ref GlossBoundaryIndicator "boundary indicators".
+   * and
+   * @ref GlossMaterialId "material indicators"
+   *
+   * COMSOL has a habit of assigning "geometric entity indicators" not only
+   * to edges and faces on the actual boundary, but also to interior faces
+   * and edges. For example, for the following volume mesh generated by
+   * COMSOL,
+   * @image html "comsol-mesh-boundary-volume-mesh.png"
+   * the marked edges and faces are as follows:
+   * @image html "comsol-mesh-marked-lines.png"
+   * @image html "comsol-mesh-marked-triangles.png"
+   * Here, some of the marked lines and faces with explicitly given
+   * geometric entity indicators are in the *interior* of the domain -- an
+   * artifact of the geometry description that was used to describe
+   * the mesh. However, we can of course not assign boundary indicators to
+   * interior edges and faces. As a consequence, this reader function simply
+   * ignores the geometric entity indicator for edges and faces that
+   * are not in fact on the boundary of the domain. The result is then a mesh
+   * in which only the following edges and faces are explicitly assigned
+   * boundary indicators:
+   * @image html "comsol-mesh-boundary-lines.png"
+   * @image html "comsol-mesh-boundary-triangles.png"
+   *
+   * @relates simplex
+   */
+  void
+  read_comsol_mphtxt(std::istream &in);
+
   /**
    * Read grid data from a file containing tecplot ASCII data. This also works
    * in the absence of any tecplot installation.
@@ -822,7 +861,17 @@ public:
                  << "15 Point (1 node, ignored when read)");
 
 
-  DeclException0(ExcGmshNoCellInformation);
+  DeclException2(
+    ExcGmshNoCellInformation,
+    unsigned int,
+    unsigned int,
+    "While reading a gmsh file, the reader function did not find "
+    "any cells. This sometimes happens if the file only contains a "
+    "surface mesh, but not a volume mesh."
+    "\n\n"
+    "The reader function did find " +
+      std::to_string(arg1) + " lines and " + std::to_string(arg2) +
+      " facets (surface triangles or quadrilaterals).");
 
 protected:
   /**
