@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2017 - 2021 by the deal.II authors
+// Copyright (C) 2017 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -77,7 +77,12 @@ namespace GridTools
     Cache(const Triangulation<dim, spacedim> &tria,
           const Mapping<dim, spacedim> &      mapping =
             (ReferenceCells::get_hypercube<dim>()
-               .template get_default_linear_mapping<dim, spacedim>()));
+#ifndef _MSC_VER
+               .template get_default_linear_mapping<dim, spacedim>()
+#else
+               .ReferenceCell::get_default_linear_mapping<dim, spacedim>()
+#endif
+               ));
 
     /**
      * Destructor.
