@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2020 by the deal.II authors
+// Copyright (C) 2004 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -2510,7 +2510,7 @@ BlockMatrixBase<MatrixType>::print(std::ostream &out,
     for (unsigned int col = 0; col < n_block_cols(); ++col)
       {
         if (!alternative_output)
-          out << "Block (" << row << ", " << col << ")" << std::endl;
+          out << "Block (" << row << ", " << col << ')' << std::endl;
 
         block(row, col).print(out, alternative_output);
       }
