@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2021 by the deal.II authors
+// Copyright (C) 2004 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -42,11 +42,24 @@ DEAL_II_NAMESPACE_OPEN
  *@{
  */
 
-// Forward declaration
-#ifndef DOXYGEN
-template <typename>
-class BlockVectorBase;
-#endif
+namespace internal
+{
+  template <typename T>
+  using has_block_t = decltype(std::declval<T const>().block(0));
+
+  template <typename T>
+  constexpr bool has_block = internal::is_supported_operation<has_block_t, T>;
+
+  template <typename T>
+  using has_n_blocks_t = decltype(std::declval<T const>().n_blocks());
+
+  template <typename T>
+  constexpr bool has_n_blocks =
+    internal::is_supported_operation<has_n_blocks_t, T>;
+
+  template <typename T>
+  constexpr bool is_block_vector = has_block<T> &&has_n_blocks<T>;
+} // namespace internal
 
 /**
  * A class that can be used to determine whether a given type is a block
@@ -65,31 +78,13 @@ class BlockVectorBase;
 template <typename VectorType>
 struct IsBlockVector
 {
-private:
-  /**
-   * Overload returning true if the class is derived from BlockVectorBase,
-   * which is what block vectors do.
-   */
-  template <typename T>
-  static std::true_type
-  check_for_block_vector(const BlockVectorBase<T> *);
-
-  /**
-   * Catch all for all other potential vector types that are not block
-   * matrices.
-   */
-  static std::false_type
-  check_for_block_vector(...);
-
 public:
   /**
    * A statically computable value that indicates whether the template
-   * argument to this class is a block vector (in fact whether the type is
-   * derived from BlockVectorBase<T>).
+   * argument to this class is a block vector (in fact whether the type has
+   * the functions `block()` and `n_blocks()`).
    */
-  static const bool value =
-    std::is_same<decltype(check_for_block_vector(std::declval<VectorType *>())),
-                 std::true_type>::value;
+  static const bool value = internal::is_block_vector<VectorType>;
 };
 
 
@@ -141,9 +136,10 @@ namespace internal
                                   typename BlockVectorType::value_type>::type;
 
       /**
-       * Declare some alias which are standard for iterators and are used
+       * Declare some aliases that are standard for iterators and are used
        * by algorithms to enquire about the specifics of the iterators they
-       * work on.
+       * work on. (Example: `std::next()`, which needs to know about a local
+       * type named `difference_type`.)
        */
       using iterator_category = std::random_access_iterator_tag;
       using difference_type   = std::ptrdiff_t;
@@ -211,13 +207,15 @@ namespace internal
        * is <tt>true</tt>, then no writing to the result is possible, making
        * this a const_iterator.
        */
-      dereference_type operator*() const;
+      dereference_type
+      operator*() const;
 
       /**
        * Random access operator, grant access to arbitrary elements relative
        * to the one presently pointed to.
        */
-      dereference_type operator[](const difference_type d) const;
+      dereference_type
+      operator[](const difference_type d) const;
 
       /**
        * Prefix increment operator. This operator advances the iterator to the
@@ -614,14 +612,16 @@ public:
    *
    * Exactly the same as operator().
    */
-  value_type operator[](const size_type i) const;
+  value_type
+  operator[](const size_type i) const;
 
   /**
    * Access components, returns U(i) as a writeable reference.
    *
    * Exactly the same as operator().
    */
-  reference operator[](const size_type i);
+  reference
+  operator[](const size_type i);
 
   /**
    * Instead of getting individual elements of a vector via operator(),
@@ -721,7 +721,8 @@ public:
   /**
    * $U = U * V$: scalar product.
    */
-  value_type operator*(const BlockVectorBase &V) const;
+  value_type
+  operator*(const BlockVectorBase &V) const;
 
   /**
    * Return the square of the $l_2$-norm.
@@ -1042,7 +1043,7 @@ namespace internal
 
     template <class BlockVectorType, bool Constness>
     inline typename Iterator<BlockVectorType, Constness>::dereference_type
-      Iterator<BlockVectorType, Constness>::operator*() const
+    Iterator<BlockVectorType, Constness>::operator*() const
     {
       return parent->block(current_block)(index_within_block);
     }
@@ -1051,8 +1052,8 @@ namespace internal
 
     template <class BlockVectorType, bool Constness>
     inline typename Iterator<BlockVectorType, Constness>::dereference_type
-      Iterator<BlockVectorType, Constness>::
-      operator[](const difference_type d) const
+    Iterator<BlockVectorType, Constness>::operator[](
+      const difference_type d) const
     {
       // if the index pointed to is
       // still within the block we
@@ -1120,8 +1121,8 @@ namespace internal
     template <class BlockVectorType, bool Constness>
     template <bool OtherConstness>
     inline bool
-    Iterator<BlockVectorType, Constness>::
-    operator==(const Iterator<BlockVectorType, OtherConstness> &i) const
+    Iterator<BlockVectorType, Constness>::operator==(
+      const Iterator<BlockVectorType, OtherConstness> &i) const
     {
       Assert(parent == i.parent, ExcPointerToDifferentVectors());
 
@@ -1133,8 +1134,8 @@ namespace internal
     template <class BlockVectorType, bool Constness>
     template <bool OtherConstness>
     inline bool
-    Iterator<BlockVectorType, Constness>::
-    operator!=(const Iterator<BlockVectorType, OtherConstness> &i) const
+    Iterator<BlockVectorType, Constness>::operator!=(
+      const Iterator<BlockVectorType, OtherConstness> &i) const
     {
       Assert(parent == i.parent, ExcPointerToDifferentVectors());
 
@@ -1146,8 +1147,8 @@ namespace internal
     template <class BlockVectorType, bool Constness>
     template <bool OtherConstness>
     inline bool
-    Iterator<BlockVectorType, Constness>::
-    operator<(const Iterator<BlockVectorType, OtherConstness> &i) const
+    Iterator<BlockVectorType, Constness>::operator<(
+      const Iterator<BlockVectorType, OtherConstness> &i) const
     {
       Assert(parent == i.parent, ExcPointerToDifferentVectors());
 
@@ -1159,8 +1160,8 @@ namespace internal
     template <class BlockVectorType, bool Constness>
     template <bool OtherConstness>
     inline bool
-    Iterator<BlockVectorType, Constness>::
-    operator<=(const Iterator<BlockVectorType, OtherConstness> &i) const
+    Iterator<BlockVectorType, Constness>::operator<=(
+      const Iterator<BlockVectorType, OtherConstness> &i) const
     {
       Assert(parent == i.parent, ExcPointerToDifferentVectors());
 
@@ -1172,8 +1173,8 @@ namespace internal
     template <class BlockVectorType, bool Constness>
     template <bool OtherConstness>
     inline bool
-    Iterator<BlockVectorType, Constness>::
-    operator>(const Iterator<BlockVectorType, OtherConstness> &i) const
+    Iterator<BlockVectorType, Constness>::operator>(
+      const Iterator<BlockVectorType, OtherConstness> &i) const
     {
       Assert(parent == i.parent, ExcPointerToDifferentVectors());
 
@@ -1185,8 +1186,8 @@ namespace internal
     template <class BlockVectorType, bool Constness>
     template <bool OtherConstness>
     inline bool
-    Iterator<BlockVectorType, Constness>::
-    operator>=(const Iterator<BlockVectorType, OtherConstness> &i) const
+    Iterator<BlockVectorType, Constness>::operator>=(
+      const Iterator<BlockVectorType, OtherConstness> &i) const
     {
       Assert(parent == i.parent, ExcPointerToDifferentVectors());
 
@@ -1198,8 +1199,8 @@ namespace internal
     template <class BlockVectorType, bool Constness>
     template <bool OtherConstness>
     inline typename Iterator<BlockVectorType, Constness>::difference_type
-    Iterator<BlockVectorType, Constness>::
-    operator-(const Iterator<BlockVectorType, OtherConstness> &i) const
+    Iterator<BlockVectorType, Constness>::operator-(
+      const Iterator<BlockVectorType, OtherConstness> &i) const
     {
       Assert(parent == i.parent, ExcPointerToDifferentVectors());
 
@@ -1211,8 +1212,8 @@ namespace internal
 
     template <class BlockVectorType, bool Constness>
     inline Iterator<BlockVectorType, Constness>
-    Iterator<BlockVectorType, Constness>::
-    operator+(const difference_type &d) const
+    Iterator<BlockVectorType, Constness>::operator+(
+      const difference_type &d) const
     {
       // if the index pointed to is
       // still within the block we
@@ -1238,8 +1239,8 @@ namespace internal
 
     template <class BlockVectorType, bool Constness>
     inline Iterator<BlockVectorType, Constness>
-    Iterator<BlockVectorType, Constness>::
-    operator-(const difference_type &d) const
+    Iterator<BlockVectorType, Constness>::operator-(
+      const difference_type &d) const
     {
       // if the index pointed to is
       // still within the block we
@@ -1606,8 +1607,9 @@ BlockVectorBase<VectorType>::is_non_negative() const
 
 
 template <class VectorType>
-typename BlockVectorBase<VectorType>::value_type BlockVectorBase<VectorType>::
-                                                 operator*(const BlockVectorBase<VectorType> &v) const
+typename BlockVectorBase<VectorType>::value_type
+BlockVectorBase<VectorType>::operator*(
+  const BlockVectorBase<VectorType> &v) const
 {
   Assert(n_blocks() == v.n_blocks(),
          ExcDimensionMismatch(n_blocks(), v.n_blocks()));
@@ -2040,8 +2042,8 @@ BlockVectorBase<VectorType>::operator=(const VectorType &v)
 template <class VectorType>
 template <class VectorType2>
 inline bool
-BlockVectorBase<VectorType>::
-operator==(const BlockVectorBase<VectorType2> &v) const
+BlockVectorBase<VectorType>::operator==(
+  const BlockVectorBase<VectorType2> &v) const
 {
   Assert(block_indices == v.block_indices, ExcDifferentBlockIndices());
 
@@ -2108,7 +2110,7 @@ BlockVectorBase<VectorType>::operator()(const size_type i)
 
 template <class VectorType>
 inline typename BlockVectorBase<VectorType>::value_type
-  BlockVectorBase<VectorType>::operator[](const size_type i) const
+BlockVectorBase<VectorType>::operator[](const size_type i) const
 {
   return operator()(i);
 }
@@ -2117,7 +2119,7 @@ inline typename BlockVectorBase<VectorType>::value_type
 
 template <class VectorType>
 inline typename BlockVectorBase<VectorType>::reference
-  BlockVectorBase<VectorType>::operator[](const size_type i)
+BlockVectorBase<VectorType>::operator[](const size_type i)
 {
   return operator()(i);
 }
