@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2020 by the deal.II authors
+// Copyright (C) 2000 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -224,7 +224,7 @@ EigenPower<VectorType>::solve(double &value, const MatrixType &A, VectorType &x)
 
   // Main loop
   int iter = 0;
-  for (; conv == SolverControl::iterate; iter++)
+  for (; conv == SolverControl::iterate; ++iter)
     {
       y.add(additional_data.shift, x);
 
@@ -321,9 +321,9 @@ EigenInverse<VectorType>::solve(double &          value,
   x *= 1. / length;
 
   // Main loop
-  double    res  = -std::numeric_limits<double>::max();
+  double    res  = std::numeric_limits<double>::lowest();
   size_type iter = 0;
-  for (; conv == SolverControl::iterate; iter++)
+  for (; conv == SolverControl::iterate; ++iter)
     {
       solver.solve(A_s, y, x, prec);
 
