@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2019 by the deal.II authors
+// Copyright (C) 2004 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -51,7 +51,7 @@ namespace PETScWrappers
 #    ifndef DOXYGEN
   class MatrixBase;
   class VectorBase;
-  class PreconditionerBase;
+  class PreconditionBase;
 #    endif
 
 
@@ -108,7 +108,6 @@ namespace PETScWrappers
    * parallel mode.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, 2004
    */
   class SolverBase
   {
@@ -137,10 +136,10 @@ namespace PETScWrappers
      * performance reasons. See class Documentation.
      */
     void
-    solve(const MatrixBase &        A,
-          VectorBase &              x,
-          const VectorBase &        b,
-          const PreconditionerBase &preconditioner);
+    solve(const MatrixBase &      A,
+          VectorBase &            x,
+          const VectorBase &      b,
+          const PreconditionBase &preconditioner);
 
 
     /**
@@ -170,7 +169,7 @@ namespace PETScWrappers
      * intended for use with SLEPc spectral transformation class.
      */
     void
-    initialize(const PreconditionerBase &preconditioner);
+    initialize(const PreconditionBase &preconditioner);
 
   protected:
     /**
@@ -187,7 +186,7 @@ namespace PETScWrappers
     const MPI_Comm mpi_communicator;
 
     /**
-     * Function that takes a Krylov Subspace Solver context object, and sets
+     * %Function that takes a Krylov Subspace Solver context object, and sets
      * the type of solver that is requested by the derived class.
      */
     virtual void
@@ -265,7 +264,6 @@ namespace PETScWrappers
    * solver.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, 2004
    */
   class SolverRichardson : public SolverBase
   {
@@ -313,7 +311,7 @@ namespace PETScWrappers
     const AdditionalData additional_data;
 
     /**
-     * Function that takes a Krylov Subspace Solver context object, and sets
+     * %Function that takes a Krylov Subspace Solver context object, and sets
      * the type of solver that is appropriate for this class.
      */
     virtual void
@@ -327,7 +325,6 @@ namespace PETScWrappers
    * prior version 3.3, Chebychev) solver.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, 2004
    */
   class SolverChebychev : public SolverBase
   {
@@ -365,7 +362,7 @@ namespace PETScWrappers
     const AdditionalData additional_data;
 
     /**
-     * Function that takes a Krylov Subspace Solver context object, and sets
+     * %Function that takes a Krylov Subspace Solver context object, and sets
      * the type of solver that is appropriate for this class.
      */
     virtual void
@@ -378,7 +375,6 @@ namespace PETScWrappers
    * An implementation of the solver interface using the PETSc CG solver.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, 2004
    */
   class SolverCG : public SolverBase
   {
@@ -416,7 +412,7 @@ namespace PETScWrappers
     const AdditionalData additional_data;
 
     /**
-     * Function that takes a Krylov Subspace Solver context object, and sets
+     * %Function that takes a Krylov Subspace Solver context object, and sets
      * the type of solver that is appropriate for this class.
      */
     virtual void
@@ -429,7 +425,6 @@ namespace PETScWrappers
    * An implementation of the solver interface using the PETSc BiCG solver.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, 2004
    */
   class SolverBiCG : public SolverBase
   {
@@ -467,7 +462,7 @@ namespace PETScWrappers
     const AdditionalData additional_data;
 
     /**
-     * Function that takes a Krylov Subspace Solver context object, and sets
+     * %Function that takes a Krylov Subspace Solver context object, and sets
      * the type of solver that is appropriate for this class.
      */
     virtual void
@@ -480,7 +475,6 @@ namespace PETScWrappers
    * An implementation of the solver interface using the PETSc GMRES solver.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, 2004
    */
   class SolverGMRES : public SolverBase
   {
@@ -535,7 +529,7 @@ namespace PETScWrappers
     const AdditionalData additional_data;
 
     /**
-     * Function that takes a Krylov Subspace Solver context object, and sets
+     * %Function that takes a Krylov Subspace Solver context object, and sets
      * the type of solver that is appropriate for this class.
      */
     virtual void
@@ -549,7 +543,6 @@ namespace PETScWrappers
    * solver.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, 2004
    */
   class SolverBicgstab : public SolverBase
   {
@@ -587,7 +580,7 @@ namespace PETScWrappers
     const AdditionalData additional_data;
 
     /**
-     * Function that takes a Krylov Subspace Solver context object, and sets
+     * %Function that takes a Krylov Subspace Solver context object, and sets
      * the type of solver that is appropriate for this class.
      */
     virtual void
@@ -599,7 +592,6 @@ namespace PETScWrappers
    * solver.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, 2004
    */
   class SolverCGS : public SolverBase
   {
@@ -637,7 +629,7 @@ namespace PETScWrappers
     const AdditionalData additional_data;
 
     /**
-     * Function that takes a Krylov Subspace Solver context object, and sets
+     * %Function that takes a Krylov Subspace Solver context object, and sets
      * the type of solver that is appropriate for this class.
      */
     virtual void
@@ -650,7 +642,6 @@ namespace PETScWrappers
    * An implementation of the solver interface using the PETSc TFQMR solver.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, 2004
    */
   class SolverTFQMR : public SolverBase
   {
@@ -688,7 +679,7 @@ namespace PETScWrappers
     const AdditionalData additional_data;
 
     /**
-     * Function that takes a Krylov Subspace Solver context object, and sets
+     * %Function that takes a Krylov Subspace Solver context object, and sets
      * the type of solver that is appropriate for this class.
      */
     virtual void
@@ -706,7 +697,6 @@ namespace PETScWrappers
    * prior. This should be fixed in later versions of PETSc, though.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, 2004
    */
   class SolverTCQMR : public SolverBase
   {
@@ -744,7 +734,7 @@ namespace PETScWrappers
     const AdditionalData additional_data;
 
     /**
-     * Function that takes a Krylov Subspace Solver context object, and sets
+     * %Function that takes a Krylov Subspace Solver context object, and sets
      * the type of solver that is appropriate for this class.
      */
     virtual void
@@ -757,7 +747,6 @@ namespace PETScWrappers
    * An implementation of the solver interface using the PETSc CR solver.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, 2004
    */
   class SolverCR : public SolverBase
   {
@@ -795,7 +784,7 @@ namespace PETScWrappers
     const AdditionalData additional_data;
 
     /**
-     * Function that takes a Krylov Subspace Solver context object, and sets
+     * %Function that takes a Krylov Subspace Solver context object, and sets
      * the type of solver that is appropriate for this class.
      */
     virtual void
@@ -809,7 +798,6 @@ namespace PETScWrappers
    * solver.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, 2004
    */
   class SolverLSQR : public SolverBase
   {
@@ -847,7 +835,7 @@ namespace PETScWrappers
     const AdditionalData additional_data;
 
     /**
-     * Function that takes a Krylov Subspace Solver context object, and sets
+     * %Function that takes a Krylov Subspace Solver context object, and sets
      * the type of solver that is appropriate for this class.
      */
     virtual void
@@ -865,7 +853,6 @@ namespace PETScWrappers
    * conjunction with this solver class becomes a direct solver.
    *
    * @ingroup PETScWrappers
-   * @author Wolfgang Bangerth, 2004, Oliver Kayser-Herold, 2004
    */
   class SolverPreOnly : public SolverBase
   {
@@ -903,7 +890,7 @@ namespace PETScWrappers
     const AdditionalData additional_data;
 
     /**
-     * Function that takes a Krylov Subspace Solver context object, and sets
+     * %Function that takes a Krylov Subspace Solver context object, and sets
      * the type of solver that is appropriate for this class.
      */
     virtual void
@@ -932,7 +919,6 @@ namespace PETScWrappers
    * http://www.mcs.anl.gov/petsc/petsc-current/docs/manualpages/Mat/MATSOLVERMUMPS.html
    *
    * @ingroup PETScWrappers
-   * @author Daniel Brauss, Alexander Grayver, 2012
    */
   class SparseDirectMUMPS : public SolverBase
   {
