@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2020 by the deal.II authors
+// Copyright (C) 1999 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -197,7 +197,7 @@ SolverRichardson<VectorType>::solve(const MatrixType &        A,
 {
   SolverControl::State conv = SolverControl::iterate;
 
-  double last_criterion = -std::numeric_limits<double>::max();
+  double last_criterion = std::numeric_limits<double>::lowest();
 
   unsigned int iter = 0;
 
@@ -253,7 +253,7 @@ SolverRichardson<VectorType>::Tsolve(const MatrixType &        A,
                                      const PreconditionerType &preconditioner)
 {
   SolverControl::State conv           = SolverControl::iterate;
-  double               last_criterion = -std::numeric_limits<double>::max();
+  double               last_criterion = std::numeric_limits<double>::lowest();
 
   unsigned int iter = 0;
 
