@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2002 - 2020 by the deal.II authors
+// Copyright (C) 2002 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -147,7 +147,7 @@ public:
   DeclException2(ExcDecompositionNotStable,
                  int,
                  double,
-                 << "The diagonal element (" << arg1 << "," << arg1 << ") is "
+                 << "The diagonal element (" << arg1 << ',' << arg1 << ") is "
                  << arg2 << ", but must be positive");
 
   //@}
