@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2019 by the deal.II authors
+// Copyright (C) 1999 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -132,9 +132,6 @@ class SparseBlockVanka;
  * section on
  * @ref Instantiations
  * in the manual).
- *
- * @author Guido Kanschat, Wolfgang Bangerth; 1999, 2000; extension for full
- * compatibility with LinearOperator class: Jean-Paul Pelteret, 2015
  */
 template <typename number>
 class SparseVanka
@@ -153,6 +150,18 @@ public:
    */
   SparseVanka();
 
+  /**
+   * Constructor which also takes two deprecated inputs.
+   *
+   * @deprecated The use of the last two parameters is deprecated. They are
+   * currently ignored.
+   */
+  DEAL_II_DEPRECATED
+  SparseVanka(const SparseMatrix<number> &M,
+              const std::vector<bool> &   selected,
+              const bool                  conserve_memory,
+              const unsigned int n_threads = MultithreadInfo::n_threads());
+
   /**
    * Constructor. Gets the matrix for preconditioning and a bit vector with
    * entries @p true for all rows to be updated. A reference to this vector
@@ -164,24 +173,8 @@ public:
    * conceivable that the preconditioner is build up for one matrix once, but
    * is used for subsequent steps in a nonlinear process as well, where the
    * matrix changes in each step slightly.
-   *
-   * If @p conserve_mem is @p false, then the inverses of the local systems
-   * are computed now; if the flag is @p true, then they are computed every
-   * time the preconditioner is applied. This saves some memory, but makes
-   * preconditioning very slow. Note also, that if the flag is @p false, then
-   * the contents of the matrix @p M at the time of calling this constructor
-   * are used, while if the flag is @p true, then the values in @p M at the
-   * time of preconditioning are used. This may lead to different results,
-   * obviously, of @p M changes.
-   *
-   * The parameter @p n_threads determines how many threads shall be used in
-   * parallel when building the inverses of the diagonal blocks. This
-   * parameter is ignored if not in multithreaded mode.
    */
-  SparseVanka(const SparseMatrix<number> &M,
-              const std::vector<bool> &   selected,
-              const bool                  conserve_memory = false,
-              const unsigned int n_threads = MultithreadInfo::n_threads());
+  SparseVanka(const SparseMatrix<number> &M, const std::vector<bool> &selected);
 
   /**
    * Destructor. Delete all allocated matrices.
@@ -197,25 +190,23 @@ public:
     /**
      * Constructor. For the parameters' description, see below.
      */
+    explicit AdditionalData(const std::vector<bool> &selected);
+
+    /**
+     * Constructor. For the parameters' description, see below.
+     *
+     * @deprecated The use of this constructor is deprecated - the second and
+     * third parameters are ignored.
+     */
+    DEAL_II_DEPRECATED
     AdditionalData(const std::vector<bool> &selected,
-                   const bool               conserve_memory = false,
+                   const bool               conserve_memory,
                    const unsigned int n_threads = MultithreadInfo::n_threads());
 
     /**
      * Indices of those degrees of freedom that we shall work on.
      */
     const std::vector<bool> &selected;
-
-    /**
-     * Conserve memory flag.
-     */
-    const bool conserve_mem;
-
-    /**
-     * Number of threads to be used when building the inverses. Only relevant
-     * in multithreaded mode.
-     */
-    const unsigned int n_threads;
   };
 
 
@@ -310,22 +301,11 @@ private:
    */
   SmartPointer<const SparseMatrix<number>, SparseVanka<number>> matrix;
 
-  /**
-   * Conserve memory flag.
-   */
-  bool conserve_mem;
-
   /**
    * Indices of those degrees of freedom that we shall work on.
    */
   const std::vector<bool> *selected;
 
-  /**
-   * Number of threads to be used when building the inverses. Only relevant in
-   * multithreaded mode.
-   */
-  unsigned int n_threads;
-
   /**
    * Array of inverse matrices, one for each degree of freedom. Only those
    * elements will be used that are tagged in @p selected.
@@ -514,8 +494,6 @@ private:
  * iterations (at most by one in each step) and they also do not change when
  * the degrees of freedom are sorted by component, while the first strategy
  * significantly deteriorated.
- *
- * @author Wolfgang Bangerth, 2000
  */
 template <typename number>
 class SparseBlockVanka : public SparseVanka<number>
@@ -544,14 +522,26 @@ public:
 
   /**
    * Constructor. Pass all arguments except for @p n_blocks to the base class.
+   *
+   * @deprecated This constructor is deprecated. The values passed to the last
+   * two arguments are ignored.
    */
+  DEAL_II_DEPRECATED
   SparseBlockVanka(const SparseMatrix<number> &M,
                    const std::vector<bool> &   selected,
                    const unsigned int          n_blocks,
                    const BlockingStrategy      blocking_strategy,
-                   const bool                  conserve_memory = false,
+                   const bool                  conserve_memory,
                    const unsigned int n_threads = MultithreadInfo::n_threads());
 
+  /**
+   * Constructor. Pass all arguments except for @p n_blocks to the base class.
+   */
+  SparseBlockVanka(const SparseMatrix<number> &M,
+                   const std::vector<bool> &   selected,
+                   const unsigned int          n_blocks,
+                   const BlockingStrategy      blocking_strategy);
+
   /**
    * Apply the preconditioner.
    */
