@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2021 by the deal.II authors
+// Copyright (C) 2000 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -306,15 +306,16 @@ namespace SparsityPatternIterators
 
 
 /**
- * A class that can store which elements of a matrix are nonzero (or, in fact,
- * <i>may</i> be nonzero) and for which we have to allocate memory to store
- * their values. This class is an example of the "static" type of sparsity
- * patters (see
+ * A class that can store which elements of a matrix are nonzero (or, to be
+ * precise, <i>may</i> be nonzero) and for which we have to allocate memory to
+ * store their values. This class is an example of the "static" type of sparsity
+ * patterns (see
  * @ref Sparsity).
  * It uses the <a
  * href="https://en.wikipedia.org/wiki/Sparse_matrix">compressed row storage
  * (CSR)</a> format to store data, and is used as the basis for the
- * derived SparsityPattern class and SparseMatrix class.
+ * derived SparsityPattern class, which is in turn used by the SparseMatrix
+ * class.
  *
  * The elements of a SparsityPatternBase, corresponding to the places where
  * SparseMatrix objects can store nonzero entries, are stored row-by-row.
@@ -365,8 +366,10 @@ public:
    *
    * Constructors, destructor, functions initializing, copying and filling an
    * object.
+   *
+   * @{
    */
-  // @{
+
   /**
    * Initialize the matrix empty, that is with no memory allocated. This is
    * useful if you want such objects as member variables in other classes. You
@@ -434,12 +437,15 @@ public:
   void
   add(const size_type i, const size_type j);
 
-  // @}
+  /**
+   * @}
+   */
 
   /**
    * @name Iterators
+   *
+   * @{
    */
-  // @{
 
   /**
    * Iterator starting at the first entry of the matrix. The resulting
@@ -484,12 +490,15 @@ public:
   end(const size_type r) const;
 
 
-  // @}
+  /**
+   * @}
+   */
 
   /**
    * @name Querying information
+   *
+   * @{
    */
-  // @{
 
   /**
    * Test for equality of two SparsityPatterns.
@@ -574,12 +583,15 @@ public:
   std::size_t
   memory_consumption() const;
 
-  // @}
+  /**
+   * @}
+   */
 
   /**
    * @name Accessing entries
+   *
+   * @{
    */
-  // @{
 
   /**
    * Access to column number field.  Return the column number of the
@@ -618,12 +630,15 @@ public:
   std::pair<size_type, size_type>
   matrix_position(const std::size_t global_index) const;
 
-  // @}
+  /**
+   * @}
+   */
 
   /**
    * @name Input/Output
+   *
+   * @{
    */
-  // @{
 
   /**
    * Print the sparsity of the matrix. The output consists of one line per row
@@ -692,10 +707,13 @@ public:
   BOOST_SERIALIZATION_SPLIT_MEMBER()
 #endif
 
-  // @}
+  /**
+   * @}
+   */
 
   /**
    * @addtogroup Exceptions
+   *
    * @{
    */
 
@@ -728,7 +746,9 @@ public:
     "The operation you attempted changes the structure of the SparsityPattern "
     "and is not possible after compress() has been called.");
 
-  // @}
+  /**
+   * @}
+   */
 
 
 protected:
@@ -901,7 +921,10 @@ public:
    * Constructors, destructor, functions initializing, copying and filling an
    * object.
    */
-  // @{
+  /**
+   * @{
+   */
+
   /**
    * Initialize the matrix empty, that is with no memory allocated. This is
    * useful if you want such objects as member variables in other classes. You
@@ -1163,13 +1186,18 @@ public:
               ForwardIterator end,
               const bool      indices_are_sorted = false);
 
-  // @}
+  /**
+   * @}
+   */
 
 
   /**
    * @name Querying information
    */
-  // @{
+  /**
+   * @{
+   */
+
   /**
    * Test for equality of two SparsityPatterns.
    */
@@ -1198,11 +1226,16 @@ public:
   std::size_t
   memory_consumption() const;
 
-  // @}
+  /**
+   * @}
+   */
+
   /**
    * @name Accessing entries
+   *
+   * @{
    */
-  // @{
+
   /**
    * Return the index of the matrix element with row number <tt>i</tt> and
    * column number <tt>j</tt>. If the matrix element is not a nonzero one,
@@ -1228,11 +1261,16 @@ public:
   size_type
   operator()(const size_type i, const size_type j) const;
 
-  // @}
+  /**
+   * @}
+   */
+
   /**
    * @name Input/Output
    */
-  // @{
+  /**
+   * @{
+   */
 
   /**
    * Write the data of this object en bloc to a file. This is done in a binary
@@ -1293,10 +1331,13 @@ public:
   BOOST_SERIALIZATION_SPLIT_MEMBER()
 #endif
 
-  // @}
+  /**
+   * @}
+   */
 
   /**
    * @addtogroup Exceptions
+   *
    * @{
    */
   /**
@@ -1314,7 +1355,9 @@ public:
                  int,
                  << "The number of partitions you gave is " << arg1
                  << ", but must be greater than zero.");
-  //@}
+  /**
+   * @}
+   */
 private:
   /**
    * Is special treatment of diagonals enabled?
@@ -1341,7 +1384,9 @@ private:
 };
 
 
-/*@}*/
+/**
+ * @}
+ */
 /*---------------------- Inline functions -----------------------------------*/
 
 #ifndef DOXYGEN
@@ -1588,8 +1633,15 @@ SparsityPatternBase::save(Archive &ar, const unsigned int) const
 
   ar &max_dim &rows &cols &max_vec_len &max_row_length &compressed;
 
-  ar &boost::serialization::make_array(rowstart.get(), max_dim + 1);
-  ar &boost::serialization::make_array(colnums.get(), max_vec_len);
+  if (max_dim != 0)
+    ar &boost::serialization::make_array(rowstart.get(), max_dim + 1);
+  else
+    Assert(rowstart.get() == nullptr, ExcInternalError());
+
+  if (max_vec_len != 0)
+    ar &boost::serialization::make_array(colnums.get(), max_vec_len);
+  else
+    Assert(colnums.get() == nullptr, ExcInternalError());
 }
 
 
@@ -1603,11 +1655,21 @@ SparsityPatternBase::load(Archive &ar, const unsigned int)
 
   ar &max_dim &rows &cols &max_vec_len &max_row_length &compressed;
 
-  rowstart = std::make_unique<std::size_t[]>(max_dim + 1);
-  colnums  = std::make_unique<size_type[]>(max_vec_len);
+  if (max_dim != 0)
+    {
+      rowstart = std::make_unique<std::size_t[]>(max_dim + 1);
+      ar &boost::serialization::make_array(rowstart.get(), max_dim + 1);
+    }
+  else
+    rowstart.reset();
 
-  ar &boost::serialization::make_array(rowstart.get(), max_dim + 1);
-  ar &boost::serialization::make_array(colnums.get(), max_vec_len);
+  if (max_vec_len != 0)
+    {
+      colnums = std::make_unique<size_type[]>(max_vec_len);
+      ar &boost::serialization::make_array(colnums.get(), max_vec_len);
+    }
+  else
+    colnums.reset();
 }
 
 
@@ -1644,13 +1706,15 @@ SparsityPatternBase::operator==(const SparsityPatternBase &sp2) const
   if (rows != sp2.rows || cols != sp2.cols || compressed != sp2.compressed)
     return false;
 
-  for (size_type i = 0; i < rows + 1; ++i)
-    if (rowstart[i] != sp2.rowstart[i])
-      return false;
+  if (rows > 0)
+    for (size_type i = 0; i < rows + 1; ++i)
+      if (rowstart[i] != sp2.rowstart[i])
+        return false;
 
-  for (size_type i = 0; i < rowstart[rows]; ++i)
-    if (colnums[i] != sp2.colnums[i])
-      return false;
+  if (rows > 0)
+    for (size_type i = 0; i < rowstart[rows]; ++i)
+      if (colnums[i] != sp2.colnums[i])
+        return false;
 
   return true;
 }
