@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2021 by the deal.II authors
+// Copyright (C) 2008 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -28,16 +28,12 @@
 
 #    include <Epetra_FECrsGraph.h>
 #    include <Epetra_Map.h>
+#    include <Epetra_MpiComm.h>
+#    include <mpi.h>
 
 #    include <cmath>
 #    include <memory>
 #    include <vector>
-#    ifdef DEAL_II_WITH_MPI
-#      include <Epetra_MpiComm.h>
-#      include <mpi.h>
-#    else
-#      include <Epetra_SerialComm.h>
-#    endif
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -207,12 +203,14 @@ namespace TrilinosWrappers
       /**
        * Dereferencing operator.
        */
-      const Accessor &operator*() const;
+      const Accessor &
+      operator*() const;
 
       /**
        * Dereferencing operator.
        */
-      const Accessor *operator->() const;
+      const Accessor *
+      operator->() const;
 
       /**
        * Comparison. True, if both iterators point to the same matrix
@@ -714,7 +712,7 @@ namespace TrilinosWrappers
     /**
      * Return the number of nonzero elements of this sparsity pattern.
      */
-    size_type
+    std::uint64_t
     n_nonzero_elements() const;
 
     /**
@@ -967,10 +965,10 @@ namespace TrilinosWrappers
                    size_type,
                    size_type,
                    size_type,
-                   << "You tried to access element (" << arg1 << "/" << arg2
-                   << ")"
+                   << "You tried to access element (" << arg1 << '/' << arg2
+                   << ')'
                    << " of a distributed matrix, but only rows in range ["
-                   << arg3 << "," << arg4
+                   << arg3 << ',' << arg4
                    << "] are stored locally and can be accessed.");
 
     /**
@@ -979,9 +977,8 @@ namespace TrilinosWrappers
     DeclException2(ExcAccessToNonPresentElement,
                    size_type,
                    size_type,
-                   << "You tried to access element (" << arg1 << "/" << arg2
-                   << ")"
-                   << " of a sparse matrix, but it appears to not"
+                   << "You tried to access element (" << arg1 << '/' << arg2
+                   << ')' << " of a sparse matrix, but it appears to not"
                    << " exist in the Trilinos sparsity pattern.");
     //@}
   private:
@@ -1118,14 +1115,16 @@ namespace TrilinosWrappers
 
 
 
-    inline const Accessor &Iterator::operator*() const
+    inline const Accessor &
+    Iterator::operator*() const
     {
       return accessor;
     }
 
 
 
-    inline const Accessor *Iterator::operator->() const
+    inline const Accessor *
+    Iterator::operator->() const
     {
       return &accessor;
     }
