@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2021 by the deal.II authors
+// Copyright (C) 1999 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -34,7 +34,6 @@
 #include <algorithm>
 #include <initializer_list>
 #include <iosfwd>
-#include <iterator>
 #include <vector>
 
 DEAL_II_NAMESPACE_OPEN
@@ -474,7 +473,8 @@ public:
    * repeatable results from one run to another.
    */
   template <typename Number2>
-  Number operator*(const Vector<Number2> &V) const;
+  Number
+  operator*(const Vector<Number2> &V) const;
 
   /**
    * Return the square of the $l_2$-norm.
@@ -625,14 +625,16 @@ public:
    *
    * Exactly the same as operator().
    */
-  Number operator[](const size_type i) const;
+  Number
+  operator[](const size_type i) const;
 
   /**
    * Access the @p ith component as a writeable reference.
    *
    * Exactly the same as operator().
    */
-  Number &operator[](const size_type i);
+  Number &
+  operator[](const size_type i);
 
   /**
    * Instead of getting individual elements of a vector via operator(),
@@ -1192,7 +1194,8 @@ Vector<Number>::operator()(const size_type i)
 
 
 template <typename Number>
-inline Number Vector<Number>::operator[](const size_type i) const
+inline Number
+Vector<Number>::operator[](const size_type i) const
 {
   return operator()(i);
 }
@@ -1200,7 +1203,8 @@ inline Number Vector<Number>::operator[](const size_type i) const
 
 
 template <typename Number>
-inline Number &Vector<Number>::operator[](const size_type i)
+inline Number &
+Vector<Number>::operator[](const size_type i)
 {
   return operator()(i);
 }
@@ -1396,13 +1400,13 @@ inline std::ostream &
 operator<<(std::ostream &out, const Vector<number> &v)
 {
   Assert(v.size() != 0, ExcEmptyObject());
-  AssertThrow(out, ExcIO());
+  AssertThrow(out.fail() == false, ExcIO());
 
   for (typename Vector<number>::size_type i = 0; i < v.size() - 1; ++i)
     out << v(i) << ' ';
   out << v(v.size() - 1);
 
-  AssertThrow(out, ExcIO());
+  AssertThrow(out.fail() == false, ExcIO());
 
   return out;
 }
