@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2021 by the deal.II authors
+// Copyright (C) 2011 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -24,10 +24,12 @@
 #include <deal.II/base/parallel.h>
 #include <deal.II/base/thread_management.h>
 
+#include <deal.II/lac/affine_constraints.h>
 #include <deal.II/lac/dynamic_sparsity_pattern.h>
 #include <deal.II/lac/sparsity_pattern.h>
 
 #include <deal.II/matrix_free/dof_info.h>
+#include <deal.II/matrix_free/hanging_nodes_internal.h>
 #include <deal.II/matrix_free/mapping_info.h>
 #include <deal.II/matrix_free/task_info.h>
 
@@ -73,7 +75,6 @@ namespace internal
               next_constraint.first[j] = constraint_entries[j].second;
             }
         }
-      next_constraint.second = constraints.size();
 
       // check whether or not constraint is already in pool. the initial
       // implementation computed a hash value based on the truncated array (to
@@ -82,13 +83,16 @@ namespace internal
       // equal. this was quite lengthy and now we use a std::map with a
       // user-defined comparator to compare floating point arrays to a
       // tolerance 1e-13.
-      const auto it = constraints.insert(next_constraint);
-
       types::global_dof_index insert_position = numbers::invalid_dof_index;
-      if (it.second == false)
-        insert_position = it.first->second;
+      const auto position = constraints.find(next_constraint.first);
+      if (position != constraints.end())
+        insert_position = position->second;
       else
-        insert_position = next_constraint.second;
+        {
+          next_constraint.second = constraints.size();
+          constraints.insert(next_constraint);
+          insert_position = next_constraint.second;
+        }
 
       // we want to store the result as a short variable, so we have to make
       // sure that the result does not exceed the limits when casting.
@@ -104,8 +108,9 @@ namespace internal
     template <typename number>
     void
     DoFInfo::read_dof_indices(
+      const std::vector<types::global_dof_index> &local_indices_resolved,
       const std::vector<types::global_dof_index> &local_indices,
-      const std::vector<unsigned int> &           lexicographic_inv,
+      const bool                                  cell_has_hanging_nodes,
       const dealii::AffineConstraints<number> &   constraints,
       const unsigned int                          cell_number,
       ConstraintValues<double> &                  constraint_values,
@@ -137,9 +142,8 @@ namespace internal
                i < component_dof_indices_offset[fe_index][comp + 1];
                i++)
             {
-              types::global_dof_index current_dof =
-                local_indices[lexicographic_inv[i]];
-              const auto *entries_ptr =
+              types::global_dof_index current_dof = local_indices_resolved[i];
+              const auto *            entries_ptr =
                 constraints.get_constraint_entries(current_dof);
 
               // dof is constrained
@@ -247,14 +251,14 @@ namespace internal
               (row_starts.size() - 1) / n_components + 1);
           row_starts_plain_indices[cell_number] = plain_dof_indices.size();
           const bool cell_has_constraints =
+            cell_has_hanging_nodes ||
             (row_starts[(cell_number + 1) * n_components].second >
              row_starts[cell_number * n_components].second);
           if (cell_has_constraints == true)
             {
               for (unsigned int i = 0; i < dofs_this_cell; ++i)
                 {
-                  types::global_dof_index current_dof =
-                    local_indices[lexicographic_inv[i]];
+                  types::global_dof_index current_dof = local_indices[i];
                   if (n_mpi_procs > 1 &&
                       (current_dof < first_owned || current_dof >= last_owned))
                     {
@@ -273,6 +277,41 @@ namespace internal
 
 
 
+    template <int dim>
+    bool
+    DoFInfo::process_hanging_node_constraints(
+      const HangingNodes<dim> &                     hanging_nodes,
+      const std::vector<std::vector<unsigned int>> &lexicographic_mapping,
+      const unsigned int                            cell_number,
+      const TriaIterator<DoFCellAccessor<dim, dim, false>> &cell,
+      std::vector<types::global_dof_index> &                dof_indices)
+    {
+      if (this->hanging_node_constraint_masks_comp.size() == 0)
+        return false;
+
+      // 2) determine the refinement configuration of the cell
+      const auto refinement_configuration =
+        hanging_nodes.compute_refinement_configuration(cell);
+
+      if (refinement_configuration == ConstraintKinds::unconstrained)
+        return false;
+
+      // 3) update DoF indices of cell for specified components
+      hanging_nodes.update_dof_indices(cell,
+                                       {},
+                                       lexicographic_mapping,
+                                       hanging_node_constraint_masks_comp,
+                                       refinement_configuration,
+                                       dof_indices);
+
+      hanging_node_constraint_masks[cell_number] =
+        compress(refinement_configuration, dim);
+
+      return true;
+    }
+
+
+
     template <int length>
     void
     DoFInfo::compute_face_index_compression(
@@ -687,7 +726,7 @@ namespace internal
               for (unsigned int j = 0; j < con_it->first; ++j, ++index)
                 {
                   Assert(glob_indices + index != end_row, ExcInternalError());
-                  out << glob_indices[index] << " ";
+                  out << glob_indices[index] << ' ';
                 }
 
               out << "[ ";
@@ -696,14 +735,14 @@ namespace internal
                    k++, index++)
                 {
                   Assert(glob_indices + index != end_row, ExcInternalError());
-                  out << glob_indices[index] << "/" << constraint_pool_data[k]
-                      << " ";
+                  out << glob_indices[index] << '/' << constraint_pool_data[k]
+                      << ' ';
                 }
               out << "] ";
             }
           glob_indices += index;
           for (; glob_indices != end_row; ++glob_indices)
-            out << *glob_indices << " ";
+            out << *glob_indices << ' ';
           out << std::endl;
         }
     }
