@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2020 - 2021 by the deal.II authors
+// Copyright (C) 2020 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -26,12 +26,15 @@
 #include <deal.II/base/vectorization.h>
 
 #include <deal.II/fe/fe_values.h>
-#include <deal.II/fe/mapping_q_generic.h>
+#include <deal.II/fe/mapping_cartesian.h>
+#include <deal.II/fe/mapping_q.h>
 
 #include <deal.II/matrix_free/evaluation_flags.h>
 #include <deal.II/matrix_free/shape_info.h>
 #include <deal.II/matrix_free/tensor_product_kernels.h>
 
+#include <deal.II/non_matching/mapping_info.h>
+
 DEAL_II_NAMESPACE_OPEN
 
 namespace internal
@@ -78,7 +81,8 @@ namespace internal
             result[i][d] = value[d][i][vector_lane];
       }
 
-      static void get_gradient(
+      static void
+      get_gradient(
         Tensor<1, dim, Tensor<1, n_components, VectorizedArray<Number>>> &value,
         const unsigned int   vector_lane,
         const gradient_type &result)
@@ -98,17 +102,18 @@ namespace internal
       }
 
       static void
-        get_value(Tensor<1, n_components, VectorizedArray<Number>> &value,
-                  const unsigned int                                vector_lane,
-                  const value_type &                                result)
+      get_value(Tensor<1, n_components, VectorizedArray<Number>> &value,
+                const unsigned int                                vector_lane,
+                const value_type &                                result)
       {
         for (unsigned int i = 0; i < n_components; ++i)
           value[i][vector_lane] = result[i];
       }
 
       template <typename Number2>
-      static Number2 &access(Tensor<1, n_components, Number2> &value,
-                             const unsigned int                component)
+      static Number2 &
+      access(Tensor<1, n_components, Number2> &value,
+             const unsigned int                component)
       {
         return value[component];
       }
@@ -153,9 +158,10 @@ namespace internal
           result[d] = value[d][vector_lane];
       }
 
-      static void get_gradient(Tensor<1, dim, VectorizedArray<Number>> &value,
-                               const unsigned int   vector_lane,
-                               const gradient_type &result)
+      static void
+      get_gradient(Tensor<1, dim, VectorizedArray<Number>> &value,
+                   const unsigned int                       vector_lane,
+                   const gradient_type &                    result)
       {
         for (unsigned int d = 0; d < dim; ++d)
           value[d][vector_lane] = result[d];
@@ -225,7 +231,8 @@ namespace internal
             result[i][d] = value[d][i][vector_lane];
       }
 
-      static void get_gradient(
+      static void
+      get_gradient(
         Tensor<1, dim, Tensor<1, dim, VectorizedArray<Number>>> &value,
         const unsigned int                                       vector_lane,
         const gradient_type &                                    result)
@@ -244,9 +251,10 @@ namespace internal
           result[i] = value[i][vector_lane];
       }
 
-      static void get_value(Tensor<1, dim, VectorizedArray<Number>> &value,
-                            const unsigned int vector_lane,
-                            const value_type & result)
+      static void
+      get_value(Tensor<1, dim, VectorizedArray<Number>> &value,
+                const unsigned int                       vector_lane,
+                const value_type &                       result)
       {
         for (unsigned int i = 0; i < dim; ++i)
           value[i][vector_lane] = result[i];
@@ -264,13 +272,13 @@ namespace internal
         return value[component];
       }
 
-      static Tensor<1, dim> &
+      static Tensor<1, dim, Number> &
       access(gradient_type &value, const unsigned int component)
       {
         return value[component];
       }
 
-      static const Tensor<1, dim> &
+      static const Tensor<1, dim, Number> &
       access(const gradient_type &value, const unsigned int component)
       {
         return value[component];
@@ -307,9 +315,10 @@ namespace internal
         result[0] = value[0][vector_lane];
       }
 
-      static void get_gradient(Tensor<1, 1, VectorizedArray<Number>> &value,
-                               const unsigned int   vector_lane,
-                               const gradient_type &result)
+      static void
+      get_gradient(Tensor<1, 1, VectorizedArray<Number>> &value,
+                   const unsigned int                     vector_lane,
+                   const gradient_type &                  result)
       {
         value[0][vector_lane] = result[0];
       }
@@ -350,6 +359,10 @@ namespace internal
     is_fast_path_supported(const FiniteElement<dim, spacedim> &fe,
                            const unsigned int base_element_number);
 
+    template <int dim, int spacedim>
+    bool
+    is_fast_path_supported(const Mapping<dim, spacedim> &mapping);
+
     template <int dim, int spacedim>
     std::vector<Polynomials::Polynomial<double>>
     get_polynomial_space(const FiniteElement<dim, spacedim> &fe);
@@ -381,8 +394,10 @@ namespace internal
  * realizations, however, there is a much more efficient implementation that
  * avoids the memory allocation and other expensive start-up cost of
  * FEValues. Currently, the functionality is specialized for mappings derived
- * from MappingQGeneric and for finite elements with tensor product structure
- * that work with the @ref matrixfree module. In those cases, the cost implied
+ * from MappingQ and MappingCartesian and for finite elements with tensor
+ * product structure that work with the
+ * @ref matrixfree
+ * module. In those cases, the cost implied
  * by this class is similar (or sometimes even somewhat lower) than using
  * `FEValues::reinit(cell)` followed by `FEValues::get_function_gradients`.
  */
@@ -421,9 +436,29 @@ public:
                     const UpdateFlags         update_flags,
                     const unsigned int        first_selected_component = 0);
 
+  /**
+   * Constructor to make the present class able to re-use the geometry
+   * data also used by other `FEPointEvaluation` objects.
+   *
+   * @param mapping_info The MappingInfo class describes the geometry-related
+   * data for evaluating finite-element solutions. This object enables to
+   * construct such an object on the outside, possibly re-using it between
+   * several objects or between several calls to the same cell and unit points.
+   *
+   * @param fe The FiniteElement object that is used for the evaluation, which
+   * is typically the same on all cells to be evaluated.
+   *
+   * @param first_selected_component For multi-component FiniteElement
+   * objects, this parameter allows to select a range of `n_components`
+   * components starting from this parameter.
+   */
+  FEPointEvaluation(NonMatching::MappingInfo<dim, spacedim> &mapping_info,
+                    const FiniteElement<dim> &               fe,
+                    const unsigned int first_selected_component = 0);
+
   /**
    * Set up the mapping information for the given cell, e.g., by computing the
-   * Jacobian of the mapping the given points if gradients of the functions
+   * Jacobian of the mapping for the given points if gradients of the functions
    * are requested.
    *
    * @param[in] cell An iterator to the current cell
@@ -566,17 +601,22 @@ public:
   unit_point(const unsigned int point_index) const;
 
 private:
+  /**
+   * Common setup function for both constructors. Does the setup for both fast
+   * and slow path.
+   *
+   * @param first_selected_component For multi-component FiniteElement
+   * objects, this parameter allows to select a range of `n_components`
+   * components starting from this parameter.
+   */
+  void
+  setup(const unsigned int first_selected_component);
+
   /**
    * Pointer to the Mapping object passed to the constructor.
    */
   SmartPointer<const Mapping<dim, spacedim>> mapping;
 
-  /**
-   * Pointer to MappingQGeneric class that enables the fast path of this
-   * class.
-   */
-  const MappingQGeneric<dim, spacedim> *mapping_q_generic;
-
   /**
    * Pointer to the FiniteElement object passed to the constructor.
    */
@@ -640,6 +680,11 @@ private:
    */
   unsigned int dofs_per_component;
 
+  /**
+   * The first selected component in the active base element.
+   */
+  unsigned int component_in_base_element;
+
   /**
    * For complicated FiniteElement objects this variable informs us about
    * which unknowns actually carry degrees of freedom in the selected
@@ -650,12 +695,7 @@ private:
   /**
    * The desired update flags for the evaluation.
    */
-  UpdateFlags update_flags;
-
-  /**
-   * The update flags specific for the mapping in the fast evaluation path.
-   */
-  UpdateFlags update_flags_mapping;
+  const UpdateFlags update_flags;
 
   /**
    * The FEValues object underlying the slow evaluation path.
@@ -663,16 +703,26 @@ private:
   std::shared_ptr<FEValues<dim, spacedim>> fe_values;
 
   /**
-   * Array to store temporary data computed by the mapping for the fast
-   * evaluation path.
+   * Pointer to mapping info on the fly computed during reinit.
    */
-  internal::FEValuesImplementation::MappingRelatedData<dim, spacedim>
-    mapping_data;
+  std::unique_ptr<NonMatching::MappingInfo<dim, spacedim>>
+    mapping_info_on_the_fly;
+
+  /**
+   * Pointer to currently used mapping info (either on the fly or external
+   * precomputed).
+   */
+  SmartPointer<NonMatching::MappingInfo<dim, spacedim>> mapping_info;
 
   /**
    * The reference points specified at reinit().
    */
   std::vector<Point<dim>> unit_points;
+
+  /**
+   * Bool indicating if fast path is chosen.
+   */
+  bool fast_path;
 };
 
 // ----------------------- template and inline function ----------------------
@@ -685,71 +735,99 @@ FEPointEvaluation<n_components, dim, spacedim, Number>::FEPointEvaluation(
   const UpdateFlags         update_flags,
   const unsigned int        first_selected_component)
   : mapping(&mapping)
-  , mapping_q_generic(
-      dynamic_cast<const MappingQGeneric<dim, spacedim> *>(&mapping))
   , fe(&fe)
   , update_flags(update_flags)
-  , update_flags_mapping(update_default)
+  , mapping_info_on_the_fly(
+      std::make_unique<NonMatching::MappingInfo<dim, spacedim>>(mapping,
+                                                                update_flags))
+  , mapping_info(mapping_info_on_the_fly.get())
 {
+  setup(first_selected_component);
+}
+
+
+
+template <int n_components, int dim, int spacedim, typename Number>
+FEPointEvaluation<n_components, dim, spacedim, Number>::FEPointEvaluation(
+  NonMatching::MappingInfo<dim, spacedim> &mapping_info,
+  const FiniteElement<dim> &               fe,
+  const unsigned int                       first_selected_component)
+  : mapping(&mapping_info.get_mapping())
+  , fe(&fe)
+  , update_flags(mapping_info.get_update_flags())
+  , mapping_info(&mapping_info)
+{
+  setup(first_selected_component);
+}
+
+
+
+template <int n_components, int dim, int spacedim, typename Number>
+void
+FEPointEvaluation<n_components, dim, spacedim, Number>::setup(
+  const unsigned int first_selected_component)
+{
+  AssertIndexRange(first_selected_component + n_components,
+                   fe->n_components() + 1);
+
   bool         same_base_element   = true;
   unsigned int base_element_number = 0;
+  component_in_base_element        = 0;
   unsigned int component           = 0;
-  for (; base_element_number < fe.n_base_elements(); ++base_element_number)
-    if (component + fe.element_multiplicity(base_element_number) >
+  for (; base_element_number < fe->n_base_elements(); ++base_element_number)
+    if (component + fe->element_multiplicity(base_element_number) >
         first_selected_component)
       {
         if (first_selected_component + n_components >
-            component + fe.element_multiplicity(base_element_number))
+            component + fe->element_multiplicity(base_element_number))
           same_base_element = false;
+        component_in_base_element = first_selected_component - component;
         break;
       }
     else
-      component += fe.element_multiplicity(base_element_number);
-  if (mapping_q_generic != nullptr &&
+      component += fe->element_multiplicity(base_element_number);
+
+  if (internal::FEPointEvaluation::is_fast_path_supported(*mapping) &&
       internal::FEPointEvaluation::is_fast_path_supported(
-        fe, base_element_number) &&
+        *fe, base_element_number) &&
       same_base_element)
     {
       internal::MatrixFreeFunctions::ShapeInfo<double> shape_info;
 
-      shape_info.reinit(QMidpoint<1>(), fe, base_element_number);
+      shape_info.reinit(QMidpoint<1>(), *fe, base_element_number);
       renumber           = shape_info.lexicographic_numbering;
       dofs_per_component = shape_info.dofs_per_component_on_cell;
       poly               = internal::FEPointEvaluation::get_polynomial_space(
-        fe.base_element(base_element_number));
+        fe->base_element(base_element_number));
 
       polynomials_are_hat_functions =
         (poly.size() == 2 && poly[0].value(0.) == 1. &&
          poly[0].value(1.) == 0. && poly[1].value(0.) == 0. &&
          poly[1].value(1.) == 1.);
+
+      fast_path = true;
     }
   else
     {
-      nonzero_shape_function_component.resize(fe.n_dofs_per_cell());
+      nonzero_shape_function_component.resize(fe->n_dofs_per_cell());
       for (unsigned int d = 0; d < n_components; ++d)
         {
           const unsigned int component = first_selected_component + d;
-          for (unsigned int i = 0; i < fe.n_dofs_per_cell(); ++i)
+          for (unsigned int i = 0; i < fe->n_dofs_per_cell(); ++i)
             {
-              const bool is_primitive = fe.is_primitive() || fe.is_primitive(i);
+              const bool is_primitive =
+                fe->is_primitive() || fe->is_primitive(i);
               if (is_primitive)
                 nonzero_shape_function_component[i][d] =
-                  (component == fe.system_to_component_index(i).first);
+                  (component == fe->system_to_component_index(i).first);
               else
                 nonzero_shape_function_component[i][d] =
-                  (fe.get_nonzero_components(i)[component] == true);
+                  (fe->get_nonzero_components(i)[component] == true);
             }
         }
-    }
 
-  // translate update flags
-  if (update_flags & update_jacobians)
-    update_flags_mapping |= update_jacobians;
-  if (update_flags & update_gradients ||
-      update_flags & update_inverse_jacobians)
-    update_flags_mapping |= update_inverse_jacobians;
-  if (update_flags & update_quadrature_points)
-    update_flags_mapping |= update_quadrature_points;
+      fast_path = false;
+    }
 }
 
 
@@ -760,33 +838,25 @@ FEPointEvaluation<n_components, dim, spacedim, Number>::reinit(
   const typename Triangulation<dim, spacedim>::cell_iterator &cell,
   const ArrayView<const Point<dim>> &                         unit_points)
 {
-  this->unit_points.resize(unit_points.size());
-  std::copy(unit_points.begin(), unit_points.end(), this->unit_points.begin());
+  // reinit is only allowed for mapping computation on the fly
+  AssertThrow(mapping_info_on_the_fly.get() != nullptr, ExcNotImplemented());
 
-  if (!poly.empty())
-    mapping_q_generic->fill_mapping_data_for_generic_points(
-      cell, unit_points, update_flags_mapping, mapping_data);
-  else
+  mapping_info->reinit(cell, unit_points);
+
+  if (!fast_path)
     {
       fe_values = std::make_shared<FEValues<dim, spacedim>>(
         *mapping,
         *fe,
         Quadrature<dim>(
           std::vector<Point<dim>>(unit_points.begin(), unit_points.end())),
-        update_flags | update_flags_mapping);
+        update_flags);
       fe_values->reinit(cell);
-      mapping_data.initialize(unit_points.size(), update_flags_mapping);
-      if (update_flags_mapping & update_jacobians)
-        for (unsigned int q = 0; q < unit_points.size(); ++q)
-          mapping_data.jacobians[q] = fe_values->jacobian(q);
-      if (update_flags_mapping & update_inverse_jacobians)
-        for (unsigned int q = 0; q < unit_points.size(); ++q)
-          mapping_data.inverse_jacobians[q] = fe_values->inverse_jacobian(q);
-      if (update_flags_mapping & update_quadrature_points)
-        for (unsigned int q = 0; q < unit_points.size(); ++q)
-          mapping_data.quadrature_points[q] = fe_values->quadrature_point(q);
     }
 
+  this->unit_points =
+    std::vector<Point<dim>>(unit_points.begin(), unit_points.end());
+
   if (update_flags & update_values)
     values.resize(unit_points.size(), numbers::signaling_nan<value_type>());
   if (update_flags & update_gradients)
@@ -802,13 +872,25 @@ FEPointEvaluation<n_components, dim, spacedim, Number>::evaluate(
   const ArrayView<const Number> &         solution_values,
   const EvaluationFlags::EvaluationFlags &evaluation_flag)
 {
+  const bool precomputed_mapping = mapping_info_on_the_fly.get() == nullptr;
+  if (precomputed_mapping)
+    {
+      unit_points = mapping_info->get_unit_points();
+
+      if (update_flags & update_values)
+        values.resize(unit_points.size(), numbers::signaling_nan<value_type>());
+      if (update_flags & update_gradients)
+        gradients.resize(unit_points.size(),
+                         numbers::signaling_nan<gradient_type>());
+    }
+
   if (unit_points.empty())
     return;
 
   AssertDimension(solution_values.size(), fe->dofs_per_cell);
   if (((evaluation_flag & EvaluationFlags::values) ||
        (evaluation_flag & EvaluationFlags::gradients)) &&
-      !poly.empty())
+      fast_path)
     {
       // fast path with tensor product evaluation
       if (solution_renumbered.size() != dofs_per_component)
@@ -817,7 +899,9 @@ FEPointEvaluation<n_components, dim, spacedim, Number>::evaluate(
         for (unsigned int i = 0; i < dofs_per_component; ++i)
           internal::FEPointEvaluation::
             EvaluatorTypeTraits<dim, n_components, Number>::read_value(
-              solution_values[renumber[comp * dofs_per_component + i]],
+              solution_values[renumber[(component_in_base_element + comp) *
+                                         dofs_per_component +
+                                       i]],
               comp,
               solution_renumbered[i]);
 
@@ -857,17 +941,21 @@ FEPointEvaluation<n_components, dim, spacedim, Number>::evaluate(
                      ExcNotInitialized());
               for (unsigned int j = 0; j < n_lanes && i + j < n_points; ++j)
                 {
-                  Assert(update_flags_mapping & update_inverse_jacobians,
-                         ExcNotInitialized());
                   internal::FEPointEvaluation::EvaluatorTypeTraits<
                     dim,
                     n_components,
                     Number>::set_gradient(val_and_grad.second,
                                           j,
                                           unit_gradients[i + j]);
-                  gradients[i + j] = apply_transformation(
-                    mapping_data.inverse_jacobians[i + j].transpose(),
-                    unit_gradients[i + j]);
+                  gradients[i + j] =
+                    static_cast<typename internal::FEPointEvaluation::
+                                  EvaluatorTypeTraits<dim,
+                                                      n_components,
+                                                      double>::gradient_type>(
+                      apply_transformation(mapping_info->get_mapping_data()
+                                             .inverse_jacobians[i + j]
+                                             .transpose(),
+                                           unit_gradients[i + j]));
                 }
             }
         }
@@ -936,6 +1024,18 @@ FEPointEvaluation<n_components, dim, spacedim, Number>::integrate(
   const ArrayView<Number> &               solution_values,
   const EvaluationFlags::EvaluationFlags &integration_flags)
 {
+  const bool precomputed_mapping = mapping_info_on_the_fly.get() == nullptr;
+  if (precomputed_mapping)
+    {
+      unit_points = mapping_info->get_unit_points();
+
+      if (update_flags & update_values)
+        values.resize(unit_points.size(), numbers::signaling_nan<value_type>());
+      if (update_flags & update_gradients)
+        gradients.resize(unit_points.size(),
+                         numbers::signaling_nan<gradient_type>());
+    }
+
   if (unit_points.size() == 0) // no evaluation points provided
     {
       std::fill(solution_values.begin(), solution_values.end(), 0.0);
@@ -945,7 +1045,7 @@ FEPointEvaluation<n_components, dim, spacedim, Number>::integrate(
   AssertDimension(solution_values.size(), fe->dofs_per_cell);
   if (((integration_flags & EvaluationFlags::values) ||
        (integration_flags & EvaluationFlags::gradients)) &&
-      !poly.empty())
+      fast_path)
     {
       // fast path with tensor product integration
 
@@ -991,11 +1091,12 @@ FEPointEvaluation<n_components, dim, spacedim, Number>::integrate(
           if (integration_flags & EvaluationFlags::gradients)
             for (unsigned int j = 0; j < n_lanes && i + j < n_points; ++j)
               {
-                Assert(update_flags_mapping & update_inverse_jacobians,
-                       ExcNotInitialized());
                 gradients[i + j] =
-                  apply_transformation(mapping_data.inverse_jacobians[i + j],
-                                       gradients[i + j]);
+                  static_cast<typename internal::FEPointEvaluation::
+                                EvaluatorTypeTraits<dim, n_components, double>::
+                                  gradient_type>(apply_transformation(
+                    mapping_info->get_mapping_data().inverse_jacobians[i + j],
+                    gradients[i + j]));
                 internal::FEPointEvaluation::
                   EvaluatorTypeTraits<dim, n_components, Number>::get_gradient(
                     gradient, j, gradients[i + j]);
@@ -1118,9 +1219,9 @@ inline const typename FEPointEvaluation<n_components, dim, spacedim, Number>::
   FEPointEvaluation<n_components, dim, spacedim, Number>::get_unit_gradient(
     const unsigned int point_index) const
 {
-  Assert(!poly.empty(),
+  Assert(fast_path,
          ExcMessage("Unit gradients are currently only implemented for tensor "
-                    "product finite elements combined with MappingQGeneric "
+                    "product finite elements combined with MappingQ "
                     "mappings"));
   AssertIndexRange(point_index, unit_gradients.size());
   return unit_gradients[point_index];
@@ -1157,9 +1258,9 @@ inline DerivativeForm<1, dim, spacedim>
 FEPointEvaluation<n_components, dim, spacedim, Number>::jacobian(
   const unsigned int point_index) const
 {
-  Assert(update_flags_mapping & update_jacobians, ExcNotInitialized());
-  AssertIndexRange(point_index, mapping_data.jacobians.size());
-  return mapping_data.jacobians[point_index];
+  AssertIndexRange(point_index,
+                   mapping_info->get_mapping_data().jacobians.size());
+  return mapping_info->get_mapping_data().jacobians[point_index];
 }
 
 
@@ -1169,11 +1270,9 @@ inline DerivativeForm<1, spacedim, dim>
 FEPointEvaluation<n_components, dim, spacedim, Number>::inverse_jacobian(
   const unsigned int point_index) const
 {
-  Assert(update_flags_mapping & update_inverse_jacobians ||
-           update_flags_mapping & update_gradients,
-         ExcNotInitialized());
-  AssertIndexRange(point_index, mapping_data.inverse_jacobians.size());
-  return mapping_data.inverse_jacobians[point_index];
+  AssertIndexRange(point_index,
+                   mapping_info->get_mapping_data().inverse_jacobians.size());
+  return mapping_info->get_mapping_data().inverse_jacobians[point_index];
 }
 
 
@@ -1183,9 +1282,9 @@ inline Point<spacedim>
 FEPointEvaluation<n_components, dim, spacedim, Number>::real_point(
   const unsigned int point_index) const
 {
-  Assert(update_flags_mapping & update_quadrature_points, ExcNotInitialized());
-  AssertIndexRange(point_index, mapping_data.quadrature_points.size());
-  return mapping_data.quadrature_points[point_index];
+  AssertIndexRange(point_index,
+                   mapping_info->get_mapping_data().quadrature_points.size());
+  return mapping_info->get_mapping_data().quadrature_points[point_index];
 }
 
 
