@@ -28,12 +28,15 @@
 #include <deal.II/base/utilities.h>
 #include <deal.II/base/vectorization.h>
 
-#include <deal.II/lac/dynamic_sparsity_pattern.h>
-
 
 DEAL_II_NAMESPACE_OPEN
 
 
+// forward declaration
+#ifndef DOXYGEN
+class DynamicSparsityPattern;
+#endif
+
 
 namespace internal
 {
@@ -614,6 +617,11 @@ namespace internal
        */
       MPI_Comm communicator_sm;
 
+      /**
+       * Assert that vectors passed to the MatrixFree loops are not ghosted.
+       */
+      bool allow_ghosted_vectors_in_loops;
+
       /**
        * Rank of MPI process
        */
