@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2020 - 2021 by the deal.II authors
+// Copyright (C) 2020 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -56,11 +56,12 @@ namespace MatrixFreeTools
             int n_q_points_1d,
             int n_components,
             typename Number,
-            typename VectorizedArrayType>
+            typename VectorizedArrayType,
+            typename VectorType>
   void
   compute_diagonal(
     const MatrixFree<dim, Number, VectorizedArrayType> &matrix_free,
-    LinearAlgebra::distributed::Vector<Number> &        diagonal_global,
+    VectorType &                                        diagonal_global,
     const std::function<void(FEEvaluation<dim,
                                           fe_degree,
                                           n_q_points_1d,
@@ -80,11 +81,12 @@ namespace MatrixFreeTools
             int n_q_points_1d,
             int n_components,
             typename Number,
-            typename VectorizedArrayType>
+            typename VectorizedArrayType,
+            typename VectorType>
   void
   compute_diagonal(
     const MatrixFree<dim, Number, VectorizedArrayType> &matrix_free,
-    LinearAlgebra::distributed::Vector<Number> &        diagonal_global,
+    VectorType &                                        diagonal_global,
     void (CLASS::*cell_operation)(FEEvaluation<dim,
                                                fe_degree,
                                                n_q_points_1d,
@@ -192,7 +194,7 @@ namespace MatrixFreeTools
 
     const auto to_category = [&](const auto &cell) {
       unsigned int c_num = 0;
-      for (unsigned int i = 0; i < GeometryInfo<dim>::faces_per_cell; i++)
+      for (unsigned int i = 0; i < GeometryInfo<dim>::faces_per_cell; ++i)
         {
           auto &face = *cell->face(i);
           if (face.at_boundary() && !cell->has_periodic_neighbor(i))
@@ -273,13 +275,18 @@ namespace MatrixFreeTools
       {
         this->phi.reinit(cell);
         // STEP 1: get relevant information from FEEvaluation
-        const unsigned int first_selected_component =
-          phi.get_first_selected_component();
         const auto &       dof_info        = phi.get_dof_info();
         const unsigned int n_fe_components = dof_info.start_components.back();
         const unsigned int dofs_per_component = phi.dofs_per_component;
         const auto &       matrix_free        = phi.get_matrix_free();
 
+        // if we have a block vector with components with the same DoFHandler,
+        // each component is described with same set of constraints and
+        // we consider the shift in components only during access of the global
+        // vector
+        const unsigned int first_selected_component =
+          n_fe_components == 1 ? 0 : phi.get_first_selected_component();
+
         const unsigned int n_lanes_filled =
           matrix_free.n_active_entries_per_cell_batch(cell);
 
@@ -322,9 +329,6 @@ namespace MatrixFreeTools
 
             if (n_components == 1 || n_fe_components == 1)
               {
-                AssertDimension(n_components,
-                                1); // TODO: currently no block vector supported
-
                 unsigned int ind_local = 0;
                 for (; index_indicators != next_index_indicators;
                      ++index_indicators, ++ind_local)
@@ -420,17 +424,16 @@ namespace MatrixFreeTools
                       }
                   }
               }
+            // STEP 2b: sort and make unique
 
-            // STEP 2b: transpose COO
-
-            // presort vector for transposed access
+            // sort vector
             std::sort(locally_relevant_constrains.begin(),
                       locally_relevant_constrains.end(),
                       [](const auto &a, const auto &b) {
-                        if (std::get<1>(a) < std::get<1>(b))
+                        if (std::get<0>(a) < std::get<0>(b))
                           return true;
-                        return (std::get<1>(a) == std::get<1>(b)) &&
-                               (std::get<0>(a) < std::get<0>(b));
+                        return (std::get<0>(a) == std::get<0>(b)) &&
+                               (std::get<1>(a) < std::get<1>(b));
                       });
 
             // make sure that all entries are unique
@@ -443,7 +446,195 @@ namespace MatrixFreeTools
                      }),
               locally_relevant_constrains.end());
 
-            // STEP 2c: translate COO to CRS
+            // STEP 2c: apply hanging-node constraints
+            if (dof_info.hanging_node_constraint_masks.size() > 0 &&
+                dof_info.hanging_node_constraint_masks_comp.size() > 0 &&
+                dof_info
+                  .hanging_node_constraint_masks_comp[phi.get_active_fe_index()]
+                                                     [first_selected_component])
+              {
+                const auto mask =
+                  dof_info.hanging_node_constraint_masks[cell * n_lanes + v];
+
+                // cell has hanging nodes
+                if (mask != dealii::internal::MatrixFreeFunctions::
+                              unconstrained_compressed_constraint_kind)
+                  {
+                    // check if hanging node internpolation matrix has been set
+                    // up
+                    if (locally_relevant_constrains_hn_map.find(mask) ==
+                        locally_relevant_constrains_hn_map.end())
+                      {
+                        // 1) collect hanging-node constraints for cell assuming
+                        // scalar finite element
+                        AlignedVector<VectorizedArrayType> values_dofs(
+                          dofs_per_component);
+
+                        std::array<dealii::internal::MatrixFreeFunctions::
+                                     compressed_constraint_kind,
+                                   VectorizedArrayType::size()>
+                          constraint_mask;
+                        constraint_mask.fill(
+                          dealii::internal::MatrixFreeFunctions::
+                            unconstrained_compressed_constraint_kind);
+
+                        constraint_mask[0] = mask;
+
+                        std::vector<
+                          std::tuple<unsigned int, unsigned int, Number>>
+                          locally_relevant_constrains_hn;
+
+                        for (unsigned int i = 0; i < dofs_per_component; ++i)
+                          {
+                            for (unsigned int j = 0; j < dofs_per_component;
+                                 ++j)
+                              values_dofs[j][0] = static_cast<Number>(i == j);
+
+                            dealii::internal::FEEvaluationHangingNodesFactory<
+                              dim,
+                              Number,
+                              VectorizedArrayType>::apply(1,
+                                                          phi.get_shape_info()
+                                                            .data.front()
+                                                            .fe_degree,
+                                                          phi.get_shape_info(),
+                                                          false,
+                                                          constraint_mask,
+                                                          values_dofs.data());
+
+                            for (unsigned int j = 0; j < dofs_per_component;
+                                 ++j)
+                              if (1e-10 < std::abs(values_dofs[j][0]) &&
+                                  (j != i ||
+                                   1e-10 < std::abs(values_dofs[j][0] - 1.0)))
+                                locally_relevant_constrains_hn.emplace_back(
+                                  j, i, values_dofs[j][0]);
+                          }
+
+
+                        std::sort(locally_relevant_constrains_hn.begin(),
+                                  locally_relevant_constrains_hn.end(),
+                                  [](const auto &a, const auto &b) {
+                                    if (std::get<0>(a) < std::get<0>(b))
+                                      return true;
+                                    return (std::get<0>(a) == std::get<0>(b)) &&
+                                           (std::get<1>(a) < std::get<1>(b));
+                                  });
+
+                        // 1b) extend for multiple components
+                        const unsigned int n_hn_constraints =
+                          locally_relevant_constrains_hn.size();
+                        locally_relevant_constrains_hn.resize(n_hn_constraints *
+                                                              n_components);
+
+                        for (unsigned int c = 0; c < n_components; ++c)
+                          for (unsigned int i = 0; i < n_hn_constraints; ++i)
+                            locally_relevant_constrains_hn[c *
+                                                             n_hn_constraints +
+                                                           i] =
+                              std::tuple<unsigned int, unsigned int, Number>{
+                                std::get<0>(locally_relevant_constrains_hn[i]) +
+                                  c * dofs_per_component,
+                                std::get<1>(locally_relevant_constrains_hn[i]) +
+                                  c * dofs_per_component,
+                                std::get<2>(locally_relevant_constrains_hn[i])};
+
+
+                        locally_relevant_constrains_hn_map[mask] =
+                          locally_relevant_constrains_hn;
+                      }
+
+                    const auto &locally_relevant_constrains_hn =
+                      locally_relevant_constrains_hn_map[mask];
+
+                    // 2) perform vmult with other constraints
+                    std::vector<std::tuple<unsigned int, unsigned int, Number>>
+                      locally_relevant_constrains_temp;
+
+                    for (unsigned int i = 0;
+                         i < dofs_per_component * n_components;
+                         ++i)
+                      {
+                        const auto lower_bound_fu = [](const auto &a,
+                                                       const auto &b) {
+                          return std::get<0>(a) < b;
+                        };
+
+                        const auto upper_bound_fu = [](const auto &a,
+                                                       const auto &b) {
+                          return a < std::get<0>(b);
+                        };
+
+                        const auto i_begin = std::lower_bound(
+                          locally_relevant_constrains_hn.begin(),
+                          locally_relevant_constrains_hn.end(),
+                          i,
+                          lower_bound_fu);
+                        const auto i_end = std::upper_bound(
+                          locally_relevant_constrains_hn.begin(),
+                          locally_relevant_constrains_hn.end(),
+                          i,
+                          upper_bound_fu);
+
+                        if (i_begin == i_end)
+                          {
+                            // dof is not constrained by hanging-node constraint
+                            // (identity matrix): simply copy constraints
+                            const auto j_begin = std::lower_bound(
+                              locally_relevant_constrains.begin(),
+                              locally_relevant_constrains.end(),
+                              i,
+                              lower_bound_fu);
+                            const auto j_end = std::upper_bound(
+                              locally_relevant_constrains.begin(),
+                              locally_relevant_constrains.end(),
+                              i,
+                              upper_bound_fu);
+
+                            for (auto v = j_begin; v != j_end; ++v)
+                              locally_relevant_constrains_temp.emplace_back(*v);
+                          }
+                        else
+                          {
+                            // dof is constrained: build transitive closure
+                            for (auto v0 = i_begin; v0 != i_end; ++v0)
+                              {
+                                const auto j_begin = std::lower_bound(
+                                  locally_relevant_constrains.begin(),
+                                  locally_relevant_constrains.end(),
+                                  std::get<1>(*v0),
+                                  lower_bound_fu);
+                                const auto j_end = std::upper_bound(
+                                  locally_relevant_constrains.begin(),
+                                  locally_relevant_constrains.end(),
+                                  std::get<1>(*v0),
+                                  upper_bound_fu);
+
+                                for (auto v1 = j_begin; v1 != j_end; ++v1)
+                                  locally_relevant_constrains_temp.emplace_back(
+                                    std::get<0>(*v0),
+                                    std::get<1>(*v1),
+                                    std::get<2>(*v0) * std::get<2>(*v1));
+                              }
+                          }
+                      }
+
+                    locally_relevant_constrains =
+                      locally_relevant_constrains_temp;
+                  }
+              }
+
+            // STEP 2d: transpose COO
+            std::sort(locally_relevant_constrains.begin(),
+                      locally_relevant_constrains.end(),
+                      [](const auto &a, const auto &b) {
+                        if (std::get<1>(a) < std::get<1>(b))
+                          return true;
+                        return (std::get<1>(a) == std::get<1>(b)) &&
+                               (std::get<0>(a) < std::get<0>(b));
+                      });
+
+            // STEP 2e: translate COO to CRS
             auto &c_pool = c_pools[v];
             {
               if (locally_relevant_constrains.size() > 0)
@@ -491,7 +682,9 @@ namespace MatrixFreeTools
         // set size locally-relevant diagonal
         for (unsigned int v = 0; v < n_lanes_filled; ++v)
           diagonals_local_constrained[v].assign(
-            c_pools[v].row_lid_to_gid.size(), Number(0.0));
+            c_pools[v].row_lid_to_gid.size() *
+              (n_fe_components == 1 ? n_components : 1),
+            Number(0.0));
       }
 
       void
@@ -509,7 +702,15 @@ namespace MatrixFreeTools
       void
       submit()
       {
-        const auto ith_column = phi.begin_dof_values();
+        // if we have a block vector with components with the same DoFHandler,
+        // we need to figure out which component and which DoF within the
+        // component are we currently considering
+        const unsigned int n_fe_components =
+          phi.get_dof_info().start_components.back();
+        const unsigned int comp =
+          n_fe_components == 1 ? i / phi.dofs_per_component : 0;
+        const unsigned int i_comp =
+          n_fe_components == 1 ? (i % phi.dofs_per_component) : i;
 
         // apply local constraint matrix from left and from right:
         // loop over all rows of transposed constrained matrix
@@ -527,43 +728,59 @@ namespace MatrixFreeTools
                 const auto scale_iterator =
                   std::lower_bound(c_pool.col.begin() + c_pool.row[j],
                                    c_pool.col.begin() + c_pool.row[j + 1],
-                                   i);
+                                   i_comp);
 
                 // explanation: j-th row of C_e^T is empty (see above)
                 if (scale_iterator == c_pool.col.begin() + c_pool.row[j + 1])
                   continue;
 
                 // explanation: C_e^T(j,i) is zero (see above)
-                if (*scale_iterator != i)
+                if (*scale_iterator != i_comp)
                   continue;
 
                 // apply constraint matrix from the left
                 Number temp = 0.0;
                 for (unsigned int k = c_pool.row[j]; k < c_pool.row[j + 1]; ++k)
-                  temp += c_pool.val[k] * ith_column[c_pool.col[k]][v];
+                  temp += c_pool.val[k] *
+                          phi.begin_dof_values()[comp * phi.dofs_per_component +
+                                                 c_pool.col[k]][v];
 
                 // apply constraint matrix from the right
-                diagonals_local_constrained[v][j] +=
+                diagonals_local_constrained
+                  [v][j + comp * c_pools[v].row_lid_to_gid.size()] +=
                   temp *
                   c_pool.val[std::distance(c_pool.col.begin(), scale_iterator)];
               }
           }
       }
 
-      void
+      template <typename VectorType>
+      inline void
       distribute_local_to_global(
-        LinearAlgebra::distributed::Vector<Number> &diagonal_global)
+        std::array<VectorType *, n_components> &diagonal_global)
       {
         // STEP 4: assembly results: add into global vector
+        const unsigned int n_fe_components =
+          phi.get_dof_info().start_components.back();
+
         for (unsigned int v = 0;
              v < phi.get_matrix_free().n_active_entries_per_cell_batch(
                    phi.get_current_cell_index());
              ++v)
+          // if we have a block vector with components with the same
+          // DoFHandler, we need to loop over all components manually and
+          // need to apply the correct shift
           for (unsigned int j = 0; j < c_pools[v].row.size() - 1; ++j)
-            ::dealii::internal::vector_access_add(
-              diagonal_global,
-              c_pools[v].row_lid_to_gid[j],
-              diagonals_local_constrained[v][j]);
+            for (unsigned int comp = 0;
+                 comp < (n_fe_components == 1 ?
+                           static_cast<unsigned int>(n_components) :
+                           1);
+                 ++comp)
+              ::dealii::internal::vector_access_add(
+                *diagonal_global[n_fe_components == 1 ? comp : 0],
+                c_pools[v].row_lid_to_gid[j],
+                diagonals_local_constrained
+                  [v][j + comp * c_pools[v].row_lid_to_gid.size()]);
       }
 
     private:
@@ -582,6 +799,11 @@ namespace MatrixFreeTools
       // note: may be larger then dofs_per_cell in the presence of
       // constraints!
       std::array<std::vector<Number>, n_lanes> diagonals_local_constrained;
+
+      std::map<
+        dealii::internal::MatrixFreeFunctions::compressed_constraint_kind,
+        std::vector<std::tuple<unsigned int, unsigned int, Number>>>
+        locally_relevant_constrains_hn_map;
     };
 
   } // namespace internal
@@ -591,11 +813,12 @@ namespace MatrixFreeTools
             int n_q_points_1d,
             int n_components,
             typename Number,
-            typename VectorizedArrayType>
+            typename VectorizedArrayType,
+            typename VectorType>
   void
   compute_diagonal(
     const MatrixFree<dim, Number, VectorizedArrayType> &matrix_free,
-    LinearAlgebra::distributed::Vector<Number> &        diagonal_global,
+    VectorType &                                        diagonal_global,
     const std::function<void(FEEvaluation<dim,
                                           fe_degree,
                                           n_q_points_1d,
@@ -606,16 +829,44 @@ namespace MatrixFreeTools
     const unsigned int                                              quad_no,
     const unsigned int first_selected_component)
   {
-    using VectorType = LinearAlgebra::distributed::Vector<Number>;
-
-    // initialize vector
-    matrix_free.initialize_dof_vector(diagonal_global, dof_no);
-
     int dummy = 0;
 
+    std::array<typename dealii::internal::BlockVectorSelector<
+                 VectorType,
+                 IsBlockVector<VectorType>::value>::BaseVectorType *,
+               n_components>
+      diagonal_global_components;
+
+    for (unsigned int d = 0; d < n_components; ++d)
+      diagonal_global_components[d] = dealii::internal::
+        BlockVectorSelector<VectorType, IsBlockVector<VectorType>::value>::
+          get_vector_component(diagonal_global, d + first_selected_component);
+
+    const auto &dof_info = matrix_free.get_dof_info(dof_no);
+
+    if (dof_info.start_components.back() == 1)
+      for (unsigned int comp = 0; comp < n_components; ++comp)
+        {
+          Assert(diagonal_global_components[comp] != nullptr,
+                 ExcMessage("The finite element underlying this FEEvaluation "
+                            "object is scalar, but you requested " +
+                            std::to_string(n_components) +
+                            " components via the template argument in "
+                            "FEEvaluation. In that case, you must pass an "
+                            "std::vector<VectorType> or a BlockVector to " +
+                            "read_dof_values and distribute_local_to_global."));
+          dealii::internal::check_vector_compatibility(
+            *diagonal_global_components[comp], matrix_free, dof_info);
+        }
+    else
+      {
+        dealii::internal::check_vector_compatibility(
+          *diagonal_global_components[0], matrix_free, dof_info);
+      }
+
     matrix_free.template cell_loop<VectorType, int>(
       [&](const MatrixFree<dim, Number, VectorizedArrayType> &matrix_free,
-          LinearAlgebra::distributed::Vector<Number> &        diagonal_global,
+          VectorType &,
           const int &,
           const std::pair<unsigned int, unsigned int> &range) mutable {
         FEEvaluation<dim,
@@ -645,7 +896,7 @@ namespace MatrixFreeTools
                 helper.submit();
               }
 
-            helper.distribute_local_to_global(diagonal_global);
+            helper.distribute_local_to_global(diagonal_global_components);
           }
       },
       diagonal_global,
@@ -659,11 +910,12 @@ namespace MatrixFreeTools
             int n_q_points_1d,
             int n_components,
             typename Number,
-            typename VectorizedArrayType>
+            typename VectorizedArrayType,
+            typename VectorType>
   void
   compute_diagonal(
     const MatrixFree<dim, Number, VectorizedArrayType> &matrix_free,
-    LinearAlgebra::distributed::Vector<Number> &        diagonal_global,
+    VectorType &                                        diagonal_global,
     void (CLASS::*cell_operation)(FEEvaluation<dim,
                                                fe_degree,
                                                n_q_points_1d,
@@ -680,7 +932,8 @@ namespace MatrixFreeTools
                      n_q_points_1d,
                      n_components,
                      Number,
-                     VectorizedArrayType>(
+                     VectorizedArrayType,
+                     VectorType>(
       matrix_free,
       diagonal_global,
       [&](auto &feeval) { (owning_class->*cell_operation)(feeval); },
@@ -761,7 +1014,7 @@ namespace MatrixFreeTools
     const unsigned int first_selected_component)
   {
     std::unique_ptr<AffineConstraints<typename MatrixType::value_type>>
-                                                              constraints_for_matrix;
+      constraints_for_matrix;
     const AffineConstraints<typename MatrixType::value_type> &constraints =
       internal::create_new_affine_constraints_if_needed(matrix,
                                                         constraints_in,
@@ -805,7 +1058,7 @@ namespace MatrixFreeTools
           {
             integrator.reinit(cell);
 
-            unsigned int const n_filled_lanes =
+            const unsigned int n_filled_lanes =
               matrix_free.n_active_entries_per_cell_batch(cell);
 
             for (unsigned int v = 0; v < n_filled_lanes; ++v)
@@ -879,15 +1132,14 @@ namespace MatrixFreeTools
                    n_components,
                    Number,
                    VectorizedArrayType,
-                   MatrixType>(matrix_free,
-                               constraints,
-                               matrix,
-                               [&](auto &feeval) {
-                                 (owning_class->*cell_operation)(feeval);
-                               },
-                               dof_no,
-                               quad_no,
-                               first_selected_component);
+                   MatrixType>(
+      matrix_free,
+      constraints,
+      matrix,
+      [&](auto &feeval) { (owning_class->*cell_operation)(feeval); },
+      dof_no,
+      quad_no,
+      first_selected_component);
   }
 
 #endif // DOXYGEN
