@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2019 - 2020 by the deal.II authors
+// Copyright (C) 2019 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -38,83 +38,34 @@ namespace internal
   // a helper type-trait that leverage SFINAE to figure out if type T has
   // ... T::local_element() const
   template <typename T>
-  struct has_local_element
-  {
-  private:
-    // this will work always.
-    // we let it be void as we know T::local_element() (if exists) should
-    // certainly return something
-    static void
-    detect(...);
+  using local_element_t = decltype(std::declval<T const>().local_element(0));
 
-    // this detecter will work only if we have "... T::local_element() const"
-    // and its return type will be the same as local_element(),
-    // that we expect to be T::value_type
-    template <typename U>
-    static decltype(std::declval<U const>().local_element(0))
-    detect(const U &);
-
-  public:
-    // finally here we check if our detector has non-void return type
-    // T::value_type. This will happen if compiler can use second detector,
-    // otherwise SFINAE let it work with the more general first one that is void
-    static const bool value =
-      !std::is_same<void, decltype(detect(std::declval<T>()))>::value;
-  };
-
-  // We need to have a separate declaration for static const members
   template <typename T>
-  const bool has_local_element<T>::value;
+  constexpr bool has_local_element = is_supported_operation<local_element_t, T>;
 
 
 
   // a helper type-trait that leverage SFINAE to figure out if type T has
   // void T::add_local_element(const uint, const typename T::value_type)
   template <typename T>
-  struct has_add_local_element
-  {
-  private:
-    static int
-    detect(...);
+  using add_local_element_t =
+    decltype(std::declval<T>().add_local_element(0, typename T::value_type()));
 
-    template <typename U>
-    static decltype(
-      std::declval<U>().add_local_element(0, typename T::value_type()))
-    detect(const U &);
-
-  public:
-    static const bool value =
-      !std::is_same<int, decltype(detect(std::declval<T>()))>::value;
-  };
-
-  // We need to have a separate declaration for static const members
   template <typename T>
-  const bool has_add_local_element<T>::value;
+  constexpr bool has_add_local_element =
+    is_supported_operation<add_local_element_t, T>;
 
 
 
   // a helper type-trait that leverage SFINAE to figure out if type T has
   // void T::set_local_element(const uint, const typename T::value_type)
   template <typename T>
-  struct has_set_local_element
-  {
-  private:
-    static int
-    detect(...);
+  using set_local_element_t =
+    decltype(std::declval<T>().set_local_element(0, typename T::value_type()));
 
-    template <typename U>
-    static decltype(
-      std::declval<U>().set_local_element(0, typename T::value_type()))
-    detect(const U &);
-
-  public:
-    static const bool value =
-      !std::is_same<int, decltype(detect(std::declval<T>()))>::value;
-  };
-
-  // We need to have a separate declaration for static const members
   template <typename T>
-  const bool has_set_local_element<T>::value;
+  constexpr bool has_set_local_element =
+    is_supported_operation<set_local_element_t, T>;
 
 
 
@@ -122,71 +73,36 @@ namespace internal
   // bool T::partitioners_are_compatible(const Utilities::MPI::Partitioner &)
   // const
   template <typename T>
-  struct has_partitioners_are_compatible
-  {
-  private:
-    static void
-    detect(...);
+  using partitioners_are_compatible_t =
+    decltype(std::declval<T const>().partitioners_are_compatible(
+      std::declval<Utilities::MPI::Partitioner>()));
 
-    template <typename U>
-    static decltype(std::declval<U const>().partitioners_are_compatible(
-      std::declval<Utilities::MPI::Partitioner>()))
-    detect(const U &);
-
-  public:
-    static const bool value =
-      std::is_same<bool, decltype(detect(std::declval<T>()))>::value;
-  };
-
-  // We need to have a separate declaration for static const members
   template <typename T>
-  const bool has_partitioners_are_compatible<T>::value;
+  constexpr bool has_partitioners_are_compatible =
+    is_supported_operation<partitioners_are_compatible_t, T>;
+
 
 
   // same as above to check
   // ... T::begin() const
   template <typename T>
-  struct has_begin
-  {
-  private:
-    static void
-    detect(...);
+  using begin_t = decltype(std::declval<T const>().begin());
 
-    template <typename U>
-    static decltype(std::declval<U const>().begin())
-    detect(const U &);
-
-  public:
-    static const bool value =
-      !std::is_same<void, decltype(detect(std::declval<T>()))>::value;
-  };
-
-  // We need to have a separate declaration for static const members
   template <typename T>
-  const bool has_begin<T>::value;
+  constexpr bool has_begin = is_supported_operation<begin_t, T>;
+
 
 
   // same as above to check
   // ... T::shared_vector_data() const
   template <typename T>
-  struct has_shared_vector_data
-  {
-  private:
-    static void
-    detect(...);
+  using shared_vector_data_t =
+    decltype(std::declval<T const>().shared_vector_data());
 
-    template <typename U>
-    static decltype(std::declval<U const>().shared_vector_data())
-    detect(const U &);
-
-  public:
-    static const bool value =
-      !std::is_same<void, decltype(detect(std::declval<T>()))>::value;
-  };
-
-  // We need to have a separate declaration for static const members
   template <typename T>
-  const bool has_shared_vector_data<T>::value;
+  constexpr bool has_shared_vector_data =
+    is_supported_operation<shared_vector_data_t, T>;
+
 
 
   // type trait for vector T and Number to see if
@@ -198,8 +114,9 @@ namespace internal
   struct is_vectorizable
   {
     static const bool value =
-      has_begin<T>::value &&
-      (has_local_element<T>::value || is_serial_vector<T>::value) &&
+      has_begin<T> &&
+      (has_local_element<T> ||
+       is_serial_vector<typename std::remove_const<T>::type>::value) &&
       std::is_same<typename T::value_type, Number>::value;
   };
 
@@ -220,48 +137,24 @@ namespace internal
   // a helper type-trait that leverage SFINAE to figure out if type T has
   // void T::update_ghost_values_start(const uint) const
   template <typename T>
-  struct has_update_ghost_values_start
-  {
-  private:
-    static bool
-    detect(...);
+  using update_ghost_values_start_t =
+    decltype(std::declval<T const>().update_ghost_values_start(0));
 
-    template <typename U>
-    static decltype(std::declval<U const>().update_ghost_values_start(0))
-    detect(const U &);
-
-  public:
-    static const bool value =
-      !std::is_same<bool, decltype(detect(std::declval<T>()))>::value;
-  };
-
-  // We need to have a separate declaration for static const members
   template <typename T>
-  const bool has_update_ghost_values_start<T>::value;
+  constexpr bool has_update_ghost_values_start =
+    is_supported_operation<update_ghost_values_start_t, T>;
 
 
 
   // a helper type-trait that leverage SFINAE to figure out if type T has
   // void T::	compress_start(const uint, VectorOperation::values)
   template <typename T>
-  struct has_compress_start
-  {
-  private:
-    static bool
-    detect(...);
+  using compress_start_t =
+    decltype(std::declval<T>().compress_start(0, VectorOperation::add));
 
-    template <typename U>
-    static decltype(std::declval<U>().compress_start(0, VectorOperation::add))
-    detect(const U &);
-
-  public:
-    static const bool value =
-      !std::is_same<bool, decltype(detect(std::declval<T>()))>::value;
-  };
-
-  // We need to have a separate declaration for static const members
   template <typename T>
-  const bool has_compress_start<T>::value;
+  constexpr bool has_compress_start =
+    is_supported_operation<compress_start_t, T>;
 
 
 
@@ -270,39 +163,19 @@ namespace internal
   // We assume that if both begin() and local_element()
   // exist, then begin() + offset == local_element(offset)
   template <typename T>
-  struct has_exchange_on_subset
-  {
-    static const bool value =
-      has_begin<T>::value && has_local_element<T>::value;
-  };
-
-  // We need to have a separate declaration for static const members
-  template <typename T>
-  const bool has_exchange_on_subset<T>::value;
+  constexpr bool   has_exchange_on_subset =
+    has_begin<T> &&has_local_element<T> &&has_partitioners_are_compatible<T>;
 
 
 
   // a helper type-trait that leverage SFINAE to figure out if type T has
   // T::communication_block_size
   template <typename T>
-  struct has_communication_block_size
-  {
-  private:
-    static void
-    detect(...);
+  using communication_block_size_t = decltype(T::communication_block_size);
 
-    template <typename U>
-    static decltype(U::communication_block_size)
-    detect(const U &);
-
-  public:
-    static const bool value =
-      !std::is_same<void, decltype(detect(std::declval<T>()))>::value;
-  };
-
-  // We need to have a separate declaration for static const members
   template <typename T>
-  const bool has_communication_block_size<T>::value;
+  constexpr bool has_communication_block_size =
+    is_supported_operation<communication_block_size_t, T>;
 
 
 
@@ -312,38 +185,35 @@ namespace internal
   // (like calculation of diagonals for matrix-free operators)
   // a dummy InVector == unsigned int is provided.
   // Thus we have to treat this case as well.
-  template <typename T>
-  struct is_serial_or_dummy
-  {
-  private:
-    // catches all cases including unsigned int
-    static void
-    detect(...);
+  template <class T, class IsSerialVectorNotSpecialized = void>
+  using not_parallel_vector_t =
+    std::integral_constant<bool, is_serial_vector<T>::value>;
 
-    // catches serial vectors
-    template <
-      typename U,
-      typename std::enable_if<is_serial_vector<U>::value, U>::type * = nullptr>
-    static void
-    detect(const U &);
-
-    // catches parallel vectors
-    template <
-      typename U,
-      typename std::enable_if<!is_serial_vector<U>::value, U>::type * = nullptr>
-    static bool
-    detect(const U &);
-
-  public:
-    static const bool value =
-      std::is_same<void, decltype(detect(std::declval<T>()))>::value;
-  };
-
-  // We need to have a separate declaration for static const members
-  template <typename T>
-  const bool is_serial_or_dummy<T>::value;
+  /**
+   * A predicate stating whether something is a vector type. We test this
+   * by seeing whether the `is_serial_vector` type is declared for the
+   * given vector type.
+   */
+  template <class VectorType>
+  using is_vector_type = decltype(is_serial_vector<VectorType>::value);
 
+  /**
+   * A predicate stating whether something is a vector type and is
+   * indeed a serial vector.
+   */
+  template <class VectorType>
+  using is_serial_vector_type = decltype(
+    typename std::enable_if<is_serial_vector<VectorType>::value, int>::type());
 
+  /**
+   * A variable that indicates that the type `T` is either (i) not
+   * a vector type at all, or (ii) if it is a vector type,
+   * that it is not a parallel vector type.
+   */
+  template <class VectorType>
+  constexpr bool is_not_parallel_vector =
+    (is_supported_operation<is_vector_type, VectorType> == false) ||
+    (is_supported_operation<is_serial_vector_type, VectorType> == true);
 } // namespace internal
 #endif
 
