@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2019 - 2021 by the deal.II authors
+// Copyright (C) 2019 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -22,8 +22,12 @@
 #include <deal.II/base/vectorization.h>
 
 #include <deal.II/matrix_free/dof_info.h>
+#include <deal.II/matrix_free/matrix_free.h>
 #include <deal.II/matrix_free/type_traits.h>
 
+#if DEBUG
+#  include <boost/algorithm/string/join.hpp>
+#endif
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -36,7 +40,7 @@ namespace internal
   // FIXME: this is wrong for Trilinos/Petsc MPI vectors
   // where we should first do Partitioner::local_to_global()
   template <typename VectorType,
-            typename std::enable_if<!has_local_element<VectorType>::value,
+            typename std::enable_if<!has_local_element<VectorType>,
                                     VectorType>::type * = nullptr>
   inline typename VectorType::value_type
   vector_access(const VectorType &vec, const unsigned int entry)
@@ -50,7 +54,7 @@ namespace internal
   // FIXME: this is wrong for Trilinos/Petsc MPI vectors
   // where we should first do Partitioner::local_to_global()
   template <typename VectorType,
-            typename std::enable_if<!has_local_element<VectorType>::value,
+            typename std::enable_if<!has_local_element<VectorType>,
                                     VectorType>::type * = nullptr>
   inline typename VectorType::value_type &
   vector_access(VectorType &vec, const unsigned int entry)
@@ -64,7 +68,7 @@ namespace internal
   // method to access data in local index space, which is what we use in
   // DoFInfo and hence in read_dof_values etc.
   template <typename VectorType,
-            typename std::enable_if<has_local_element<VectorType>::value,
+            typename std::enable_if<has_local_element<VectorType>,
                                     VectorType>::type * = nullptr>
   inline typename VectorType::value_type &
   vector_access(VectorType &vec, const unsigned int entry)
@@ -76,7 +80,7 @@ namespace internal
 
   // same for const access
   template <typename VectorType,
-            typename std::enable_if<has_local_element<VectorType>::value,
+            typename std::enable_if<has_local_element<VectorType>,
                                     VectorType>::type * = nullptr>
   inline typename VectorType::value_type
   vector_access(const VectorType &vec, const unsigned int entry)
@@ -87,7 +91,7 @@ namespace internal
 
 
   template <typename VectorType,
-            typename std::enable_if<has_add_local_element<VectorType>::value,
+            typename std::enable_if<has_add_local_element<VectorType>,
                                     VectorType>::type * = nullptr>
   inline void
   vector_access_add(VectorType &                           vec,
@@ -100,7 +104,7 @@ namespace internal
 
 
   template <typename VectorType,
-            typename std::enable_if<!has_add_local_element<VectorType>::value,
+            typename std::enable_if<!has_add_local_element<VectorType>,
                                     VectorType>::type * = nullptr>
   inline void
   vector_access_add(VectorType &                           vec,
@@ -113,7 +117,7 @@ namespace internal
 
 
   template <typename VectorType,
-            typename std::enable_if<has_add_local_element<VectorType>::value,
+            typename std::enable_if<has_add_local_element<VectorType>,
                                     VectorType>::type * = nullptr>
   inline void
   vector_access_add_global(VectorType &                           vec,
@@ -126,7 +130,7 @@ namespace internal
 
 
   template <typename VectorType,
-            typename std::enable_if<!has_add_local_element<VectorType>::value,
+            typename std::enable_if<!has_add_local_element<VectorType>,
                                     VectorType>::type * = nullptr>
   inline void
   vector_access_add_global(VectorType &                           vec,
@@ -139,7 +143,7 @@ namespace internal
 
 
   template <typename VectorType,
-            typename std::enable_if<has_set_local_element<VectorType>::value,
+            typename std::enable_if<has_set_local_element<VectorType>,
                                     VectorType>::type * = nullptr>
   inline void
   vector_access_set(VectorType &                           vec,
@@ -152,7 +156,7 @@ namespace internal
 
 
   template <typename VectorType,
-            typename std::enable_if<!has_set_local_element<VectorType>::value,
+            typename std::enable_if<!has_set_local_element<VectorType>,
                                     VectorType>::type * = nullptr>
   inline void
   vector_access_set(VectorType &                           vec,
@@ -169,15 +173,20 @@ namespace internal
   // version below is when has_partitioners_are_compatible == false
   // FIXME: this is incorrect for PETSc/Trilinos MPI vectors
   template <
+    int dim,
+    typename Number,
+    typename VectorizedArrayType,
     typename VectorType,
-    typename std::enable_if<!has_partitioners_are_compatible<VectorType>::value,
+    typename std::enable_if<!has_partitioners_are_compatible<VectorType>,
                             VectorType>::type * = nullptr>
   inline void
   check_vector_compatibility(
-    const VectorType &                            vec,
-    const internal::MatrixFreeFunctions::DoFInfo &dof_info)
+    const VectorType &                                  vec,
+    const MatrixFree<dim, Number, VectorizedArrayType> &matrix_free,
+    const internal::MatrixFreeFunctions::DoFInfo &      dof_info)
   {
     (void)vec;
+    (void)matrix_free;
     (void)dof_info;
 
     AssertDimension(vec.size(), dof_info.vector_partitioner->size());
@@ -186,26 +195,80 @@ namespace internal
 
 
   // same as above for has_partitioners_are_compatible == true
-  template <
-    typename VectorType,
-    typename std::enable_if<has_partitioners_are_compatible<VectorType>::value,
-                            VectorType>::type * = nullptr>
+  template <int dim,
+            typename Number,
+            typename VectorizedArrayType,
+            typename VectorType,
+            typename std::enable_if<has_partitioners_are_compatible<VectorType>,
+                                    VectorType>::type * = nullptr>
   inline void
   check_vector_compatibility(
-    const VectorType &                            vec,
-    const internal::MatrixFreeFunctions::DoFInfo &dof_info)
+    const VectorType &                                  vec,
+    const MatrixFree<dim, Number, VectorizedArrayType> &matrix_free,
+    const internal::MatrixFreeFunctions::DoFInfo &      dof_info)
   {
     (void)vec;
+    (void)matrix_free;
     (void)dof_info;
-    Assert(vec.partitioners_are_compatible(*dof_info.vector_partitioner),
-           ExcMessage(
-             "The parallel layout of the given vector is not "
-             "compatible with the parallel partitioning in MatrixFree. "
-             "A potential reason is that you did not use "
-             "MatrixFree::initialize_dof_vector() to get a "
-             "compatible vector. If you did, the dof_handler_index "
-             "used there and the one passed to the constructor of "
-             "FEEvaluation do not match."));
+
+#if DEBUG
+    if (vec.partitioners_are_compatible(*dof_info.vector_partitioner) == false)
+      {
+        unsigned int dof_index = numbers::invalid_unsigned_int;
+
+        for (unsigned int i = 0; i < matrix_free.n_components(); ++i)
+          if (&matrix_free.get_dof_info(i) == &dof_info)
+            {
+              dof_index = i;
+              break;
+            }
+
+        Assert(dof_index != numbers::invalid_unsigned_int, ExcInternalError());
+
+        std::vector<std::string> dof_indices_with_compatible_partitioners;
+
+        for (unsigned int i = 0; i < matrix_free.n_components(); ++i)
+          if (vec.partitioners_are_compatible(
+                *matrix_free.get_dof_info(i).vector_partitioner))
+            dof_indices_with_compatible_partitioners.push_back(
+              std::to_string(i));
+
+        if (dof_indices_with_compatible_partitioners.empty())
+          {
+            Assert(false,
+                   ExcMessage("The parallel layout of the given vector is "
+                              "compatible neither with the Partitioner of the "
+                              "current FEEvaluation with dof_handler_index=" +
+                              std::to_string(dof_index) +
+                              " nor with any Partitioner in MatrixFree. A "
+                              "potential reason is that you did not use "
+                              "MatrixFree::initialize_dof_vector() to get a "
+                              "compatible vector."));
+          }
+        else
+          {
+            Assert(
+              false,
+              ExcMessage(
+                "The parallel layout of the given vector is "
+                "not compatible with the Partitioner of the "
+                "current FEEvaluation with dof_handler_index=" +
+                std::to_string(dof_index) +
+                ". However, the underlying "
+                "MatrixFree contains Partitioner objects that are compatible. "
+                "They have the following dof_handler_index values: " +
+                boost::algorithm::join(dof_indices_with_compatible_partitioners,
+                                       ", ") +
+                ". Did you want to pass any of these values to the "
+                "constructor of the current FEEvaluation object or "
+                "did you not use MatrixFree::initialize_dof_vector() "
+                "with dof_handler_index=" +
+                std::to_string(dof_index) +
+                " to get a "
+                "compatible vector?"));
+          }
+      }
+#endif
   }
 
 
@@ -247,10 +310,20 @@ namespace internal
                             VectorizedArrayType *dof_values,
                             std::integral_constant<bool, true>) const
     {
+#ifdef DEBUG
+      // in debug mode, run non-vectorized version because this path
+      // has additional checks (e.g., regarding ghosting)
+      process_dofs_vectorized(dofs_per_cell,
+                              dof_index,
+                              vec,
+                              dof_values,
+                              std::integral_constant<bool, false>());
+#else
       const Number *vec_ptr = vec.begin() + dof_index;
       for (unsigned int i = 0; i < dofs_per_cell;
            ++i, vec_ptr += VectorizedArrayType::size())
         dof_values[i].load(vec_ptr);
+#endif
     }
 
 
@@ -340,7 +413,17 @@ namespace internal
                        VectorizedArrayType &res,
                        std::integral_constant<bool, true>) const
     {
+#ifdef DEBUG
+      // in debug mode, run non-vectorized version because this path
+      // has additional checks (e.g., regarding ghosting)
+      process_dof_gather(indices,
+                         vec,
+                         constant_offset,
+                         res,
+                         std::integral_constant<bool, false>());
+#else
       res.gather(vec.begin() + constant_offset, indices);
+#endif
     }
 
 
