@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2002 - 2020 by the deal.II authors
+// Copyright (C) 2002 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -281,7 +281,7 @@ MGCoarseGridApplySmoother<VectorType>::operator()(const unsigned int level,
                                                   VectorType &       dst,
                                                   const VectorType & src) const
 {
-  coarse_smooth->smooth(level, dst, src);
+  coarse_smooth->apply(level, dst, src);
 }
 
 /* ------------------ Functions for MGCoarseGridIterativeSolver ------------ */
@@ -356,23 +356,78 @@ MGCoarseGridIterativeSolver<VectorType,
 
 
 
+namespace internal
+{
+  namespace MGCoarseGridIterativeSolver
+  {
+    template <
+      class VectorType,
+      class SolverType,
+      class MatrixType,
+      class PreconditionerType,
+      typename std::enable_if<
+        std::is_same<VectorType, typename SolverType::vector_type>::value,
+        VectorType>::type * = nullptr>
+    void
+    solve(SolverType &              solver,
+          const MatrixType &        matrix,
+          const PreconditionerType &preconditioner,
+          VectorType &              dst,
+          const VectorType &        src)
+    {
+      solver.solve(matrix, dst, src, preconditioner);
+    }
+
+    template <
+      class VectorType,
+      class SolverType,
+      class MatrixType,
+      class PreconditionerType,
+      typename std::enable_if<
+        !std::is_same<VectorType, typename SolverType::vector_type>::value,
+        VectorType>::type * = nullptr>
+    void
+    solve(SolverType &              solver,
+          const MatrixType &        matrix,
+          const PreconditionerType &preconditioner,
+          VectorType &              dst,
+          const VectorType &        src)
+    {
+      typename SolverType::vector_type src_;
+      typename SolverType::vector_type dst_;
+
+      src_ = src;
+      dst_ = dst;
+
+      solver.solve(matrix, dst_, src_, preconditioner);
+
+      dst = dst_;
+    }
+  } // namespace MGCoarseGridIterativeSolver
+} // namespace internal
+
+
+
 template <class VectorType,
           class SolverType,
           class MatrixType,
           class PreconditionerType>
 void
-MGCoarseGridIterativeSolver<VectorType,
-                            SolverType,
-                            MatrixType,
-                            PreconditionerType>::
-operator()(const unsigned int /*level*/,
-           VectorType &      dst,
-           const VectorType &src) const
+                       MGCoarseGridIterativeSolver<
+                         VectorType,
+                         SolverType,
+                         MatrixType,
+                         PreconditionerType>::operator()(const unsigned int /*level*/,
+                                  VectorType &      dst,
+                                  const VectorType &src) const
 {
   Assert(solver != nullptr, ExcNotInitialized());
   Assert(matrix != nullptr, ExcNotInitialized());
   Assert(preconditioner != nullptr, ExcNotInitialized());
-  solver->solve(*matrix, dst, src, *preconditioner);
+
+  dst = 0;
+  internal::MGCoarseGridIterativeSolver::solve(
+    *solver, *matrix, *preconditioner, dst, src);
 }
 
 
@@ -401,10 +456,10 @@ MGCoarseGridHouseholder<number, VectorType>::initialize(
 
 template <typename number, class VectorType>
 void
-MGCoarseGridHouseholder<number, VectorType>::
-operator()(const unsigned int /*level*/,
-           VectorType &      dst,
-           const VectorType &src) const
+MGCoarseGridHouseholder<number, VectorType>::operator()(
+  const unsigned int /*level*/,
+  VectorType &      dst,
+  const VectorType &src) const
 {
   householder.least_squares(dst, src);
 }
