@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2020 by the deal.II authors
+// Copyright (C) 2010 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -95,6 +95,18 @@ public:
     const std::set<types::boundary_id> &boundary_ids,
     const ComponentMask &               component_mask = ComponentMask());
 
+  /**
+   * Add Dirichlet boundary dofs to the internal data structures
+   * on level @p level.
+   * The indices are restricted to the set of locally relevant
+   * level dofs.
+   */
+  template <int dim, int spacedim>
+  void
+  add_boundary_indices(const DoFHandler<dim, spacedim> &dof,
+                       const unsigned int               level,
+                       const IndexSet &                 boundary_indices);
+
   /**
    * Add user defined constraints to be used on level @p level.
    *
@@ -247,7 +259,7 @@ MGConstrainedDoFs::initialize(
   const unsigned int max_level = (level_relevant_dofs.max_level() == 0) ?
                                    nlevels - 1 :
                                    level_relevant_dofs.max_level();
-  const bool user_level_dofs =
+  const bool         user_level_dofs =
     (level_relevant_dofs.max_level() == 0) ? false : true;
 
   // At this point level_constraint and refinement_edge_indices are empty.
@@ -262,8 +274,8 @@ MGConstrainedDoFs::initialize(
         }
       else
         {
-          IndexSet relevant_dofs;
-          DoFTools::extract_locally_relevant_level_dofs(dof, l, relevant_dofs);
+          const IndexSet relevant_dofs =
+            DoFTools::extract_locally_relevant_level_dofs(dof, l);
           level_constraints[l].reinit(relevant_dofs);
         }
 
@@ -355,6 +367,25 @@ MGConstrainedDoFs::make_zero_boundary_constraints(
 
 
 
+template <int dim, int spacedim>
+inline void
+MGConstrainedDoFs::add_boundary_indices(const DoFHandler<dim, spacedim> &dof,
+                                        const unsigned int               level,
+                                        const IndexSet &level_boundary_indices)
+{
+  const unsigned int n_levels = dof.get_triangulation().n_global_levels();
+  if (boundary_indices.size() == 0)
+    {
+      boundary_indices.resize(n_levels);
+      for (unsigned int i = 0; i < n_levels; ++i)
+        boundary_indices[i] = IndexSet(dof.n_dofs(i));
+    }
+  AssertDimension(boundary_indices.size(), n_levels);
+  boundary_indices[level].add_indices(level_boundary_indices);
+}
+
+
+
 template <int dim, int spacedim>
 inline void
 MGConstrainedDoFs::make_no_normal_flux_constraints(
