@@ -73,7 +73,8 @@ namespace mg
     /**
      * Access matrix on a level.
      */
-    const LinearOperator<VectorType> &operator[](unsigned int level) const;
+    const LinearOperator<VectorType> &
+    operator[](unsigned int level) const;
 
     virtual void
     vmult(const unsigned int level,
@@ -235,8 +236,8 @@ namespace mg
 
 
   template <typename VectorType>
-  inline const LinearOperator<VectorType> &Matrix<VectorType>::
-                                           operator[](unsigned int level) const
+  inline const LinearOperator<VectorType> &
+  Matrix<VectorType>::operator[](unsigned int level) const
   {
     return matrices[level];
   }
