@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2020 by the deal.II authors
+// Copyright (C) 1999 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -118,11 +118,25 @@ namespace mg
     /**
      * This signal is triggered before (@p before is true) and after (@p before
      * is false) the call to a post-smoothing step via MGPostSmoother::apply()
-     * on
-     * @p level.
+     * on @p level.
      */
     boost::signals2::signal<void(const bool before, const unsigned int level)>
       post_smoother_step;
+
+    /**
+     * This signal is triggered before (@p before is true) and after (@p before
+     * is false) the computation of the residual vector on @p level, including
+     * the result of edge_out and edge_down.
+     */
+    boost::signals2::signal<void(const bool before, const unsigned int level)>
+      residual_step;
+
+    /**
+     * This signal is triggered before (@p before is true) and after (@p before
+     * is false) the execution of edge_in and edge_up.
+     */
+    boost::signals2::signal<void(const bool before, const unsigned int level)>
+      edge_prolongation;
   };
 } // namespace mg
 
@@ -142,11 +156,6 @@ namespace mg
  * vector. This is a nontrivial operation, usually initiated automatically by
  * the class PreconditionMG and performed by the classes derived from
  * MGTransferBase.
- *
- * @note The interface of this class is still very clumsy. In particular, you
- * will have to set up quite a few auxiliary objects before you can use it.
- * Unfortunately, it seems that this can be avoided only be restricting the
- * flexibility of this class in an unacceptable way.
  */
 template <typename VectorType>
 class Multigrid : public Subscriptor
@@ -228,6 +237,15 @@ public:
   set_edge_matrices(const MGMatrixBase<VectorType> &edge_out,
                     const MGMatrixBase<VectorType> &edge_in);
 
+  /**
+   * Similar to the function above: however, only @p edge_in is set. This
+   * is useful if the matrix attached to this class ignores the edge
+   * constraints during vmult(), which is only used during the computation
+   * of the residual.
+   */
+  void
+  set_edge_in_matrix(const MGMatrixBase<VectorType> &edge_in);
+
   /**
    * Set additional matrices to correct residual computation at refinement
    * edges. These matrices originate from discontinuous Galerkin methods (see
@@ -288,10 +306,17 @@ public:
   void set_cycle(Cycle);
 
   /**
-   * Connect a function to mg::Signals::coarse_solve.
+   * Connect a function to mg::Signals::pre_smoother_step.
    */
   boost::signals2::connection
-  connect_coarse_solve(
+  connect_pre_smoother_step(
+    const std::function<void(const bool, const unsigned int)> &slot);
+
+  /**
+   * Connect a function to mg::Signals::residual_step.
+   */
+  boost::signals2::connection
+  connect_residual_step(
     const std::function<void(const bool, const unsigned int)> &slot);
 
   /**
@@ -301,6 +326,13 @@ public:
   connect_restriction(
     const std::function<void(const bool, const unsigned int)> &slot);
 
+  /**
+   * Connect a function to mg::Signals::coarse_solve.
+   */
+  boost::signals2::connection
+  connect_coarse_solve(
+    const std::function<void(const bool, const unsigned int)> &slot);
+
   /**
    * Connect a function to mg::Signals::prolongation.
    */
@@ -309,10 +341,10 @@ public:
     const std::function<void(const bool, const unsigned int)> &slot);
 
   /**
-   * Connect a function to mg::Signals::pre_smoother_step.
+   * Connect a function to mg::Signals::edge_prolongation.
    */
   boost::signals2::connection
-  connect_pre_smoother_step(
+  connect_edge_prolongation(
     const std::function<void(const bool, const unsigned int)> &slot);
 
   /**
@@ -561,6 +593,18 @@ public:
   boost::signals2::connection
   connect_transfer_to_global(const std::function<void(bool)> &slot);
 
+  /**
+   * Return the Multigrid object passed to the constructor.
+   */
+  Multigrid<VectorType> &
+  get_multigrid();
+
+  /**
+   * Return the Multigrid object passed to the constructor.
+   */
+  const Multigrid<VectorType> &
+  get_multigrid() const;
+
 private:
   /**
    * Associated @p DoFHandler.
@@ -935,6 +979,22 @@ PreconditionMG<dim, VectorType, TRANSFER>::Tvmult_add(
   Assert(false, ExcNotImplemented());
 }
 
+
+template <int dim, typename VectorType, class TRANSFER>
+Multigrid<VectorType> &
+PreconditionMG<dim, VectorType, TRANSFER>::get_multigrid()
+{
+  return *this->multigrid;
+}
+
+
+template <int dim, typename VectorType, class TRANSFER>
+const Multigrid<VectorType> &
+PreconditionMG<dim, VectorType, TRANSFER>::get_multigrid() const
+{
+  return *this->multigrid;
+}
+
 #endif // DOXYGEN
 
 DEAL_II_NAMESPACE_CLOSE
