@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2021 by the deal.II authors
+// Copyright (C) 1999 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -66,7 +66,9 @@ namespace Exceptions
     DeclExceptionMsg(ExcNoTriangulationSelected,
                      "For the operation you are attempting, you first need to "
                      "tell the DataOut or related object which DoFHandler or "
-                     "triangulation you would like to work on.");
+                     "triangulation you would like to work on. This is "
+                     "generally done using the 'attach_dof_handler()' or "
+                     "'attach_triangulation()' member functions.");
 
     /**
      * Exception
@@ -74,7 +76,9 @@ namespace Exceptions
     DeclExceptionMsg(ExcNoDoFHandlerSelected,
                      "For the operation you are attempting, you first need to "
                      "tell the DataOut or related object which DoFHandler "
-                     "you would like to work on.");
+                     "you would like to work on. This is "
+                     "generally done using the 'attach_dof_handler()' "
+                     "member function.");
 
     /**
      * Exception
@@ -121,7 +125,7 @@ namespace Exceptions
                    int,
                    << "You have to give one name per component in your "
                    << "data vector. The number you gave was " << arg1
-                   << ", but the number of components is " << arg2 << ".");
+                   << ", but the number of components is " << arg2 << '.');
     /**
      * Exception
      */
@@ -522,11 +526,6 @@ namespace internal
  * @ref step_22 "step-22"
  * tutorial program).
  *
- * This class does not copy the vector given to it through the
- * add_data_vector() functions, for memory consumption reasons. It only stores
- * a reference to it, so it is in your responsibility to make sure that the
- * data vectors exist long enough.
- *
  * After adding all data vectors, you need to call a function which generates
  * the patches (i.e., some intermediate data representation) for output from
  * the stored data. Derived classes name this function build_patches().
@@ -591,19 +590,18 @@ namespace internal
  *
  * @ingroup output
  */
-template <typename DoFHandlerType,
+template <int dim,
           int patch_dim,
-          int patch_space_dim = patch_dim>
-class DataOut_DoFData : public DataOutInterface<patch_dim, patch_space_dim>
+          int spacedim       = dim,
+          int patch_spacedim = patch_dim>
+class DataOut_DoFData : public DataOutInterface<patch_dim, patch_spacedim>
 {
 public:
   /**
    * Typedef to the iterator type of the dof handler class under
    * consideration.
    */
-  using cell_iterator =
-    typename Triangulation<DoFHandlerType::dimension,
-                           DoFHandlerType::space_dimension>::cell_iterator;
+  using cell_iterator = typename Triangulation<dim, spacedim>::cell_iterator;
 
 public:
   /**
@@ -651,7 +649,7 @@ public:
    * object, then that contains all information needed to generate the output.
    */
   void
-  attach_dof_handler(const DoFHandlerType &);
+  attach_dof_handler(const DoFHandler<dim, spacedim> &);
 
   /**
    * Designate a triangulation to be used to extract geometry data and the
@@ -663,8 +661,7 @@ public:
    * at all, in which case it provides the geometry.
    */
   void
-  attach_triangulation(const Triangulation<DoFHandlerType::dimension,
-                                           DoFHandlerType::space_dimension> &);
+  attach_triangulation(const Triangulation<dim, spacedim> &);
 
   /**
    * Add a data vector together with its name.
@@ -728,11 +725,6 @@ public:
    * @note The actual type for the vector argument may be any vector type from
    * which FEValues can extract values on a cell using the
    * FEValuesBase::get_function_values() function.
-   *
-   * @note When working in parallel, the vector to be written needs to be ghosted
-   * with read access to all degrees of freedom on the locally owned cells, see
-   * the step-40 or step-37 tutorial programs for details, i.e., it might be
-   * necessary to call data.update_ghost_values().
    */
   template <class VectorType>
   void
@@ -741,8 +733,7 @@ public:
     const std::vector<std::string> &names,
     const DataVectorType            type = type_automatic,
     const std::vector<DataComponentInterpretation::DataComponentInterpretation>
-      &data_component_interpretation = std::vector<
-        DataComponentInterpretation::DataComponentInterpretation>());
+      &data_component_interpretation = {});
 
   /**
    * This function is an abbreviation to the above one (see there for a
@@ -767,8 +758,7 @@ public:
     const std::string &  name,
     const DataVectorType type = type_automatic,
     const std::vector<DataComponentInterpretation::DataComponentInterpretation>
-      &data_component_interpretation = std::vector<
-        DataComponentInterpretation::DataComponentInterpretation>());
+      &data_component_interpretation = {});
 
   /**
    * This function is an extension of the above one (see there for a
@@ -788,12 +778,11 @@ public:
   template <class VectorType>
   void
   add_data_vector(
-    const DoFHandlerType &          dof_handler,
-    const VectorType &              data,
-    const std::vector<std::string> &names,
+    const DoFHandler<dim, spacedim> &dof_handler,
+    const VectorType &               data,
+    const std::vector<std::string> & names,
     const std::vector<DataComponentInterpretation::DataComponentInterpretation>
-      &data_component_interpretation = std::vector<
-        DataComponentInterpretation::DataComponentInterpretation>());
+      &data_component_interpretation = {});
 
 
   /**
@@ -803,12 +792,11 @@ public:
   template <class VectorType>
   void
   add_data_vector(
-    const DoFHandlerType &dof_handler,
-    const VectorType &    data,
-    const std::string &   name,
+    const DoFHandler<dim, spacedim> &dof_handler,
+    const VectorType &               data,
+    const std::string &              name,
     const std::vector<DataComponentInterpretation::DataComponentInterpretation>
-      &data_component_interpretation = std::vector<
-        DataComponentInterpretation::DataComponentInterpretation>());
+      &data_component_interpretation = {});
 
   /**
    * This function is an alternative to the above ones, allowing the output of
@@ -838,9 +826,8 @@ public:
    */
   template <class VectorType>
   void
-  add_data_vector(const VectorType &data,
-                  const DataPostprocessor<DoFHandlerType::space_dimension>
-                    &data_postprocessor);
+  add_data_vector(const VectorType &                 data,
+                  const DataPostprocessor<spacedim> &data_postprocessor);
 
   /**
    * Same function as above, but with a DoFHandler object that does not need
@@ -850,10 +837,9 @@ public:
    */
   template <class VectorType>
   void
-  add_data_vector(const DoFHandlerType &dof_handler,
-                  const VectorType &    data,
-                  const DataPostprocessor<DoFHandlerType::space_dimension>
-                    &data_postprocessor);
+  add_data_vector(const DoFHandler<dim, spacedim> &  dof_handler,
+                  const VectorType &                 data,
+                  const DataPostprocessor<spacedim> &data_postprocessor);
 
   /**
    * Add a multilevel data vector.
@@ -875,7 +861,7 @@ public:
   template <class VectorType>
   void
   add_mg_data_vector(
-    const DoFHandlerType &           dof_handler,
+    const DoFHandler<dim, spacedim> &dof_handler,
     const MGLevelObject<VectorType> &data,
     const std::vector<std::string> & names,
     const std::vector<DataComponentInterpretation::DataComponentInterpretation>
@@ -887,7 +873,7 @@ public:
    */
   template <class VectorType>
   void
-  add_mg_data_vector(const DoFHandlerType &           dof_handler,
+  add_mg_data_vector(const DoFHandler<dim, spacedim> &dof_handler,
                      const MGLevelObject<VectorType> &data,
                      const std::string &              name);
 
@@ -901,13 +887,13 @@ public:
   clear_data_vectors();
 
   /**
-   * Release pointers to all input data elements, i.e. pointers to data
-   * vectors and to the DoF handler object. This function may be useful when
+   * Release pointers to all input data elements, i.e. pointers to
+   * to the DoF handler object. This function may be useful when
    * you have called the @p build_patches function of derived class, since
    * then the patches are built and the input data is no more needed, nor is
    * there a need to reference it. You can then output the patches detached
    * from the main thread and need not make sure anymore that the DoF handler
-   * object and vectors must not be deleted before the output thread is
+   * object must not be deleted before the output thread is
    * finished.
    */
   void
@@ -936,11 +922,23 @@ public:
    * This function will fail if either this or the other object did not yet
    * set up any patches.
    */
-  template <typename DoFHandlerType2>
+  template <int dim2, int spacedim2>
   void
   merge_patches(
-    const DataOut_DoFData<DoFHandlerType2, patch_dim, patch_space_dim> &source,
-    const Point<patch_space_dim> &shift = Point<patch_space_dim>());
+    const DataOut_DoFData<dim2, patch_dim, spacedim2, patch_spacedim> &source,
+    const Point<patch_spacedim> &shift = Point<patch_spacedim>());
+
+  /**
+   * @deprecated Use merge_patches() without the DoFHandlerType2 template
+   * instead.
+   */
+  template <typename DoFHandlerType2>
+  DEAL_II_DEPRECATED void
+  merge_patches(const DataOut_DoFData<DoFHandlerType2::dimension,
+                                      patch_dim,
+                                      DoFHandlerType2::space_dimension,
+                                      patch_spacedim> &source,
+                const Point<patch_spacedim> &shift = Point<patch_spacedim>());
 
   /**
    * Release the pointers to the data vectors and the DoF handler. You have to
@@ -962,34 +960,30 @@ protected:
   /**
    * Abbreviate the somewhat lengthy name for the Patch class.
    */
-  using Patch = dealii::DataOutBase::Patch<patch_dim, patch_space_dim>;
+  using Patch = dealii::DataOutBase::Patch<patch_dim, patch_spacedim>;
 
   /**
    * Pointer to the triangulation object.
    */
-  SmartPointer<const Triangulation<DoFHandlerType::dimension,
-                                   DoFHandlerType::space_dimension>>
-    triangulation;
+  SmartPointer<const Triangulation<dim, spacedim>> triangulation;
 
   /**
    * Pointer to the optional handler object.
    */
-  SmartPointer<const DoFHandlerType> dofs;
+  SmartPointer<const DoFHandler<dim, spacedim>> dofs;
 
   /**
    * List of data elements with vectors of values for each degree of freedom.
    */
-  std::vector<std::shared_ptr<internal::DataOutImplementation::DataEntryBase<
-    DoFHandlerType::dimension,
-    DoFHandlerType::space_dimension>>>
+  std::vector<std::shared_ptr<
+    internal::DataOutImplementation::DataEntryBase<dim, spacedim>>>
     dof_data;
 
   /**
    * List of data elements with vectors of values for each cell.
    */
-  std::vector<std::shared_ptr<internal::DataOutImplementation::DataEntryBase<
-    DoFHandlerType::dimension,
-    DoFHandlerType::space_dimension>>>
+  std::vector<std::shared_ptr<
+    internal::DataOutImplementation::DataEntryBase<dim, spacedim>>>
     cell_data;
 
   /**
@@ -1003,9 +997,11 @@ protected:
    * %Function by which the base class's functions get to know what patches
    * they shall write to a file.
    */
+public:
   virtual const std::vector<Patch> &
   get_patches() const override;
 
+protected:
   /**
    * Virtual function through which the names of data sets are obtained by the
    * output functions of the base class.
@@ -1013,15 +1009,6 @@ protected:
   virtual std::vector<std::string>
   get_dataset_names() const override;
 
-  /**
-   * Extracts the finite elements stored in the dof_data object, including a
-   * dummy object of FE_DGQ<dim>(0) in case only the triangulation is used.
-   */
-  std::vector<
-    std::shared_ptr<dealii::hp::FECollection<DoFHandlerType::dimension,
-                                             DoFHandlerType::space_dimension>>>
-  get_fes() const;
-
   /**
    * Overload of the respective DataOutInterface::get_nonscalar_data_ranges()
    * function. See there for a more extensive documentation.
@@ -1033,9 +1020,17 @@ protected:
                DataComponentInterpretation::DataComponentInterpretation>>
   get_nonscalar_data_ranges() const override;
 
+protected:
+  /**
+   * Extracts the finite elements stored in the dof_data object, including a
+   * dummy object of FE_DGQ<dim>(0) in case only the triangulation is used.
+   */
+  std::vector<std::shared_ptr<dealii::hp::FECollection<dim, spacedim>>>
+  get_fes() const;
+
   // Make all template siblings friends. Needed for the merge_patches()
   // function.
-  template <class, int, int>
+  template <int, int, int, int>
   friend class DataOut_DoFData;
 
   /**
@@ -1050,10 +1045,10 @@ private:
   template <class VectorType>
   void
   add_data_vector_internal(
-    const DoFHandlerType *          dof_handler,
-    const VectorType &              data,
-    const std::vector<std::string> &names,
-    const DataVectorType            type,
+    const DoFHandler<dim, spacedim> *dof_handler,
+    const VectorType &               data,
+    const std::vector<std::string> & names,
+    const DataVectorType             type,
     const std::vector<DataComponentInterpretation::DataComponentInterpretation>
       &        data_component_interpretation,
     const bool deduce_output_names);
@@ -1062,10 +1057,10 @@ private:
 
 
 // -------------------- template and inline functions ------------------------
-template <typename DoFHandlerType, int patch_dim, int patch_space_dim>
+template <int dim, int patch_dim, int spacedim, int patch_spacedim>
 template <typename VectorType>
 void
-DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>::add_data_vector(
+DataOut_DoFData<dim, patch_dim, spacedim, patch_spacedim>::add_data_vector(
   const VectorType &   vec,
   const std::string &  name,
   const DataVectorType type,
@@ -1081,10 +1076,10 @@ DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>::add_data_vector(
 
 
 
-template <typename DoFHandlerType, int patch_dim, int patch_space_dim>
+template <int dim, int patch_dim, int spacedim, int patch_spacedim>
 template <typename VectorType>
 void
-DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>::add_data_vector(
+DataOut_DoFData<dim, patch_dim, spacedim, patch_spacedim>::add_data_vector(
   const VectorType &              vec,
   const std::vector<std::string> &names,
   const DataVectorType            type,
@@ -1099,13 +1094,13 @@ DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>::add_data_vector(
 
 
 
-template <typename DoFHandlerType, int patch_dim, int patch_space_dim>
+template <int dim, int patch_dim, int spacedim, int patch_spacedim>
 template <typename VectorType>
 void
-DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>::add_data_vector(
-  const DoFHandlerType &dof_handler,
-  const VectorType &    data,
-  const std::string &   name,
+DataOut_DoFData<dim, patch_dim, spacedim, patch_spacedim>::add_data_vector(
+  const DoFHandler<dim, spacedim> &dof_handler,
+  const VectorType &               data,
+  const std::string &              name,
   const std::vector<DataComponentInterpretation::DataComponentInterpretation>
     &data_component_interpretation)
 {
@@ -1120,13 +1115,13 @@ DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>::add_data_vector(
 
 
 
-template <typename DoFHandlerType, int patch_dim, int patch_space_dim>
+template <int dim, int patch_dim, int spacedim, int patch_spacedim>
 template <typename VectorType>
 void
-DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>::add_data_vector(
-  const DoFHandlerType &          dof_handler,
-  const VectorType &              data,
-  const std::vector<std::string> &names,
+DataOut_DoFData<dim, patch_dim, spacedim, patch_spacedim>::add_data_vector(
+  const DoFHandler<dim, spacedim> &dof_handler,
+  const VectorType &               data,
+  const std::vector<std::string> & names,
   const std::vector<DataComponentInterpretation::DataComponentInterpretation>
     &data_component_interpretation)
 {
@@ -1140,12 +1135,12 @@ DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>::add_data_vector(
 
 
 
-template <typename DoFHandlerType, int patch_dim, int patch_space_dim>
+template <int dim, int patch_dim, int spacedim, int patch_spacedim>
 template <typename VectorType>
 void
-DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>::add_data_vector(
-  const VectorType &                                        vec,
-  const DataPostprocessor<DoFHandlerType::space_dimension> &data_postprocessor)
+DataOut_DoFData<dim, patch_dim, spacedim, patch_spacedim>::add_data_vector(
+  const VectorType &                 vec,
+  const DataPostprocessor<spacedim> &data_postprocessor)
 {
   Assert(dofs != nullptr,
          Exceptions::DataOutImplementation::ExcNoDoFHandlerSelected());
@@ -1154,12 +1149,12 @@ DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>::add_data_vector(
 
 
 
-template <typename DoFHandlerType, int patch_dim, int patch_space_dim>
-template <typename DoFHandlerType2>
+template <int dim, int patch_dim, int spacedim, int patch_spacedim>
+template <int dim2, int spacedim2>
 void
-DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>::merge_patches(
-  const DataOut_DoFData<DoFHandlerType2, patch_dim, patch_space_dim> &source,
-  const Point<patch_space_dim> &                                      shift)
+DataOut_DoFData<dim, patch_dim, spacedim, patch_spacedim>::merge_patches(
+  const DataOut_DoFData<dim2, patch_dim, spacedim2, patch_spacedim> &source,
+  const Point<patch_spacedim> &                                      shift)
 {
   const std::vector<Patch> &source_patches = source.get_patches();
   Assert((patches.size() != 0) && (source_patches.size() != 0),
@@ -1192,9 +1187,9 @@ DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>::merge_patches(
   Assert(patches[0].data.n_cols() == source_patches[0].data.n_cols(),
          Exceptions::DataOutImplementation::ExcIncompatiblePatchLists());
   Assert((patches[0].data.n_rows() +
-          (patches[0].points_are_available ? 0 : patch_space_dim)) ==
+          (patches[0].points_are_available ? 0 : patch_spacedim)) ==
            (source_patches[0].data.n_rows() +
-            (source_patches[0].points_are_available ? 0 : patch_space_dim)),
+            (source_patches[0].points_are_available ? 0 : patch_spacedim)),
          Exceptions::DataOutImplementation::ExcIncompatiblePatchLists());
 
   // check equality of the vector data
@@ -1227,7 +1222,7 @@ DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>::merge_patches(
   patches.insert(patches.end(), source_patches.begin(), source_patches.end());
 
   // perform shift, if so desired
-  if (shift != Point<patch_space_dim>())
+  if (shift != Point<patch_spacedim>())
     for (unsigned int i = old_n_patches; i < patches.size(); ++i)
       for (const unsigned int v : GeometryInfo<patch_dim>::vertex_indices())
         patches[i].vertices[v] += shift;
@@ -1243,19 +1238,38 @@ DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>::merge_patches(
         patches[i].neighbors[n] += old_n_patches;
 }
 
+
+
+template <int dim, int patch_dim, int spacedim, int patch_spacedim>
+template <typename DoFHandlerType2>
+void
+DataOut_DoFData<dim, patch_dim, spacedim, patch_spacedim>::merge_patches(
+  const DataOut_DoFData<DoFHandlerType2::dimension,
+                        patch_dim,
+                        DoFHandlerType2::space_dimension,
+                        patch_spacedim> &source,
+  const Point<patch_spacedim> &          shift)
+{
+  this->merge_patches<DoFHandlerType2::dimension,
+                      DoFHandlerType2::space_dimension>(source, shift);
+}
+
+
+
 namespace Legacy
 {
   /**
-   * The template arguments of the original dealii::DataOut_DoFData class will
-   * change in a future release. If for some reason, you need a code that is
-   * compatible with deal.II 9.3 and the subsequent release, use this alias
-   * instead.
+   * @deprecated Use dealii::DataOut_DoFData without the DoFHandlerType
+   * template instead.
    */
   template <typename DoFHandlerType,
             int patch_dim,
             int patch_space_dim = patch_dim>
-  using DataOut_DoFData =
-    dealii::DataOut_DoFData<DoFHandlerType, patch_dim, patch_space_dim>;
+  using DataOut_DoFData DEAL_II_DEPRECATED =
+    dealii::DataOut_DoFData<DoFHandlerType::dimension,
+                            patch_dim,
+                            DoFHandlerType::space_dimension,
+                            patch_space_dim>;
 } // namespace Legacy
 
 
