@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2007 - 2020 by the deal.II authors
+// Copyright (C) 2007 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -43,11 +43,11 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace Functions
 {
-  template <int dim, typename DoFHandlerType, typename VectorType>
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::FEFieldFunction(
-    const DoFHandlerType &mydh,
-    const VectorType &    myv,
-    const Mapping<dim> &  mymapping)
+  template <int dim, typename VectorType, int spacedim>
+  FEFieldFunction<dim, VectorType, spacedim>::FEFieldFunction(
+    const DoFHandler<dim, spacedim> &mydh,
+    const VectorType &               myv,
+    const Mapping<dim> &             mymapping)
     : Function<dim, typename VectorType::value_type>(
         mydh.get_fe(0).n_components())
     , dh(&mydh, "FEFieldFunction")
@@ -59,34 +59,36 @@ namespace Functions
 
 
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   void
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::set_active_cell(
-    const typename DoFHandlerType::active_cell_iterator &newcell)
+  FEFieldFunction<dim, VectorType, spacedim>::set_active_cell(
+    const typename DoFHandler<dim, spacedim>::active_cell_iterator &newcell)
   {
     cell_hint.get() = newcell;
   }
 
 
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   void
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::vector_value(
+  FEFieldFunction<dim, VectorType, spacedim>::vector_value(
     const Point<dim> &                       p,
     Vector<typename VectorType::value_type> &values) const
   {
     Assert(values.size() == this->n_components,
            ExcDimensionMismatch(values.size(), this->n_components));
-    typename DoFHandlerType::active_cell_iterator cell = cell_hint.get();
+    typename DoFHandler<dim, spacedim>::active_cell_iterator cell =
+      cell_hint.get();
     if (cell == dh->end())
       cell = dh->begin_active();
 
     std_cxx17::optional<Point<dim>> qp = get_reference_coordinates(cell, p);
     if (!qp)
       {
-        const std::pair<typename dealii::internal::
-                          ActiveCellIterator<dim, dim, DoFHandlerType>::type,
-                        Point<dim>>
+        const std::pair<
+          typename dealii::internal::
+            ActiveCellIterator<dim, dim, DoFHandler<dim, spacedim>>::type,
+          Point<dim>>
           my_pair = GridTools::find_active_cell_around_point(mapping, *dh, p);
         AssertThrow(my_pair.first.state() == IteratorState::valid &&
                       !my_pair.first->is_artificial(),
@@ -114,9 +116,9 @@ namespace Functions
 
 
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   typename VectorType::value_type
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::value(
+  FEFieldFunction<dim, VectorType, spacedim>::value(
     const Point<dim> & p,
     const unsigned int comp) const
   {
@@ -127,9 +129,9 @@ namespace Functions
 
 
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   void
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::vector_gradient(
+  FEFieldFunction<dim, VectorType, spacedim>::vector_gradient(
     const Point<dim> &                                            p,
     std::vector<Tensor<1, dim, typename VectorType::value_type>> &gradients)
     const
@@ -137,16 +139,18 @@ namespace Functions
     using number = typename VectorType::value_type;
     Assert(gradients.size() == this->n_components,
            ExcDimensionMismatch(gradients.size(), this->n_components));
-    typename DoFHandlerType::active_cell_iterator cell = cell_hint.get();
+    typename DoFHandler<dim, spacedim>::active_cell_iterator cell =
+      cell_hint.get();
     if (cell == dh->end())
       cell = dh->begin_active();
 
     std_cxx17::optional<Point<dim>> qp = get_reference_coordinates(cell, p);
     if (!qp)
       {
-        const std::pair<typename dealii::internal::
-                          ActiveCellIterator<dim, dim, DoFHandlerType>::type,
-                        Point<dim>>
+        const std::pair<
+          typename dealii::internal::
+            ActiveCellIterator<dim, dim, DoFHandler<dim, spacedim>>::type,
+          Point<dim>>
           my_pair = GridTools::find_active_cell_around_point(mapping, *dh, p);
         AssertThrow(my_pair.first.state() == IteratorState::valid &&
                       !my_pair.first->is_artificial(),
@@ -189,9 +193,9 @@ namespace Functions
 
 
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   Tensor<1, dim, typename VectorType::value_type>
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::gradient(
+  FEFieldFunction<dim, VectorType, spacedim>::gradient(
     const Point<dim> & p,
     const unsigned int comp) const
   {
@@ -203,24 +207,26 @@ namespace Functions
 
 
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   void
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::vector_laplacian(
+  FEFieldFunction<dim, VectorType, spacedim>::vector_laplacian(
     const Point<dim> &                       p,
     Vector<typename VectorType::value_type> &values) const
   {
     Assert(values.size() == this->n_components,
            ExcDimensionMismatch(values.size(), this->n_components));
-    typename DoFHandlerType::active_cell_iterator cell = cell_hint.get();
+    typename DoFHandler<dim, spacedim>::active_cell_iterator cell =
+      cell_hint.get();
     if (cell == dh->end())
       cell = dh->begin_active();
 
     std_cxx17::optional<Point<dim>> qp = get_reference_coordinates(cell, p);
     if (!qp)
       {
-        const std::pair<typename dealii::internal::
-                          ActiveCellIterator<dim, dim, DoFHandlerType>::type,
-                        Point<dim>>
+        const std::pair<
+          typename dealii::internal::
+            ActiveCellIterator<dim, dim, DoFHandler<dim, spacedim>>::type,
+          Point<dim>>
           my_pair = GridTools::find_active_cell_around_point(mapping, *dh, p);
         AssertThrow(my_pair.first.state() == IteratorState::valid &&
                       !my_pair.first->is_artificial(),
@@ -248,9 +254,9 @@ namespace Functions
 
 
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   typename VectorType::value_type
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::laplacian(
+  FEFieldFunction<dim, VectorType, spacedim>::laplacian(
     const Point<dim> & p,
     const unsigned int comp) const
   {
@@ -263,18 +269,18 @@ namespace Functions
   // Now the list versions
   // ==============================
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   void
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::vector_value_list(
+  FEFieldFunction<dim, VectorType, spacedim>::vector_value_list(
     const std::vector<Point<dim>> &                       points,
     std::vector<Vector<typename VectorType::value_type>> &values) const
   {
     Assert(points.size() == values.size(),
            ExcDimensionMismatch(points.size(), values.size()));
 
-    std::vector<typename DoFHandlerType::active_cell_iterator> cells;
-    std::vector<std::vector<Point<dim>>>                       qpoints;
-    std::vector<std::vector<unsigned int>>                     maps;
+    std::vector<typename DoFHandler<dim, spacedim>::active_cell_iterator> cells;
+    std::vector<std::vector<Point<dim>>>   qpoints;
+    std::vector<std::vector<unsigned int>> maps;
 
     const unsigned int n_cells =
       compute_point_locations(points, cells, qpoints, maps);
@@ -316,9 +322,9 @@ namespace Functions
 
 
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   void
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::value_list(
+  FEFieldFunction<dim, VectorType, spacedim>::value_list(
     const std::vector<Point<dim>> &               points,
     std::vector<typename VectorType::value_type> &values,
     const unsigned int                            component) const
@@ -342,9 +348,9 @@ namespace Functions
 
 
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   void
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::vector_gradient_list(
+  FEFieldFunction<dim, VectorType, spacedim>::vector_gradient_list(
     const std::vector<Point<dim>> &points,
     std::vector<std::vector<Tensor<1, dim, typename VectorType::value_type>>>
       &values) const
@@ -352,9 +358,9 @@ namespace Functions
     Assert(points.size() == values.size(),
            ExcDimensionMismatch(points.size(), values.size()));
 
-    std::vector<typename DoFHandlerType::active_cell_iterator> cells;
-    std::vector<std::vector<Point<dim>>>                       qpoints;
-    std::vector<std::vector<unsigned int>>                     maps;
+    std::vector<typename DoFHandler<dim, spacedim>::active_cell_iterator> cells;
+    std::vector<std::vector<Point<dim>>>   qpoints;
+    std::vector<std::vector<unsigned int>> maps;
 
     const unsigned int n_cells =
       compute_point_locations(points, cells, qpoints, maps);
@@ -397,7 +403,7 @@ namespace Functions
           {
             const unsigned int s = vgrads[q].size();
             values[maps[i][q]].resize(s);
-            for (unsigned int l = 0; l < s; l++)
+            for (unsigned int l = 0; l < s; ++l)
               values[maps[i][q]][l] = vgrads[q][l];
           }
       }
@@ -405,9 +411,9 @@ namespace Functions
 
 
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   void
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::gradient_list(
+  FEFieldFunction<dim, VectorType, spacedim>::gradient_list(
     const std::vector<Point<dim>> &                               points,
     std::vector<Tensor<1, dim, typename VectorType::value_type>> &values,
     const unsigned int component) const
@@ -432,18 +438,18 @@ namespace Functions
 
 
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   void
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::vector_laplacian_list(
+  FEFieldFunction<dim, VectorType, spacedim>::vector_laplacian_list(
     const std::vector<Point<dim>> &                       points,
     std::vector<Vector<typename VectorType::value_type>> &values) const
   {
     Assert(points.size() == values.size(),
            ExcDimensionMismatch(points.size(), values.size()));
 
-    std::vector<typename DoFHandlerType::active_cell_iterator> cells;
-    std::vector<std::vector<Point<dim>>>                       qpoints;
-    std::vector<std::vector<unsigned int>>                     maps;
+    std::vector<typename DoFHandler<dim, spacedim>::active_cell_iterator> cells;
+    std::vector<std::vector<Point<dim>>>   qpoints;
+    std::vector<std::vector<unsigned int>> maps;
 
     const unsigned int n_cells =
       compute_point_locations(points, cells, qpoints, maps);
@@ -487,9 +493,9 @@ namespace Functions
 
 
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   void
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::laplacian_list(
+  FEFieldFunction<dim, VectorType, spacedim>::laplacian_list(
     const std::vector<Point<dim>> &               points,
     std::vector<typename VectorType::value_type> &values,
     const unsigned int                            component) const
@@ -513,13 +519,14 @@ namespace Functions
 
 
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   unsigned int
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::compute_point_locations(
-    const std::vector<Point<dim>> &                             points,
-    std::vector<typename DoFHandlerType::active_cell_iterator> &cells,
-    std::vector<std::vector<Point<dim>>> &                      qpoints,
-    std::vector<std::vector<unsigned int>> &                    maps) const
+  FEFieldFunction<dim, VectorType, spacedim>::compute_point_locations(
+    const std::vector<Point<dim>> &points,
+    std::vector<typename DoFHandler<dim, spacedim>::active_cell_iterator>
+      &                                     cells,
+    std::vector<std::vector<Point<dim>>> &  qpoints,
+    std::vector<std::vector<unsigned int>> &maps) const
   {
     // Calling the GridTools routine and preparing output
     auto cell_qpoint_map =
@@ -530,7 +537,7 @@ namespace Functions
     cells.resize(tria_cells.size());
     unsigned int i = 0;
     for (const auto &c : tria_cells)
-      cells[i++] = typename DoFHandlerType::cell_iterator(*c, dh);
+      cells[i++] = typename DoFHandler<dim, spacedim>::cell_iterator(*c, dh);
     qpoints = std::get<1>(cell_qpoint_map);
     maps    = std::get<2>(cell_qpoint_map);
 
@@ -541,11 +548,11 @@ namespace Functions
   }
 
 
-  template <int dim, typename DoFHandlerType, typename VectorType>
+  template <int dim, typename VectorType, int spacedim>
   std_cxx17::optional<Point<dim>>
-  FEFieldFunction<dim, DoFHandlerType, VectorType>::get_reference_coordinates(
-    const typename DoFHandlerType::active_cell_iterator &cell,
-    const Point<dim> &                                   point) const
+  FEFieldFunction<dim, VectorType, spacedim>::get_reference_coordinates(
+    const typename DoFHandler<dim, spacedim>::active_cell_iterator &cell,
+    const Point<dim> &                                              point) const
   {
     try
       {
