@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2020 by the deal.II authors
+// Copyright (C) 2009 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -34,8 +34,6 @@
 #include <deal.II/fe/mapping.h>
 #include <deal.II/fe/mapping_q1.h>
 
-#include <deal.II/grid/grid_tools.h>
-
 #include <deal.II/lac/vector.h>
 
 #include <deal.II/numerics/data_postprocessor.h>
@@ -61,6 +59,9 @@ namespace internal
     class PointGeometryData
     {
     public:
+      /**
+       * Only a constructor needed for this class (a struct really)
+       */
       PointGeometryData(
         const Point<dim> &                          new_requested_location,
         const std::vector<Point<dim>> &             new_locations,
