@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2018 - 2020 by the deal.II authors
+// Copyright (C) 2018 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -35,6 +35,9 @@ DEAL_II_NAMESPACE_OPEN
 template <typename Number>
 class Vector;
 
+template <int dim, int spacedim>
+class DoFHandler;
+
 namespace FESeries
 {
   template <int dim, int spacedim>
@@ -45,8 +48,6 @@ namespace FESeries
 
 namespace hp
 {
-  template <int dim, int spacedim>
-  class DoFHandler;
   template <int dim, int spacedim>
   class FECollection;
 } // namespace hp
@@ -101,8 +102,6 @@ namespace SmoothnessEstimator
    * The rate of the decay $\sigma$ can be used to estimate the smoothness. For
    * example, one strategy to implement hp-refinement criteria is to perform
    * p-refinement if $\sigma>1$ (see @cite mavriplis1994hp).
-   *
-   * @author Denis Davydov, 2018, Marc Fehling, 2018 - 2020
    */
   namespace Legendre
   {
@@ -129,7 +128,7 @@ namespace SmoothnessEstimator
      * This object needs to be initialized to have at least $p+1$ coefficients
      * in each direction for every finite element in the collection, where $p$
      * is its polynomial degree.
-     * @param [in] dof_handler An hp::DoFHandler.
+     * @param [in] dof_handler A DoFHandler.
      * @param [in] solution A solution vector.
      * @param [out] smoothness_indicators A vector for smoothness indicators.
      * @param [in] regression_strategy Determines which norm will be used on the
@@ -159,11 +158,11 @@ namespace SmoothnessEstimator
      */
     template <int dim, int spacedim, typename VectorType>
     void
-    coefficient_decay(FESeries::Legendre<dim, spacedim> &  fe_legendre,
-                      const hp::DoFHandler<dim, spacedim> &dof_handler,
-                      const VectorType &                   solution,
-                      Vector<float> &             smoothness_indicators,
-                      const VectorTools::NormType regression_strategy =
+    coefficient_decay(FESeries::Legendre<dim, spacedim> &fe_legendre,
+                      const DoFHandler<dim, spacedim> &  dof_handler,
+                      const VectorType &                 solution,
+                      Vector<float> &                    smoothness_indicators,
+                      const VectorTools::NormType        regression_strategy =
                         VectorTools::Linfty_norm,
                       const double smallest_abs_coefficient = 1e-10,
                       const bool   only_flagged_cells       = false);
@@ -181,7 +180,7 @@ namespace SmoothnessEstimator
      * @param [in] fe_legendre FESeries::Legendre object to calculate coefficients.
      * This object needs to be initialized to have at least $p+1$ coefficients
      * in each direction, where $p$ is the maximum polynomial degree to be used.
-     * @param [in] dof_handler An hp::DoFHandler
+     * @param [in] dof_handler A DoFHandler
      * @param [in] solution A solution vector
      * @param [out] smoothness_indicators A vector for smoothness indicators
      * @param [in] coefficients_predicate A predicate to select Legendre
@@ -216,10 +215,10 @@ namespace SmoothnessEstimator
     template <int dim, int spacedim, typename VectorType>
     void
     coefficient_decay_per_direction(
-      FESeries::Legendre<dim, spacedim> &  fe_legendre,
-      const hp::DoFHandler<dim, spacedim> &dof_handler,
-      const VectorType &                   solution,
-      Vector<float> &                      smoothness_indicators,
+      FESeries::Legendre<dim, spacedim> &fe_legendre,
+      const DoFHandler<dim, spacedim> &  dof_handler,
+      const VectorType &                 solution,
+      Vector<float> &                    smoothness_indicators,
       const ComponentMask &coefficients_predicate   = ComponentMask(),
       const double         smallest_abs_coefficient = 1e-10,
       const bool           only_flagged_cells       = false);
@@ -229,14 +228,26 @@ namespace SmoothnessEstimator
      * the default configuration for smoothness estimation purposes.
      *
      * For each finite element of the provided @p fe_collection, we use as many
-     * modes as its polynomial degree plus one, since we start with the first
-     * Legendre polynomial which is just a constant. Further for each element,
-     * we use a Gaussian quadrature designed to yield exact results for the
-     * highest order Legendre polynomial used.
+     * modes as its polynomial degree plus two. This includes the first Legendre
+     * polynomial which is just a constant. Further for each element, we use a
+     * Gaussian quadrature designed to yield exact results for the highest order
+     * Legendre polynomial used.
+     *
+     * As the Legendre expansion can only be performed on scalar fields, this
+     * class does not operate on vector-valued finite elements and will
+     * therefore throw an assertion. However, each component of a finite element
+     * field can be treated as a scalar field, respectively, on which Legendre
+     * expansions are again possible. For this purpose, the optional parameter
+     * @p component defines which component of each FiniteElement will be used.
+     * The default value of @p component only applies to scalar FEs, in which
+     * case it indicates that the sole component is to be decomposed. For
+     * vector-valued FEs, a non-default value must be explicitly provided.
      */
     template <int dim, int spacedim>
     FESeries::Legendre<dim, spacedim>
-    default_fe_series(const hp::FECollection<dim, spacedim> &fe_collection);
+    default_fe_series(
+      const hp::FECollection<dim, spacedim> &fe_collection,
+      const unsigned int component = numbers::invalid_unsigned_int);
   } // namespace Legendre
 
 
@@ -256,7 +267,7 @@ namespace SmoothnessEstimator
    * @f}
    * with $u_j$ the degrees of freedom and $\varphi_j$ the corresponding shape
    * functions. $\{\phi_{\bf k}({\bf x}) = \exp(i \, 2 \pi \, {\bf k} \cdot
-   * {\bf x}) \}$ are expoential functions on cell $K$. $a_{\bf k}$ and ${\cal
+   * {\bf x}) \}$ are exponential functions on cell $K$. $a_{\bf k}$ and ${\cal
    * F}_{{\bf k},j}$ are coefficients and transformation matrices from the
    * Fourier expansion of each shape function. For practical reasons, we will
    * perform the calculation of these matrices and coefficients only on the
@@ -344,9 +355,6 @@ namespace SmoothnessEstimator
    *
    * @note An extensive demonstration of the use of these functions is
    * provided in step-27.
-   *
-   * @author Wolfgang Bangerth 2006 - 2007, Denis Davydov, 2016,
-   * Marc Fehling, 2018 - 2020
    */
   namespace Fourier
   {
@@ -395,11 +403,11 @@ namespace SmoothnessEstimator
      */
     template <int dim, int spacedim, typename VectorType>
     void
-    coefficient_decay(FESeries::Fourier<dim, spacedim> &   fe_fourier,
-                      const hp::DoFHandler<dim, spacedim> &dof_handler,
-                      const VectorType &                   solution,
-                      Vector<float> &             smoothness_indicators,
-                      const VectorTools::NormType regression_strategy =
+    coefficient_decay(FESeries::Fourier<dim, spacedim> &fe_fourier,
+                      const DoFHandler<dim, spacedim> & dof_handler,
+                      const VectorType &                solution,
+                      Vector<float> &                   smoothness_indicators,
+                      const VectorTools::NormType       regression_strategy =
                         VectorTools::Linfty_norm,
                       const double smallest_abs_coefficient = 1e-10,
                       const bool   only_flagged_cells       = false);
@@ -445,10 +453,10 @@ namespace SmoothnessEstimator
     template <int dim, int spacedim, typename VectorType>
     void
     coefficient_decay_per_direction(
-      FESeries::Fourier<dim, spacedim> &   fe_fourier,
-      const hp::DoFHandler<dim, spacedim> &dof_handler,
-      const VectorType &                   solution,
-      Vector<float> &                      smoothness_indicators,
+      FESeries::Fourier<dim, spacedim> &fe_fourier,
+      const DoFHandler<dim, spacedim> & dof_handler,
+      const VectorType &                solution,
+      Vector<float> &                   smoothness_indicators,
       const ComponentMask &coefficients_predicate   = ComponentMask(),
       const double         smallest_abs_coefficient = 1e-10,
       const bool           only_flagged_cells       = false);
@@ -458,14 +466,26 @@ namespace SmoothnessEstimator
      * the default configuration for smoothness estimation purposes.
      *
      * For each finite element of the provided @p fe_collection, we use as many
-     * modes as its polynomial degree plus one, and at least three modes.
-     * Further for each element, we use a 4-point Gaussian quarature iterated in
-     * each dimension by the maximal wave number, which is the number of modes
-     * decreased by one since we start with $k = 0$.
+     * modes as its polynomial degree plus two. Further for each element, we use
+     * a 5-point Gaussian quarature iterated in each dimension by the maximal
+     * wave number, which is the number of modes decreased by one since we start
+     * with $k = 0$.
+     *
+     * As the Fourier expansion can only be performed on scalar fields, this
+     * class does not operate on vector-valued finite elements and will
+     * therefore throw an assertion. However, each component of a finite element
+     * field can be treated as a scalar field, respectively, on which Fourier
+     * expansions are again possible. For this purpose, the optional parameter
+     * @p component defines which component of each FiniteElement will be used.
+     * The default value of @p component only applies to scalar FEs, in which
+     * case it indicates that the sole component is to be decomposed. For
+     * vector-valued FEs, a non-default value must be explicitly provided.
      */
     template <int dim, int spacedim>
     FESeries::Fourier<dim, spacedim>
-    default_fe_series(const hp::FECollection<dim, spacedim> &fe_collection);
+    default_fe_series(
+      const hp::FECollection<dim, spacedim> &fe_collection,
+      const unsigned int component = numbers::invalid_unsigned_int);
   } // namespace Fourier
 } // namespace SmoothnessEstimator
 
