@@ -194,7 +194,7 @@ namespace VectorTools
 
   /**
    * Call the other interpolate_boundary_values() function, see above, with
-   * <tt>mapping=MappingQGeneric@<dim,spacedim@>(1)</tt>. The same comments
+   * <tt>mapping=MappingQ@<dim,spacedim@>(1)</tt>. The same comments
    * apply as for the previous function, in particular about the use of the
    * component mask and the requires size of the function object.
    *
@@ -213,7 +213,7 @@ namespace VectorTools
 
   /**
    * Call the other interpolate_boundary_values() function, see above, with
-   * <tt>mapping=MappingQGeneric@<dim,spacedim@>(1)</tt>. The same comments
+   * <tt>mapping=MappingQ@<dim,spacedim@>(1)</tt>. The same comments
    * apply as for the previous function, in particular about the use of the
    * component mask and the requires size of the function object.
    */
@@ -316,7 +316,7 @@ namespace VectorTools
 
   /**
    * Call the other interpolate_boundary_values() function, see above, with
-   * <tt>mapping=MappingQGeneric@<dim,spacedim@>(1)</tt>. The same comments
+   * <tt>mapping=MappingQ@<dim,spacedim@>(1)</tt>. The same comments
    * apply as for the previous function, in particular about the use of the
    * component mask and the requires size of the function object.
    *
@@ -337,7 +337,7 @@ namespace VectorTools
 
   /**
    * Call the other interpolate_boundary_values() function, see above, with
-   * <tt>mapping=MappingQGeneric@<dim,spacedim@>(1)</tt>. The same comments
+   * <tt>mapping=MappingQ@<dim,spacedim@>(1)</tt>. The same comments
    * apply as for the previous function, in particular about the use of the
    * component mask and the requires size of the function object.
    *
@@ -385,8 +385,8 @@ namespace VectorTools
    * where $\vec{n}$ is an outward normal vector.
    *
    * This function throws an exception if used with $H_\text{curl}$ conforming
-   * elements, so the project_boundary_values_curl_conforming() should be used
-   * instead.
+   * elements, so the project_boundary_values_curl_conforming_l2() should be
+   * used instead.
    *
    * @param[in] mapping The mapping that will be used in the transformations
    * necessary to integrate along the boundary.
@@ -443,7 +443,7 @@ namespace VectorTools
 
   /**
    * Call the project_boundary_values() function, see above, with
-   * <tt>mapping=MappingQGeneric@<dim,spacedim@>(1)</tt>.
+   * <tt>mapping=MappingQ@<dim,spacedim@>(1)</tt>.
    */
   template <int dim, int spacedim, typename number>
   void
@@ -471,7 +471,7 @@ namespace VectorTools
 
   /**
    * Call the project_boundary_values() function, see above, with
-   * <tt>mapping=MappingQGeneric@<dim,spacedim@>(1)</tt>.
+   * <tt>mapping=MappingQ@<dim,spacedim@>(1)</tt>.
    */
   template <int dim, int spacedim, typename number>
   void
@@ -534,7 +534,7 @@ namespace VectorTools
 
   /**
    * Call the project_boundary_values() function, see above, with
-   * <tt>mapping=MappingQGeneric@<dim,spacedim@>(1)</tt>.
+   * <tt>mapping=MappingQ@<dim,spacedim@>(1)</tt>.
    *
    * @ingroup constraints
    */
@@ -548,96 +548,6 @@ namespace VectorTools
     AffineConstraints<number> &constraints,
     std::vector<unsigned int>  component_mapping = {});
 
-
-  /**
-   * Compute constraints that correspond to boundary conditions of the form
-   * $\vec{n}\times\vec{u}=\vec{n}\times\vec{f}$, i.e. the tangential
-   * components of $u$ and $f$ shall coincide.
-   *
-   * If the AffineConstraints @p constraints contained values or other
-   * constraints before, the new ones are added or the old ones overwritten,
-   * if a node of the boundary part to be used was already in the list of
-   * constraints. This is handled by using inhomogeneous constraints. Please
-   * note that when combining adaptive meshes and this kind of constraints,
-   * the Dirichlet conditions should be set first, and then completed by
-   * hanging node constraints, in order to make sure that the discretization
-   * remains consistent. See the discussion on conflicting constraints in the
-   * module on
-   * @ref constraints.
-   *
-   * This function is explicitly written to use with the FE_Nedelec elements.
-   * Thus it throws an exception, if it is called with other finite elements.
-   *
-   * The second argument of this function denotes the first vector component
-   * in the finite element that corresponds to the vector function that you
-   * want to constrain. For example, if we want to solve Maxwell's equations
-   * in 3d and the finite element has components $(E_x,E_y,E_z,B_x,B_y,B_z)$
-   * and we want the boundary conditions
-   * $\vec{n}\times\vec{B}=\vec{n}\times\vec{f}$, then @p
-   * first_vector_component would be 3. Vectors are implicitly assumed to have
-   * exactly <code>dim</code> components that are ordered in the same way as
-   * we usually order the coordinate directions, i.e. $x$-, $y$-, and finally
-   * $z$-component.
-   *
-   * The parameter @p boundary_component corresponds to the number @p
-   * boundary_id of the face. numbers::internal_face_boundary_id is an illegal
-   * value, since it is reserved for interior faces.
-   *
-   * The last argument is denoted to compute the normal vector $\vec{n}$ at
-   * the boundary points.
-   *
-   * <h4>Computing constraints</h4>
-   *
-   * To compute the constraints we use projection-based interpolation as
-   * proposed in Solin, Segeth and Dolezel (Higher order finite elements,
-   * Chapman&amp;Hall, 2004) on every face located at the boundary.
-   *
-   * First one projects $\vec{f}$ on the lowest-order edge shape functions.
-   * Then the remaining part $(I-P_0)\vec{f}$ of the function is projected on
-   * the remaining higher-order edge shape functions. In the last step we
-   * project $(I-P_0-P_e)\vec{f}$ on the bubble shape functions defined on the
-   * face.
-   *
-   * @deprecated Use the project_boundary_values_curl_conforming_l2() function
-   * instead of this one.
-   *
-   * @ingroup constraints
-   *
-   * @see
-   * @ref GlossBoundaryIndicator "Glossary entry on boundary indicators"
-   */
-  template <int dim>
-  DEAL_II_DEPRECATED void
-  project_boundary_values_curl_conforming(
-    const DoFHandler<dim, dim> & dof_handler,
-    const unsigned int           first_vector_component,
-    const Function<dim, double> &boundary_function,
-    const types::boundary_id     boundary_component,
-    AffineConstraints<double> &  constraints,
-    const Mapping<dim> &         mapping);
-
-  /**
-   * Same as above for the hp-namespace.
-   *
-   * @deprecated Use the project_boundary_values_curl_conforming_l2() function
-   * instead of this one.
-   *
-   * @ingroup constraints
-   *
-   * @see
-   * @ref GlossBoundaryIndicator "Glossary entry on boundary indicators"
-   */
-  template <int dim>
-  DEAL_II_DEPRECATED void
-  project_boundary_values_curl_conforming(
-    const DoFHandler<dim, dim> &           dof_handler,
-    const unsigned int                     first_vector_component,
-    const Function<dim, double> &          boundary_function,
-    const types::boundary_id               boundary_component,
-    AffineConstraints<double> &            constraints,
-    const hp::MappingCollection<dim, dim> &mapping_collection =
-      hp::StaticMappingQ1<dim>::mapping_collection);
-
   /**
    * This function is an updated version of the
    * project_boundary_values_curl_conforming function. The intention is to fix
@@ -808,15 +718,15 @@ namespace VectorTools
    * @see
    * @ref GlossBoundaryIndicator "Glossary entry on boundary indicators"
    */
-  template <int dim>
+  template <int dim, typename number, typename number2 = number>
   void
   project_boundary_values_div_conforming(
-    const DoFHandler<dim, dim> & dof_handler,
-    const unsigned int           first_vector_component,
-    const Function<dim, double> &boundary_function,
-    const types::boundary_id     boundary_component,
-    AffineConstraints<double> &  constraints,
-    const Mapping<dim> &         mapping);
+    const DoFHandler<dim, dim> &  dof_handler,
+    const unsigned int            first_vector_component,
+    const Function<dim, number2> &boundary_function,
+    const types::boundary_id      boundary_component,
+    AffineConstraints<number> &   constraints,
+    const Mapping<dim> &          mapping);
 
   /**
    * Same as above for the hp-namespace.
@@ -826,14 +736,14 @@ namespace VectorTools
    * @see
    * @ref GlossBoundaryIndicator "Glossary entry on boundary indicators"
    */
-  template <int dim>
+  template <int dim, typename number, typename number2 = number>
   void
   project_boundary_values_div_conforming(
     const DoFHandler<dim, dim> &           dof_handler,
     const unsigned int                     first_vector_component,
-    const Function<dim, double> &          boundary_function,
+    const Function<dim, number2> &         boundary_function,
     const types::boundary_id               boundary_component,
-    AffineConstraints<double> &            constraints,
+    AffineConstraints<number> &            constraints,
     const hp::MappingCollection<dim, dim> &mapping_collection =
       hp::StaticMappingQ1<dim>::mapping_collection);
 
