@@ -197,7 +197,6 @@ namespace Particles
           return; // nothing else to do here
         }
 
-      const auto &tria     = field_dh.get_triangulation();
       const auto &fe       = field_dh.get_fe();
       auto        particle = particle_handler.begin();
 
@@ -224,7 +223,7 @@ namespace Particles
 
       while (particle != particle_handler.end())
         {
-          const auto &cell = particle->get_surrounding_cell(tria);
+          const auto &cell = particle->get_surrounding_cell();
           const auto &dh_cell =
             typename DoFHandler<dim, spacedim>::cell_iterator(*cell, &field_dh);
           dh_cell->get_dof_indices(dof_indices);
