@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2017 - 2020 by the deal.II authors
+// Copyright (C) 2017 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -293,7 +293,7 @@ namespace Physics
                      int,
                      << "The number of rows in the input matrix is " << arg1
                      << ", but needs to be either " << arg2 << " or " << arg3
-                     << ".");
+                     << '.');
 
 
       /**
@@ -305,8 +305,8 @@ namespace Physics
                      int,
                      int,
                      << "The number of rows in the input matrix is " << arg1
-                     << ", but needs to be either " << arg2 << "," << arg3
-                     << ", or " << arg4 << ".");
+                     << ", but needs to be either " << arg2 << ',' << arg3
+                     << ", or " << arg4 << '.');
 
 
       /**
@@ -318,7 +318,7 @@ namespace Physics
                      int,
                      << "The number of columns in the input matrix is " << arg1
                      << ", but needs to be either " << arg2 << " or " << arg3
-                     << ".");
+                     << '.');
 
 
       /**
@@ -330,8 +330,8 @@ namespace Physics
                      int,
                      int,
                      << "The number of columns in the input matrix is " << arg1
-                     << ", but needs to be either " << arg2 << "," << arg3
-                     << ", or " << arg4 << ".");
+                     << ", but needs to be either " << arg2 << ',' << arg3
+                     << ", or " << arg4 << '.');
 
 
       /**
