@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2019 by the deal.II authors
+// Copyright (C) 2016 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,7 +18,6 @@
 
 #include <deal.II/base/config.h>
 
-#include <deal.II/base/point.h>
 #include <deal.II/base/symmetric_tensor.h>
 #include <deal.II/base/tensor.h>
 
@@ -32,8 +31,6 @@ namespace Physics
     /**
      * Transformation functions and tensors that are defined in terms of
      * rotation angles and axes of rotation.
-     *
-     * @author Jean-Paul Pelteret, 2017
      */
     namespace Rotations
     {
@@ -92,6 +89,15 @@ namespace Physics
        */
       template <typename Number>
       Tensor<2, 3, Number>
+      rotation_matrix_3d(const Tensor<1, 3, Number> &axis, const Number &angle);
+
+      /**
+       * @copydoc Physics::Transformations::Rotations::rotation_matrix_3d()
+       *
+       * @deprecated Use the variant with a Tensor as an axis.
+       */
+      template <typename Number>
+      DEAL_II_DEPRECATED_EARLY Tensor<2, 3, Number>
       rotation_matrix_3d(const Point<3, Number> &axis, const Number &angle);
 
       //@}
@@ -113,8 +119,6 @@ namespace Physics
      * reference and spatial configurations, and their surfaces $\partial
      * V_{0}$ and $\partial V_{t}$ have the outwards facing normals
      * $\mathbf{N}$ and $\mathbf{n}$.
-     *
-     * @author Jean-Paul Pelteret, Andrew McBride, 2016
      */
     namespace Contravariant
     {
@@ -182,7 +186,7 @@ namespace Physics
 
       /**
        * Return the result of the push forward transformation on a rank-4
-       * contravariant tensor, i.e. (in index notation)
+       * contravariant tensor, i.e. (in index notation):
        * @f[
        *  \left[ \chi\left(\bullet\right)^{\sharp} \right]_{ijkl}
        *    \dealcoloneq F_{iI} F_{jJ}
@@ -201,7 +205,7 @@ namespace Physics
 
       /**
        * Return the result of the push forward transformation on a rank-4
-       * contravariant symmetric tensor, i.e. (in index notation)
+       * contravariant symmetric tensor, i.e. (in index notation):
        * @f[
        *  \left[ \chi\left(\bullet\right)^{\sharp} \right]_{ijkl}
        *    \dealcoloneq F_{iI} F_{jJ}
@@ -284,7 +288,7 @@ namespace Physics
 
       /**
        * Return the result of the pull back transformation on a rank-4
-       * contravariant tensor, i.e. (in index notation)
+       * contravariant tensor, i.e. (in index notation):
        * @f[
        *  \left[ \chi^{-1}\left(\bullet\right)^{\sharp} \right]_{IJKL}
        *    \dealcoloneq F^{-1}_{Ii} F^{-1}_{Jj}
@@ -303,7 +307,7 @@ namespace Physics
 
       /**
        * Return the result of the pull back transformation on a rank-4
-       * contravariant symmetric tensor, i.e. (in index notation)
+       * contravariant symmetric tensor, i.e. (in index notation):
        * @f[
        *  \left[ \chi^{-1}\left(\bullet\right)^{\sharp} \right]_{IJKL}
        *    \dealcoloneq F^{-1}_{Ii} F^{-1}_{Jj}
@@ -340,8 +344,6 @@ namespace Physics
      * the curves $\partial A_{0}$ and $\partial A_{t}$ that are,
      * respectively, associated with the line directors $\mathbf{L}$ and
      * $\mathbf{l}$.
-     *
-     * @author Jean-Paul Pelteret, Andrew McBride, 2016
      */
     namespace Covariant
     {
@@ -409,7 +411,7 @@ namespace Physics
 
       /**
        * Return the result of the push forward transformation on a rank-4
-       * covariant tensor, i.e. (in index notation)
+       * covariant tensor, i.e. (in index notation):
        * @f[
        *  \left[ \chi\left(\bullet\right)^{\flat} \right]_{ijkl}
        *    \dealcoloneq F^{-T}_{iI} F^{-T}_{jJ}
@@ -428,7 +430,7 @@ namespace Physics
 
       /**
        * Return the result of the push forward transformation on a rank-4
-       * covariant symmetric tensor, i.e. (in index notation)
+       * covariant symmetric tensor, i.e. (in index notation):
        * @f[
        *  \left[ \chi\left(\bullet\right)^{\flat} \right]_{ijkl}
        *    \dealcoloneq F^{-T}_{iI} F^{-T}_{jJ}
@@ -511,7 +513,7 @@ namespace Physics
 
       /**
        * Return the result of the pull back transformation on a rank-4
-       * contravariant tensor, i.e. (in index notation)
+       * contravariant tensor, i.e. (in index notation):
        * @f[
        *  \left[ \chi^{-1}\left(\bullet\right)^{\flat} \right]_{IJKL}
        *  \dealcoloneq F^{T}_{Ii} F^{T}_{Jj}
@@ -530,7 +532,7 @@ namespace Physics
 
       /**
        * Return the result of the pull back transformation on a rank-4
-       * contravariant symmetric tensor, i.e. (in index notation)
+       * contravariant symmetric tensor, i.e. (in index notation):
        * @f[
        *  \left[ \chi^{-1}\left(\bullet\right)^{\flat} \right]_{IJKL}
        *  \dealcoloneq F^{T}_{Ii} F^{T}_{Jj}
@@ -554,8 +556,6 @@ namespace Physics
      * Transformation of tensors that are defined in terms of a set of
      * contravariant basis vectors and scale with the inverse of the volume
      * change associated with the mapping.
-     *
-     * @author Jean-Paul Pelteret, Andrew McBride, 2016
      */
     namespace Piola
     {
@@ -627,7 +627,7 @@ namespace Physics
 
       /**
        * Return the result of the push forward transformation on a rank-4
-       * contravariant tensor, i.e. (in index notation)
+       * contravariant tensor, i.e. (in index notation):
        * @f[
        *  \textrm{det} \mathbf{F}^{-1} \; \left[
        * \chi\left(\bullet\right)^{\sharp} \right]_{ijkl}
@@ -648,7 +648,7 @@ namespace Physics
 
       /**
        * Return the result of the push forward transformation on a rank-4
-       * contravariant symmetric tensor, i.e. (in index notation)
+       * contravariant symmetric tensor, i.e. (in index notation):
        * @f[
        *  \textrm{det} \mathbf{F}^{-1} \; \left[
        * \chi\left(\bullet\right)^{\sharp} \right]_{ijkl}
@@ -737,7 +737,7 @@ namespace Physics
 
       /**
        * Return the result of the pull back transformation on a rank-4
-       * contravariant tensor, i.e. (in index notation)
+       * contravariant tensor, i.e. (in index notation):
        * @f[
        *  \textrm{det} \mathbf{F} \; \left[
        * \chi^{-1}\left(\bullet\right)^{\sharp} \right]_{IJKL}
@@ -758,7 +758,7 @@ namespace Physics
 
       /**
        * Return the result of the pull back transformation on a rank-4
-       * contravariant symmetric tensor, i.e. (in index notation)
+       * contravariant symmetric tensor, i.e. (in index notation):
        * @f[
        *  \textrm{det} \mathbf{F} \; \left[
        * \chi^{-1}\left(\bullet\right)^{\sharp} \right]_{IJKL}
@@ -813,6 +813,92 @@ namespace Physics
                     const Tensor<2, dim, Number> &F);
 
     //@}
+
+    /**
+     * @name Basis transformations
+     */
+    //@{
+
+    /**
+     * Return a vector with a changed basis, i.e.
+     * @f[
+     *  \mathbf{V}^{\prime} \dealcoloneq \mathbf{B} \cdot \mathbf{V}
+     * @f]
+     *
+     * @param[in] V The vector to be transformed $\mathbf{V}$
+     * @param[in] B The transformation matrix $\mathbf{B}$
+     * @return      $\mathbf{V}^{\prime}$
+     */
+    template <int dim, typename Number>
+    Tensor<1, dim, Number>
+    basis_transformation(const Tensor<1, dim, Number> &V,
+                         const Tensor<2, dim, Number> &B);
+
+    /**
+     * Return a rank-2 tensor with a changed basis, i.e.
+     * @f[
+     *  \mathbf{T}^{\prime} \dealcoloneq \mathbf{B} \cdot \mathbf{T} \cdot
+     * \mathbf{B}^{T}
+     * @f]
+     *
+     * @param[in] T The tensor to be transformed $\mathbf{T}$
+     * @param[in] B The transformation matrix $\mathbf{B}$
+     * @return      $\mathbf{T}^{\prime}$
+     */
+    template <int dim, typename Number>
+    Tensor<2, dim, Number>
+    basis_transformation(const Tensor<2, dim, Number> &T,
+                         const Tensor<2, dim, Number> &B);
+
+    /**
+     * Return a symmetric rank-2 tensor with a changed basis, i.e.
+     * @f[
+     *  \mathbf{T}^{\prime} \dealcoloneq \mathbf{B} \cdot \mathbf{T} \cdot
+     * \mathbf{B}^{T}
+     * @f]
+     *
+     * @param[in] T The tensor to be transformed $\mathbf{T}$
+     * @param[in] B The transformation matrix $\mathbf{B}$
+     * @return      $\mathbf{T}^{\prime}$
+     */
+    template <int dim, typename Number>
+    SymmetricTensor<2, dim, Number>
+    basis_transformation(const SymmetricTensor<2, dim, Number> &T,
+                         const Tensor<2, dim, Number> &         B);
+
+    /**
+     * Return a rank-4 tensor with a changed basis, i.e. (in index notation):
+     * @f[
+     *  H_{ijkl}^{\prime} \dealcoloneq B_{iI} B_{jJ} H_{IJKL} B_{kK} B_{lL}
+     * @f]
+     *
+     * @param[in] H The tensor to be transformed $\mathbf{T}$
+     * @param[in] B The transformation matrix $\mathbf{B}$
+     * @return      $\mathbf{H}^{\prime}$
+     */
+    template <int dim, typename Number>
+    Tensor<4, dim, Number>
+    basis_transformation(const Tensor<4, dim, Number> &H,
+                         const Tensor<2, dim, Number> &B);
+
+    /**
+     * Return a symmetric rank-4 tensor with a changed basis, i.e. (in index
+     * notation):
+     * @f[
+     *  H_{ijkl}^{\prime} \dealcoloneq B_{iI} B_{jJ} H_{IJKL} B_{kK} B_{lL}
+     * @f]
+     *
+     * @param[in] H The tensor to be transformed $\mathbf{T}$
+     * @param[in] B The transformation matrix $\mathbf{B}$
+     * @return      $\mathbf{H}^{\prime}$
+     */
+    template <int dim, typename Number>
+    SymmetricTensor<4, dim, Number>
+    basis_transformation(const SymmetricTensor<4, dim, Number> &H,
+                         const Tensor<2, dim, Number> &         B);
+
+    //@}
+
   } // namespace Transformations
 } // namespace Physics
 
@@ -820,139 +906,13 @@ namespace Physics
 
 #ifndef DOXYGEN
 
-// ------------------------- inline functions ------------------------
-
-namespace internal
-{
-  namespace Physics
-  {
-    template <int dim, typename Number>
-    inline Tensor<1, dim, Number>
-    transformation_contraction(const Tensor<1, dim, Number> &V,
-                               const Tensor<2, dim, Number> &F)
-    {
-      return contract<1, 0>(F, V);
-    }
-
-
-
-    template <int dim, typename Number>
-    inline Tensor<2, dim, Number>
-    transformation_contraction(const Tensor<2, dim, Number> &T,
-                               const Tensor<2, dim, Number> &F)
-    {
-      return contract<1, 0>(F, contract<1, 1>(T, F));
-    }
-
-
-
-    template <int dim, typename Number>
-    inline dealii::SymmetricTensor<2, dim, Number>
-    transformation_contraction(const dealii::SymmetricTensor<2, dim, Number> &T,
-                               const Tensor<2, dim, Number> &                 F)
-    {
-      Tensor<2, dim, Number> tmp_1;
-      for (unsigned int i = 0; i < dim; ++i)
-        for (unsigned int J = 0; J < dim; ++J)
-          // Loop over I but complex.h defines a macro I, so use I_ instead
-          for (unsigned int I_ = 0; I_ < dim; ++I_)
-            tmp_1[i][J] += F[i][I_] * T[I_][J];
-
-      dealii::SymmetricTensor<2, dim, Number> out;
-      for (unsigned int i = 0; i < dim; ++i)
-        for (unsigned int j = i; j < dim; ++j)
-          for (unsigned int J = 0; J < dim; ++J)
-            out[i][j] += F[j][J] * tmp_1[i][J];
-
-      return out;
-    }
-
-
-
-    template <int dim, typename Number>
-    inline Tensor<4, dim, Number>
-    transformation_contraction(const Tensor<4, dim, Number> &H,
-                               const Tensor<2, dim, Number> &F)
-    {
-      // This contraction order and indexing might look a bit dubious, so a
-      // quick explanation as to what's going on is probably in order:
-      //
-      // When the contract() function operates on the inner indices, the
-      // result has the inner index and outer index transposed, i.e.
-      // contract<2,1>(H,F) implies
-      // T_{IJLk} = (H_{IJMN} F_{mM}) \delta_{mL} \delta_{Nk}
-      // rather than T_{IJkL} (the desired result).
-      // So, in effect, contraction of the 3rd (inner) index with F as the
-      // second argument results in its transposition with respect to its
-      // adjacent neighbor. This is due to the position of the argument F,
-      // leading to the free index being on the right hand side of the result.
-      // However, given that we can do two transformations from the LHS of H
-      // and two from the right we can undo the otherwise erroneous
-      // swapping of the outer indices upon application of the second
-      // sets of contractions.
-      //
-      // Note: Its significantly quicker (in 3d) to push forward
-      // each index individually
-      return contract<1, 1>(
-        F, contract<1, 1>(F, contract<2, 1>(contract<2, 1>(H, F), F)));
-    }
-
-
-
-    template <int dim, typename Number>
-    inline dealii::SymmetricTensor<4, dim, Number>
-    transformation_contraction(const dealii::SymmetricTensor<4, dim, Number> &H,
-                               const Tensor<2, dim, Number> &                 F)
-    {
-      // The first and last transformation operations respectively
-      // break and recover the symmetry properties of the tensors.
-      // We also want to perform a minimal number of operations here
-      // and avoid some complications related to the transposition of
-      // tensor indices when contracting inner indices using the contract()
-      // function. (For an explanation of the contraction operations,
-      // please see the note in the equivalent function for standard
-      // Tensors.) So what we'll do here is manually perform the first
-      // and last contractions that break/recover the tensor symmetries
-      // on the inner indices, and use the contract() function only on
-      // the outer indices.
-      //
-      // Note: Its significantly quicker (in 3d) to push forward
-      // each index individually
-
-      // Push forward (inner) index 1
-      Tensor<4, dim, Number> tmp;
-      // Loop over I but complex.h defines a macro I, so use I_ instead
-      for (unsigned int I_ = 0; I_ < dim; ++I_)
-        for (unsigned int j = 0; j < dim; ++j)
-          for (unsigned int K = 0; K < dim; ++K)
-            for (unsigned int L = 0; L < dim; ++L)
-              for (unsigned int J = 0; J < dim; ++J)
-                tmp[I_][j][K][L] += F[j][J] * H[I_][J][K][L];
-
-      // Push forward (outer) indices 0 and 3
-      tmp = contract<1, 0>(F, contract<3, 1>(tmp, F));
-
-      // Push forward (inner) index 2
-      dealii::SymmetricTensor<4, dim, Number> out;
-      for (unsigned int i = 0; i < dim; ++i)
-        for (unsigned int j = i; j < dim; ++j)
-          for (unsigned int k = 0; k < dim; ++k)
-            for (unsigned int l = k; l < dim; ++l)
-              for (unsigned int K = 0; K < dim; ++K)
-                out[i][j][k][l] += F[k][K] * tmp[i][j][K][l];
-
-      return out;
-    }
-  } // namespace Physics
-} // namespace internal
-
 
 
 template <typename Number>
 Tensor<2, 2, Number>
 Physics::Transformations::Rotations::rotation_matrix_2d(const Number &angle)
 {
-  const double rotation[2][2] = {{std::cos(angle), -std::sin(angle)},
+  const Number rotation[2][2] = {{std::cos(angle), -std::sin(angle)},
                                  {std::sin(angle), std::cos(angle)}};
   return Tensor<2, 2>(rotation);
 }
@@ -962,15 +922,15 @@ Physics::Transformations::Rotations::rotation_matrix_2d(const Number &angle)
 template <typename Number>
 Tensor<2, 3, Number>
 Physics::Transformations::Rotations::rotation_matrix_3d(
-  const Point<3, Number> &axis,
-  const Number &          angle)
+  const Tensor<1, 3, Number> &axis,
+  const Number &              angle)
 {
   Assert(std::abs(axis.norm() - 1.0) < 1e-9,
          ExcMessage("The supplied axial vector is not a unit vector."));
   const Number c              = std::cos(angle);
   const Number s              = std::sin(angle);
   const Number t              = 1. - c;
-  const double rotation[3][3] = {{t * axis[0] * axis[0] + c,
+  const Number rotation[3][3] = {{t * axis[0] * axis[0] + c,
                                   t * axis[0] * axis[1] - s * axis[2],
                                   t * axis[0] * axis[2] + s * axis[1]},
                                  {t * axis[0] * axis[1] + s * axis[2],
@@ -984,13 +944,24 @@ Physics::Transformations::Rotations::rotation_matrix_3d(
 
 
 
+template <typename Number>
+Tensor<2, 3, Number>
+Physics::Transformations::Rotations::rotation_matrix_3d(
+  const Point<3, Number> &axis,
+  const Number &          angle)
+{
+  return rotation_matrix_3d(static_cast<Tensor<1, 3, Number>>(axis), angle);
+}
+
+
+
 template <int dim, typename Number>
 inline Tensor<1, dim, Number>
 Physics::Transformations::Contravariant::push_forward(
   const Tensor<1, dim, Number> &V,
   const Tensor<2, dim, Number> &F)
 {
-  return internal::Physics::transformation_contraction(V, F);
+  return Physics::Transformations::basis_transformation(V, F);
 }
 
 
@@ -1001,7 +972,7 @@ Physics::Transformations::Contravariant::push_forward(
   const Tensor<2, dim, Number> &T,
   const Tensor<2, dim, Number> &F)
 {
-  return internal::Physics::transformation_contraction(T, F);
+  return Physics::Transformations::basis_transformation(T, F);
 }
 
 
@@ -1012,7 +983,7 @@ Physics::Transformations::Contravariant::push_forward(
   const SymmetricTensor<2, dim, Number> &T,
   const Tensor<2, dim, Number> &         F)
 {
-  return internal::Physics::transformation_contraction(T, F);
+  return Physics::Transformations::basis_transformation(T, F);
 }
 
 
@@ -1023,7 +994,7 @@ Physics::Transformations::Contravariant::push_forward(
   const Tensor<4, dim, Number> &H,
   const Tensor<2, dim, Number> &F)
 {
-  return internal::Physics::transformation_contraction(H, F);
+  return Physics::Transformations::basis_transformation(H, F);
 }
 
 
@@ -1034,7 +1005,7 @@ Physics::Transformations::Contravariant::push_forward(
   const SymmetricTensor<4, dim, Number> &H,
   const Tensor<2, dim, Number> &         F)
 {
-  return internal::Physics::transformation_contraction(H, F);
+  return Physics::Transformations::basis_transformation(H, F);
 }
 
 
@@ -1045,7 +1016,7 @@ Physics::Transformations::Contravariant::pull_back(
   const Tensor<1, dim, Number> &v,
   const Tensor<2, dim, Number> &F)
 {
-  return internal::Physics::transformation_contraction(v, invert(F));
+  return Physics::Transformations::basis_transformation(v, invert(F));
 }
 
 
@@ -1056,7 +1027,7 @@ Physics::Transformations::Contravariant::pull_back(
   const Tensor<2, dim, Number> &t,
   const Tensor<2, dim, Number> &F)
 {
-  return internal::Physics::transformation_contraction(t, invert(F));
+  return Physics::Transformations::basis_transformation(t, invert(F));
 }
 
 
@@ -1067,7 +1038,7 @@ Physics::Transformations::Contravariant::pull_back(
   const SymmetricTensor<2, dim, Number> &t,
   const Tensor<2, dim, Number> &         F)
 {
-  return internal::Physics::transformation_contraction(t, invert(F));
+  return Physics::Transformations::basis_transformation(t, invert(F));
 }
 
 
@@ -1078,7 +1049,7 @@ Physics::Transformations::Contravariant::pull_back(
   const Tensor<4, dim, Number> &h,
   const Tensor<2, dim, Number> &F)
 {
-  return internal::Physics::transformation_contraction(h, invert(F));
+  return Physics::Transformations::basis_transformation(h, invert(F));
 }
 
 
@@ -1089,7 +1060,7 @@ Physics::Transformations::Contravariant::pull_back(
   const SymmetricTensor<4, dim, Number> &h,
   const Tensor<2, dim, Number> &         F)
 {
-  return internal::Physics::transformation_contraction(h, invert(F));
+  return Physics::Transformations::basis_transformation(h, invert(F));
 }
 
 
@@ -1100,7 +1071,8 @@ Physics::Transformations::Covariant::push_forward(
   const Tensor<1, dim, Number> &V,
   const Tensor<2, dim, Number> &F)
 {
-  return internal::Physics::transformation_contraction(V, transpose(invert(F)));
+  return Physics::Transformations::basis_transformation(V,
+                                                        transpose(invert(F)));
 }
 
 
@@ -1111,7 +1083,8 @@ Physics::Transformations::Covariant::push_forward(
   const Tensor<2, dim, Number> &T,
   const Tensor<2, dim, Number> &F)
 {
-  return internal::Physics::transformation_contraction(T, transpose(invert(F)));
+  return Physics::Transformations::basis_transformation(T,
+                                                        transpose(invert(F)));
 }
 
 
@@ -1122,7 +1095,8 @@ Physics::Transformations::Covariant::push_forward(
   const SymmetricTensor<2, dim, Number> &T,
   const Tensor<2, dim, Number> &         F)
 {
-  return internal::Physics::transformation_contraction(T, transpose(invert(F)));
+  return Physics::Transformations::basis_transformation(T,
+                                                        transpose(invert(F)));
 }
 
 
@@ -1133,7 +1107,8 @@ Physics::Transformations::Covariant::push_forward(
   const Tensor<4, dim, Number> &H,
   const Tensor<2, dim, Number> &F)
 {
-  return internal::Physics::transformation_contraction(H, transpose(invert(F)));
+  return Physics::Transformations::basis_transformation(H,
+                                                        transpose(invert(F)));
 }
 
 
@@ -1144,7 +1119,8 @@ Physics::Transformations::Covariant::push_forward(
   const SymmetricTensor<4, dim, Number> &H,
   const Tensor<2, dim, Number> &         F)
 {
-  return internal::Physics::transformation_contraction(H, transpose(invert(F)));
+  return Physics::Transformations::basis_transformation(H,
+                                                        transpose(invert(F)));
 }
 
 
@@ -1154,7 +1130,7 @@ inline Tensor<1, dim, Number>
 Physics::Transformations::Covariant::pull_back(const Tensor<1, dim, Number> &v,
                                                const Tensor<2, dim, Number> &F)
 {
-  return internal::Physics::transformation_contraction(v, transpose(F));
+  return Physics::Transformations::basis_transformation(v, transpose(F));
 }
 
 
@@ -1164,7 +1140,7 @@ inline Tensor<2, dim, Number>
 Physics::Transformations::Covariant::pull_back(const Tensor<2, dim, Number> &t,
                                                const Tensor<2, dim, Number> &F)
 {
-  return internal::Physics::transformation_contraction(t, transpose(F));
+  return Physics::Transformations::basis_transformation(t, transpose(F));
 }
 
 
@@ -1175,7 +1151,7 @@ Physics::Transformations::Covariant::pull_back(
   const SymmetricTensor<2, dim, Number> &t,
   const Tensor<2, dim, Number> &         F)
 {
-  return internal::Physics::transformation_contraction(t, transpose(F));
+  return Physics::Transformations::basis_transformation(t, transpose(F));
 }
 
 
@@ -1185,7 +1161,7 @@ inline Tensor<4, dim, Number>
 Physics::Transformations::Covariant::pull_back(const Tensor<4, dim, Number> &h,
                                                const Tensor<2, dim, Number> &F)
 {
-  return internal::Physics::transformation_contraction(h, transpose(F));
+  return Physics::Transformations::basis_transformation(h, transpose(F));
 }
 
 
@@ -1196,7 +1172,7 @@ Physics::Transformations::Covariant::pull_back(
   const SymmetricTensor<4, dim, Number> &h,
   const Tensor<2, dim, Number> &         F)
 {
-  return internal::Physics::transformation_contraction(h, transpose(F));
+  return Physics::Transformations::basis_transformation(h, transpose(F));
 }
 
 
@@ -1313,6 +1289,127 @@ Physics::Transformations::nansons_formula(const Tensor<1, dim, Number> &N,
   return cofactor(F) * N;
 }
 
+
+template <int dim, typename Number>
+inline Tensor<1, dim, Number>
+Physics::Transformations::basis_transformation(const Tensor<1, dim, Number> &V,
+                                               const Tensor<2, dim, Number> &B)
+{
+  return contract<1, 0>(B, V);
+}
+
+
+
+template <int dim, typename Number>
+inline Tensor<2, dim, Number>
+Physics::Transformations::basis_transformation(const Tensor<2, dim, Number> &T,
+                                               const Tensor<2, dim, Number> &B)
+{
+  return contract<1, 0>(B, contract<1, 1>(T, B));
+}
+
+
+
+template <int dim, typename Number>
+inline SymmetricTensor<2, dim, Number>
+Physics::Transformations::basis_transformation(
+  const SymmetricTensor<2, dim, Number> &T,
+  const Tensor<2, dim, Number> &         B)
+{
+  Tensor<2, dim, Number> tmp_1;
+  for (unsigned int i = 0; i < dim; ++i)
+    for (unsigned int J = 0; J < dim; ++J)
+      // Loop over I but complex.h defines a macro I, so use I_ instead
+      for (unsigned int I_ = 0; I_ < dim; ++I_)
+        tmp_1[i][J] += B[i][I_] * T[I_][J];
+
+  SymmetricTensor<2, dim, Number> out;
+  for (unsigned int i = 0; i < dim; ++i)
+    for (unsigned int j = i; j < dim; ++j)
+      for (unsigned int J = 0; J < dim; ++J)
+        out[i][j] += B[j][J] * tmp_1[i][J];
+
+  return out;
+}
+
+
+
+template <int dim, typename Number>
+inline Tensor<4, dim, Number>
+Physics::Transformations::basis_transformation(const Tensor<4, dim, Number> &H,
+                                               const Tensor<2, dim, Number> &B)
+{
+  // This contraction order and indexing might look a bit dubious, so a
+  // quick explanation as to what's going on is probably in order:
+  //
+  // When the contract() function operates on the inner indices, the
+  // result has the inner index and outer index transposed, i.e.
+  // contract<2,1>(H,F) implies
+  // T_{IJLk} = (H_{IJMN} F_{mM}) \delta_{mL} \delta_{Nk}
+  // rather than T_{IJkL} (the desired result).
+  // So, in effect, contraction of the 3rd (inner) index with F as the
+  // second argument results in its transposition with respect to its
+  // adjacent neighbor. This is due to the position of the argument F,
+  // leading to the free index being on the right hand side of the result.
+  // However, given that we can do two transformations from the LHS of H
+  // and two from the right we can undo the otherwise erroneous
+  // swapping of the outer indices upon application of the second
+  // sets of contractions.
+  //
+  // Note: Its significantly quicker (in 3d) to push forward
+  // each index individually
+  return contract<1, 1>(
+    B, contract<1, 1>(B, contract<2, 1>(contract<2, 1>(H, B), B)));
+}
+
+
+
+template <int dim, typename Number>
+inline SymmetricTensor<4, dim, Number>
+Physics::Transformations::basis_transformation(
+  const SymmetricTensor<4, dim, Number> &H,
+  const Tensor<2, dim, Number> &         B)
+{
+  // The first and last transformation operations respectively
+  // break and recover the symmetry properties of the tensors.
+  // We also want to perform a minimal number of operations here
+  // and avoid some complications related to the transposition of
+  // tensor indices when contracting inner indices using the contract()
+  // function. (For an explanation of the contraction operations,
+  // please see the note in the equivalent function for standard
+  // Tensors.) So what we'll do here is manually perform the first
+  // and last contractions that break/recover the tensor symmetries
+  // on the inner indices, and use the contract() function only on
+  // the outer indices.
+  //
+  // Note: Its significantly quicker (in 3d) to push forward
+  // each index individually
+
+  // Push forward (inner) index 1
+  Tensor<4, dim, Number> tmp;
+  // Loop over I but complex.h defines a macro I, so use I_ instead
+  for (unsigned int I_ = 0; I_ < dim; ++I_)
+    for (unsigned int j = 0; j < dim; ++j)
+      for (unsigned int K = 0; K < dim; ++K)
+        for (unsigned int L = 0; L < dim; ++L)
+          for (unsigned int J = 0; J < dim; ++J)
+            tmp[I_][j][K][L] += B[j][J] * H[I_][J][K][L];
+
+  // Push forward (outer) indices 0 and 3
+  tmp = contract<1, 0>(B, contract<3, 1>(tmp, B));
+
+  // Push forward (inner) index 2
+  SymmetricTensor<4, dim, Number> out;
+  for (unsigned int i = 0; i < dim; ++i)
+    for (unsigned int j = i; j < dim; ++j)
+      for (unsigned int k = 0; k < dim; ++k)
+        for (unsigned int l = k; l < dim; ++l)
+          for (unsigned int K = 0; K < dim; ++K)
+            out[i][j][k][l] += B[k][K] * tmp[i][j][K][l];
+
+  return out;
+}
+
 #endif // DOXYGEN
 
 DEAL_II_NAMESPACE_CLOSE
