@@ -1,17 +1,18 @@
-//-----------------------------------------------------------
+// ---------------------------------------------------------------------
 //
-//    Copyright (C) 2017 - 2021 by the deal.II authors
+// Copyright (C) 2017 - 2022 by the deal.II authors
 //
-//    This file is part of the deal.II library.
+// This file is part of the deal.II library.
 //
-//    The deal.II library is free software; you can use it, redistribute
-//    it, and/or modify it under the terms of the GNU Lesser General
-//    Public License as published by the Free Software Foundation; either
-//    version 2.1 of the License, or (at your option) any later version.
-//    The full text of the license can be found in the file LICENSE.md at
-//    the top level directory of deal.II.
+// The deal.II library is free software; you can use it, redistribute
+// it, and/or modify it under the terms of the GNU Lesser General
+// Public License as published by the Free Software Foundation; either
+// version 2.1 of the License, or (at your option) any later version.
+// The full text of the license can be found in the file LICENSE.md at
+// the top level directory of deal.II.
 //
-//---------------------------------------------------------------
+// ---------------------------------------------------------------------
+
 
 #ifndef dealii_sundials_kinsol_h
 #define dealii_sundials_kinsol_h
@@ -33,9 +34,6 @@
 #  include <boost/signals2.hpp>
 
 #  include <kinsol/kinsol.h>
-#  if DEAL_II_SUNDIALS_VERSION_LT(4, 1, 0)
-#    include <kinsol/kinsol_impl.h>
-#  endif
 #  include <nvector/nvector_serial.h>
 #  include <sundials/sundials_math.h>
 #  include <sundials/sundials_types.h>
@@ -66,10 +64,7 @@ namespace SUNDIALS
    *
    * KINSOL's Newton solver employs the inexact Newton method. As this solver
    * is intended mainly for large systems, the user is required to provide
-   * their own solver function. If a solver function is not provided, the
-   * internal dense solver of KINSOL is used. Be warned that this solver
-   * computes the Jacobian approximately, and may be efficient only for small
-   * systems.
+   * their own solver function.
    *
    * At the highest level, KINSOL implements the following iteration
    * scheme:
@@ -136,22 +131,6 @@ namespace SUNDIALS
    * algorithm is that the full Newton step tends to be taken close to the
    * solution.
    *
-   * As a user option, KINSOL permits the application of inequality
-   * constraints, $u_i > 0$ and $u_i < 0$, as well as $u_i \geq 0$ and $u_i
-   * \leq 0$, where $u_i$ is the $i$-th component of $u$. Any such constraint,
-   * or no constraint, may be imposed on each component by providing the
-   * optional functions
-   * - get_lower_than_zero_constrained_entries()
-   * - get_greater_than_zero_constrained_entries()
-   * - get_lower_equal_than_zero_constrained_entries()
-   * - get_greater_or_equal_than_zero_constrained_entries()
-   *
-   * KINSOL will reduce step lengths in order to ensure that no constraint is
-   * violated. Specifically, if a new Newton iterate will violate a constraint,
-   * the maximum step length along the Newton direction that will satisfy all
-   * constraints is found, and $\delta_n$ is scaled to take a step of that
-   * length.
-   *
    * The basic fixed-point iteration scheme implemented in KINSOL is given by:
    * - Set $u_0 =$ an initial guess
    * - For $n = 0, 1, 2, \dots$ until convergence do:
@@ -180,20 +159,17 @@ namespace SUNDIALS
    * or
    *  - iteration_function;
    *
-   * Specifying residual() allows the user to use Newton strategies (i.e.,
-   * $F(u)=0$ will be solved), while specifying iteration_function(), fixed
-   * point iteration or Picard iteration will be used (i.e., $G(u)=u$ will be
-   * solved).
+   * Specifying residual() allows the user to use Newton and Picard strategies
+   * (i.e., $F(u)=0$ will be solved), while specifying iteration_function(), a
+   * fixed point iteration will be used (i.e., $G(u)=u$ will be solved).
    *
-   * If the use of a Newton method is desired, then the user should also supply
-   *  - solve_jacobian_system;
+   * If the use of a Newton or Picard method is desired, then the user should
+   * also supply
+   *  - solve_jacobian_system or solve_with_jacobian;
    * and optionally
    *  - setup_jacobian;
    *
-   * If the solve_jacobian_system() function is not supplied, then KINSOL will
-   * use its internal dense solver for Newton methods, with approximate
-   * Jacobian. This may be very expensive for large systems. Fixed point
-   * iteration does not require the solution of any linear system.
+   * Fixed point iteration does not require the solution of any linear system.
    *
    * Also the following functions could be rewritten, to provide additional
    * scaling factors for both the solution and the residual evaluation during
@@ -398,15 +374,24 @@ namespace SUNDIALS
     };
 
     /**
-     * Constructor. It is possible to fine tune the SUNDIALS KINSOL solver by
-     * passing an AdditionalData() object that sets all of the solver
-     * parameters.
+     * Constructor, with class parameters set by the AdditionalData object.
      *
      * @param data KINSOL configuration data
-     * @param mpi_comm MPI communicator
+     *
+     * @note With SUNDIALS 6 and later this constructor sets up logging
+     * objects to only work on the present processor (i.e., results are only
+     * communicated over MPI_COMM_SELF).
      */
-    KINSOL(const AdditionalData &data     = AdditionalData(),
-           const MPI_Comm &      mpi_comm = MPI_COMM_WORLD);
+    KINSOL(const AdditionalData &data = AdditionalData());
+
+    /**
+     * Constructor.
+     *
+     * @param data KINSOL configuration data
+     * @param mpi_comm MPI Communicator over which logging operations are
+     * computed. Only used in SUNDIALS 6 and newer.
+     */
+    KINSOL(const AdditionalData &data, const MPI_Comm &mpi_comm);
 
     /**
      * Destructor.
@@ -567,7 +552,7 @@ namespace SUNDIALS
      *   AdditionalData::maximum_newton_step variable to one, indicating
      *   that the Jacobian should be re-computed in every iteration.
      */
-    DEAL_II_DEPRECATED_EARLY
+    DEAL_II_DEPRECATED
     std::function<int(const VectorType &ycur,
                       const VectorType &fcur,
                       const VectorType &rhs,
@@ -706,11 +691,23 @@ namespace SUNDIALS
      */
     AdditionalData data;
 
+    /**
+     * The MPI communicator to be used by this solver, if any.
+     */
+    MPI_Comm mpi_communicator;
+
     /**
      * KINSOL memory object.
      */
     void *kinsol_mem;
 
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+    /**
+     * A context object associated with the KINSOL solver.
+     */
+    SUNContext kinsol_ctx;
+#  endif
+
     /**
      * KINSOL solution vector.
      */
@@ -726,11 +723,6 @@ namespace SUNDIALS
      */
     N_Vector f_scale;
 
-    /**
-     * MPI communicator. SUNDIALS solver runs happily in parallel.
-     */
-    MPI_Comm communicator;
-
     /**
      * Memory pool of vectors.
      */
