@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2014 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -8,8 +8,8 @@
 // it, and/or modify it under the terms of the GNU Lesser General
 // Public License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// The full text of the license can be found in the file LICENSE at
-// the top level of the deal.II distribution.
+// The full text of the license can be found in the file LICENSE.md at
+// the top level directory of deal.II.
 //
 // ---------------------------------------------------------------------
 
@@ -18,21 +18,21 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-ConditionalOStream::ConditionalOStream(std::ostream &stream,
-                                       const bool    active)
-  :
-  output_stream (stream),
-  active_flag(active)
+ConditionalOStream::ConditionalOStream(std::ostream &stream, const bool active)
+  : output_stream(stream)
+  , active_flag(active)
 {}
 
 
-void ConditionalOStream::set_condition(bool flag)
+void
+ConditionalOStream::set_condition(bool flag)
 {
   active_flag = flag;
 }
 
 
-bool ConditionalOStream::is_active() const
+bool
+ConditionalOStream::is_active() const
 {
   return active_flag;
 }
