@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2021 by the deal.II authors
+// Copyright (C) 1998 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -59,23 +59,37 @@ namespace deal_II_exceptions
     bool allow_abort_on_exception = true;
   } // namespace internals
 
+
+
   void
   set_additional_assert_output(const std::string &p)
   {
     internals::get_additional_assert_output() = p;
   }
 
+
+
   void
   suppress_stacktrace_in_exceptions()
   {
     internals::show_stacktrace = false;
   }
 
+
+
   void
   disable_abort_on_exception()
   {
     internals::allow_abort_on_exception = false;
   }
+
+
+
+  void
+  enable_abort_on_exception()
+  {
+    internals::allow_abort_on_exception = true;
+  }
 } // namespace deal_II_exceptions
 
 
@@ -86,7 +100,6 @@ ExceptionBase::ExceptionBase()
   , function("")
   , cond("")
   , exc("")
-  , stacktrace(nullptr)
   , n_stacktrace_frames(0)
   , what_str("")
 {
@@ -103,27 +116,22 @@ ExceptionBase::ExceptionBase(const ExceptionBase &exc)
   , function(exc.function)
   , cond(exc.cond)
   , exc(exc.exc)
-  , stacktrace(nullptr)
-  , // don't copy stacktrace to avoid double de-allocation problem
-  n_stacktrace_frames(0)
+  , n_stacktrace_frames(exc.n_stacktrace_frames)
   , what_str("") // don't copy the error message, it gets generated dynamically
                  // by what()
 {
 #ifdef DEAL_II_HAVE_GLIBC_STACKTRACE
-  std::fill(std::begin(raw_stacktrace), std::end(raw_stacktrace), nullptr);
+  // Copy the raw_stacktrace pointers. We don't own them, they just point to the
+  // addresses of symbols in the executable's/library's symbol tables -- and as
+  // a consequence, it is safe to copy these pointers
+  std::copy(std::begin(exc.raw_stacktrace),
+            std::end(exc.raw_stacktrace),
+            std::begin(raw_stacktrace));
 #endif
 }
 
 
 
-ExceptionBase::~ExceptionBase() noexcept
-{
-  free(stacktrace); // free(nullptr) is allowed
-  stacktrace = nullptr;
-}
-
-
-
 void
 ExceptionBase::set_fields(const char *f,
                           const int   l,
@@ -147,29 +155,20 @@ ExceptionBase::set_fields(const char *f,
 #endif
 }
 
+
+
 const char *
 ExceptionBase::what() const noexcept
 {
   // If no error c_string was generated so far, do it now:
   if (what_str.empty())
-    {
-#ifdef DEAL_II_HAVE_GLIBC_STACKTRACE
-      // We have deferred the symbol lookup to this point to avoid costly
-      // runtime penalties due to linkage of external libraries by
-      // backtrace_symbols.
-
-      // first delete old stacktrace if necessary
-      free(stacktrace); // free(nullptr) is allowed
-      stacktrace = backtrace_symbols(raw_stacktrace, n_stacktrace_frames);
-#endif
-
-      generate_message();
-    }
+    generate_message();
 
   return what_str.c_str();
 }
 
 
+
 const char *
 ExceptionBase::get_exc_name() const
 {
@@ -227,6 +226,15 @@ ExceptionBase::print_stack_trace(std::ostream &out) const
   if (deal_II_exceptions::internals::show_stacktrace == false)
     return;
 
+  char **stacktrace = nullptr;
+#ifdef DEAL_II_HAVE_GLIBC_STACKTRACE
+  // We have deferred the symbol lookup to this point to avoid costly
+  // runtime penalties due to linkage of external libraries by
+  // backtrace_symbols.
+  stacktrace = backtrace_symbols(raw_stacktrace, n_stacktrace_frames);
+#endif
+
+
   // if there is a stackframe stored, print it
   out << std::endl;
   out << "Stacktrace:" << std::endl << "-----------" << std::endl;
@@ -303,6 +311,9 @@ ExceptionBase::print_stack_trace(std::ostream &out) const
       if (functionname == "main")
         break;
     }
+
+  free(stacktrace); // free(nullptr) is allowed
+  stacktrace = nullptr;
 }
 
 
@@ -414,7 +425,7 @@ namespace StandardExceptions
           << std::endl;
       }
     out << "The numerical value of the original error code is " << error_code
-        << "." << std::endl;
+        << '.' << std::endl;
   }
 #endif // DEAL_II_WITH_MPI
 
@@ -466,7 +477,7 @@ namespace deal_II_exceptions
 #ifdef DEAL_II_WITH_MPI
       int is_initialized;
       MPI_Initialized(&is_initialized);
-      if (is_initialized)
+      if (is_initialized != 0)
         {
           // do the same as in Utilities::MPI::n_mpi_processes() here,
           // but without error checking to not throw again.
