@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2020 by the deal.II authors
+// Copyright (C) 2000 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,13 +18,24 @@
 
 #include <algorithm>
 #include <cstdlib> // for std::getenv
+#include <mutex>
 #include <thread>
 
 #ifdef DEAL_II_WITH_TBB
-#  include <tbb/task_scheduler_init.h>
+#  ifdef DEAL_II_TBB_WITH_ONEAPI
+#    include <tbb/global_control.h>
+#  else
+#    include <tbb/task_scheduler_init.h>
+#  endif
 #endif
 
 
+#ifdef DEAL_II_WITH_TASKFLOW
+DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
+#  include <taskflow/taskflow.hpp>
+DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
+#endif
+
 DEAL_II_NAMESPACE_OPEN
 
 
@@ -86,11 +97,30 @@ MultithreadInfo::set_thread_limit(const unsigned int max_threads)
     n_max_threads = n_cores();
 
 #ifdef DEAL_II_WITH_TBB
+#  ifdef DEAL_II_TBB_WITH_ONEAPI
+  // tbb::global_control is a class that affects the specified behavior of
+  // tbb during its lifetime. Thus, in order to set a global thread limit
+  // for tbb we have to maintain the object throughout the execution of the
+  // program. We do this by maintaining a static std::unique_ptr.
+  //
+  // A std::unique_ptr is a good choice here because tbb::global_control
+  // does not provide a mechanism to override its setting - we can only
+  // delete the old and replace it with a new one.
+  static std::unique_ptr<tbb::global_control> tbb_global_control;
+  tbb_global_control = std::make_unique<tbb::global_control>(
+    tbb::global_control::max_allowed_parallelism, n_max_threads);
+
+#  else
   // Initialize the scheduler and destroy the old one before doing so
   static tbb::task_scheduler_init dummy(tbb::task_scheduler_init::deferred);
   if (dummy.is_active())
     dummy.terminate();
   dummy.initialize(n_max_threads);
+#  endif
+#endif
+
+#ifdef DEAL_II_WITH_TASKFLOW
+  executor = std::make_unique<tf::Executor>(n_max_threads);
 #endif
 }
 
@@ -125,14 +155,31 @@ MultithreadInfo::memory_consumption()
 void
 MultithreadInfo::initialize_multithreading()
 {
-  static bool done = false;
-  if (done)
-    return;
-
-  MultithreadInfo::set_thread_limit(numbers::invalid_unsigned_int);
-  done = true;
+  static std::once_flag is_initialized;
+  std::call_once(is_initialized, []() {
+    MultithreadInfo::set_thread_limit(numbers::invalid_unsigned_int);
+  });
 }
 
+
+
+#ifdef DEAL_II_WITH_TASKFLOW
+tf::Executor &
+MultithreadInfo::get_taskflow_executor()
+{
+  // This should not trigger in normal user code, because we initialize the
+  // Executor in the static DoOnce struct at the end of this file unless you
+  // ask for the Executor before this static object gets constructed.
+  Assert(
+    executor.get() != nullptr,
+    ExcMessage(
+      "Please initialize multithreading using MultithreadInfo::set_thread_limit() first."));
+  return *(executor.get());
+}
+
+std::unique_ptr<tf::Executor> MultithreadInfo::executor = nullptr;
+#endif
+
 unsigned int MultithreadInfo::n_max_threads = numbers::invalid_unsigned_int;
 
 namespace
