@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2021 by the deal.II authors
+// Copyright (C) 1998 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -333,6 +333,39 @@ namespace
       }
   }
 
+  /**
+   * Demangle all parameters recursively and attach them to @p tree_out.
+   * @p is_parameter_or_alias_node indicates, whether a given node
+   * @p tree_in is a parameter node or an alias node (as opposed to being
+   * a subsection).
+   */
+  void
+  recursively_demangle(const boost::property_tree::ptree &tree_in,
+                       boost::property_tree::ptree &      tree_out,
+                       const bool is_parameter_or_alias_node = false)
+  {
+    for (const auto &p : tree_in)
+      {
+        if (is_parameter_or_alias_node)
+          {
+            tree_out.put_child(p.first, p.second);
+          }
+        else
+          {
+            boost::property_tree::ptree temp;
+
+            if (const auto val = p.second.get_value_optional<std::string>())
+              temp.put_value<std::string>(*val);
+
+            recursively_demangle(p.second,
+                                 temp,
+                                 is_parameter_node(p.second) ||
+                                   is_alias_node(p.second));
+            tree_out.put_child(demangle(p.first), temp);
+          }
+      }
+  }
+
   /**
    * Assert validity of of given output @p style.
    */
@@ -401,7 +434,7 @@ ParameterHandler::parse_input(std::istream &     input,
                               const std::string &last_line,
                               const bool         skip_undefined)
 {
-  AssertThrow(input, ExcIO());
+  AssertThrow(input.fail() == false, ExcIO());
 
   // store subsections we are currently in
   const std::vector<std::string> saved_path = subsection_path;
@@ -457,15 +490,15 @@ ParameterHandler::parse_input(std::istream &     input,
 
       // If we see the line which is the same as @p last_line ,
       // terminate the parsing.
-      if (last_line.length() != 0 && input_line == last_line)
+      if (last_line.size() != 0 && input_line == last_line)
         break;
 
       // Check whether or not the current line should be joined with the next
       // line before calling scan_line.
-      if (input_line.length() != 0 &&
-          input_line.find_last_of('\\') == input_line.length() - 1)
+      if (input_line.size() != 0 &&
+          input_line.find_last_of('\\') == input_line.size() - 1)
         {
-          input_line.erase(input_line.length() - 1); // remove the last '\'
+          input_line.erase(input_line.size() - 1); // remove the last '\'
           is_concatenated = true;
 
           fully_concatenated_line += input_line;
@@ -700,7 +733,7 @@ void
 ParameterHandler::parse_input_from_xml(std::istream &in,
                                        const bool    skip_undefined)
 {
-  AssertThrow(in, ExcIO());
+  AssertThrow(in.fail() == false, ExcIO());
   // read the XML tree assuming that (as we
   // do in print_parameters(XML) it has only
   // a single top-level node called
@@ -756,7 +789,7 @@ void
 ParameterHandler::parse_input_from_json(std::istream &in,
                                         const bool    skip_undefined)
 {
-  AssertThrow(in, ExcIO());
+  AssertThrow(in.fail() == false, ExcIO());
 
   boost::property_tree::ptree node_tree;
   // This boost function will raise an exception if this is not a valid JSON
@@ -1250,7 +1283,7 @@ std::ostream &
 ParameterHandler::print_parameters(std::ostream &    out,
                                    const OutputStyle style) const
 {
-  AssertThrow(out, ExcIO());
+  AssertThrow(out.fail() == false, ExcIO());
 
   assert_validity_of_output_style(style);
 
@@ -1260,7 +1293,7 @@ ParameterHandler::print_parameters(std::ostream &    out,
   boost::property_tree::ptree current_entries = *entries.get();
 
   // Sort parameters alphabetically, if needed.
-  if (!(style & KeepDeclarationOrder))
+  if ((style & KeepDeclarationOrder) == 0)
     {
       // Dive recursively into the subsections,
       // starting from the top level.
@@ -1282,13 +1315,13 @@ ParameterHandler::print_parameters(std::ostream &    out,
   // first
 
   // explicitly compress the tree if requested
-  if ((style & Short) && (style & (XML | JSON)))
+  if (((style & Short) != 0) && ((style & (XML | JSON)) != 0))
     {
       // modify the copy of the tree
       recursively_compress_tree(current_entries);
     }
 
-  if (style & XML)
+  if ((style & XML) != 0)
     {
       // call the writer function and exit as there is nothing
       // further to do down in this function
@@ -1306,29 +1339,31 @@ ParameterHandler::print_parameters(std::ostream &    out,
       return out;
     }
 
-  if (style & JSON)
+  if ((style & JSON) != 0)
     {
-      write_json(out, current_entries);
+      boost::property_tree::ptree current_entries_damangled;
+      recursively_demangle(current_entries, current_entries_damangled);
+      write_json(out, current_entries_damangled);
       return out;
     }
 
   // for all of the other formats, print a preamble:
-  if ((style & Short) && (style & Text))
+  if (((style & Short) != 0) && ((style & Text) != 0))
     {
       // nothing to do
     }
-  else if (style & Text)
+  else if ((style & Text) != 0)
     {
       out << "# Listing of Parameters" << std::endl
           << "# ---------------------" << std::endl;
     }
-  else if (style & LaTeX)
+  else if ((style & LaTeX) != 0)
     {
       out << "\\subsection{Global parameters}" << std::endl;
       out << "\\label{parameters:global}" << std::endl;
       out << std::endl << std::endl;
     }
-  else if (style & Description)
+  else if ((style & Description) != 0)
     {
       out << "Listing of Parameters:" << std::endl << std::endl;
     }
@@ -1369,7 +1404,7 @@ ParameterHandler::print_parameters(
     output_style = style | LaTeX;
 
   std::ofstream out(filename);
-  AssertThrow(out, ExcIO());
+  AssertThrow(out.fail() == false, ExcIO());
   print_parameters(out, output_style);
 }
 
@@ -1383,7 +1418,7 @@ ParameterHandler::recursively_print_parameters(
   const unsigned int                 indent_level,
   std::ostream &                     out) const
 {
-  AssertThrow(out, ExcIO());
+  AssertThrow(out.fail() == false, ExcIO());
 
   // this function should not be necessary for XML or JSON output...
   Assert(!(style & (XML | JSON)), ExcInternalError());
@@ -1393,9 +1428,9 @@ ParameterHandler::recursively_print_parameters(
 
   unsigned int overall_indent_level = indent_level;
 
-  const bool is_short = style & Short;
+  const bool is_short = (style & Short) != 0;
 
-  if (style & Text)
+  if ((style & Text) != 0)
     {
       // first find out the longest entry name to be able to align the
       // equal signs to do this loop over all nodes of the current
@@ -1409,10 +1444,9 @@ ParameterHandler::recursively_print_parameters(
       for (const auto &p : current_section)
         if (is_parameter_node(p.second) == true)
           {
-            longest_name = std::max(longest_name, demangle(p.first).length());
-            longest_value =
-              std::max(longest_value,
-                       p.second.get<std::string>("value").length());
+            longest_name  = std::max(longest_name, demangle(p.first).size());
+            longest_value = std::max(longest_value,
+                                     p.second.get<std::string>("value").size());
           }
 
       // print entries one by one
@@ -1448,8 +1482,7 @@ ParameterHandler::recursively_print_parameters(
             // print name and value of this entry
             out << std::setw(overall_indent_level * 2) << ""
                 << "set " << demangle(p.first)
-                << std::setw(longest_name - demangle(p.first).length() + 1)
-                << " "
+                << std::setw(longest_name - demangle(p.first).size() + 1) << " "
                 << "= " << value;
 
             // finally print the default value, but only if it differs
@@ -1457,7 +1490,7 @@ ParameterHandler::recursively_print_parameters(
             if (!is_short &&
                 value != p.second.get<std::string>("default_value"))
               {
-                out << std::setw(longest_value - value.length() + 1) << ' '
+                out << std::setw(longest_value - value.size() + 1) << ' '
                     << "# ";
                 out << "default: "
                     << p.second.get<std::string>("default_value");
@@ -1466,7 +1499,7 @@ ParameterHandler::recursively_print_parameters(
             out << '\n';
           }
     }
-  else if (style & LaTeX)
+  else if ((style & LaTeX) != 0)
     {
       auto escape = [](const std::string &input) {
         return Patterns::internal::escape(input, Patterns::PatternBase::LaTeX);
@@ -1598,14 +1631,14 @@ ParameterHandler::recursively_print_parameters(
           out << "\\end{itemize}" << '\n';
         }
     }
-  else if (style & Description)
+  else if ((style & Description) != 0)
     {
       // first find out the longest entry name to be able to align the
       // equal signs
       std::size_t longest_name = 0;
       for (const auto &p : current_section)
         if (is_parameter_node(p.second) == true)
-          longest_name = std::max(longest_name, demangle(p.first).length());
+          longest_name = std::max(longest_name, demangle(p.first).size());
 
       // print entries one by one
       for (const auto &p : current_section)
@@ -1614,8 +1647,7 @@ ParameterHandler::recursively_print_parameters(
             // print name and value
             out << std::setw(overall_indent_level * 2) << ""
                 << "set " << demangle(p.first)
-                << std::setw(longest_name - demangle(p.first).length() + 1)
-                << " "
+                << std::setw(longest_name - demangle(p.first).size() + 1) << " "
                 << " = ";
 
             // print possible values:
@@ -1640,7 +1672,7 @@ ParameterHandler::recursively_print_parameters(
 
             // if there is a documenting string, print it as well
             if (!is_short &&
-                p.second.get<std::string>("documentation").length() != 0)
+                p.second.get<std::string>("documentation").size() != 0)
               out << std::setw(overall_indent_level * 2 + longest_name + 10)
                   << ""
                   << "(" << p.second.get<std::string>("documentation") << ")"
@@ -1664,8 +1696,9 @@ ParameterHandler::recursively_print_parameters(
       else if (is_alias_node(p.second) == false)
         ++n_sections;
 
-    if (!(style & Description) && (!((style & Text) && is_short)) &&
-        (n_parameters != 0) && (n_sections != 0))
+    if (((style & Description) == 0) &&
+        (!(((style & Text) != 0) && is_short)) && (n_parameters != 0) &&
+        (n_sections != 0))
       out << "\n\n";
   }
 
@@ -1675,12 +1708,12 @@ ParameterHandler::recursively_print_parameters(
         (is_alias_node(p.second) == false))
       {
         // first print the subsection header
-        if ((style & Text) || (style & Description))
+        if (((style & Text) != 0) || ((style & Description) != 0))
           {
             out << std::setw(overall_indent_level * 2) << ""
                 << "subsection " << demangle(p.first) << '\n';
           }
-        else if (style & LaTeX)
+        else if ((style & LaTeX) != 0)
           {
             auto escape = [](const std::string &input) {
               return Patterns::internal::escape(input,
@@ -1717,13 +1750,13 @@ ParameterHandler::recursively_print_parameters(
         recursively_print_parameters(
           tree, directory_path, style, overall_indent_level + 1, out);
 
-        if (is_short && (style & Text))
+        if (is_short && ((style & Text) != 0))
           {
             // write end of subsection.
             out << std::setw(overall_indent_level * 2) << ""
                 << "end" << '\n';
           }
-        else if (style & Text)
+        else if ((style & Text) != 0)
           {
             // write end of subsection. one blank line after each
             // subsection
@@ -1736,11 +1769,11 @@ ParameterHandler::recursively_print_parameters(
             if (overall_indent_level == 0)
               out << '\n';
           }
-        else if (style & Description)
+        else if ((style & Description) != 0)
           {
             // nothing to do
           }
-        else if (style & LaTeX)
+        else if ((style & LaTeX) != 0)
           {
             // nothing to do
           }
@@ -1775,7 +1808,7 @@ ParameterHandler::log_parameters_section(LogStream &       out,
   boost::property_tree::ptree *current_entries = entries.get();
 
   // Sort parameters alphabetically, if needed.
-  if (!(style & KeepDeclarationOrder))
+  if ((style & KeepDeclarationOrder) == 0)
     {
       sorted_entries  = *entries;
       current_entries = &sorted_entries;
@@ -1831,7 +1864,7 @@ ParameterHandler::scan_line(std::string        line,
   line = Utilities::trim(line);
 
   // if line is now empty: leave
-  if (line.length() == 0)
+  if (line.size() == 0)
     {
       return;
     }
@@ -1840,7 +1873,7 @@ ParameterHandler::scan_line(std::string        line,
            Utilities::match_at_string_start(line, "subsection "))
     {
       // delete this prefix
-      line.erase(0, std::string("subsection").length() + 1);
+      line.erase(0, std::string("subsection").size() + 1);
 
       const std::string subsection = Utilities::trim(line);
 
@@ -1859,7 +1892,7 @@ ParameterHandler::scan_line(std::string        line,
            Utilities::match_at_string_start(line, "end"))
     {
       line.erase(0, 3);
-      while ((line.size() > 0) && (std::isspace(line[0])))
+      while ((line.size() > 0) && ((std::isspace(line[0])) != 0))
         line.erase(0, 1);
 
       AssertThrow(
@@ -1951,6 +1984,15 @@ ParameterHandler::scan_line(std::string        line,
 
           // finally write the new value into the database
           entries->put(path + path_separator + "value", entry_value);
+
+          auto map_iter = entries_set_status.find(path);
+          if (map_iter != entries_set_status.end())
+            map_iter->second =
+              std::pair<bool, bool>(map_iter->second.first, true);
+          else
+            AssertThrow(false,
+                        ExcMessage("Could not find parameter " + path +
+                                   " in map entries_set_status."));
         }
       else
         {
@@ -2092,7 +2134,7 @@ MultipleParameterLoop::parse_input(std::istream &     input,
                                    const std::string &last_line,
                                    const bool         skip_undefined)
 {
-  AssertThrow(input, ExcIO());
+  AssertThrow(input.fail() == false, ExcIO());
 
   // Note that (to avoid infinite recursion) we have to explicitly call the
   // base class version of parse_input and *not* a wrapper (which may be
@@ -2278,13 +2320,13 @@ MultipleParameterLoop::Entry::split_different_values()
   // pair of braces
   if (multiple[0] == '{')
     multiple.erase(0, 1);
-  if (multiple[multiple.size() - 1] == '}')
+  if (multiple.back() == '}')
     multiple.erase(multiple.size() - 1, 1);
   // erase leading and trailing spaces
   // in multiple
-  while (std::isspace(multiple[0]))
+  while (std::isspace(multiple[0]) != 0)
     multiple.erase(0, 1);
-  while (std::isspace(multiple[multiple.size() - 1]))
+  while (std::isspace(multiple.back()) != 0)
     multiple.erase(multiple.size() - 1, 1);
 
   // delete spaces around '|'
