@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2020 by the deal.II authors
+// Copyright (C) 1999 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -36,13 +36,13 @@ namespace internal
   {
     // we don't quite know the data type in 'value', but
     // it must be one of the ones in the type list of the
-    // boost::variant. Go through this list and return
+    // std_cxx17::variant. Go through this list and return
     // the value if this happens to be a number
     //
     // first try with int
     try
       {
-        return boost::get<int>(value);
+        return std_cxx17::get<int>(value);
       }
     catch (...)
       {}
@@ -51,7 +51,7 @@ namespace internal
     // ... then with unsigned int...
     try
       {
-        return boost::get<unsigned int>(value);
+        return std_cxx17::get<unsigned int>(value);
       }
     catch (...)
       {}
@@ -59,7 +59,7 @@ namespace internal
     // ... then with std::uint64_t...
     try
       {
-        return boost::get<std::uint64_t>(value);
+        return std_cxx17::get<std::uint64_t>(value);
       }
     catch (...)
       {}
@@ -67,7 +67,7 @@ namespace internal
     // ...and finally with double precision:
     try
       {
-        return boost::get<double>(value);
+        return std_cxx17::get<double>(value);
       }
     catch (...)
       {
@@ -91,7 +91,11 @@ namespace internal
     else
       ss.setf(std::ios::fixed, std::ios::floatfield);
 
+#ifdef DEAL_II_HAVE_CXX17
+    std::visit([&ss](auto &v) { ss << v; }, value);
+#else
     ss << value;
+#endif
 
     cached_value = ss.str();
     if (cached_value.size() == 0)
@@ -105,11 +109,12 @@ namespace internal
   }
 
 
+#ifndef DEAL_II_HAVE_CXX17
   namespace Local
   {
     // see which type we can cast to, then use this type to create
     // a default constructed object
-    struct GetDefaultValue : public boost::static_visitor<>
+    struct SetValueToDefault : public boost::static_visitor<>
     {
       template <typename T>
       void
@@ -119,12 +124,22 @@ namespace internal
       }
     };
   } // namespace Local
+#endif
 
   TableEntry
   TableEntry::get_default_constructed_copy() const
   {
     TableEntry new_entry = *this;
-    boost::apply_visitor(Local::GetDefaultValue(), new_entry.value);
+#ifndef DEAL_II_HAVE_CXX17
+    boost::apply_visitor(Local::SetValueToDefault(), new_entry.value);
+#else
+    // Let std::visit figure out which data type is actually stored,
+    // and then set the object so stored to a default-constructed
+    // one.
+    std::visit([](
+                 auto &arg) { arg = std::remove_reference_t<decltype(arg)>(); },
+               new_entry.value);
+#endif
 
     return new_entry;
   }
@@ -171,7 +186,7 @@ TableHandler::Column::pad_column_below(const unsigned int size)
       entry.cache_string(scientific, precision);
       max_length =
         std::max(max_length,
-                 static_cast<unsigned int>(entry.get_cached_string().length()));
+                 static_cast<unsigned int>(entry.get_cached_string().size()));
     }
 }
 
@@ -186,7 +201,7 @@ TableHandler::Column::invalidate_cache()
       entry.cache_string(this->scientific, this->precision);
       max_length =
         std::max(max_length,
-                 static_cast<unsigned int>(entry.get_cached_string().length()));
+                 static_cast<unsigned int>(entry.get_cached_string().size()));
     }
 }
 
@@ -234,8 +249,7 @@ TableHandler::start_new_row()
         entry.cache_string(column.second.scientific, column.second.precision);
         column.second.max_length =
           std::max(column.second.max_length,
-                   static_cast<unsigned int>(
-                     entry.get_cached_string().length()));
+                   static_cast<unsigned int>(entry.get_cached_string().size()));
       }
 }
 
@@ -254,7 +268,7 @@ TableHandler::add_column_to_supercolumn(const std::string &key,
 {
   Assert(columns.count(key), ExcColumnNotExistent(key));
 
-  if (!supercolumns.count(superkey))
+  if (supercolumns.count(superkey) == 0u)
     {
       std::pair<std::string, std::vector<std::string>> new_column(
         superkey, std::vector<std::string>());
@@ -277,7 +291,7 @@ TableHandler::add_column_to_supercolumn(const std::string &key,
         column_order.erase(order_iter);
     }
 
-  if (supercolumns.count(superkey))
+  if (supercolumns.count(superkey) != 0u)
     {
       supercolumns[superkey].push_back(key);
       // By default set the
@@ -383,7 +397,7 @@ TableHandler::set_scientific(const std::string &key, const bool scientific)
 void
 TableHandler::write_text(std::ostream &out, const TextOutputFormat format) const
 {
-  AssertThrow(out, ExcIO());
+  AssertThrow(out.fail() == false, ExcIO());
   boost::io::ios_flags_saver restore_flags(out);
 
   // first pad the table from below if necessary
@@ -430,7 +444,7 @@ TableHandler::write_text(std::ostream &out, const TextOutputFormat format) const
               const std::string &key = sel_columns[j];
               column_widths[j] =
                 std::max(column_widths[j],
-                         static_cast<unsigned int>(key.length()));
+                         static_cast<unsigned int>(key.size()));
               out << std::setw(column_widths[j]);
               out << key << " | ";
             }
@@ -529,7 +543,7 @@ TableHandler::write_text(std::ostream &out, const TextOutputFormat format) const
               }
 
               // header is longer than the column(s) under it
-              if (width < key.length())
+              if (width < key.size())
                 {
                   // make the column or the last column in this
                   // supercolumn wide enough
@@ -548,18 +562,18 @@ TableHandler::write_text(std::ostream &out, const TextOutputFormat format) const
                     {
                       if (sel_columns[i] == colname)
                         {
-                          column_widths[i] += key.length() - width;
+                          column_widths[i] += key.size() - width;
                           break;
                         }
                     }
 
-                  width = key.length();
+                  width = key.size();
                 }
 
               // now write key. try to center it somehow
-              const unsigned int front_padding = (width - key.length()) / 2,
+              const unsigned int front_padding = (width - key.size()) / 2,
                                  rear_padding =
-                                   (width - key.length()) - front_padding;
+                                   (width - key.size()) - front_padding;
               for (unsigned int i = 0; i < front_padding; ++i)
                 out << ' ';
               out << key;
@@ -601,7 +615,7 @@ TableHandler::write_tex(std::ostream &out, const bool with_header) const
 {
   // TODO[TH]: update code similar to
   // write_text() to use the cache
-  AssertThrow(out, ExcIO());
+  AssertThrow(out.fail() == false, ExcIO());
   if (with_header)
     out << "\\documentclass[10pt]{report}" << '\n'
         << "\\usepackage{float}" << '\n'
@@ -714,7 +728,11 @@ TableHandler::write_tex(std::ostream &out, const bool with_header) const
           else
             out.setf(std::ios::fixed, std::ios::floatfield);
 
+#ifdef DEAL_II_HAVE_CXX17
+          std::visit([&out](auto &v) { out << v; }, column.entries[i].value);
+#else
           out << column.entries[i].value;
+#endif
 
           if (j < n_cols - 1)
             out << " & ";
