@@ -500,7 +500,7 @@ FE_ABF<dim>::get_dpo_vector(const unsigned int rt_order)
   // Initiative...), and we have
   // (rt_order+1)^(dim-1) DoFs per face
   unsigned int dofs_per_face = 1;
-  for (int d = 0; d < dim - 1; ++d)
+  for (unsigned int d = 0; d < dim - 1; ++d)
     dofs_per_face *= rt_order + 1;
 
   // and then there are interior dofs
