@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2012 - 2021 by the deal.II authors
+// Copyright (C) 2012 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -106,7 +106,7 @@ FE_Q_DG0<dim, spacedim>::get_name() const
   unsigned int index = 0;
 
   // Decode the support points in one coordinate direction.
-  for (unsigned int j = 0; j < dofs_per_cell; j++)
+  for (unsigned int j = 0; j < dofs_per_cell; ++j)
     {
       if ((dim > 1) ? (unit_support_points[j](1) == 0 &&
                        ((dim > 2) ? unit_support_points[j](2) == 0 : true)) :
@@ -128,7 +128,7 @@ FE_Q_DG0<dim, spacedim>::get_name() const
            "Could not decode support points in one coordinate direction."));
 
   // Check whether the support points are equidistant.
-  for (unsigned int j = 0; j < n_points; j++)
+  for (unsigned int j = 0; j < n_points; ++j)
     if (std::fabs(points[j] - static_cast<double>(j) / this->degree) > 1e-15)
       {
         type = false;
@@ -149,7 +149,7 @@ FE_Q_DG0<dim, spacedim>::get_name() const
       // Check whether the support points come from QGaussLobatto.
       const QGaussLobatto<1> points_gl(n_points);
       type = true;
-      for (unsigned int j = 0; j < n_points; j++)
+      for (unsigned int j = 0; j < n_points; ++j)
         if (points[j] != points_gl.point(j)(0))
           {
             type = false;
@@ -199,7 +199,7 @@ FE_Q_DG0<dim, spacedim>::convert_generalized_support_point_values_to_dof_values(
     }
 
   // We don't need the discontinuous function for local interpolation
-  nodal_dofs[nodal_dofs.size() - 1] = 0.;
+  nodal_dofs.back() = 0.;
 }
 
 
@@ -236,7 +236,7 @@ std::vector<bool>
 FE_Q_DG0<dim, spacedim>::get_riaf_vector(const unsigned int deg)
 {
   std::vector<bool> riaf(Utilities::fixed_power<dim>(deg + 1) + 1, false);
-  riaf[riaf.size() - 1] = true;
+  riaf.back() = true;
   return riaf;
 }
 
