@@ -23,7 +23,7 @@ DEAL_II_NAMESPACE_OPEN
 
 template <int dim, int spacedim>
 MappingQ1<dim, spacedim>::MappingQ1()
-  : MappingQGeneric<dim, spacedim>(1)
+  : MappingQ<dim, spacedim>(1)
 {}
 
 
@@ -39,8 +39,8 @@ MappingQ1<dim, spacedim>::clone() const
 
 
 template <int dim, int spacedim>
-MappingQGeneric<dim, spacedim>
-  StaticMappingQ1<dim, spacedim>::mapping = MappingQGeneric<dim, spacedim>(1);
+MappingQ<dim, spacedim>
+  StaticMappingQ1<dim, spacedim>::mapping = MappingQ<dim, spacedim>(1);
 
 
 
