@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2021 by the deal.II authors
+// Copyright (C) 2001 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -24,8 +24,8 @@
 #include <deal.II/dofs/dof_accessor.h>
 #include <deal.II/dofs/dof_handler.h>
 
+#include <deal.II/fe/mapping_q.h>
 #include <deal.II/fe/mapping_q1.h>
-#include <deal.II/fe/mapping_q_generic.h>
 
 #include <deal.II/grid/filtered_iterator.h>
 #include <deal.II/grid/grid_tools.h>
@@ -112,7 +112,7 @@ namespace GridTools
 
     // For all remaining vertices, test
     // whether they are any closer
-    for (unsigned int j = best_vertex + 1; j < vertices.size(); j++)
+    for (unsigned int j = best_vertex + 1; j < vertices.size(); ++j)
       if (used[j])
         {
           double dist = (p - vertices[j]).norm_square();
@@ -512,8 +512,7 @@ namespace GridTools
                       }
                   }
                 catch (
-                  typename MappingQGeneric<dim,
-                                           spacedim>::ExcTransformationFailed &)
+                  typename MappingQ<dim, spacedim>::ExcTransformationFailed &)
                   {
                     // ok, the transformation
                     // failed presumably
@@ -882,7 +881,7 @@ namespace GridTools
     const double layer_thickness)
   {
     std::vector<typename MeshType::active_cell_iterator>
-                      subdomain_boundary_cells, active_cell_layer_within_distance;
+      subdomain_boundary_cells, active_cell_layer_within_distance;
     std::vector<bool> vertices_outside_subdomain(
       mesh.get_triangulation().n_vertices(), false);
 
@@ -1347,8 +1346,7 @@ namespace GridTools
                       }
                   }
                 catch (
-                  typename MappingQGeneric<dim,
-                                           spacedim>::ExcTransformationFailed &)
+                  typename MappingQ<dim, spacedim>::ExcTransformationFailed &)
                   {
                     // ok, the transformation
                     // failed presumably
@@ -1460,7 +1458,7 @@ namespace GridTools
         else
           // If not, it asks for the parent of the cell, until it finds the
           // parent cell with the refinement level equal to the min_level and
-          // inserts that parent cell into the the set of uniform_cells, as the
+          // inserts that parent cell into the set of uniform_cells, as the
           // set of cells with the coarsest common refinement level.
           {
             typename Container::cell_iterator parent = *patch_cell;
@@ -2015,7 +2013,7 @@ namespace GridTools
     std::set<std::pair<CellIterator, unsigned int>> &pairs1,
     std::set<std::pair<typename identity<CellIterator>::type, unsigned int>>
       &                                          pairs2,
-    const int                                    direction,
+    const unsigned int                           direction,
     std::vector<PeriodicFacePair<CellIterator>> &matched_pairs,
     const dealii::Tensor<1, CellIterator::AccessorType::space_dimension>
       &                       offset,
@@ -2108,7 +2106,7 @@ namespace GridTools
   collect_periodic_faces(
     const MeshType &         mesh,
     const types::boundary_id b_id,
-    const int                direction,
+    const unsigned int       direction,
     std::vector<PeriodicFacePair<typename MeshType::cell_iterator>>
       &                                         matched_pairs,
     const Tensor<1, MeshType::space_dimension> &offset,
@@ -2190,7 +2188,7 @@ namespace GridTools
     const MeshType &         mesh,
     const types::boundary_id b_id1,
     const types::boundary_id b_id2,
-    const int                direction,
+    const unsigned int       direction,
     std::vector<PeriodicFacePair<typename MeshType::cell_iterator>>
       &                                         matched_pairs,
     const Tensor<1, MeshType::space_dimension> &offset,
@@ -2277,7 +2275,7 @@ namespace GridTools
   inline bool
   orthogonal_equality(const Point<spacedim> &    point1,
                       const Point<spacedim> &    point2,
-                      const int                  direction,
+                      const unsigned int         direction,
                       const Tensor<1, spacedim> &offset,
                       const FullMatrix<double> & matrix)
   {
@@ -2288,15 +2286,15 @@ namespace GridTools
     Point<spacedim> distance;
 
     if (matrix.m() == spacedim)
-      for (int i = 0; i < spacedim; ++i)
-        for (int j = 0; j < spacedim; ++j)
+      for (unsigned int i = 0; i < spacedim; ++i)
+        for (unsigned int j = 0; j < spacedim; ++j)
           distance(i) += matrix(i, j) * point1(j);
     else
       distance = point1;
 
     distance += offset - point2;
 
-    for (int i = 0; i < spacedim; ++i)
+    for (unsigned int i = 0; i < spacedim; ++i)
       {
         // Only compare coordinate-components != direction:
         if (i == direction)
@@ -2411,11 +2409,12 @@ namespace GridTools
 
 
   template <typename FaceIterator>
-  inline bool orthogonal_equality(
+  inline bool
+  orthogonal_equality(
     std::bitset<3> &                                              orientation,
     const FaceIterator &                                          face1,
     const FaceIterator &                                          face2,
-    const int                                                     direction,
+    const unsigned int                                            direction,
     const Tensor<1, FaceIterator::AccessorType::space_dimension> &offset,
     const FullMatrix<double> &                                    matrix)
   {
@@ -2467,7 +2466,7 @@ namespace GridTools
   orthogonal_equality(
     const FaceIterator &                                          face1,
     const FaceIterator &                                          face2,
-    const int                                                     direction,
+    const unsigned int                                            direction,
     const Tensor<1, FaceIterator::AccessorType::space_dimension> &offset,
     const FullMatrix<double> &                                    matrix)
   {
