@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2018 by the deal.II authors
+// Copyright (C) 2006 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -25,26 +25,23 @@ namespace internal
 {
   namespace TriangulationImplementation
   {
+    TriaFaces::TriaFaces(const unsigned int dim)
+      : dim(dim)
+      , quads(2)
+      , lines(1)
+    {}
+
     std::size_t
-    TriaFaces<1>::memory_consumption() const
+    TriaFaces::memory_consumption() const
     {
+      if (dim == 2)
+        return MemoryConsumption::memory_consumption(lines);
+      if (dim == 3)
+        return (MemoryConsumption::memory_consumption(quads) +
+                MemoryConsumption::memory_consumption(lines));
+
       return 0;
     }
-
-
-    std::size_t
-    TriaFaces<2>::memory_consumption() const
-    {
-      return MemoryConsumption::memory_consumption(lines);
-    }
-
-
-    std::size_t
-    TriaFaces<3>::memory_consumption() const
-    {
-      return (MemoryConsumption::memory_consumption(quads) +
-              MemoryConsumption::memory_consumption(lines));
-    }
   } // namespace TriangulationImplementation
 } // namespace internal
 
