@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2017 by the deal.II authors
+// Copyright (C) 2016 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -8,8 +8,8 @@
 // it, and/or modify it under the terms of the GNU Lesser General
 // Public License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// The full text of the license can be found in the file LICENSE at
-// the top level of the deal.II distribution.
+// The full text of the license can be found in the file LICENSE.md at
+// the top level directory of deal.II.
 //
 // ---------------------------------------------------------------------
 
@@ -29,18 +29,21 @@ namespace LinearAlgebra
 {
   namespace distributed
   {
-#define TEMPL_COPY_CONSTRUCTOR(S1,S2)                                   \
-  template Vector<S1>& Vector<S1>::operator=<S2> (const Vector<S2> &)
+#define TEMPL_COPY_CONSTRUCTOR(S1, S2)                  \
+  template Vector<S1, ::dealii::MemorySpace::Host>      \
+    &Vector<S1, ::dealii::MemorySpace::Host>::operator= \
+      <S2>(const Vector<S2, ::dealii::MemorySpace::Host> &)
 
-    TEMPL_COPY_CONSTRUCTOR(double,float);
-    TEMPL_COPY_CONSTRUCTOR(float,double);
-
-    TEMPL_COPY_CONSTRUCTOR(std::complex<double>,std::complex<float>);
-    TEMPL_COPY_CONSTRUCTOR(std::complex<float>,std::complex<double>);
+    TEMPL_COPY_CONSTRUCTOR(double, float);
+    TEMPL_COPY_CONSTRUCTOR(float, double);
+#ifdef DEAL_II_WITH_COMPLEX_VALUES
+    TEMPL_COPY_CONSTRUCTOR(std::complex<double>, std::complex<float>);
+    TEMPL_COPY_CONSTRUCTOR(std::complex<float>, std::complex<double>);
+#endif
 
 #undef TEMPL_COPY_CONSTRUCTOR
-  }
-}
+  } // namespace distributed
+} // namespace LinearAlgebra
 
 
 DEAL_II_NAMESPACE_CLOSE
