@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2017 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -8,12 +8,12 @@
 // it, and/or modify it under the terms of the GNU Lesser General
 // Public License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// The full text of the license can be found in the file LICENSE at
-// the top level of the deal.II distribution.
+// The full text of the license can be found in the file LICENSE.md at
+// the top level directory of deal.II.
 //
 // ---------------------------------------------------------------------
 
-#include <deal.II/lac/petsc_parallel_block_vector.h>
+#include <deal.II/lac/petsc_block_vector.h>
 
 #ifdef DEAL_II_WITH_PETSC
 
@@ -23,24 +23,24 @@ namespace PETScWrappers
 {
   namespace MPI
   {
-    typedef types::global_dof_index size_type;
+    using size_type = types::global_dof_index;
 
     void
-    BlockVector::reinit (const unsigned int num_blocks)
+    BlockVector::reinit(const unsigned int num_blocks)
     {
-      std::vector<size_type> block_sizes (num_blocks, 0);
-      this->block_indices.reinit (block_sizes);
+      std::vector<size_type> block_sizes(num_blocks, 0);
+      this->block_indices.reinit(block_sizes);
       if (this->components.size() != this->n_blocks())
         this->components.resize(this->n_blocks());
 
-      for (unsigned int i=0; i<this->n_blocks(); ++i)
-        components[i].reinit (MPI_COMM_SELF, 0, 0);
+      for (unsigned int i = 0; i < this->n_blocks(); ++i)
+        components[i].reinit(MPI_COMM_SELF, 0, 0);
 
       collect_sizes();
     }
-  }
+  } // namespace MPI
 
-}
+} // namespace PETScWrappers
 
 DEAL_II_NAMESPACE_CLOSE
 
