@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2018 - 2020 by the deal.II authors
+// Copyright (C) 2018 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,13 +17,11 @@
 
 #ifdef DEAL_II_TRILINOS_WITH_TPETRA
 
-#  ifdef DEAL_II_WITH_MPI
+#  include <deal.II/base/index_set.h>
 
-#    include <deal.II/base/index_set.h>
+#  include <Tpetra_Map.hpp>
 
-#    include <Tpetra_Map.hpp>
-
-#    include <memory>
+#  include <memory>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -99,6 +97,4 @@ namespace LinearAlgebra
 
 DEAL_II_NAMESPACE_CLOSE
 
-#  endif
-
 #endif
