@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2020 by the deal.II authors
+// Copyright (C) 2020 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -28,6 +28,13 @@ namespace internal
 {
   namespace MatrixFreeFunctions
   {
+    // ensure that the type defined in both dof_info.h and
+    // hanging_nodes_internal.h is consistent
+    static_assert(std::is_same<compressed_constraint_kind, std::uint8_t>::value,
+                  "Unexpected type for compressed hanging node indicators!");
+
+
+
     DoFInfo::DoFInfo()
     {
       clear();
@@ -103,7 +110,12 @@ namespace internal
           // shift for this cell within the block as compared to the next
           // one
           const bool has_constraints =
-            row_starts[ib].second != row_starts[ib + n_fe_components].second;
+            (hanging_node_constraint_masks.size() != 0 &&
+             hanging_node_constraint_masks_comp.size() != 0 &&
+             hanging_node_constraint_masks[cell * n_vectorization + v] !=
+               unconstrained_compressed_constraint_kind &&
+             hanging_node_constraint_masks_comp[fe_index][0 /*TODO*/]) ||
+            (row_starts[ib].second != row_starts[ib + n_fe_components].second);
 
           auto do_copy = [&](const unsigned int *begin,
                              const unsigned int *end) {
@@ -173,7 +185,7 @@ namespace internal
 
           types::global_dof_index last_contiguous_start = ghost_origin[0].first;
           ghost_numbering[ghost_origin[0].second]       = 0;
-          for (std::size_t i = 1; i < n_ghosts; i++)
+          for (std::size_t i = 1; i < n_ghosts; ++i)
             {
               if (ghost_origin[i].first > ghost_origin[i - 1].first + 1)
                 {
@@ -221,18 +233,31 @@ namespace internal
               // now the same procedure for plain indices
               if (store_plain_indices == true)
                 {
-                  if (row_starts[boundary_cells[i] * n_components].second !=
-                      row_starts[(boundary_cells[i] + 1) * n_components].second)
+                  bool has_hanging_nodes = false;
+
+                  const unsigned int fe_index =
+                    (cell_active_fe_index.size() == 0 ||
+                     dofs_per_cell.size() == 1) ?
+                      0 :
+                      cell_active_fe_index[boundary_cells[i]];
+                  AssertIndexRange(fe_index, dofs_per_cell.size());
+
+                  if (hanging_node_constraint_masks.size() > 0 &&
+                      hanging_node_constraint_masks_comp.size() > 0 &&
+                      hanging_node_constraint_masks[boundary_cells[i]] !=
+                        unconstrained_compressed_constraint_kind)
+                    for (unsigned int comp = 0; comp < n_components; ++comp)
+                      has_hanging_nodes |=
+                        hanging_node_constraint_masks_comp[fe_index][comp];
+
+                  if (has_hanging_nodes ||
+                      row_starts[boundary_cells[i] * n_components].second !=
+                        row_starts[(boundary_cells[i] + 1) * n_components]
+                          .second)
                     {
                       unsigned int *data_ptr =
                         plain_dof_indices.data() +
                         row_starts_plain_indices[boundary_cells[i]];
-                      const unsigned int fe_index =
-                        (cell_active_fe_index.size() == 0 ||
-                         dofs_per_cell.size() == 1) ?
-                          0 :
-                          cell_active_fe_index[boundary_cells[i]];
-                      AssertIndexRange(fe_index, dofs_per_cell.size());
                       const unsigned int *row_end =
                         data_ptr + dofs_per_cell[fe_index];
                       for (; data_ptr != row_end; ++data_ptr)
@@ -322,6 +347,11 @@ namespace internal
       unsigned int              position_cell = 0;
       new_dof_indices.reserve(dof_indices.size());
       new_constraint_indicator.reserve(constraint_indicator.size());
+
+      std::vector<compressed_constraint_kind> new_hanging_node_constraint_masks;
+      new_hanging_node_constraint_masks.reserve(
+        hanging_node_constraint_masks.size());
+
       if (store_plain_indices == true)
         {
           new_rowstart_plain.resize(vectorization_length *
@@ -350,6 +380,23 @@ namespace internal
             {
               const unsigned int cell_no =
                 renumbering[position_cell + j] * n_components;
+
+              bool has_hanging_nodes = false;
+
+              if (hanging_node_constraint_masks.size() > 0 &&
+                  hanging_node_constraint_masks_comp.size() > 0)
+                {
+                  const auto mask =
+                    hanging_node_constraint_masks[renumbering[position_cell +
+                                                              j]];
+                  new_hanging_node_constraint_masks.push_back(mask);
+
+                  if (mask != unconstrained_compressed_constraint_kind)
+                    for (unsigned int comp = 0; comp < n_components; ++comp)
+                      has_hanging_nodes |= hanging_node_constraint_masks_comp
+                        [have_hp ? cell_active_fe_index[i] : 0][comp];
+                }
+
               for (unsigned int comp = 0; comp < n_components; ++comp)
                 {
                   new_row_starts[(i * vectorization_length + j) * n_components +
@@ -370,8 +417,9 @@ namespace internal
                       constraint_indicator[index]);
                 }
               if (store_plain_indices &&
-                  row_starts[cell_no].second !=
-                    row_starts[cell_no + n_components].second)
+                  ((row_starts[cell_no].second !=
+                    row_starts[cell_no + n_components].second) ||
+                   has_hanging_nodes))
                 {
                   new_rowstart_plain[i * vectorization_length + j] =
                     new_plain_indices.size();
@@ -394,6 +442,12 @@ namespace internal
                                comp]
                   .second = new_constraint_indicator.size();
               }
+
+          for (unsigned int j = n_vect; j < vectorization_length; ++j)
+            if (hanging_node_constraint_masks.size() > 0)
+              new_hanging_node_constraint_masks.push_back(
+                unconstrained_compressed_constraint_kind);
+
           position_cell += n_vect;
         }
       AssertDimension(position_cell * n_components + 1, row_starts.size());
@@ -414,6 +468,7 @@ namespace internal
       new_constraint_indicator.swap(constraint_indicator);
       new_plain_indices.swap(plain_dof_indices);
       new_rowstart_plain.swap(row_starts_plain_indices);
+      new_hanging_node_constraint_masks.swap(hanging_node_constraint_masks);
 
 #ifdef DEBUG
       // sanity check 1: all indices should be smaller than the number of dofs
@@ -573,10 +628,12 @@ namespace internal
                   for (unsigned int j = 0; j < n_comp; ++j)
                     dof_indices_contiguous
                       [dof_access_cell][i * vectorization_length + j] =
-                        this->dof_indices[row_starts[(i * vectorization_length +
-                                                      j) *
-                                                     n_components]
-                                            .first];
+                        this->dof_indices.size() == 0 ?
+                          0 :
+                          this->dof_indices
+                            [row_starts[(i * vectorization_length + j) *
+                                        n_components]
+                               .first];
                 }
 
               if (indices_are_interleaved_and_contiguous)
@@ -1440,6 +1497,8 @@ namespace internal
       memory +=
         (row_starts.capacity() * sizeof(std::pair<unsigned int, unsigned int>));
       memory += MemoryConsumption::memory_consumption(dof_indices);
+      memory +=
+        MemoryConsumption::memory_consumption(hanging_node_constraint_masks);
       memory += MemoryConsumption::memory_consumption(row_starts_plain_indices);
       memory += MemoryConsumption::memory_consumption(plain_dof_indices);
       memory += MemoryConsumption::memory_consumption(constraint_indicator);
@@ -1454,12 +1513,20 @@ namespace internal
   namespace MatrixFreeFunctions
   {
     template struct ConstraintValues<double>;
-    template struct ConstraintValues<float>;
+
+    template unsigned short
+    ConstraintValues<double>::insert_entries(
+      const std::vector<std::pair<types::global_dof_index, float>> &);
+    template unsigned short
+    ConstraintValues<double>::insert_entries(
+      const std::vector<std::pair<types::global_dof_index, double>> &);
+
 
     template void
     DoFInfo::read_dof_indices<double>(
       const std::vector<types::global_dof_index> &,
-      const std::vector<unsigned int> &,
+      const std::vector<types::global_dof_index> &,
+      const bool,
       const dealii::AffineConstraints<double> &,
       const unsigned int,
       ConstraintValues<double> &,
@@ -1468,12 +1535,35 @@ namespace internal
     template void
     DoFInfo::read_dof_indices<float>(
       const std::vector<types::global_dof_index> &,
-      const std::vector<unsigned int> &,
+      const std::vector<types::global_dof_index> &,
+      const bool,
       const dealii::AffineConstraints<float> &,
       const unsigned int,
       ConstraintValues<double> &,
       bool &);
 
+    template bool
+    DoFInfo::process_hanging_node_constraints<1>(
+      const HangingNodes<1> &,
+      const std::vector<std::vector<unsigned int>> &,
+      const unsigned int,
+      const TriaIterator<DoFCellAccessor<1, 1, false>> &,
+      std::vector<types::global_dof_index> &);
+    template bool
+    DoFInfo::process_hanging_node_constraints<2>(
+      const HangingNodes<2> &,
+      const std::vector<std::vector<unsigned int>> &,
+      const unsigned int,
+      const TriaIterator<DoFCellAccessor<2, 2, false>> &,
+      std::vector<types::global_dof_index> &);
+    template bool
+    DoFInfo::process_hanging_node_constraints<3>(
+      const HangingNodes<3> &,
+      const std::vector<std::vector<unsigned int>> &,
+      const unsigned int,
+      const TriaIterator<DoFCellAccessor<3, 3, false>> &,
+      std::vector<types::global_dof_index> &);
+
     template void
     DoFInfo::compute_face_index_compression<1>(
       const std::vector<FaceToCellTopology<1>> &);
