@@ -24,17 +24,6 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-#define SPLIT_INSTANTIATIONS_COUNT 3
-#ifndef SPLIT_INSTANTIATIONS_INDEX
-#  define SPLIT_INSTANTIATIONS_INDEX 0
-#endif
 #include "matrix_free.inst"
 
-#if SPLIT_INSTANTIATIONS_INDEX == 0
-
-template struct internal::MatrixFreeFunctions::ShapeInfo<double>;
-template struct internal::MatrixFreeFunctions::ShapeInfo<float>;
-
-#endif
-
 DEAL_II_NAMESPACE_CLOSE
