@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2018 - 2021 by the deal.II authors
+// Copyright (C) 2018 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -21,6 +21,8 @@
 #include <deal.II/base/parallel.h>
 #include <deal.II/base/utilities.h>
 
+#include <deal.II/lac/dynamic_sparsity_pattern.h>
+
 #include <deal.II/matrix_free/task_info.h>
 
 
@@ -28,12 +30,30 @@
 #  include <tbb/blocked_range.h>
 #  include <tbb/parallel_for.h>
 #  include <tbb/task.h>
-#  include <tbb/task_scheduler_init.h>
+#  ifndef DEAL_II_TBB_WITH_ONEAPI
+#    include <tbb/task_scheduler_init.h>
+#  endif
 #endif
 
 #include <iostream>
 #include <set>
 
+//
+// TBB with oneAPI API has deprecated and removed the
+// <code>tbb::tasks</code> backend. With this it is no longer possible to
+// compile the following code that builds a directed acyclic graph (DAG) of
+// (thread parallel) tasks without a major porting effort. It turned out
+// that such a dynamic handling of dependencies and structures is not as
+// competitive as initially assumed. Consequently, this part of the matrix
+// free infrastructure has seen less attention than the rest over the last
+// years and is (presumably) not used that often.
+//
+// In case of detected oneAPI backend we simply disable threading in the
+// matrix free backend for now.
+//
+// Matthias Maier, Martin Kronbichler, 2021
+//
+
 DEAL_II_NAMESPACE_OPEN
 
 
@@ -43,7 +63,7 @@ namespace internal
 {
   namespace MatrixFreeFunctions
   {
-#ifdef DEAL_II_WITH_TBB
+#if defined(DEAL_II_WITH_TBB) && !defined(DEAL_II_TBB_WITH_ONEAPI)
 
     // This defines the TBB data structures that are needed to schedule the
     // partition-partition variant
@@ -153,7 +173,7 @@ namespace internal
           std::vector<CellWork *> blocked_worker(n_blocked_workers);
 
           root->set_ref_count(evens + 1);
-          for (unsigned int j = 0; j < evens; j++)
+          for (unsigned int j = 0; j < evens; ++j)
             {
               worker[j] = new (root->allocate_child())
                 CellWork(function,
@@ -341,7 +361,7 @@ namespace internal
 
       funct.vector_update_ghosts_start();
 
-#ifdef DEAL_II_WITH_TBB
+#if defined(DEAL_II_WITH_TBB) && !defined(DEAL_II_TBB_WITH_ONEAPI)
 
       if (scheme != none)
         {
@@ -357,7 +377,7 @@ namespace internal
               MPICommunication *worker_compr =
                 new (root->allocate_child()) MPICommunication(funct, true);
               worker_compr->set_ref_count(1);
-              for (unsigned int j = 0; j < evens; j++)
+              for (unsigned int j = 0; j < evens; ++j)
                 {
                   if (j > 0)
                     {
@@ -807,7 +827,7 @@ namespace internal
       // power of two, which allows it to replace integer divisions by shifts
       unsigned int vectorization_length_bits = 0;
       unsigned int my_length                 = vectorization_length;
-      while (my_length >>= 1)
+      while ((my_length >>= 1) != 0u)
         ++vectorization_length_bits;
       const unsigned int n_lanes = 1 << vectorization_length_bits;
 
@@ -852,9 +872,9 @@ namespace internal
         for (unsigned int j = n_categories - 1; j > 0; --j)
           {
             unsigned int lower_index = j - 1;
-            while (renumbering_category[j].size() % n_lanes)
+            while ((renumbering_category[j].size() % n_lanes) != 0u)
               {
-                while (renumbering_category[j].size() % n_lanes &&
+                while (((renumbering_category[j].size() % n_lanes) != 0u) &&
                        !renumbering_category[lower_index].empty())
                   {
                     renumbering_category[j].push_back(
@@ -1044,7 +1064,7 @@ namespace internal
                             cell_vectorization_categories[n_active_cells]);
           renumbering[cell] = cell;
         }
-      if (n_ghost_cells % n_lanes)
+      if ((n_ghost_cells % n_lanes) != 0u)
         incompletely_filled_vectorization.back() = n_ghost_cells % n_lanes;
       cell_partition_data.push_back(n_cell_batches + n_ghost_batches);
       partition_row_index.back() = cell_partition_data.size() - 1;
@@ -1232,7 +1252,7 @@ namespace internal
 
       unsigned int mcell_start = 0;
       block_start[0]           = 0;
-      for (unsigned int block = 0; block < n_blocks; block++)
+      for (unsigned int block = 0; block < n_blocks; ++block)
         {
           block_start[block + 1] = block_start[block];
           for (unsigned int mcell = mcell_start;
@@ -1255,7 +1275,7 @@ namespace internal
         block_size_last = block_size;
 
       unsigned int tick = 0;
-      for (unsigned int block = 0; block < n_blocks; block++)
+      for (unsigned int block = 0; block < n_blocks; ++block)
         {
           unsigned int present_block = partition_color_list[block];
           for (unsigned int cell = block_start[present_block];
@@ -1270,7 +1290,7 @@ namespace internal
           if (cell_partition_data[tick] == block)
             cell_partition_data[tick++] = counter_macro;
 
-          for (unsigned int j = 0; j < this_block_size; j++)
+          for (unsigned int j = 0; j < this_block_size; ++j)
             irregular[counter_macro++] =
               irregular_cells[present_block * block_size + j];
         }
@@ -1414,7 +1434,7 @@ namespace internal
           // This is the simple case. The renumbering is just a combination of
           // the renumbering that we were given as an input and the
           // renumbering of partition/coloring given in partition_2layers_list
-          for (unsigned int j = 0; j < renumbering.size(); j++)
+          for (unsigned int j = 0; j < renumbering.size(); ++j)
             renumbering[j] = renumbering_in[partition_2layers_list[j]];
           // Account for the ghost cells, finally.
           for (unsigned int i = 0; i < n_ghost_cells; ++i)
@@ -1430,7 +1450,7 @@ namespace internal
           unsigned int counter     = 0;
           unsigned int mcell_start = 0;
           block_start[0]           = 0;
-          for (unsigned int block = 0; block < n_blocks; block++)
+          for (unsigned int block = 0; block < n_blocks; ++block)
             {
               block_start[block + 1] = block_start[block];
               for (unsigned int mcell = mcell_start;
@@ -1453,7 +1473,7 @@ namespace internal
             block_size_last = block_size;
 
           unsigned int tick = 0;
-          for (unsigned int block = 0; block < n_blocks; block++)
+          for (unsigned int block = 0; block < n_blocks; ++block)
             {
               unsigned int present_block = partition_2layers_list[block];
               for (unsigned int cell = block_start[present_block];
@@ -1468,7 +1488,7 @@ namespace internal
               if (cell_partition_data[tick] == block)
                 cell_partition_data[tick++] = counter_macro;
 
-              for (unsigned int j = 0; j < this_block_size; j++)
+              for (unsigned int j = 0; j < this_block_size; ++j)
                 irregular[counter_macro++] =
                   irregular_cells[present_block * block_size + j];
             }
@@ -1552,7 +1572,7 @@ namespace internal
 
       partition_list.swap(renumbering);
 
-      for (unsigned int j = 0; j < renumbering.size(); j++)
+      for (unsigned int j = 0; j < renumbering.size(); ++j)
         renumbering[j] = partition_list[partition_partition_list[j]];
 
       for (unsigned int i = 0; i < n_ghost_cells; ++i)
@@ -1751,7 +1771,7 @@ namespace internal
                                   .push_back(partition_partition_list[cell]);
                             }
                           // check how many more cells are needed in the lists
-                          for (unsigned int j = 0; j < max_fe_index + 1; j++)
+                          for (unsigned int j = 0; j < max_fe_index + 1; ++j)
                             {
                               remaining_per_cell_batch[j] =
                                 renumbering_fe_index[j].size() %
@@ -1876,7 +1896,7 @@ namespace internal
                           // index within one partition-partition which was
                           // implicitly assumed above
                           cell = counter - partition_counter;
-                          for (unsigned int j = 0; j < max_fe_index + 1; j++)
+                          for (unsigned int j = 0; j < max_fe_index + 1; ++j)
                             {
                               for (const unsigned int jj :
                                    renumbering_fe_index[j])
@@ -1944,7 +1964,7 @@ namespace internal
       partition_row_index.resize(partition + 1);
       cell_partition_data.clear();
       unsigned int color_counter = 0, index_counter = 0;
-      for (unsigned int part = 0; part < partition; part++)
+      for (unsigned int part = 0; part < partition; ++part)
         {
           partition_row_index[part] = index_counter;
           unsigned int max_color    = 0;
@@ -1981,7 +2001,7 @@ namespace internal
           // Reorder within partition: First, all blocks that belong the 0 and
           // then so on until those with color max (Note that the smaller the
           // number the larger the partition)
-          for (unsigned int color = 0; color <= max_color; color++)
+          for (unsigned int color = 0; color <= max_color; ++color)
             {
               cell_partition_data.push_back(color_counter);
               index_counter++;
@@ -2245,7 +2265,7 @@ namespace internal
       partition_odds.resize(partition);
       partition_n_blocked_workers.resize(partition);
       partition_n_workers.resize(partition);
-      for (unsigned int part = 0; part < partition; part++)
+      for (unsigned int part = 0; part < partition; ++part)
         {
           partition_evens[part] =
             (partition_row_index[part + 1] - partition_row_index[part] + 1) / 2;
