@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2020 by the deal.II authors
+// Copyright (C) 2003 - 2021 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -202,17 +202,6 @@ MGTransferBlock<number>::copy_to_mg(
 
 
 
-template <int dim, int spacedim>
-void
-MGTransferBlockBase::build_matrices(
-  const DoFHandler<dim, spacedim> & /*dof*/,
-  const DoFHandler<dim, spacedim> &mg_dof_handler)
-{
-  build(mg_dof_handler);
-}
-
-
-
 template <int dim, int spacedim>
 void
 MGTransferBlockBase::build(const DoFHandler<dim, spacedim> &dof_handler)
@@ -477,17 +466,6 @@ MGTransferBlockBase::build(const DoFHandler<dim, spacedim> &dof_handler)
     }
 }
 
-template <typename number>
-template <int dim, int spacedim>
-void
-MGTransferBlockSelect<number>::build_matrices(
-  const DoFHandler<dim, spacedim> & /*dof*/,
-  const DoFHandler<dim, spacedim> &mg_dof,
-  unsigned int                     select)
-{
-  build(mg_dof, select);
-}
-
 template <typename number>
 template <int dim, int spacedim>
 void
@@ -576,17 +554,6 @@ MGTransferBlockSelect<number>::build(
 }
 
 
-template <typename number>
-template <int dim, int spacedim>
-void
-MGTransferBlock<number>::build_matrices(
-  const DoFHandler<dim, spacedim> & /*dof*/,
-  const DoFHandler<dim, spacedim> &mg_dof,
-  const std::vector<bool> &        sel)
-{
-  build(mg_dof, sel);
-}
-
 template <typename number>
 template <int dim, int spacedim>
 void
