@@ -287,7 +287,7 @@ namespace internal
               AssertThrow(level_dof_indices.size() == index_owner.size(),
                           ExcMessage("Size does not match!"));
 
-              for (unsigned int i = 0; i < index_owner.size(); i++)
+              for (unsigned int i = 0; i < index_owner.size(); ++i)
                 send_data[index_owner[i]].emplace_back(level,
                                                        global_dof_indices[i],
                                                        level_dof_indices[i]);
@@ -641,39 +641,6 @@ namespace internal
 
 
 
-    namespace
-    {
-      /**
-       * Helper function for setup_transfer. Checks for identity constrained
-       * dofs and replace with the indices of the dofs to which they are
-       * constrained
-       */
-      void
-      replace(const MGConstrainedDoFs *             mg_constrained_dofs,
-              const unsigned int                    level,
-              std::vector<types::global_dof_index> &dof_indices)
-      {
-        if (mg_constrained_dofs != nullptr &&
-            mg_constrained_dofs->get_level_constraints(level).n_constraints() >
-              0)
-          for (auto &ind : dof_indices)
-            if (mg_constrained_dofs->get_level_constraints(level)
-                  .is_identity_constrained(ind))
-              {
-                Assert(mg_constrained_dofs->get_level_constraints(level)
-                           .get_constraint_entries(ind)
-                           ->size() == 1,
-                       ExcInternalError());
-                ind = mg_constrained_dofs->get_level_constraints(level)
-                        .get_constraint_entries(ind)
-                        ->front()
-                        .first;
-              }
-      }
-    } // namespace
-
-
-
     // Sets up most of the internal data structures of the MGTransferMatrixFree
     // class
     template <int dim, typename Number>
@@ -809,7 +776,9 @@ namespace internal
                     continue;
                   cell->child(c)->get_mg_dof_indices(local_dof_indices);
 
-                  replace(mg_constrained_dofs, level, local_dof_indices);
+                  resolve_identity_constraints(mg_constrained_dofs,
+                                               level,
+                                               local_dof_indices);
 
                   const IndexSet &owned_level_dofs =
                     dof_handler.locally_owned_mg_dofs(level);
@@ -880,7 +849,9 @@ namespace internal
                 {
                   cell->get_mg_dof_indices(local_dof_indices);
 
-                  replace(mg_constrained_dofs, level - 1, local_dof_indices);
+                  resolve_identity_constraints(mg_constrained_dofs,
+                                               level - 1,
+                                               local_dof_indices);
 
                   const IndexSet &owned_level_dofs_l0 =
                     dof_handler.locally_owned_mg_dofs(0);
@@ -1030,6 +1001,31 @@ namespace internal
         }
     }
 
+
+
+    void
+    resolve_identity_constraints(
+      const MGConstrainedDoFs *             mg_constrained_dofs,
+      const unsigned int                    level,
+      std::vector<types::global_dof_index> &dof_indices)
+    {
+      if (mg_constrained_dofs != nullptr &&
+          mg_constrained_dofs->get_level_constraints(level).n_constraints() > 0)
+        for (auto &ind : dof_indices)
+          if (mg_constrained_dofs->get_level_constraints(level)
+                .is_identity_constrained(ind))
+            {
+              Assert(mg_constrained_dofs->get_level_constraints(level)
+                         .get_constraint_entries(ind)
+                         ->size() == 1,
+                     ExcInternalError());
+              ind = mg_constrained_dofs->get_level_constraints(level)
+                      .get_constraint_entries(ind)
+                      ->front()
+                      .first;
+            }
+    }
+
   } // namespace MGTransfer
 } // namespace internal
 
