@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2007 - 2020 by the deal.II authors
+// Copyright (C) 2007 - 2022 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -19,6 +19,29 @@ DEAL_II_NAMESPACE_OPEN
 
 
 
+namespace DataPostprocessorInputs
+{
+  template <int spacedim>
+  CommonInputs<spacedim>::CommonInputs()
+    : face_number(numbers::invalid_unsigned_int)
+  {}
+
+
+
+  template <int spacedim>
+  unsigned int
+  CommonInputs<spacedim>::get_face_number() const
+  {
+    Assert(
+      face_number != numbers::invalid_unsigned_int,
+      ExcMessage(
+        "This function can only be called if set_cell_and_face() has "
+        "previously been called. Typically, this would be by using DataOutFaces "
+        "or a related class."));
+    return face_number;
+  }
+} // namespace DataPostprocessorInputs
+
 // -------------------------- DataPostprocessor ---------------------------
 
 template <int dim>
@@ -54,7 +77,7 @@ DataPostprocessor<dim>::get_data_component_interpretation() const
 }
 
 
-// -------------------------- DataPostprocessorScalar -------------------------
+// -------------------- DataPostprocessorScalar -------------------------
 
 template <int dim>
 DataPostprocessorScalar<dim>::DataPostprocessorScalar(
@@ -93,7 +116,8 @@ DataPostprocessorScalar<dim>::get_needed_update_flags() const
 
 
 
-// -------------------------- DataPostprocessorVector -------------------------
+// -------------------------- DataPostprocessorVector
+// -------------------------
 
 template <int dim>
 DataPostprocessorVector<dim>::DataPostprocessorVector(
@@ -132,7 +156,8 @@ DataPostprocessorVector<dim>::get_needed_update_flags() const
 
 
 
-// -------------------------- DataPostprocessorTensor -------------------------
+// -------------------------- DataPostprocessorTensor
+// -------------------------
 
 template <int dim>
 DataPostprocessorTensor<dim>::DataPostprocessorTensor(
@@ -171,6 +196,44 @@ DataPostprocessorTensor<dim>::get_needed_update_flags() const
 
 
 
+namespace DataPostprocessors
+{
+  template <int dim>
+  BoundaryIds<dim>::BoundaryIds()
+    : DataPostprocessorScalar<dim>("boundary_id", update_quadrature_points)
+  {}
+
+
+  template <int dim>
+  void
+  BoundaryIds<dim>::evaluate_scalar_field(
+    const DataPostprocessorInputs::Scalar<dim> &inputs,
+    std::vector<Vector<double>> &               computed_quantities) const
+  {
+    AssertDimension(computed_quantities.size(), inputs.solution_values.size());
+
+    const typename DoFHandler<dim>::active_cell_iterator cell =
+      inputs.template get_cell<dim>();
+    const unsigned int face = inputs.get_face_number();
+
+    for (auto &output : computed_quantities)
+      {
+        AssertDimension(output.size(), 1);
+
+        // By default, DataOutFaces is only run on faces at the boundary of the
+        // domain. But one can instruct it to also run on internal faces, and in
+        // that case we cannot ask for the boundary id. Rather, we output -1, as
+        // described in the documentation.
+        if (cell->at_boundary(face))
+          output(0) = cell->face(face)->boundary_id();
+        else
+          output(0) = -1;
+      }
+  }
+} // namespace DataPostprocessors
+
+
+
 // explicit instantiation
 #include "data_postprocessor.inst"
 
