@@ -13,6 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
+#include <deal.II/base/config.h>
+
 #include <deal.II/opencascade/utilities.h>
 
 #ifdef DEAL_II_WITH_OPENCASCADE
@@ -27,7 +29,6 @@
 #  include <STEPControl_Controller.hxx>
 #  include <STEPControl_Reader.hxx>
 #  include <STEPControl_Writer.hxx>
-#  include <Standard_Version.hxx>
 #  include <TopExp_Explorer.hxx>
 #  include <TopoDS.hxx>
 #  include <TopoDS_Edge.hxx>
@@ -37,17 +38,21 @@
 #  include <cstdio>
 #  include <iostream>
 #  include <set>
-#  if (OCC_VERSION_MAJOR < 7)
-#    include <Handle_Standard_Transient.hxx>
-#  else
+#  if DEAL_II_OPENCASCADE_VERSION_GTE(7, 0, 0)
 #    include <Standard_Transient.hxx>
+#  else
+#    include <Handle_Standard_Transient.hxx>
 #  endif
 
 #  include <BRepAdaptor_Curve.hxx>
-#  include <BRepAdaptor_HCompCurve.hxx>
-#  include <BRepAdaptor_HCurve.hxx>
+#  if DEAL_II_OPENCASCADE_VERSION_GTE(7, 6, 0)
+#    include <BRepAlgoAPI_Section.hxx>
+#  else
+#    include <BRepAdaptor_HCompCurve.hxx>
+#    include <BRepAdaptor_HCurve.hxx>
+#    include <BRepAlgo_Section.hxx>
+#  endif
 #  include <BRepAdaptor_Surface.hxx>
-#  include <BRepAlgo_Section.hxx>
 #  include <BRepBndLib.hxx>
 #  include <BRepBuilderAPI_MakeEdge.hxx>
 #  include <BRepBuilderAPI_Sewing.hxx>
@@ -90,14 +95,11 @@ namespace OpenCASCADE
     TopExp_Explorer exp;
     unsigned int    n_faces = 0, n_edges = 0, n_vertices = 0;
     for (exp.Init(shape, TopAbs_FACE); exp.More(); exp.Next(), ++n_faces)
-      {
-      }
+      {}
     for (exp.Init(shape, TopAbs_EDGE); exp.More(); exp.Next(), ++n_edges)
-      {
-      }
+      {}
     for (exp.Init(shape, TopAbs_VERTEX); exp.More(); exp.Next(), ++n_vertices)
-      {
-      }
+      {}
     return std::tuple<unsigned int, unsigned int, unsigned int>(n_faces,
                                                                 n_edges,
                                                                 n_vertices);
@@ -331,12 +333,12 @@ namespace OpenCASCADE
 
     StlAPI_Writer writer;
 
-#  if ((OCC_VERSION_MAJOR * 100 + OCC_VERSION_MINOR * 10) >= 690)
+#  if DEAL_II_OPENCASCADE_VERSION_GTE(6, 9, 0)
     // opencascade versions 6.9.0 onwards return an error status
     const auto error = writer.Write(shape_to_be_written, filename.c_str());
 
     // which is a custom type between 6.9.0 and 7.1.0
-#    if ((OCC_VERSION_MAJOR * 100 + OCC_VERSION_MINOR * 10) < 720)
+#    if !DEAL_II_OPENCASCADE_VERSION_GTE(7, 2, 0)
     AssertThrow(error == StlAPI_StatusOK,
                 ExcMessage("Error writing STL from shape."));
 #    else
@@ -429,8 +431,12 @@ namespace OpenCASCADE
                   const double /*tolerance*/)
   {
     Handle(Geom_Plane) plane = new Geom_Plane(c_x, c_y, c_z, c);
+#  if DEAL_II_OPENCASCADE_VERSION_GTE(7, 6, 0)
+    BRepAlgoAPI_Section section(in_shape, plane);
+#  else
     BRepAlgo_Section section(in_shape, plane);
-    TopoDS_Shape     edges = section.Shape();
+#  endif
+    TopoDS_Shape edges = section.Shape();
     return edges;
   }
 
