@@ -1,17 +1,18 @@
-//-----------------------------------------------------------
+// ---------------------------------------------------------------------
 //
-//    Copyright (C) 2017 - 2021 by the deal.II authors
+// Copyright (C) 2017 - 2022 by the deal.II authors
 //
-//    This file is part of the deal.II library.
+// This file is part of the deal.II library.
 //
-//    The deal.II library is free software; you can use it, redistribute
-//    it, and/or modify it under the terms of the GNU Lesser General
-//    Public License as published by the Free Software Foundation; either
-//    version 2.1 of the License, or (at your option) any later version.
-//    The full text of the license can be found in the file LICENSE.md at
-//    the top level directory of deal.II.
+// The deal.II library is free software; you can use it, redistribute
+// it, and/or modify it under the terms of the GNU Lesser General
+// Public License as published by the Free Software Foundation; either
+// version 2.1 of the License, or (at your option) any later version.
+// The full text of the license can be found in the file LICENSE.md at
+// the top level directory of deal.II.
 //
-//-----------------------------------------------------------
+// ---------------------------------------------------------------------
+
 
 
 #include <deal.II/base/config.h>
@@ -38,36 +39,21 @@
 #  include <deal.II/sundials/n_vector.h>
 #  include <deal.II/sundials/sunlinsol_wrapper.h>
 
-#  if DEAL_II_SUNDIALS_VERSION_LT(4, 0, 0)
-#    include <arkode/arkode_impl.h>
-#    include <sundials/sundials_config.h>
-#  else
-#    include <arkode/arkode_arkstep.h>
-#    include <sunlinsol/sunlinsol_spgmr.h>
-#    include <sunnonlinsol/sunnonlinsol_fixedpoint.h>
-#    if DEAL_II_SUNDIALS_VERSION_LT(5, 0, 0)
-#      include <deal.II/sundials/sunlinsol_newempty.h>
-#    endif
-#  endif
+#  include <arkode/arkode_arkstep.h>
+#  include <sunlinsol/sunlinsol_spgmr.h>
+#  include <sunnonlinsol/sunnonlinsol_fixedpoint.h>
 
 #  include <iostream>
 
-#  if DEAL_II_SUNDIALS_VERSION_LT(4, 0, 0)
-// Make sure we know how to call sundials own ARKode() function
-const auto &SundialsARKode = ARKode;
-#  endif
-
 DEAL_II_NAMESPACE_OPEN
 
 namespace SUNDIALS
 {
-  using namespace internal;
-
   namespace
   {
     template <typename VectorType>
     int
-    t_arkode_explicit_function(realtype tt,
+    explicit_function_callback(realtype tt,
                                N_Vector yy,
                                N_Vector yp,
                                void *   user_data)
@@ -86,7 +72,7 @@ namespace SUNDIALS
 
     template <typename VectorType>
     int
-    t_arkode_implicit_function(realtype tt,
+    implicit_function_callback(realtype tt,
                                N_Vector yy,
                                N_Vector yp,
                                void *   user_data)
@@ -103,125 +89,15 @@ namespace SUNDIALS
 
 
 
-#  if DEAL_II_SUNDIALS_VERSION_LT(4, 0, 0)
     template <typename VectorType>
     int
-    t_arkode_setup_jacobian(ARKodeMem    arkode_mem,
-                            int          convfail,
-                            N_Vector     ypred,
-                            N_Vector     fpred,
-                            booleantype *jcurPtr,
-                            N_Vector,
-                            N_Vector,
-                            N_Vector)
-    {
-      Assert(arkode_mem->ark_user_data != nullptr, ExcInternalError());
-      ARKode<VectorType> &solver =
-        *static_cast<ARKode<VectorType> *>(arkode_mem->ark_user_data);
-
-      auto *src_ypred = internal::unwrap_nvector_const<VectorType>(ypred);
-      auto *src_fpred = internal::unwrap_nvector_const<VectorType>(fpred);
-      // avoid reinterpret_cast
-      bool jcurPtr_tmp = false;
-      int  err         = solver.setup_jacobian(convfail,
-                                      arkode_mem->ark_tn,
-                                      arkode_mem->ark_gamma,
-                                      *src_ypred,
-                                      *src_fpred,
-                                      jcurPtr_tmp);
-#    if DEAL_II_SUNDIALS_VERSION_GTE(2, 0, 0)
-      *jcurPtr = jcurPtr_tmp ? SUNTRUE : SUNFALSE;
-#    else
-      *jcurPtr = jcurPtr_tmp ? TRUE : FALSE;
-#    endif
-
-      return err;
-    }
-
-
-
-    template <typename VectorType>
-    int
-    t_arkode_solve_jacobian(ARKodeMem arkode_mem,
-                            N_Vector  b,
-#    if DEAL_II_SUNDIALS_VERSION_LT(3, 0, 0)
-                            N_Vector,
-#    endif
-                            N_Vector ycur,
-                            N_Vector fcur)
-    {
-      Assert(arkode_mem->ark_user_data != nullptr, ExcInternalError());
-      ARKode<VectorType> &solver =
-        *static_cast<ARKode<VectorType> *>(arkode_mem->ark_user_data);
-
-      auto *dst      = internal::unwrap_nvector<VectorType>(b);
-      auto *src_ycur = internal::unwrap_nvector_const<VectorType>(ycur);
-      auto *src_fcur = internal::unwrap_nvector_const<VectorType>(fcur);
-
-      // make a temporary copy to work on in the user call
-      VectorType src = *dst;
-
-      int err = solver.solve_jacobian_system(arkode_mem->ark_tn,
-                                             arkode_mem->ark_gamma,
-                                             *src_ycur,
-                                             *src_fcur,
-                                             src,
-                                             *dst);
-
-
-      return err;
-    }
-
-
-
-    template <typename VectorType>
-    int
-    t_arkode_setup_mass(ARKodeMem arkode_mem, N_Vector, N_Vector, N_Vector)
-    {
-      Assert(arkode_mem->ark_user_data != nullptr, ExcInternalError());
-      ARKode<VectorType> &solver =
-        *static_cast<ARKode<VectorType> *>(arkode_mem->ark_user_data);
-      int err = solver.setup_mass(arkode_mem->ark_tn);
-      return err;
-    }
-
-
-
-    template <typename VectorType>
-    int
-    t_arkode_solve_mass(ARKodeMem arkode_mem,
-#    if DEAL_II_SUNDIALS_VERSION_LT(3, 0, 0)
-                        N_Vector b,
-                        N_Vector
-#    else
-                        N_Vector b
-#    endif
-    )
-    {
-      Assert(arkode_mem->ark_user_data != nullptr, ExcInternalError());
-      ARKode<VectorType> &solver =
-        *static_cast<ARKode<VectorType> *>(arkode_mem->ark_user_data);
-
-      auto *dst = internal::unwrap_nvector<VectorType>(b);
-
-      // make a temporary copy to work on in the user call
-      VectorType src = *dst;
-
-      int err = solver.solve_mass_system(src, *dst);
-
-      return err;
-    }
-#  else
-
-    template <typename VectorType>
-    int
-    t_arkode_jac_times_vec_function(N_Vector v,
-                                    N_Vector Jv,
-                                    realtype t,
-                                    N_Vector y,
-                                    N_Vector fy,
-                                    void *   user_data,
-                                    N_Vector)
+    jacobian_times_vector_callback(N_Vector v,
+                                   N_Vector Jv,
+                                   realtype t,
+                                   N_Vector y,
+                                   N_Vector fy,
+                                   void *   user_data,
+                                   N_Vector)
     {
       Assert(user_data != nullptr, ExcInternalError());
       ARKode<VectorType> &solver =
@@ -240,10 +116,10 @@ namespace SUNDIALS
 
     template <typename VectorType>
     int
-    t_arkode_jac_times_setup_function(realtype t,
-                                      N_Vector y,
-                                      N_Vector fy,
-                                      void *   user_data)
+    jacobian_times_vector_setup_callback(realtype t,
+                                         N_Vector y,
+                                         N_Vector fy,
+                                         void *   user_data)
     {
       Assert(user_data != nullptr, ExcInternalError());
       ARKode<VectorType> &solver =
@@ -259,7 +135,7 @@ namespace SUNDIALS
 
     template <typename VectorType>
     int
-    t_arkode_prec_solve_function(realtype t,
+    solve_with_jacobian_callback(realtype t,
                                  N_Vector y,
                                  N_Vector fy,
                                  N_Vector r,
@@ -287,13 +163,13 @@ namespace SUNDIALS
 
     template <typename VectorType>
     int
-    t_arkode_prec_setup_function(realtype     t,
-                                 N_Vector     y,
-                                 N_Vector     fy,
-                                 booleantype  jok,
-                                 booleantype *jcurPtr,
-                                 realtype     gamma,
-                                 void *       user_data)
+    jacobian_solver_setup_callback(realtype     t,
+                                   N_Vector     y,
+                                   N_Vector     fy,
+                                   booleantype  jok,
+                                   booleantype *jcurPtr,
+                                   realtype     gamma,
+                                   void *       user_data)
     {
       Assert(user_data != nullptr, ExcInternalError());
       ARKode<VectorType> &solver =
@@ -310,10 +186,10 @@ namespace SUNDIALS
 
     template <typename VectorType>
     int
-    t_arkode_mass_times_vec_function(N_Vector v,
-                                     N_Vector Mv,
-                                     realtype t,
-                                     void *   mtimes_data)
+    mass_matrix_times_vector_callback(N_Vector v,
+                                      N_Vector Mv,
+                                      realtype t,
+                                      void *   mtimes_data)
     {
       Assert(mtimes_data != nullptr, ExcInternalError());
       ARKode<VectorType> &solver =
@@ -329,7 +205,7 @@ namespace SUNDIALS
 
     template <typename VectorType>
     int
-    t_arkode_mass_times_setup_function(realtype t, void *mtimes_data)
+    mass_matrix_times_vector_setup_callback(realtype t, void *mtimes_data)
     {
       Assert(mtimes_data != nullptr, ExcInternalError());
       ARKode<VectorType> &solver =
@@ -342,12 +218,12 @@ namespace SUNDIALS
 
     template <typename VectorType>
     int
-    t_arkode_mass_prec_solve_function(realtype t,
-                                      N_Vector r,
-                                      N_Vector z,
-                                      realtype delta,
-                                      int      lr,
-                                      void *   user_data)
+    solve_with_mass_matrix_callback(realtype t,
+                                    N_Vector r,
+                                    N_Vector z,
+                                    realtype delta,
+                                    int      lr,
+                                    void *   user_data)
     {
       Assert(user_data != nullptr, ExcInternalError());
       ARKode<VectorType> &solver =
@@ -363,7 +239,7 @@ namespace SUNDIALS
 
     template <typename VectorType>
     int
-    t_arkode_mass_prec_setup_function(realtype t, void *user_data)
+    mass_matrix_solver_setup_callback(realtype t, void *user_data)
     {
       Assert(user_data != nullptr, ExcInternalError());
       ARKode<VectorType> &solver =
@@ -371,41 +247,54 @@ namespace SUNDIALS
 
       return solver.mass_preconditioner_setup(t);
     }
-#  endif
-
   } // namespace
 
 
+  template <typename VectorType>
+  ARKode<VectorType>::ARKode(const AdditionalData &data)
+    : ARKode(data, MPI_COMM_SELF)
+  {}
+
+
   template <typename VectorType>
   ARKode<VectorType>::ARKode(const AdditionalData &data,
                              const MPI_Comm &      mpi_comm)
     : data(data)
     , arkode_mem(nullptr)
-    , communicator(is_serial_vector<VectorType>::value ?
-                     MPI_COMM_SELF :
-                     Utilities::MPI::duplicate_communicator(mpi_comm))
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+    , arkode_ctx(nullptr)
+#  endif
+    , mpi_communicator(mpi_comm)
     , last_end_time(data.initial_time)
   {
     set_functions_to_trigger_an_assert();
+
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+    // SUNDIALS will always duplicate communicators if we provide them. This
+    // can cause problems if SUNDIALS is configured with MPI and we pass along
+    // MPI_COMM_SELF in a serial application as MPI won't be
+    // initialized. Hence, work around that by just not providing a
+    // communicator in that case.
+    const int status =
+      SUNContext_Create(mpi_communicator == MPI_COMM_SELF ? nullptr :
+                                                            &mpi_communicator,
+                        &arkode_ctx);
+    (void)status;
+    AssertARKode(status);
+#  endif
   }
 
+
+
   template <typename VectorType>
   ARKode<VectorType>::~ARKode()
   {
-    if (arkode_mem)
-#  if DEAL_II_SUNDIALS_VERSION_LT(4, 0, 0)
-      ARKodeFree(&arkode_mem);
-#  else
-      ARKStepFree(&arkode_mem);
-#  endif
+    ARKStepFree(&arkode_mem);
 
-#  ifdef DEAL_II_WITH_MPI
-    if (is_serial_vector<VectorType>::value == false)
-      {
-        const int ierr = MPI_Comm_free(&communicator);
-        (void)ierr;
-        AssertNothrow(ierr == MPI_SUCCESS, ExcMPI(ierr));
-      }
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+    const int status = SUNContext_Free(&arkode_ctx);
+    (void)status;
+    AssertARKode(status);
 #  endif
   }
 
@@ -457,32 +346,44 @@ namespace SUNDIALS
                       solution,
                       time.get_step_number());
       }
+    else
+      {
+        // If we don't do a full reset then we still need to fix the end time.
+        // In SUNDIALS 6 and later, SUNDIALS will not do timesteps if the
+        // current time is past the set end point (i.e., ARKStepEvolve will
+        // return ARK_TSTOP_RETURN).
+        const int status = ARKStepSetStopTime(arkode_mem, time.get_end_time());
+        (void)status;
+        AssertARKode(status);
+      }
 
-    auto solution_nvector = internal::make_nvector_view(solution);
+    auto solution_nvector = internal::make_nvector_view(solution
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+                                                        ,
+                                                        arkode_ctx
+#  endif
+    );
 
     while (!time.is_at_end())
       {
         time.set_desired_next_step_size(data.output_period);
-        double actual_next_time;
-#  if DEAL_II_SUNDIALS_VERSION_LT(4, 0, 0)
-        const auto status = SundialsARKode(arkode_mem,
-                                           time.get_next_time(),
-                                           solution_nvector,
-                                           &actual_next_time,
-                                           ARK_NORMAL);
-#  else
+
+        // Let ARKode advance time by one period:
+        double     actual_next_time;
         const auto status = ARKStepEvolve(arkode_mem,
                                           time.get_next_time(),
                                           solution_nvector,
                                           &actual_next_time,
                                           ARK_NORMAL);
-#  endif
         (void)status;
         AssertARKode(status);
 
+        // Then reflect this time advancement in our own DiscreteTime object:
         time.set_next_step_size(actual_next_time - time.get_current_time());
         time.advance_time();
 
+        // Finally check whether resets or output calls are desired at this
+        // time:
         while (solver_should_restart(time.get_current_time(), solution))
           reset(time.get_current_time(),
                 time.get_previous_step_size(),
@@ -499,7 +400,6 @@ namespace SUNDIALS
 
 
 
-#  if DEAL_II_SUNDIALS_VERSION_LT(4, 0, 0)
   template <typename VectorType>
   void
   ARKode<VectorType>::reset(const double      current_time,
@@ -507,143 +407,58 @@ namespace SUNDIALS
                             const VectorType &solution)
   {
     last_end_time = current_time;
-    if (arkode_mem)
-      ARKodeFree(&arkode_mem);
-
-    arkode_mem = ARKodeCreate();
-
     int status;
     (void)status;
 
-    Assert(explicit_function || implicit_function,
-           ExcFunctionNotProvided("explicit_function || implicit_function"));
-
-    // just a view on the memory in solution, all write operations on yy by
-    // ARKODE will automatically be mirrored to solution
-    auto initial_condition_nvector = internal::make_nvector_view(solution);
-
-    status = ARKodeInit(
-      arkode_mem,
-      explicit_function ? &t_arkode_explicit_function<VectorType> : nullptr,
-      implicit_function ? &t_arkode_implicit_function<VectorType> : nullptr,
-      current_time,
-      initial_condition_nvector);
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+    status = SUNContext_Free(&arkode_ctx);
     AssertARKode(status);
-
-    if (get_local_tolerances)
-      {
-        const auto abs_tols = make_nvector_view(get_local_tolerances());
-        status =
-          ARKodeSVtolerances(arkode_mem, data.relative_tolerance, abs_tols);
-        AssertARKode(status);
-      }
-    else
-      {
-        status = ARKodeSStolerances(arkode_mem,
-                                    data.relative_tolerance,
-                                    data.absolute_tolerance);
-        AssertARKode(status);
-      }
-
-    status = ARKodeSetInitStep(arkode_mem, current_time_step);
-    AssertARKode(status);
-
-    status = ARKodeSetUserData(arkode_mem, this);
-    AssertARKode(status);
-
-    status = ARKodeSetStopTime(arkode_mem, data.final_time);
-    AssertARKode(status);
-
+    // Same comment applies as in class constructor:
     status =
-      ARKodeSetMaxNonlinIters(arkode_mem, data.maximum_non_linear_iterations);
+      SUNContext_Create(mpi_communicator == MPI_COMM_SELF ? nullptr :
+                                                            &mpi_communicator,
+                        &arkode_ctx);
     AssertARKode(status);
+#  endif
 
-    // Initialize solver
-    auto ARKode_mem = static_cast<ARKodeMem>(arkode_mem);
-
-    if (solve_jacobian_system)
-      {
-        status = ARKodeSetNewton(arkode_mem);
-        AssertARKode(status);
-        if (data.implicit_function_is_linear)
-          {
-            status = ARKodeSetLinear(
-              arkode_mem, data.implicit_function_is_time_independent ? 0 : 1);
-            AssertARKode(status);
-          }
-
-
-        ARKode_mem->ark_lsolve = t_arkode_solve_jacobian<VectorType>;
-        if (setup_jacobian)
-          {
-            ARKode_mem->ark_lsetup = t_arkode_setup_jacobian<VectorType>;
-#    if DEAL_II_SUNDIALS_VERSION_LT(3, 0, 0)
-            ARKode_mem->ark_setupNonNull = true;
-#    endif
-          }
-      }
-    else
-      {
-        status =
-          ARKodeSetFixedPoint(arkode_mem, data.maximum_non_linear_iterations);
-        AssertARKode(status);
-      }
-
-
-    if (solve_mass_system)
-      {
-        ARKode_mem->ark_msolve = t_arkode_solve_mass<VectorType>;
-
-        if (setup_mass)
-          {
-            ARKode_mem->ark_msetup = t_arkode_setup_mass<VectorType>;
-#    if DEAL_II_SUNDIALS_VERSION_LT(3, 0, 0)
-            ARKode_mem->ark_MassSetupNonNull = true;
-#    endif
-          }
-      }
-
-    status = ARKodeSetOrder(arkode_mem, data.maximum_order);
-    AssertARKode(status);
-
-    if (custom_setup)
-      custom_setup(arkode_mem);
-  }
-
-#  else
-
-  template <typename VectorType>
-  void
-  ARKode<VectorType>::reset(const double current_time,
-                            const double current_time_step,
-                            const VectorType &solution)
-  {
-    last_end_time = current_time;
     if (arkode_mem)
-      ARKStepFree(&arkode_mem);
-
-    int status;
-    (void)status;
+      {
+        ARKStepFree(&arkode_mem);
+        // Initialization is version-dependent: do that in a moment
+      }
 
     // just a view on the memory in solution, all write operations on yy by
     // ARKODE will automatically be mirrored to solution
-    auto initial_condition_nvector = internal::make_nvector_view(solution);
+    auto initial_condition_nvector = internal::make_nvector_view(solution
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+                                                                 ,
+                                                                 arkode_ctx
+#  endif
+    );
 
     Assert(explicit_function || implicit_function,
            ExcFunctionNotProvided("explicit_function || implicit_function"));
 
     arkode_mem = ARKStepCreate(
-      explicit_function ? &t_arkode_explicit_function<VectorType> : nullptr,
-      implicit_function ? &t_arkode_implicit_function<VectorType> : nullptr,
+      explicit_function ? &explicit_function_callback<VectorType> : nullptr,
+      implicit_function ? &implicit_function_callback<VectorType> : nullptr,
       current_time,
-      initial_condition_nvector);
-
+      initial_condition_nvector
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+      ,
+      arkode_ctx
+#  endif
+    );
     Assert(arkode_mem != nullptr, ExcInternalError());
 
     if (get_local_tolerances)
       {
-        const auto abs_tols =
-          internal::make_nvector_view(get_local_tolerances());
+        const auto abs_tols = internal::make_nvector_view(get_local_tolerances()
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+                                                            ,
+                                                          arkode_ctx
+#  endif
+        );
         status =
           ARKStepSVtolerances(arkode_mem, data.relative_tolerance, abs_tols);
         AssertARKode(status);
@@ -698,36 +513,54 @@ namespace SUNDIALS
           {
             linear_solver =
               std::make_unique<internal::LinearSolverWrapper<VectorType>>(
-                solve_linearized_system);
+                solve_linearized_system
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+                ,
+                arkode_ctx
+#  endif
+              );
             sun_linear_solver = *linear_solver;
           }
         else
           {
             // use default solver from SUNDIALS
             // TODO give user options
-            auto y_template = internal::make_nvector_view(solution);
+            auto y_template = internal::make_nvector_view(solution
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+                                                          ,
+                                                          arkode_ctx
+#  endif
+            );
+#  if DEAL_II_SUNDIALS_VERSION_LT(6, 0, 0)
             sun_linear_solver =
               SUNLinSol_SPGMR(y_template,
                               PREC_NONE,
                               0 /*krylov subvectors, 0 uses default*/);
+#  else
+            sun_linear_solver =
+              SUNLinSol_SPGMR(y_template,
+                              PREC_NONE,
+                              0 /*krylov subvectors, 0 uses default*/,
+                              arkode_ctx);
+#  endif
           }
         status = ARKStepSetLinearSolver(arkode_mem, sun_linear_solver, nullptr);
         AssertARKode(status);
-        status =
-          ARKStepSetJacTimes(arkode_mem,
-                             jacobian_times_setup ?
-                               t_arkode_jac_times_setup_function<VectorType> :
-                               nullptr,
-                             t_arkode_jac_times_vec_function<VectorType>);
+        status = ARKStepSetJacTimes(
+          arkode_mem,
+          jacobian_times_setup ?
+            jacobian_times_vector_setup_callback<VectorType> :
+            nullptr,
+          jacobian_times_vector_callback<VectorType>);
         AssertARKode(status);
         if (jacobian_preconditioner_solve)
           {
             status = ARKStepSetPreconditioner(
               arkode_mem,
               jacobian_preconditioner_setup ?
-                t_arkode_prec_setup_function<VectorType> :
+                jacobian_solver_setup_callback<VectorType> :
                 nullptr,
-              t_arkode_prec_solve_function<VectorType>);
+              solve_with_jacobian_callback<VectorType>);
             AssertARKode(status);
           }
         if (data.implicit_function_is_linear)
@@ -739,11 +572,23 @@ namespace SUNDIALS
       }
     else
       {
-        auto y_template = internal::make_nvector_view(solution);
+        auto y_template = internal::make_nvector_view(solution
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+                                                      ,
+                                                      arkode_ctx
+#  endif
+        );
 
+#  if DEAL_II_SUNDIALS_VERSION_LT(6, 0, 0)
         SUNNonlinearSolver fixed_point_solver =
           SUNNonlinSol_FixedPoint(y_template,
                                   data.anderson_acceleration_subspace);
+#  else
+        SUNNonlinearSolver fixed_point_solver =
+          SUNNonlinSol_FixedPoint(y_template,
+                                  data.anderson_acceleration_subspace,
+                                  arkode_ctx);
+#  endif
 
         status = ARKStepSetNonlinearSolver(arkode_mem, fixed_point_solver);
         AssertARKode(status);
@@ -770,16 +615,34 @@ namespace SUNDIALS
           {
             mass_solver =
               std::make_unique<internal::LinearSolverWrapper<VectorType>>(
-                solve_mass);
+                solve_mass
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+                ,
+                arkode_ctx
+#  endif
+              );
             sun_mass_linear_solver = *mass_solver;
           }
         else
           {
-            auto y_template = internal::make_nvector_view(solution);
+            auto y_template = internal::make_nvector_view(solution
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+                                                          ,
+                                                          arkode_ctx
+#  endif
+            );
+#  if DEAL_II_SUNDIALS_VERSION_LT(6, 0, 0)
             sun_mass_linear_solver =
               SUNLinSol_SPGMR(y_template,
                               PREC_NONE,
                               0 /*krylov subvectors, 0 uses default*/);
+#  else
+            sun_mass_linear_solver =
+              SUNLinSol_SPGMR(y_template,
+                              PREC_NONE,
+                              0 /*krylov subvectors, 0 uses default*/,
+                              arkode_ctx);
+#  endif
           }
         booleantype mass_time_dependent =
           data.mass_is_time_independent ? SUNFALSE : SUNTRUE;
@@ -788,13 +651,13 @@ namespace SUNDIALS
                                             nullptr,
                                             mass_time_dependent);
         AssertARKode(status);
-        status =
-          ARKStepSetMassTimes(arkode_mem,
-                              mass_times_setup ?
-                                t_arkode_mass_times_setup_function<VectorType> :
-                                nullptr,
-                              t_arkode_mass_times_vec_function<VectorType>,
-                              this);
+        status = ARKStepSetMassTimes(
+          arkode_mem,
+          mass_times_setup ?
+            mass_matrix_times_vector_setup_callback<VectorType> :
+            nullptr,
+          mass_matrix_times_vector_callback<VectorType>,
+          this);
         AssertARKode(status);
 
         if (mass_preconditioner_solve)
@@ -802,14 +665,13 @@ namespace SUNDIALS
             status = ARKStepSetMassPreconditioner(
               arkode_mem,
               mass_preconditioner_setup ?
-                t_arkode_mass_prec_setup_function<VectorType> :
+                mass_matrix_solver_setup_callback<VectorType> :
                 nullptr,
-              t_arkode_mass_prec_solve_function<VectorType>);
+              solve_with_mass_matrix_callback<VectorType>);
             AssertARKode(status);
           }
       }
   }
-#  endif
 
 
 
