@@ -1,17 +1,18 @@
-//-----------------------------------------------------------
+// ---------------------------------------------------------------------
 //
-//    Copyright (C) 2017 - 2021 by the deal.II authors
+// Copyright (C) 2017 - 2022 by the deal.II authors
 //
-//    This file is part of the deal.II library.
+// This file is part of the deal.II library.
 //
-//    The deal.II library is free software; you can use it, redistribute
-//    it, and/or modify it under the terms of the GNU Lesser General
-//    Public License as published by the Free Software Foundation; either
-//    version 2.1 of the License, or (at your option) any later version.
-//    The full text of the license can be found in the file LICENSE.md at
-//    the top level directory of deal.II.
+// The deal.II library is free software; you can use it, redistribute
+// it, and/or modify it under the terms of the GNU Lesser General
+// Public License as published by the Free Software Foundation; either
+// version 2.1 of the License, or (at your option) any later version.
+// The full text of the license can be found in the file LICENSE.md at
+// the top level directory of deal.II.
 //
-//-----------------------------------------------------------
+// ---------------------------------------------------------------------
+
 
 
 #include <deal.II/base/config.h>
@@ -35,6 +36,7 @@
 #  endif
 
 #  include <deal.II/sundials/copy.h>
+#  include <deal.II/sundials/n_vector.h>
 
 // Make sure we #include the SUNDIALS config file...
 #  include <sundials/sundials_config.h>
@@ -50,9 +52,6 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace SUNDIALS
 {
-  using namespace internal;
-
-
   template <typename VectorType>
   KINSOL<VectorType>::AdditionalData::AdditionalData(
     const SolutionStrategy &strategy,
@@ -133,7 +132,7 @@ namespace SUNDIALS
   {
     template <typename VectorType>
     int
-    residual_or_iteration_callback(N_Vector yy, N_Vector FF, void *user_data)
+    residual_callback(N_Vector yy, N_Vector FF, void *user_data)
     {
       KINSOL<VectorType> &solver =
         *static_cast<KINSOL<VectorType> *>(user_data);
@@ -145,89 +144,50 @@ namespace SUNDIALS
       typename VectorMemory<VectorType>::Pointer dst_FF(mem);
       solver.reinit_vector(*dst_FF);
 
-      copy(*src_yy, yy);
+      internal::copy(*src_yy, yy);
 
       int err = 0;
       if (solver.residual)
         err = solver.residual(*src_yy, *dst_FF);
-      else if (solver.iteration_function)
+      else
+        Assert(false, ExcInternalError());
+
+      internal::copy(FF, *dst_FF);
+
+      return err;
+    }
+
+
+
+    template <typename VectorType>
+    int
+    iteration_callback(N_Vector yy, N_Vector FF, void *user_data)
+    {
+      KINSOL<VectorType> &solver =
+        *static_cast<KINSOL<VectorType> *>(user_data);
+      GrowingVectorMemory<VectorType> mem;
+
+      typename VectorMemory<VectorType>::Pointer src_yy(mem);
+      solver.reinit_vector(*src_yy);
+
+      typename VectorMemory<VectorType>::Pointer dst_FF(mem);
+      solver.reinit_vector(*dst_FF);
+
+      internal::copy(*src_yy, yy);
+
+      int err = 0;
+      if (solver.iteration_function)
         err = solver.iteration_function(*src_yy, *dst_FF);
       else
         Assert(false, ExcInternalError());
 
-      copy(FF, *dst_FF);
+      internal::copy(FF, *dst_FF);
 
       return err;
     }
 
 
 
-#  if DEAL_II_SUNDIALS_VERSION_LT(4, 1, 0)
-    template <typename VectorType>
-    int
-    setup_jacobian_callback(KINMem kinsol_mem)
-    {
-      KINSOL<VectorType> &solver =
-        *static_cast<KINSOL<VectorType> *>(kinsol_mem->kin_user_data);
-      GrowingVectorMemory<VectorType> mem;
-
-      typename VectorMemory<VectorType>::Pointer src_ycur(mem);
-      solver.reinit_vector(*src_ycur);
-
-      typename VectorMemory<VectorType>::Pointer src_fcur(mem);
-      solver.reinit_vector(*src_fcur);
-
-      copy(*src_ycur, kinsol_mem->kin_uu);
-      copy(*src_fcur, kinsol_mem->kin_fval);
-
-      int err = solver.setup_jacobian(*src_ycur, *src_fcur);
-      return err;
-    }
-
-
-
-    template <typename VectorType>
-    int
-    solve_with_jacobian_callback(KINMem    kinsol_mem,
-                                 N_Vector  x,
-                                 N_Vector  b,
-                                 realtype *sJpnorm,
-                                 realtype *sFdotJp)
-    {
-      KINSOL<VectorType> &solver =
-        *static_cast<KINSOL<VectorType> *>(kinsol_mem->kin_user_data);
-      GrowingVectorMemory<VectorType> mem;
-
-      typename VectorMemory<VectorType>::Pointer src_ycur(mem);
-      solver.reinit_vector(*src_ycur);
-
-      typename VectorMemory<VectorType>::Pointer src_fcur(mem);
-      solver.reinit_vector(*src_fcur);
-
-      copy(*src_ycur, kinsol_mem->kin_uu);
-      copy(*src_fcur, kinsol_mem->kin_fval);
-
-      typename VectorMemory<VectorType>::Pointer src(mem);
-      solver.reinit_vector(*src);
-
-      typename VectorMemory<VectorType>::Pointer dst(mem);
-      solver.reinit_vector(*dst);
-
-      copy(*src, b);
-
-      int err = solver.solve_jacobian_system(*src_ycur, *src_fcur, *src, *dst);
-      copy(x, *dst);
-
-      *sJpnorm = N_VWL2Norm(b, kinsol_mem->kin_fscale);
-      N_VProd(b, kinsol_mem->kin_fscale, b);
-      N_VProd(b, kinsol_mem->kin_fscale, b);
-      *sFdotJp = N_VDotProd(kinsol_mem->kin_fval, b);
-
-      return err;
-    }
-
-#  else // SUNDIALS 5.0 or later
-
     template <typename VectorType>
     int
     setup_jacobian_callback(N_Vector u,
@@ -251,8 +211,8 @@ namespace SUNDIALS
       solver.reinit_vector(*ycur);
       solver.reinit_vector(*fcur);
 
-      copy(*ycur, u);
-      copy(*fcur, f);
+      internal::copy(*ycur, u);
+      internal::copy(*fcur, f);
 
       // Call the user-provided setup function with these arguments:
       solver.setup_jacobian(*ycur, *fcur);
@@ -289,11 +249,11 @@ namespace SUNDIALS
           solver.reinit_vector(*src_b);
           solver.reinit_vector(*dst_x);
 
-          copy(*src_b, b);
+          internal::copy(*src_b, b);
 
           const int err = solver.solve_with_jacobian(*src_b, *dst_x, tol);
 
-          copy(x, *dst_x);
+          internal::copy(x, *dst_x);
 
           return err;
         }
@@ -315,7 +275,7 @@ namespace SUNDIALS
           solver.reinit_vector(*src_b);
           solver.reinit_vector(*dst_x);
 
-          copy(*src_b, b);
+          internal::copy(*src_b, b);
 
           // Call the user-provided setup function with these arguments. Note
           // that Sundials 4.x and later no longer provide values for
@@ -325,30 +285,50 @@ namespace SUNDIALS
           const int err =
             solver.solve_jacobian_system(*src_ycur, *src_fcur, *src_b, *dst_x);
 
-          copy(x, *dst_x);
+          internal::copy(x, *dst_x);
 
           return err;
         }
     }
-
-#  endif
   } // namespace
 
 
 
+  template <typename VectorType>
+  KINSOL<VectorType>::KINSOL(const AdditionalData &data)
+    : KINSOL(data, MPI_COMM_SELF)
+  {}
+
+
+
   template <typename VectorType>
   KINSOL<VectorType>::KINSOL(const AdditionalData &data,
                              const MPI_Comm &      mpi_comm)
     : data(data)
+    , mpi_communicator(mpi_comm)
     , kinsol_mem(nullptr)
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+    , kinsol_ctx(nullptr)
+#  endif
     , solution(nullptr)
     , u_scale(nullptr)
     , f_scale(nullptr)
-    , communicator(is_serial_vector<VectorType>::value ?
-                     MPI_COMM_SELF :
-                     Utilities::MPI::duplicate_communicator(mpi_comm))
   {
     set_functions_to_trigger_an_assert();
+
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+    // SUNDIALS will always duplicate communicators if we provide them. This
+    // can cause problems if SUNDIALS is configured with MPI and we pass along
+    // MPI_COMM_SELF in a serial application as MPI won't be
+    // initialized. Hence, work around that by just not providing a
+    // communicator in that case.
+    const int status =
+      SUNContext_Create(mpi_communicator == MPI_COMM_SELF ? nullptr :
+                                                            &mpi_communicator,
+                        &kinsol_ctx);
+    (void)status;
+    AssertKINSOL(status);
+#  endif
   }
 
 
@@ -356,16 +336,11 @@ namespace SUNDIALS
   template <typename VectorType>
   KINSOL<VectorType>::~KINSOL()
   {
-    if (kinsol_mem)
-      KINFree(&kinsol_mem);
-
-#  ifdef DEAL_II_WITH_MPI
-    if (is_serial_vector<VectorType>::value == false)
-      {
-        const int ierr = MPI_Comm_free(&communicator);
-        (void)ierr;
-        AssertNothrow(ierr == MPI_SUCCESS, ExcMPI(ierr));
-      }
+    KINFree(&kinsol_mem);
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+    const int status = SUNContext_Free(&kinsol_ctx);
+    (void)status;
+    AssertKINSOL(status);
 #  endif
   }
 
@@ -375,63 +350,133 @@ namespace SUNDIALS
   unsigned int
   KINSOL<VectorType>::solve(VectorType &initial_guess_and_solution)
   {
-    unsigned int system_size = initial_guess_and_solution.size();
+    // Make sure we have what we need
+    if (data.strategy == AdditionalData::fixed_point)
+      {
+        Assert(iteration_function,
+               ExcFunctionNotProvided("iteration_function"));
+      }
+    else
+      {
+        Assert(residual, ExcFunctionNotProvided("residual"));
+        Assert(solve_jacobian_system || solve_with_jacobian,
+               ExcFunctionNotProvided(
+                 "solve_jacobian_system || solve_with_jacobian"));
+      }
 
-    // The solution is stored in
-    // solution. Here we take only a
-    // view of it.
+    // Create a new solver object:
+    int status = 0;
+    (void)status;
+
+    KINFree(&kinsol_mem);
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+    status = SUNContext_Free(&kinsol_ctx);
+    AssertKINSOL(status);
+#  endif
+
+#  if DEAL_II_SUNDIALS_VERSION_LT(6, 0, 0)
+    kinsol_mem = KINCreate();
+#  else
+    // Same comment applies as in class constructor:
+    status =
+      SUNContext_Create(mpi_communicator == MPI_COMM_SELF ? nullptr :
+                                                            &mpi_communicator,
+                        &kinsol_ctx);
+    AssertKINSOL(status);
+
+    kinsol_mem = KINCreate(kinsol_ctx);
+#  endif
+
+    status = KINSetUserData(kinsol_mem, static_cast<void *>(this));
+    AssertKINSOL(status);
+
+    const auto system_size = initial_guess_and_solution.size();
+
+    // The solution is stored in solution. Here we take only a view of it.
 #  ifdef DEAL_II_WITH_MPI
     if (is_serial_vector<VectorType>::value == false)
       {
-        const IndexSet is = initial_guess_and_solution.locally_owned_elements();
-        const unsigned int local_system_size = is.n_elements();
+        const IndexSet &is =
+          initial_guess_and_solution.locally_owned_elements();
+        const auto local_system_size = is.n_elements();
 
-        solution =
-          N_VNew_Parallel(communicator, local_system_size, system_size);
+        solution = N_VNew_Parallel(mpi_communicator,
+                                   local_system_size,
+                                   system_size
+#    if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+                                   ,
+                                   kinsol_ctx
+#    endif
+        );
 
-        u_scale = N_VNew_Parallel(communicator, local_system_size, system_size);
+        u_scale = N_VNew_Parallel(mpi_communicator,
+                                  local_system_size,
+                                  system_size
+#    if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+                                  ,
+                                  kinsol_ctx
+#    endif
+        );
         N_VConst_Parallel(1.e0, u_scale);
 
-        f_scale = N_VNew_Parallel(communicator, local_system_size, system_size);
+        f_scale = N_VNew_Parallel(mpi_communicator,
+                                  local_system_size,
+                                  system_size
+#    if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+                                  ,
+                                  kinsol_ctx
+#    endif
+        );
         N_VConst_Parallel(1.e0, f_scale);
       }
     else
 #  endif
       {
-        Assert(is_serial_vector<VectorType>::value,
-               ExcInternalError(
-                 "Trying to use a serial code with a parallel vector."));
-        solution = N_VNew_Serial(system_size);
-        u_scale  = N_VNew_Serial(system_size);
+        Assert(is_serial_vector<VectorType>::value, ExcInternalError());
+        solution = N_VNew_Serial(system_size
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+                                 ,
+                                 kinsol_ctx
+#  endif
+        );
+        u_scale = N_VNew_Serial(system_size
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+                                ,
+                                kinsol_ctx
+#  endif
+        );
         N_VConst_Serial(1.e0, u_scale);
-        f_scale = N_VNew_Serial(system_size);
+        f_scale = N_VNew_Serial(system_size
+#  if DEAL_II_SUNDIALS_VERSION_GTE(6, 0, 0)
+                                ,
+                                kinsol_ctx
+#  endif
+        );
         N_VConst_Serial(1.e0, f_scale);
       }
 
     if (get_solution_scaling)
-      copy(u_scale, get_solution_scaling());
+      internal::copy(u_scale, get_solution_scaling());
 
     if (get_function_scaling)
-      copy(f_scale, get_function_scaling());
+      internal::copy(f_scale, get_function_scaling());
 
-    copy(solution, initial_guess_and_solution);
-
-    if (kinsol_mem)
-      KINFree(&kinsol_mem);
-
-    kinsol_mem = KINCreate();
-
-    int status =
-      KINInit(kinsol_mem, residual_or_iteration_callback<VectorType>, solution);
-    (void)status;
-    AssertKINSOL(status);
-
-    status = KINSetUserData(kinsol_mem, static_cast<void *>(this));
-    AssertKINSOL(status);
+    internal::copy(solution, initial_guess_and_solution);
 
+    // This must be called before KINSetMAA
     status = KINSetNumMaxIters(kinsol_mem, data.maximum_non_linear_iterations);
     AssertKINSOL(status);
 
+    // From the manual: this must be called BEFORE KINInit
+    status = KINSetMAA(kinsol_mem, data.anderson_subspace_size);
+    AssertKINSOL(status);
+
+    if (data.strategy == AdditionalData::fixed_point)
+      status = KINInit(kinsol_mem, iteration_callback<VectorType>, solution);
+    else
+      status = KINInit(kinsol_mem, residual_callback<VectorType>, solution);
+    AssertKINSOL(status);
+
     status = KINSetFuncNormTol(kinsol_mem, data.function_tolerance);
     AssertKINSOL(status);
 
@@ -450,9 +495,6 @@ namespace SUNDIALS
     status = KINSetMaxBetaFails(kinsol_mem, data.maximum_beta_failures);
     AssertKINSOL(status);
 
-    status = KINSetMAA(kinsol_mem, data.anderson_subspace_size);
-    AssertKINSOL(status);
-
     status = KINSetRelErrFunc(kinsol_mem, data.dq_relative_error);
     AssertKINSOL(status);
 
@@ -463,31 +505,16 @@ namespace SUNDIALS
         solve_with_jacobian) // user assigned a function object to the solver
                              // slot
       {
-/* interface up to and including 4.0 */
-#  if DEAL_II_SUNDIALS_VERSION_LT(4, 1, 0)
-        auto KIN_mem        = static_cast<KINMem>(kinsol_mem);
-        KIN_mem->kin_lsolve = solve_with_jacobian_callback<VectorType>;
-        if (setup_jacobian) // user assigned a function object to the Jacobian
-          // set-up slot
-          KIN_mem->kin_lsetup = setup_jacobian_callback<VectorType>;
-
-/* interface up to and including 4.1 */
-#  elif DEAL_II_SUNDIALS_VERSION_LT(5, 0, 0)
-
-        // deal.II does not currently have support for KINSOL in
-        // SUNDIALS 4.1. One could write this and update this section,
-        // but it does not seem worthwhile spending the time to
-        // interface with an old version of SUNDIAL given that the
-        // code below supports modern SUNDIAL versions just fine.
-        Assert(false, ExcNotImplemented());
-
-#  else /* interface starting with SUNDIALS 5.0 */
         // Set the operations we care for in the sun_linear_solver object
         // and attach it to the KINSOL object. The functions that will get
         // called do not actually receive the KINSOL object, just the LS
         // object, so we have to store a pointer to the current
         // object in the LS object
-        LS          = SUNLinSolNewEmpty();
+#  if DEAL_II_SUNDIALS_VERSION_LT(6, 0, 0)
+        LS = SUNLinSolNewEmpty();
+#  else
+        LS = SUNLinSolNewEmpty(kinsol_ctx);
+#  endif
         LS->content = this;
 
         LS->ops->gettype =
@@ -517,7 +544,11 @@ namespace SUNDIALS
         // if we don't set it, it won't call the functions that set up
         // the matrix object (i.e., the argument to the 'KINSetJacFn'
         // function below).
-        J          = SUNMatNewEmpty();
+#  if DEAL_II_SUNDIALS_VERSION_LT(6, 0, 0)
+        J = SUNMatNewEmpty();
+#  else
+        J  = SUNMatNewEmpty(kinsol_ctx);
+#  endif
         J->content = this;
 
         J->ops->getid = [](SUNMatrix /*ignored*/) -> SUNMatrix_ID {
@@ -545,35 +576,19 @@ namespace SUNDIALS
         // Finally, if we were given a set-up function, tell KINSOL about
         // it as well. The manual says that this must happen *after*
         // calling KINSetLinearSolver
-        if (setup_jacobian)
-          {
-            status =
-              KINSetJacFn(kinsol_mem, &setup_jacobian_callback<VectorType>);
-            AssertKINSOL(status);
-          }
-#  endif
-      }
-    else
-      {
-        J      = SUNDenseMatrix(system_size, system_size);
-        LS     = SUNDenseLinearSolver(u_scale, J);
-        status = KINDlsSetLinearSolver(kinsol_mem, LS, J);
+        if (!setup_jacobian)
+          setup_jacobian = [](const VectorType &, const VectorType &) {
+            return 0;
+          };
+        status = KINSetJacFn(kinsol_mem, &setup_jacobian_callback<VectorType>);
         AssertKINSOL(status);
       }
 
-    if (data.strategy == AdditionalData::newton ||
-        data.strategy == AdditionalData::linesearch)
-      Assert(residual, ExcFunctionNotProvided("residual"));
-
-    if (data.strategy == AdditionalData::fixed_point ||
-        data.strategy == AdditionalData::picard)
-      Assert(iteration_function, ExcFunctionNotProvided("iteration_function"));
-
     // call to KINSol
     status = KINSol(kinsol_mem, solution, data.strategy, u_scale, f_scale);
     AssertKINSOL(status);
 
-    copy(initial_guess_and_solution, solution);
+    internal::copy(initial_guess_and_solution, solution);
 
     // Free the vectors which are no longer used.
 #  ifdef DEAL_II_WITH_MPI
@@ -595,8 +610,10 @@ namespace SUNDIALS
     status = KINGetNumNonlinSolvIters(kinsol_mem, &nniters);
     AssertKINSOL(status);
 
-    SUNMatDestroy(J);
-    SUNLinSolFree(LS);
+    if (J != nullptr)
+      SUNMatDestroy(J);
+    if (LS != nullptr)
+      SUNLinSolFree(LS);
     KINFree(&kinsol_mem);
 
     return static_cast<unsigned int>(nniters);
